!=============================================================================
!  AHPS2LBRM constructs the necessary files for the LBRM from the rest of
!  the files used in the AHPS.  LBRM2AHPS converts the LBRM output back 
!  into the formats used by the rest of the AHPS.
!
!  Written by Tim Hunter, 21sep2000
!
!
!     FILE FORMAT DESCRIPTIONS
!     ------------------------
!
!     ********** LBRM-format files **********
!
!     LBRM.CFG
!        This is the configuration (or control) file which contains the
!        file names.  That's all it has in it.  The format is similar to
!        the older Windows ".INI" files.  That is, all lines will be ignored
!        except for those that start with the key string we are looking for
!        followed by an "=" sign.  The rest of the line will be considered
!        a file name.  An example file might look like the following:
!
!        |** This is the configuration file for the LBRM **
!        |This line will be ignored.
!        |$$$ So will this one.
!        |   spaces at the beginning of a line are insignificant.
!        | The ordering of the lines is also insignificant.
!        |The next 4 lines are the only ones we need to have in here.
!        |PARMFILE=PARAM.TXT
!        |BCFILE = BNDRCOND.TXT
!        |  METFILE =   METDATA.TXT
!        |OUTFILE = OUTPUT.TXT
!        |DATAFILE =FLOWDATA.TXT
!        | WORKDIR =  C:\LBRM\
!
!
!        Note that the entry "WORKDIR=" was included for use by the Windows-based
!        applications.  It is not used in this Fortran code and will be ignored.
!
!     ----------------------
!     PARAMETER FILE  (could be named anything)
!        This file contains what would be considered the control information
!        for the LBRM in simulation or calibration.  That includes dates,
!        watershed area, and parameters.
!
!        The format is as follows:
!
!        1st record: Identifier (FORMAT: A)
!                    |        |
!                    |________|___Any string the user wants to use which will
!                                 help identify this model run. This will be
!                                 echoed into the output file simply for the
!                                 user's help later.  It can be left blank with
!                                 no consequences if the user so desires.
!
!        2nd record: Area (FORMAT: E13.6E2)
!                    |  |
!                    |__|____watershed area in square meters.
!
!        3rd record: iD iM iY iD iM iY (FORMAT: 2I2, I4, 2I2, I4)
!                    || || || || || ||
!                    || || || || || ||___starting year for calibration run
!                    || || || || ||______starting month for calibration run
!                    || || || ||_________starting day for calibration run
!                    || || ||
!                    || || ||   NOTE: calibration date unused in simulation
!                    || || ||
!                    || || ||___starting year in data file for model run
!                    || ||______starting month in data file for model run
!                    ||_________starting day in data file for model run
!
!        4th record: iD iM iY (FORMAT: 2I2, I4)
!                    || || ||
!                    || || ||___ending year in data file for model run
!                    || ||______ending month in data file for model run
!                    ||_________ending day in data file for model run
!
!        5th through 13th records: model parameter and calibration info
!
!                    Parameter I J Minimum Maximum (FORMAT: E13.6E2)
!                    |       | | | |     | |     |
!                    |       | | | |     | |_____|_allowable parameter maximum (calibration only)
!                    |       | | | |_____|_allowable parameter minimum (calibration only)
!                    |       | | |_maximum number significant digits (calibration only)
!                    |       | |_starting significant digit (calibration only)
!                    |_______|_model parameter as follows:
!
!             Tbase   base temperature for computation of heat available
!                     for evapotranspiration
!             AlbedS  melt factor (proportionality constant) for computation
!                     of degree-day snowmelt
!             AlpPer  linear reservoir coefficient for percolation from
!                     the upper soil zone moisture storage
!             AlpUEv  partial linear reservoir coefficient for evapo-
!                     transpiration from the upper soil zone
!             AlpInt  linear reservoir coefficient for interflow from the
!                     lower soil zone moisture storage
!             AlpDPr  linear reservoir coefficient for deep percolation
!                     from the lower soil zone moisture storage
!             AlpLEv  partial linear reservoir coefficient for evapo-
!                     transpiration from the lower soil zone
!             AlpGw   linear reservoir coefficient for groundwater flow
!                     from the groundwater zone moisture storage
!             AlpSf   linear reservoir coefficient for basin outflow from
!                     the surface zone moisture storage
!
!        14th through 29th records: (FORMAT: E13.6E2)
!
!             Cons    proportionality constant for computation of heat
!                     available for evapotranspiration; CANNOT BE ZERO DURING
!                     PRODUCTION RUNS; IF ZERO DURING CALIBRATION, "Cons" IS
!                     COMPUTED FROM THE FOLLOWING 14 VALUES:
!             Rjan    January average mid-monthly daily cloudless solar
! 		      insolation at the surface (langleys)
!             Rfeb    February insolation
!             Rmar    March insolation
!             Rapr    April insolation
!             Rmay    May insolation
!             Rjun    June insolation
!             Rjul    July insolation
!             Raug    August insolation
!             Rsep    September insolation
!             Roct    October insolation
!             Rnov    November insolation
!             Rdec    December insolation
!             B1      empirical linear coefficient [ordinate-intercept]
!		      relating ratio of cloudy insolation/cloudless insolation
!                     to "cloud cover"
!             B2      empirical linear coefficient [slope] relating ratio of
!                     cloudy insolation/cloudless insolation to "cloud cover"
!             USZC    upper soil zone capacity (cm)
!
!       30th record: iParm Corr RMSE (FORMAT: I2, 2E13.6E2)
!                    |   | |  | |  |
!                    |   | |  | |__|__root mean square error (calibration only)
!                    |   | |__|_______correlation (calibration only)
!                    |___|____________parameter in start calibration with (calibration only)
!
!     ----------------------
!     BOUNDARY CONDITIONS FILE (could be named anything)
!        This file contains boundary conditions that the model outputs must adhere
!        to.  During the model run, values of the various storages are (re)set to
!        the boundary conditions on the indicated dates if the boundary conditions
!        are non-negative.  A date, corresponding to the starting date in the
!        parameter file, must be present in this file and would, by definition,
!        contain the initial conditions (which CANNOT be negative).  There may be
!        any number of boundary conditions present in this file, but THEY MUST BE
!        IN CHRONOLOGICAL ORDER!
!
!        1st record: skipped (may be anything)
!
!        2nd record: skipped (may be anything)
!
!        3-? records: iD iM iY Uszm Lszm Gzm Ss Snw (FORMAT: 2I2, I4, 5E13.6E2)
!                     || || || |  | |  | | | || | |
!                     || || || |  | |  | | | || |_|___snowpack moisture (cm.)
!                     || || || |  | |  | | | ||_______surface storage (cm.)
!                     || || || |  | |  | |_|__________groundwater storage (cm.)
!                     || || || |  | |__|______________lower soil zone storage (cm.)
!                     || || || |__|___________________upper soil zone storage (cm.)
!                     || || ||________________________date's year
!                     || ||___________________________date's month
!                     ||______________________________date's day
!
!
!     ----------------------
!     METEOROLOGICAL DATA FILE  (could be named anything)
!        This file contains the daily air temperatures (maximum and minimum)
!        and precipitation for each day of the model run period specified
!        in the parameter file.  It may contain data for dates outside that
!        period as well.  If so, only the data for the specified period will
!        actually be used.
!
!        Note that NO MISSING DATA IS ALLOWED!!!!!!  If the LBRM encounters
!        missing data during a run it will immediately stop with an error message.
!
!        The format is as follows:
!
!        1st record: Identifier (FORMAT: A)
!                    |        |
!                    |________|___Any string the user wants to use which will
!                                 help identify this data set.  It will not be
!                                 used by the LBRM for anything.
!
!        2nd record: "DDMMYYYY TMIN TMAX PREC" (FORMAT: A)
!                     |                     |
!                     |_____________________|__header info string - not used
!
!        Records 3 through the end of the file have daily data.  These records
!        must be in chronological order with no missing days.
!
!        dd mm yyyy Tmin Tmax Precip (FORMAT: 2I2, I4, 3I5)
!        || || |  | |  | |  | |    |
!        || || |  | |  | |  | |____|__depth of daily precipitation in
!        || || |  | |  | |  |         hundredths of millimeters.
!        || || |  | |  | |__|_________maximum daily air temperature in 
!        || || |  | |  |              hundredths of degrees Celsius.
!        || || |  | |__|______________minimum daily air temperature in
!        || || |  |                   hundredths of degrees Celsius.
!        || || |__|___________________four-digit year of data's date
!        || ||________________________two-digit month of data's date
!        ||___________________________two-digit day of data's date
!
!
!     ----------------------
!     LBRM OUTPUT DATA FILE  (could be named anything)
!        This file contains the daily output from the LBRM for each day
!        of the model run period specified in the parameter file.
!
!        The format is as follows:
!
!        1st record: Identifier (FORMAT: A)
!                    |        |
!                    |________|___This is the string that was read from record
!                                 1 of the PARAMETER file.
!
!        2nd record: "Output from the LBRM.  Runoff and storages are in mm over the basin."
!
!        3rd record: "DDMMYYYY  RUNOFF   USZM   LSZM GROUND SURFAC   SNOW"
!
!        Records 4 through the end of the file have the daily output (end-of-day).
!
!           dd mm yyyy Run USZ LSZ GZM SRF SNW (FORMAT: 2I2, I4, 6E11.4E2)
!           || || |  | | | | | | | | | | | | |
!           || || |  | | | | | | | | | | | |_|__snowpack moisture storage (cm)
!           || || |  | | | | | | | | | |_|______surface moisture storage (cm)
!           || || |  | | | | | | | |_|__________groundwater moisture storage (cm)
!           || || |  | | | | | |_|______________lower soil zone moisture storage (cm)
!           || || |  | | | |_|__________________upper soil zone moisture storage (cm)
!           || || |  | |_|______________________runoff volume for this day (cm)
!           || || |__|__________________________four-digit year of data's date
!           || ||_______________________________two-digit month of data's date
!           ||__________________________________two-digit day of data's date
!
!
!
!     ********** AHPS-format files **********
!
!     The file: PARMTABL.xxx contains the model parameters and initial and
!     boundary conditions for each subbasin.  Currently, up to 25 sets of
!     boundary conditions are allowed for each subbasin parameter set.  This
!     file has the following format:
!
!     1st record: name (FORMAT: A10)
!                 |  |
!                 |__|_____basin/subbasin name, left adjusted, in upper case,
!                          the left-most three characters of which must equal
!                          the basin name (xxx).
!
!     2nd record through 11th record: model parameter (FORMAT: e13.6e2)
!
!              TBASE    base Temperature for heat available for evapotrans.
!              ALBEDS   degree-day factor for snowmelt computations
!              ALPPER   coefficient for percolation from upper soil zone
!              ALPUEV   coefficient for evapotrans. from upper soil zone
!              ALPINT   coefficient for interflow from lower soil zone
!              ALPDPR   coefficient for deep percolation from lower soil zone
!              ALPLEV   coefficient for evapotrans. from lower soil zone
!              ALPGW    coefficient for groundwater flow from groundwater zone
!              ALPSF    coefficient for basin outflow from surface storage zone
!              CONS     coefficient for heat available for evapotrans.
!
!    12th record: id im iy uszm lszm gzm ss snw (FORMAT: 2I2, I4, 5E13.6E2)
!                 || || || |  | |  | | | || | |
!                 || || || |  | |  | | | || |_|_snow pack storage (centimeters)
!                 || || || |  | |  | | | ||___surface storage (centimeters)
!                 || || || |  | |  | |_|____groundwater storage (centimeters)
!                 || || || |  | |__|______lower soil zone storage (centimeters)
!                 || || || |__|_________upper soil zone storage (centimeters)
!                 || || ||____________date's year
!                 || ||_____________date's month
!                 ||______________date's day
!
!     This record represents initial model storage conditions on the first day
!     of the calibration period.  These initial conditions are never used in
!     any of the forecast programs but are necessary for climatic data
!     preparation programs and are included for completeness.  The format of
!     record 12 is repeated between 0 and 24 more times for various boundary
!     conditions that the user might wish to include; these may represent field
!     measurements, for example.  Any values present in this manner will be
!     used by the forecast package to replace model generated storage values
!     on the dates specified.
!
!     The format of records 1 through 12-36 is repeated successively for each
!     subbasin, respectively. 
!
!     ----------------------
!     The summary storage files, xxx??.SUM, are direct access files with a
!     record length of 12 bytes and are unformatted.  They have the following
!     format:
!
!      1st record: I1 I2 I3 (UNFORMATTED: 2, 4, 2)
!                  || || ||
!                  || || ||___scale number
!                  || ||___sequence number corresponding to the starting
!                  ||      date of the file.
!                  ||___basin/subbasin identifier (e. g., 119 corresponds
!                       to Lake Superior, subbasin 19)
!
!      2nd record: I1 I2  (UNFORMATTED: 4, 4)
!                  || ||
!                  || ||_____ Sequence number corresponding to the ending
!                  ||         date in the file.
!                  ||________ Sequence number corresponding to the last
!                             valid date in the file.
!
!      Remaining records: I1 I2 I3 I4 I5 I6 (UNFORMATTED: 2, 2, 2, 2, 2, 2)
!                         || || || || || ||
!                         || || || || || ||__daily snowpack moisture (cm/100)
!                         || || || || ||___daily surface storage (cm/100)
!                         || || || ||____daily groundwater storage (cm/100)
!                         || || ||_____daily lower soil zone storage (cm/100)
!                         || ||______daily upper soil zone storage (cm/100)
!                         ||_______daily runoff volume (cm/100)
!
!
!-----------------------------------------------------------------

MODULE LBRM_FILES

      USE FcstUtil
      USE ReadData

CONTAINS

! ----------------------------------------------------------------------
!
!     Read the start/end dates from BSNNM or met data files.
!     The source is dependent upon the value of the PF
!     variable passed in to this routine.
!          1 = We are processing the provisional data period.
!              Read the earliest_new_data record (REC=8) for start date
!              Read the prv_data_end_date record (REC=4) for end date
!          2 = We are processing the forecast period.
!              Read the forecast_start_date record (REC=5) for start date
!              Read the forecast_end_date record   (REC=6) for end date
!         -9 = We are processing the entire period of the CLM (& PRV) files.
!              Read the start date from xxx01.CLM
!              Read the end date from xxx01.PRV (if it exists, else xxx01.CLM)
!
      SUBROUTINE GetLBRMStartEndDates(PF, Bsn, RunStart, RunEnd)
      IMPLICIT   NONE
      INTEGER, INTENT(IN)           :: PF
      INTEGER, INTENT(OUT)          :: RunStart, RunEnd
      CHARACTER (LEN=3), INTENT(IN) :: Bsn


      INTEGER (KIND=INTEGER2) :: I2
      INTEGER                 :: Dy, Mn, Yr, Seq
      LOGICAL                 :: FileExists
      CHARACTER (LEN=200)     :: Line, File11


      IF (PF .GT. 0) THEN

         File11 = TRIM(ThisLakeDir) // 'BSNNM'
         OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', RECL=12,           &
     &        ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
         CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
         IF (PF .EQ. 1) THEN
            READ(11, 1111, REC=8, ERR=812) Dy, Mn, Yr
            CALL DateSequence(Dy, Mn, Yr, Seq); IF (ErrorLevel .NE. 0) GOTO 899
            RunStart = Seq
            READ(11, 1111, REC=4, ERR=812) Dy, Mn, Yr
            CALL DateSequence(Dy, Mn, Yr, Seq); IF (ErrorLevel .NE. 0) GOTO 899
            RunEnd = Seq
         END IF
         IF (PF .EQ. 2) THEN
            READ(11, 1111, REC=5, ERR=812) Dy, Mn, Yr
            CALL DateSequence(Dy, Mn, Yr, Seq); IF (ErrorLevel .NE. 0) GOTO 899
            RunStart = Seq
            READ(11, 1111, REC=6, ERR=812) Dy, Mn, Yr
            CALL DateSequence(Dy, Mn, Yr, Seq); IF (ErrorLevel .NE. 0) GOTO 899
            RunEnd = Seq
         END IF
         CLOSE(11)
         CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

      ELSE

         File11 = TRIM(ThisLakeDir) // TRIM(Bsn) // '01.CLM'
         OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', RECL=6,            &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=811)
         CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
         READ(11, REC=1, ERR=812) I2, RunStart
         CLOSE(11)
         CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899 

         File11 = TRIM(ThisLakeDir) // TRIM(Bsn) // '01.PRV'
         INQUIRE(FILE=TRIM(File11), EXIST=FileExists)
         IF (FileExists) THEN
            OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', RECL=6,         &
     &           ACCESS='DIRECT', FORM='UNFORMATTED', ERR=811)
            CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
            READ(11, REC=2, ERR=812) RunEnd
            CLOSE(11)
            CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899
         ELSE
            File11 = TRIM(ThisLakeDir) // TRIM(Bsn) // '01.CLM'
            OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', RECL=6,         &
     &           ACCESS='DIRECT', FORM='UNFORMATTED', ERR=811)
            CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
            READ(11, REC=2, ERR=812) I2, RunEnd
            CLOSE(11)
            CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899
         END IF

      END IF   ! (PF .GT. 0)
      RETURN


  811 ErrorMessage = 'LBRM: Error opening file '//TRIM(File11)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'LBRM: Error reading file '//TRIM(File11)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1  ! Return with an error indication.
  899 ErrorMessage = '[traceback] : GetLBRMStartEndDates...'
      CALL PassMsg
      RETURN

 1111 FORMAT(2I3, I5)

      END SUBROUTINE GetLBRMStartEndDates

! ----------------------------------------------------------------------
      SUBROUTINE AHPS_FindArraySizes(PF, Sub, Lake, RunMetSize, BoundSize)
      IMPLICIT  NONE
      INTEGER, INTENT(IN)     ::  PF, Sub
      INTEGER, INTENT(INOUT)  ::  RunMetSize, BoundSize
      CHARACTER (LEN=3), INTENT(IN) :: Lake

      INTEGER (KIND=INTEGER2) :: I2
      INTEGER :: I, IOS, U1, Dy, Mn, Yr, LakeNum, Seq, RunStart, RunEnd
      LOGICAL :: InTheSubbasin, EndOfEntries
      CHARACTER (LEN=2)   :: SB
      CHARACTER (LEN=3)   :: Bsn
      CHARACTER (LEN=20)  :: S
      CHARACTER (LEN=80)  :: Line
      CHARACTER (LEN=120) :: File11

      ErrorLevel = 0

!
!  Convert sub from a number to a 2-char string
!
      WRITE(SB, '(I2.2)', IOSTAT=IOS) Sub
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'AHPS_FindArraySizes : Invalid subbasin specified.'
         CALL PassMsg
         GOTO 898
      END IF

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) THEN
            ThisLakeDir = TRIM(CfgDirs(I))
            LakeNum = I
         END IF
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Read the start/end dates
!
      CALL GetLBRMStartEndDates(PF, Lake, RunStart, RunEnd)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     Compute number of entries to be put into the runoff (output)
!     data file.  This is the same as the number of entries to be
!     read from the meteorological data (input) file.
!
      RunMetSize = RunEnd-RunStart+1
      IF (RunMetSize .GT. RD_MaxDays) THEN
         ErrorMessage = 'AHPS_FindArraySizes : Daily data array is statically sized too small.'
         CALL PassMsg
         GOTO 898
      END IF

!
!     Read the LBRMBNDC.xxx file and determine number of relevant entries.
!     Don't get the values yet, just how many.
!     If we are doing a forecast, then we will only have 1 entry -- the initial conditions.
!     If we are processing for a historical data period, though, we will have the
!     initial conditions plus any other values that fall within the period.
!
      IF (PF .EQ. 2) THEN
         BoundSize = 1
      ELSE
         BoundSize = 1
         File11  = TRIM(ThisLakeDir) // 'LBRMBNDC.'//TRIM(Lake)
         OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', ERR=811)
         CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
         InTheSubbasin = .FALSE.
         DO WHILE (.NOT. InTheSubbasin)
            READ(11, 1121, IOSTAT=IOS) Line
            CALL Caps(Line)
            IF ((Line(1:8) .EQ. 'SUBBASIN') .AND. (Line(13:14) .EQ. SB)) &
     &         InTheSubbasin = .TRUE.
         END DO
         IF (IOS .NE. 0) GOTO 812
         EndOfEntries = .FALSE.
         DO WHILE ((IOS .EQ. 0) .AND. (.NOT. EndOfEntries))
            READ(11, 1121, IOSTAT=IOS) Line
            IF (IOS .NE. 0) CYCLE
            IF (Line(1:3) .EQ. 'SUB') THEN
               EndOfEntries = .TRUE.
               CYCLE
            END IF
            IF (LEN_TRIM(Line) .LT. 50) CYCLE
            READ(Line, 1131, IOSTAT=IOS) Dy, Mn, Yr
            IF (IOS .NE. 0) CYCLE
            CALL DateSequence(Dy, Mn, Yr, Seq); IF (ErrorLevel .NE. 0) GOTO 899
            IF ((Seq .GE. RunStart) .AND. (Seq .LE. RunEnd)) THEN
               BoundSize = BoundSize + 1
            END IF
         END DO
         CLOSE(11)
         CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899
      END IF

      RETURN


  811 ErrorMessage = 'LBRM: Error opening file '//TRIM(File11)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'LBRM: Error reading file '//TRIM(File11)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1  ! Return with an error indication.
  899 ErrorMessage = '[traceback] : AHPS_FindArraySizes...'
      CALL PassMsg
      RETURN

 1111 FORMAT(2I3, I5)
 1121 FORMAT(A80)
 1131 FORMAT(2X, 2I2, I4, 5E13.6E2)

      END SUBROUTINE AHPS_FindArraySizes

! ----------------------------------------------------------------------
      SUBROUTINE AHPSInputData(PF,Sub,Lake, BCnt,DCnt, BCSeqs, BCU,BCL,BCG,BCS,BCN, Tx,Tn,Pc)
      IMPLICIT  NONE
      INTEGER, INTENT(IN)     :: PF, Sub
      INTEGER, INTENT(INOUT)  :: BCnt, DCnt
      INTEGER (KIND=INTEGER2), DIMENSION(DCnt), INTENT(INOUT) :: Tx, Tn, Pc
      INTEGER, DIMENSION(BCnt), INTENT(INOUT) :: BCSeqs
      REAL,    DIMENSION(BCnt), INTENT(INOUT) :: BCU, BCL, BCG, BCS, BCN
      CHARACTER (LEN=3), INTENT(IN) :: Lake

      INTEGER (KIND=INTEGER2) :: I2
      INTEGER :: I, J, K, IOS, IOS2, LakeNum, U1, Dy, Mn, Yr, Seq
      INTEGER :: RunStart, RunEnd, MetStart, MetEnd, FillStart, FillEnd
      LOGICAL :: InTheSubbasin, EndOfEntries, SomeIsMissing, FileExists
      REAL    :: Area

      REAL, DIMENSION(6)      :: RunDat
      REAL, DIMENSION(1, 6)   :: SumDat
      REAL, DIMENSION(RD_MaxDays, 4) :: MetDat

      CHARACTER (LEN=2)   :: SB
      CHARACTER (LEN=3)   :: Bsn
      CHARACTER (LEN=20)  :: S
      CHARACTER (LEN=200) :: Line, File11

      ErrorLevel = 0

!
!  Convert sub from a number to a 2-char string
!
      WRITE(SB, '(I2.2)', IOSTAT=IOS) Sub
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'AHPSInputData : Invalid subbasin specified.'
         CALL PassMsg
         GOTO 898
      END IF

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) THEN
            ThisLakeDir = TRIM(CfgDirs(I))
            LakeNum = I
         END IF
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Read the start/end dates
!
      CALL GetLBRMStartEndDates(PF, Lake, RunStart, RunEnd)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     If we are doing the forecast (outlook) period, then we don't read the
!     met data from the actual forecast period, rather we read it from 
!     somewhere during the historical time period (CLM or PRV files) starting
!     on the date specified in DATEFILE.INT
!
      MetStart = RunStart
      MetEnd   = RunEnd
      IF (PF .EQ. 2) THEN
         File11 = TRIM(ThisLakeDir) // 'DATEFILE.INT'
         OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', ERR=811)
         CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
         READ(11, 1161, ERR=812) Dy
         READ(11, 1161, ERR=812) Mn
         READ(11, 1161, ERR=812) Yr
         CLOSE(11)
         CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899
         CALL DateSequence(Dy,Mn,Yr,MetStart); IF (ErrorLevel .NE. 0) GOTO 899
         MetEnd = MetStart + (RunEnd-RunStart)
      END IF

!
!     Read the met data from the *.CLM and/or *.PRV file(s).  
!     The data is read as: 
!        Tmin, Tmax = degrees C
!        Precip     = millimeters
!
!     By setting RD_FillMetMissing to TRUE, any time the data is missing it will be
!     filled with the long-term average value for that particular day of the year
!     (i.e. Julian day).
!
      RD_FillMetMissing = .TRUE.
      RD_LastMissingBsnSub = 'xxx99'    !  arbitrary initial value that is not valid
      MetDat(:,:) = -99.99
      K = Sub
      Bsn = Lake(1:3)
      CALL ReadMetData(MetStart, MetEnd, MetDat, Bsn, K, ThisLakeDir)
      IF (ErrorLevel .NE. 0) GOTO 899
      RD_FillMetMissing = .FALSE.          !  set it back to default value

!
!     Copy the met data into the appropriate arrays.
!     Convert the data units back to:
!        Tmin, Tmax = hundredths of degrees C
!        Precip     = hundredths of millimeters
!
!     The LBRM program needs the data in these units
!
      DO Seq = MetStart, MetEnd
         I = Seq - MetStart + 1
         Tn(I) = -9999.9
         Tx(I) = -9999.9
         Pc(I) = -9999.9
         IF (MetDat(I,1) .GT. -98.) Tn(I) = NINT(MetDat(I,1) * 100)      ! tmin  C -> C/100
         IF (MetDat(I,2) .GT. -98.) Tx(I) = NINT(MetDat(I,2) * 100)      ! tmax  C -> C/100
         IF (MetDat(I,3) .GT. -98.) Pc(I) = NINT(MetDat(I,3) * 100)      ! prec  mm -> mm/100
      END DO

!
!     Read the data from the xxxnn.SUM files.  We need the values on the
!     date before the run start date in order to set the appropriate 
!     boundary conditions for the run.  If the requested date falls
!     outside the bounds of the SUM files, then we will use the 
!     long-term average values on that day. 
!       (e.g. Suppose we are doing a run from the very beginning of of
!        CLM files [1/1/1948]. Then we need the values from the SUM
!        files for 12/31/1947, but they don't extend back that far. We
!        will get all of the 12/31 values and average them.)
!
!     These values will be used as the values on the start date, not the
!     day before, because values in the *.SUM files are the END-OF-DAY
!     values while the values in the boundary files are the START-OF-DAY
!     values. 
!
      SumDat(:,:) = -9999.9
      I = Sub
      Bsn = Lake(1:3)
      J = RunStart - 1
      K = RunStart - 1
      RD_FillRunMissing = .TRUE.                            !  set it TRUE
      CALL ReadRunData(J, K, SumDat, Bsn, I, ThisLakeDir)
      RD_FillRunMissing = .FALSE.                           !  set it back to default value
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     Get subbasin area for use when storing boundary conditions
!
      File11  = TRIM(ThisLakeDir) // 'AREA.'//TRIM(Lake)
      OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      DO I = 1, Sub
         READ(11, 1123, ERR=812) Area
      END DO
      CLOSE(11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Read the boundary conditions file (LBRMBNDC.xxx) and keep the relevant values.
!     If we are doing a forecast, then we will only have 1 entry -- the initial conditions.
!     If we are processing for the provisional data period, though, we will have the
!     initial conditions plus any other values that fall within the period.
!
!     Values are converted to cubic meters when stored.  Thus we need to first get 
!     the subbasin area (just done above).
!
      K = 1
      BCSeqs(K) = RunStart
      BCU(K) = SumDat(1,2) / 100 * Area    ! cm -> cubic meters
      BCL(K) = SumDat(1,3) / 100 * Area    ! cm -> cubic meters
      BCG(K) = SumDat(1,4) / 100 * Area    ! cm -> cubic meters
      BCS(K) = SumDat(1,5) / 100 * Area    ! cm -> cubic meters
      BCN(K) = SumDat(1,6) / 100 * Area    ! cm -> cubic meters

      IF ((PF .EQ. 1) .OR. (PF .EQ. -9)) THEN
         File11  = TRIM(ThisLakeDir) // 'LBRMBNDC.'//TRIM(Lake)
         OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', ERR=811)
         CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
         InTheSubbasin = .FALSE.
         DO WHILE (.NOT. InTheSubbasin)
            READ(11, 1121, IOSTAT=IOS) Line
            CALL Caps(Line)
            IF ((Line(1:8) .EQ. 'SUBBASIN') .AND. (Line(13:14) .EQ. SB)) &
     &         InTheSubbasin = .TRUE.
         END DO

         IF (IOS .EQ. 0) THEN       ! we are ready to read data for the subbasin
            EndOfEntries = .FALSE.
            DO WHILE ((IOS .EQ. 0) .AND. (.NOT. EndOfEntries))
               READ(11, 1121, IOSTAT=IOS) Line
               IF (IOS .NE. 0) CYCLE
               CALL Caps(Line); IF (ErrorLevel .NE. 0) GOTO 899
               IF (Line(1:3) .EQ. 'SUB') THEN
                  EndOfEntries = .TRUE.
                  CYCLE
               END IF
               IF (LEN_TRIM(Line) .LT. 50) CYCLE
               READ(Line, 1131, IOSTAT=IOS2) Dy, Mn, Yr, (RunDat(I), I=1,5)
               IF (IOS2 .NE. 0) CYCLE
               CALL DateSequence(Dy, Mn, Yr, Seq); IF (ErrorLevel .NE. 0) GOTO 899
               IF (Seq .EQ. RunStart) THEN
                  IF (BCU(1) .LT. -99) BCU(1) = RunDat(1) / 100 * Area
                  IF (BCL(1) .LT. -99) BCL(1) = RunDat(2) / 100 * Area
                  IF (BCG(1) .LT. -99) BCG(1) = RunDat(3) / 100 * Area
                  IF (BCS(1) .LT. -99) BCS(1) = RunDat(4) / 100 * Area
                  IF (BCN(1) .LT. -99) BCN(1) = RunDat(5) / 100 * Area
               END IF
               IF ((Seq .GT. RunStart) .AND. (Seq .LE. RunEnd)) THEN
                  K = K + 1
                  BCSeqs(K) = Seq
                  BCU(K) = RunDat(1) / 100 * Area    ! cm -> cubic meters
                  BCL(K) = RunDat(2) / 100 * Area    ! cm -> cubic meters
                  BCG(K) = RunDat(3) / 100 * Area    ! cm -> cubic meters
                  BCS(K) = RunDat(4) / 100 * Area    ! cm -> cubic meters
                  BCN(K) = RunDat(5) / 100 * Area    ! cm -> cubic meters
               END IF
            END DO
         END IF            
         CLOSE(11)
         CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899
      END IF
      RETURN


  811 ErrorMessage = 'LBRM: Error opening file '//TRIM(File11)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'LBRM: Error reading file '//TRIM(File11)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1  ! Return with an error indication.
  899 ErrorMessage = '[traceback] : AHPSInputData...'
      CALL PassMsg
      RETURN

 1111 FORMAT(2I3, I5)
 1121 FORMAT(A)
 1122 FORMAT(2I2, I4, 2I2, I4)
 1123 FORMAT(E13.6E2)
 1131 FORMAT(2X, 2I2, I4, 5E13.6E2)
 1161 FORMAT(I4)

      END SUBROUTINE AHPSInputData

! ----------------------------------------------------------------------
      SUBROUTINE AHPS2LBRM(ErrStatus, HandleIn, PrvFcst, Sub, Lake)
      IMPLICIT  NONE
      INTEGER, INTENT(INOUT)  ::  ErrStatus
      INTEGER, INTENT(IN)     ::  HandleIn, PrvFcst, Sub
      CHARACTER (LEN=3), INTENT(IN) :: Lake

      INTEGER  :: I, J, K, IOS, LakeNum, BoundVals, InitDate
      INTEGER  :: Dy, Mn, Yr, Seq, RunStart, RunEnd, MetStart, MetEnd
      INTEGER, DIMENSION(200) :: BCDates

      LOGICAL :: FoundIt

      REAL  ::  ParmVal, InsolationVal, Cons, Coeff, SoilCap, Area
      REAL, DIMENSION(6)      :: RunDat
      REAL, DIMENSION(1, 6)   :: SumDat
      REAL, DIMENSION(200, 5) :: Bound
      REAL, DIMENSION(RD_MaxDays, 4) :: MetDat

      CHARACTER (LEN=2)   :: SB
      CHARACTER (LEN=3)   :: Bsn
      CHARACTER (LEN=20)  :: S
      CHARACTER (LEN=99)  :: ParmFile, BoundFile, MetFile, OutFile
      CHARACTER (LEN=99)  :: File11, File21
      CHARACTER (LEN=200) :: Line

      WinHandle = HandleIn

!
!  Convert sub from a number to a 2-char string
!
      WRITE(SB, '(I2.2)', IOSTAT=I) Sub
      IF (I .NE. 0) THEN
         ErrorMessage = 'AHPS2LBRM : Invalid subbasin specified.'
         CALL PassMsg
         GOTO 898
      END IF

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) THEN
            ThisLakeDir = TRIM(CfgDirs(I))
            LakeNum = I
         END IF
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF
      Bsn = LakeName3(LakeNum)

!
!     Read the start/end dates
!
      CALL GetLBRMStartEndDates(PrvFcst, Lake, RunStart, RunEnd)
      IF (ErrorLevel .NE. 0) GOTO 899

      IF (RunEnd-RunStart+1  .GT.  RD_MaxDays) THEN
         ErrorMessage = 'AHPS2LBRM : Daily data array is too small.'
         CALL PassMsg
         GOTO 898
      END IF

!
!     Get subbasin area for output to PARAM??.xxx file
!
      File11  = TRIM(ThisLakeDir) // 'AREA.' // Bsn
      OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      DO I = 1, Sub
         READ(11, 1123, ERR=812) Area
      END DO
      CLOSE(11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Read the LBRMPARM.xxx file and write the PARAM??.INT file
!
      CALL BuildParmFile(ThisLakeDir, Bsn, SB, Area, RunStart, RunEnd)
      IF (ErrorLevel .NE. 0) GOTO 899

!
!     Build the LBRM.CFG file
!     There is no "DATAFILE" entry because we are not calibrating.
!     'METFILE' will not be used, but is written for reference.
!      
      ParmFile  = TRIM(ThisLakeDir) // 'PARAM' //SB//'.INT'
      BoundFile = TRIM(ThisLakeDir) // 'BOUND' //SB//'.INT'
      MetFile   = TRIM(ThisLakeDir) // 'METDAT'//SB//'.INT'
      OutFile   = TRIM(ThisLakeDir) // 'RUNDAT'//SB//'.INT'

      File21 = TRIM(ThisLakeDir) // 'LBRM.CFG'
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', ERR=30)
      CLOSE(21, STATUS='DELETE')
  30  OPEN(UNIT=21, FILE=TRIM(File21), STATUS='NEW', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(21, 2101, ERR=813) TRIM(ParmFile)
      WRITE(21, 2102, ERR=813) TRIM(BoundFile)
      WRITE(21, 2103, ERR=813) TRIM(MetFile)
      WRITE(21, 2104, ERR=813) TRIM(OutFile)
      WRITE(21, 2105, ERR=813) TRIM(ThisLakeDir)
      CLOSE(21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Error Handling
! 
 811  ErrorMessage = 'AHPS2LBRM : Error opening file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 812  ErrorMessage = 'AHPS2LBRM : Error reading file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 813  ErrorMessage = 'AHPS2LBRM : Error writing file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 821  ErrorMessage = 'AHPS2LBRM : Error opening file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898
 822  ErrorMessage = 'AHPS2LBRM : Error reading file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898
 823  ErrorMessage = 'AHPS2LBRM : Error writing file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : AHPS2LBRM...'
      CALL PassMsg
      GOTO 999

!
!  Final cleanup
!
 999  CALL CloseAllFiles()
      ErrStatus = ErrorLevel

      RETURN

!
!  FORMATs
!
 1111 FORMAT(2I3, I5)
 1121 FORMAT(A)
 1122 FORMAT(2I2, I4, 2I2, I4)
 1123 FORMAT(E13.6E2)
 1131 FORMAT(2I2, I4, 5E13.6E2)
 1161 FORMAT(I4)

 2101 FORMAT(' PARMFILE = ', A)
 2102 FORMAT(' BCFILE   = ', A)
 2103 FORMAT(' METFILE  = ', A)
 2104 FORMAT(' OUTFILE  = ', A)
 2105 FORMAT(' WORKDIR  = ', A)

 2122 FORMAT(2I2, I4, 2I2, I4)
 2123 FORMAT(E13.6E2, ' 9 9 9.999999E-09 9.999999E+09  (parameter ', I2, ')')
 2124 FORMAT(E13.6E2, '                                (Cons, cal/cm2)')
 2125 FORMAT(E13.6E2, '                                (insolation - ',A3,')')
 2126 FORMAT(E13.6E2, '                                ', A)
 2127 FORMAT(' 1 9.999999E+09 9.999999E+09             (dummy calibration stuff)')

 2131 FORMAT(A, A2, '    # of days = ', I3)
 2132 FORMAT('DDMMYYYY TMIN TMAX PREC')
 2133 FORMAT(2I2, I4, 3I5)

      END SUBROUTINE AHPS2LBRM



! ----------------------------------------------------------------------
      SUBROUTINE BuildParmFile(Dir, Bsn, SB, Area, SSeq, ESeq)
      IMPLICIT  NONE
      CHARACTER (LEN=*), INTENT(IN) :: Dir
      CHARACTER (LEN=3), INTENT(IN) :: Bsn
      CHARACTER (LEN=2), INTENT(IN) :: SB
      REAL, INTENT(IN)              :: Area
      INTEGER, INTENT(IN)           :: SSeq, ESeq

      INTEGER  :: I, J, K, IOS, Dy, Mn, Yr

      LOGICAL :: FoundIt

      REAL  ::  ParmVal, InsolationVal, Cons, Coeff, SoilCap
      REAL, DIMENSION(6)      :: RunDat
      REAL, DIMENSION(1, 6)   :: SumDat

      CHARACTER (LEN=20)  :: S
      CHARACTER (LEN=99)  :: File11, File21
      CHARACTER (LEN=200) :: Line

!
!     Read the LBRMPARM.xxx file and write the PARAM??.INT file
!
      File11  = TRIM(Dir) // 'LBRMPARM.'//TRIM(Bsn)
      OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899

      File21  = TRIM(Dir) // 'PARAM'//SB//'.INT'
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', ERR=10)
      CLOSE(21, STATUS='DELETE')
  10  OPEN(UNIT=21, FILE=TRIM(File21), STATUS='NEW', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Find the header for this subbasin
!
      S = 'SUBBASIN ' // TRIM(Bsn) // SB       ! e.g. 'SUBBASIN SUP01'
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      FoundIt = .FALSE.
      READ(11, 1121, IOSTAT=IOS) Line
      DO WHILE ((.NOT. FoundIt) .AND. (IOS .EQ. 0))
         CALL Caps(Line); IF (ErrorLevel .NE. 0) GOTO 899
         IF (Line(1:14) .EQ. S) THEN
            FoundIt = .TRUE.
         ELSE
            READ(11, 1121, IOSTAT=IOS) Line
         END IF
      END DO
      IF (IOS .NE. 0) GOTO 812

!
!     Write the header type info into the output file
!
      WRITE(21, 1121, ERR=823) TRIM(Line)          ! header line (ID)
      WRITE(21, 1123, ERR=823) Area                ! subbasin area (sq meters)
      CALL SequenceDate(Dy, Mn, Yr, SSeq); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(Line, 2122) Dy, Mn, Yr, 9, 9, 9999
      WRITE(21, 1121, ERR=823) TRIM(Line)//'      (start date, unused)'
      CALL SequenceDate(Dy, Mn, Yr, ESeq); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(Line, 2122) Dy, Mn, Yr
      WRITE(21, 1121, ERR=823) TRIM(Line)//'              (end date)'

!
!     Read/write the parameters
!
      DO I = 1, 9
         READ(11, 1123, ERR=812) ParmVal
         WRITE(21, 2123, ERR=823) ParmVal, I
      END DO
      READ(11, 1123, ERR=812) Cons
      WRITE(21, 2124, ERR=823) Cons
      DO I = 1, 12
         READ(11, 1123, ERR=812) InsolationVal
         WRITE(21, 2125, ERR=823) InsolationVal, MonthName3(I)
      END DO
      READ (11, 1123, ERR=812) Coeff
      WRITE(21, 2126, ERR=823) Coeff, '(1st empirical linear coefficient [ordinate-intercept])'
      READ (11, 1123, ERR=812) Coeff
      WRITE(21, 2126, ERR=823) Coeff, '(2nd empirical linear coefficient [slope])'
      READ (11, 1123, ERR=812) SoilCap
      WRITE(21, 2126, ERR=823) SoilCap, '(upper soil zone capacity, cm)'
      WRITE(21, 2127, ERR=823) 
      CLOSE(21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE(11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

      RETURN

!
!  Error Handling
! 
 811  ErrorMessage = 'BuildParmFile : Error opening file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 812  ErrorMessage = 'BuildParmFile : Error reading file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 813  ErrorMessage = 'BuildParmFile : Error writing file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 821  ErrorMessage = 'BuildParmFile : Error opening file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898
 822  ErrorMessage = 'BuildParmFile : Error reading file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898
 823  ErrorMessage = 'BuildParmFile : Error writing file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : BuildParmFile...'
      CALL PassMsg
      RETURN

!
!  FORMATs
!
 1121 FORMAT(A)
 1122 FORMAT(2I2, I4, 2I2, I4)
 1123 FORMAT(E13.6E2)

 2122 FORMAT(2I2.2, I4.4, 2I2.2, I4.4)
 2123 FORMAT(E13.6E2, ' 9 9 9.999999E-09 9.999999E+09  (parameter ', I2, ')')
 2124 FORMAT(E13.6E2, '                                (Cons, cal/cm2)')
 2125 FORMAT(E13.6E2, '                                (insolation - ',A3,')')
 2126 FORMAT(E13.6E2, '                                ', A)
 2127 FORMAT(' 1 9.999999E+09 9.999999E+09             (dummy calibration stuff)')

      END SUBROUTINE BuildParmFile


! ----------------------------------------------------------------------
! ----------------------------------------------------------------------

      SUBROUTINE LBRM2AHPS(ErrStatus, HandleIn, PrvFcst, Sub, Lake)
      IMPLICIT  NONE
      INTEGER, INTENT(INOUT)  ::  ErrStatus
      INTEGER, INTENT(IN)     ::  HandleIn, PrvFcst, Sub
      CHARACTER (LEN=3), INTENT(IN) :: Lake

      INTEGER  :: I, J, K, IOS, LakeNum, SSeq, VSeq, TSeq, RN
      INTEGER  :: Dy, Mn, Yr, Seq, RunStart, RunEnd 
      INTEGER (KIND=INTEGER2) :: I2, Scale, IDat(6)

      REAL, DIMENSION(6)             :: TDat
      REAL, DIMENSION(RD_MaxDays, 6) :: RunData

      CHARACTER (LEN=2)   :: SB
      CHARACTER (LEN=20)  :: S
      CHARACTER (LEN=99)  :: OutFile, Item
      CHARACTER (LEN=99)  :: File11, File21
      CHARACTER (LEN=200) :: Line, Value

      WinHandle = HandleIn

!
!  Convert sub from a number to a 2-char string
!
      WRITE(SB, '(I2.2)', IOSTAT=I) Sub
      IF (I .NE. 0) THEN
         ErrorMessage = 'LBRM2AHPS : Invalid subbasin specified.'
         CALL PassMsg
         GOTO 898
      END IF

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) THEN
            ThisLakeDir = TRIM(CfgDirs(I))
            LakeNum = I
         END IF
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Read the LBRM.CFG file to find out name of output file
!      
      OutFile = '-----'
      File11 = TRIM(ThisLakeDir) // 'LBRM.CFG'
      OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(11, 1101, IOSTAT=IOS) Line
      DO WHILE (IOS .EQ. 0) 
         CALL ParseCfgLine(Line, Item, Value); IF (ErrorLevel .NE. 0) GOTO 899      
         CALL CAPS(Item); IF (ErrorLevel .NE. 0) GOTO 899
         IF (TRIM(Item) .EQ. 'OUTFILE') OutFile = TRIM(Value)
         READ(11, 1101, IOSTAT=IOS) Line
      END DO
      CLOSE(11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899
      IF (OutFile(1:5) .EQ. '-----') THEN
         ErrorMessage = ' LBRM2AHPS : No entry in LBRM.CFG for output file!'
         CALL PassMsg
         GOTO 898
      END IF

!
!     Read the start/end dates
!
      CALL GetLBRMStartEndDates(PrvFcst, Lake, RunStart, RunEnd)
      IF (ErrorLevel .NE. 0) GOTO 899

      IF (RunEnd-RunStart+1  .GT.  RD_MaxDays) THEN
         ErrorMessage = 'LBRM2AHPS : Daily data array is too small.'
         CALL PassMsg
         GOTO 898
      END IF

!
!     Read the data generated by the LBRM
!
      File11 = OutFile
      OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(11, *, ERR=812)
      READ(11, *, ERR=812)
      READ(11, *, ERR=812)
      DO I = RunStart, RunEnd
         J = I - RunStart + 1
         READ(11, 1121, ERR=812) Dy, Mn, Yr, (TDat(K), K=1,6)
         CALL DateSequence(Dy, Mn, Yr, Seq); IF (ErrorLevel .NE. 0) GOTO 899
         IF (Seq .NE. I) THEN
            ErrorMessage = 'LBRM2AHPS : Error reading runoff data (bad date)'
            CALL PassMsg
            GOTO 898
         END IF
         RunData(J,:) = TDat(:)
      END DO
      CLOSE(11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!
!     If RunStart is earlier than the start of the existing *.SUM file, then
!     we need to re-create the SUM file with an earlier start date.
!
      File21 = TRIM(ThisLakeDir) // Lake // SB // '.SUM' 
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', RECL=12,          &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      READ(21, REC=1, ERR=822) I2, SSeq, Scale
      READ(21, REC=2, ERR=822) VSeq, TSeq
      CLOSE(21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899
      IF (RunStart .LT. SSeq) CALL ReBuildSumFile(File21, RunStart)

!
!     Write the data into the xxxnn.SUM file
!
      File21 = TRIM(ThisLakeDir) // Lake // SB // '.SUM' 
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', RECL=12,          &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      READ(21, REC=1, ERR=822) I2, SSeq, Scale
      READ(21, REC=2, ERR=822) VSeq, TSeq
      DO I = RunStart, RunEnd
         J  = I - RunStart + 1         ! index into array
         RN = I - SSeq + 1 + SumHdr    ! record number in file
         IDat(:) = NINT(RunData(J,:) * 100.0)
         IDat(4) = NINT(RunData(J,4) * 100.0 / Scale)
         WRITE(21, REC=RN, ERR=823) (IDat(K), K=1,6)
      END DO
      IF ((PrvFcst .EQ. 1) .AND. (RunEnd .GT. VSeq)) VSeq = RunEnd
      IF (RunEnd .GT. TSeq) TSeq = RunEnd
      WRITE(21, REC=2, ERR=823) VSeq, TSeq
      CLOSE(21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Error Handling
!
 811  ErrorMessage = 'LBRM2AHPS : Error opening file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 812  ErrorMessage = 'LBRM2AHPS : Error reading file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 813  ErrorMessage = 'LBRM2AHPS : Error writing file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 821  ErrorMessage = 'LBRM2AHPS : Error opening file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898
 822  ErrorMessage = 'LBRM2AHPS : Error reading file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898
 823  ErrorMessage = 'LBRM2AHPS : Error writing file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898

 
 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : LBRM2AHPS...'
      CALL PassMsg
      GOTO 999

!
!  Final cleanup
!
 999  CALL CloseAllFiles()
      ErrStatus = ErrorLevel

      RETURN

!
!  FORMATs
!
 1101 FORMAT(A)
 1111 FORMAT(2I3, I5)
 1121 FORMAT(2I2, I4, 6E11.4E2)
 2120 FORMAT(2I3, I5, A1)

      END SUBROUTINE LBRM2AHPS

! ----------------------------------------------------------------------
      SUBROUTINE LBRM2AHPSWriteOnly(ErrStatus, HandleIn, PrvFcst, Sub,    &
     &                         DS, Run, Uszm, Lszm, Gzm, Ss, Snow, Lake)
      IMPLICIT  NONE
      INTEGER, INTENT(INOUT)  ::  ErrStatus
      INTEGER, INTENT(IN)     ::  HandleIn, PrvFcst, Sub
      INTEGER, INTENT(INOUT)  ::  DS
      REAL, DIMENSION(DS), INTENT(INOUT) :: Run, Uszm, Lszm, Gzm, Ss, Snow
      CHARACTER (LEN=3), INTENT(IN) :: Lake

      INTEGER (KIND=INTEGER2) :: I2, Scale, IDat(6)
      INTEGER  :: I, J, K, IOS, LakeNum, SSeq, VSeq, TSeq, RN
      INTEGER  :: Dy, Mn, Yr, Seq, RunStart, RunEnd 
      REAL     :: Area

      CHARACTER (LEN=2)   :: SB
      CHARACTER (LEN=20)  :: S
      CHARACTER (LEN=99)  :: OutFile, Item
      CHARACTER (LEN=99)  :: File11, File21
      CHARACTER (LEN=200) :: Line, Value

      WinHandle = HandleIn
!
!  Convert sub from a number to a 2-char string
!
      WRITE(SB, '(I2.2)', IOSTAT=I) Sub
      IF (I .NE. 0) THEN
         ErrorMessage = 'LBRM2AHPSWriteOnly : Invalid subbasin specified.'
         CALL PassMsg
         GOTO 898
      END IF

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) THEN
            ThisLakeDir = TRIM(CfgDirs(I))
            LakeNum = I
         END IF
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Read the start/end dates
!
      CALL GetLBRMStartEndDates(PrvFcst, Lake, RunStart, RunEnd)
      IF (ErrorLevel .NE. 0) GOTO 899

      IF (RunEnd-RunStart+1  .GT.  RD_MaxDays) THEN
         ErrorMessage = 'LBRM2AHPSWriteOnly : Daily data array is too small.'
         CALL PassMsg
         GOTO 898
      END IF

!
!     Get subbasin area for converting data values from cubic meters to hundredths of cm
!
      File11  = TRIM(ThisLakeDir) // 'AREA.' // TRIM(Lake)
      OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      DO I = 1, Sub
         READ(11, 1123, ERR=812) Area
      END DO
      CLOSE(11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!
!     If RunStart is earlier than the start of the existing *.SUM file, then
!     we need to re-create the SUM file with an earlier start date.
!
      File21 = TRIM(ThisLakeDir) // Lake // SB // '.SUM' 
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', RECL=12,          &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      READ(21, REC=1, ERR=822) I2, SSeq, Scale
      READ(21, REC=2, ERR=822) VSeq, TSeq
      CLOSE(21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899
      IF (RunStart .LT. SSeq) CALL ReBuildSumFile(File21, RunStart)

!
!     Write the data into the xxxnn.SUM file
!     Convert values from cubic meters to hundredths of cm.
!
      File21 = TRIM(ThisLakeDir) // Lake // SB // '.SUM' 
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', RECL=12,          &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      READ(21, REC=1, ERR=822) I2, SSeq, Scale
      READ(21, REC=2, ERR=822) VSeq, TSeq
      DO I = RunStart, RunEnd
         J  = I - RunStart + 1         ! index into array
         RN = I - SSeq + 1 + SumHdr    ! record number in file
         IDat(1) = NINT(Run (J) / Area * 10000.0)
         IDat(2) = NINT(Uszm(J) / Area * 10000.0)
         IDat(3) = NINT(Lszm(J) / Area * 10000.0)
         IDat(4) = NINT(Gzm (J) / Area * 10000.0 / Scale)
         IDat(5) = NINT(Ss  (J) / Area * 10000.0)
         IDat(6) = NINT(Snow(J) / Area * 10000.0)
         WRITE(21, REC=RN, ERR=823) (IDat(K), K=1,6)
      END DO
      IF ((PrvFcst .EQ. 1) .AND. (RunEnd .GT. VSeq)) VSeq = RunEnd
      IF (RunEnd .GT. TSeq) TSeq = RunEnd
      WRITE(21, REC=2, ERR=823) VSeq, TSeq
      CLOSE(21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Error Handling
!
 811  ErrorMessage = 'LBRM2AHPSWriteOnly : Error opening file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 812  ErrorMessage = 'LBRM2AHPSWriteOnly : Error reading file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 813  ErrorMessage = 'LBRM2AHPSWriteOnly : Error writing file ' // TRIM(File11)
      CALL PassMsg;  GOTO 898
 821  ErrorMessage = 'LBRM2AHPSWriteOnly : Error opening file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898
 822  ErrorMessage = 'LBRM2AHPSWriteOnly : Error reading file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898
 823  ErrorMessage = 'LBRM2AHPSWriteOnly : Error writing file ' // TRIM(File21)
      CALL PassMsg;  GOTO 898

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : LBRM2AHPSWriteOnly...'
      CALL PassMsg
      GOTO 999

!
!  Final cleanup
!
 999  CALL CloseAllFiles()
      ErrStatus = ErrorLevel

      RETURN

!
!  FORMATs
!
 1101 FORMAT(A)
 1111 FORMAT(2I3, I5)
 1121 FORMAT(2I2, I4, 6E11.4E2)
 1123 FORMAT(E13.6E3)
 2120 FORMAT(2I3, I5, A1)

      END SUBROUTINE LBRM2AHPSWriteOnly


! ----------------------------------------------------------------------
      SUBROUTINE RebuildSumFile(FileName, NewStart)
      IMPLICIT   NONE
      INTEGER, INTENT(IN) :: NewStart
      CHARACTER (LEN=*), INTENT(IN) :: FileName

      INTEGER (KIND=INTEGER2) :: I2, Miss, TDat(6)
      INTEGER :: U1, U2, SSeq, Scale, VSeq, TSeq, I, J, R1, R2
      LOGICAL :: ErrStat      

      Miss = -9999

      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) RETURN
      OPEN(UNIT=U1, FILE=TRIM(FileName), STATUS='OLD', RECL=12,           &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U1, REC=1, ERR=812) I2, SSeq, Scale
      READ(U1, REC=2, ERR=812) VSeq, TSeq

      U2 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) RETURN
      OPEN(UNIT=U2, FILE='TEMP_SUM.INT', STATUS='REPLACE', RECL=12,       &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=821)
      CALL FileWasOpened(U2); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(U2, REC=1, ERR=823) I2, NewStart, Scale
      WRITE(U2, REC=2, ERR=823) VSeq, TSeq

      DO I = NewStart, SSeq-1
        R2 = I - NewStart + 3
        WRITE(U2, REC=R2, ERR=823) Miss, Miss, Miss, Miss, Miss, Miss
      END DO

      DO I = SSeq, TSeq
        R1 = I - SSeq + 3
        R2 = I - NewStart + 3
        READ (U1, REC=R1, ERR=812) (TDat(J), J=1,6)
        WRITE(U2, REC=R2, ERR=823) (TDat(J), J=1,6)
      END DO


      CLOSE(U2)
      CALL FileWasClosed(U2); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE(U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      CALL RenameFile('TEMP_SUM.INT', TRIM(FileName), ErrStat)
      RETURN

 811  ErrorMessage = 'RebuildSumFile : Error opening file ' // TRIM(FileName)
      CALL PassMsg;  GOTO 898
 812  ErrorMessage = 'RebuildSumFile : Error reading file ' // TRIM(FileName)
      CALL PassMsg;  GOTO 898
 821  ErrorMessage = 'RebuildSumFile : Error opening file TEMP_SUM.INT'
      CALL PassMsg;  GOTO 898
 823  ErrorMessage = 'RebuildSumFile : Error writing file TEMP_SUM.INT'
      CALL PassMsg;  GOTO 898

 898  ErrorLevel = 1
 899  ErrorMessage = '[traceback] : RebuildSumFile...'
      CALL PassMsg
      CALL CloseAllFiles()
      RETURN

      END SUBROUTINE RebuildSumFile

END MODULE LBRM_FILES




