!
!     Written by TSH 17sep2007
!
!     This routine extracts the subbasin outflow timeseries data from
!     each subbasin, writing it to ASCII output files.
!     The files will/may be named one of the following:
!         DLYFLOW_HIST.TXT   (daily flows for the entire historical data period - (Pd<>5, Pd<>8))
!         DLYFLOW_PRV.TXT    (daily flows for the provisional data period)
!         DLYFLOW_yyyy.TXT   (daily flows for the outlook data period using met from year yyyy)
!         MONFLOW_HIST.TXT   (monthly flows for the entire historical data period - (Pd<>5, Pd<>8))
!         MONFLOW_PRV.TXT    (monthly flows for the provisional data period)
!         MONFLOW_yyyy.TXT   (monthly flows for the outlook data period using met from year yyyy)
!
!     If we are extracting the data for the outlook period, then we will read the
!     DATENAME.LIS file to get the year corresponding to the input met data. That
!     year will be used in the output file name.
!
!     We can use this program in one of 3 ways:
!       (1) If PD=5 then LBRM was just run to update the *.SUM files for 
!           the provisional met data period.  
!
!           Daily: We will output the daily data for the entire period from 
!                  the base date to the provisional data end date.
!
!           Monthly: We will output the monthly data for all of the complete
!                    months in the same period (provisional data period). 
!                    Note that if the provisional data end date is not the
!                    last day of the month, then we will truncate the output 
!                    at the end of the previous month.
!
!       (2) If PD=8 then LBRM was just run the 2nd time.  That is, it was
!           run to update the *.SUM files for the outlook/forecast
!           period.  
!
!           Daily: We will output the daily data for the forecast period from 
!                  the forecast start date to forecast end date.
!
!           Monthly: We will output the monthly values for the forecast period.
!                    Note that if the forecast starts on anything but the first 
!                    day of the month, then that first month's value will need 
!                    to use daily values from the historical data set.
!
!       (3) If PD is anything else, we will assume that the LBRM was just
!           run as part of a "setup" or simulation procedure to update the
!           *.SUM files for a historical/simulation period.
!
!           Daily: We will get the start/end dates from ???01.SUM and use those
!                  dates (start to the "valid end" date) as the period of record.
!                  We will assume that they are the same for all files.
!
!           Monthly: We will use those same dates, but any incomplete months at the
!                    start or end will be ignored/truncated.
!
!
MODULE ExtractSubFlwMain

CONTAINS

      SUBROUTINE DO_GetSubFlw(ErrStatus, HandleIn, Lake)

      USE FcstUtil

      IMPLICIT NONE

      INTEGER, INTENT(INOUT) :: ErrStatus
      INTEGER, INTENT(IN)    :: HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      !
      !   iNSbsns = Maximum number of subbasins.
      !   Hdr     = number of header records in *.SUM
      !
      INTEGER (KIND=INTEGER2), PARAMETER :: iNSbsns = 35
      INTEGER (KIND=INTEGER2), PARAMETER :: Hdr = 2

      INTEGER (KIND=INTEGER2)  :: Skip2, Flw

      INTEGER :: I, J, IOS, Pd
      INTEGER :: Day, Mon, Year, Sub, RNum, Seq, Ndx
      INTEGER :: BaseDay, BaseMon, BaseYear, PEDay, PEMon, PEYear
      INTEGER :: FSDay, FSMon, FSYear, FEDay, FEMon, FEYear
      INTEGER :: BaseDateSeq, ProvEndSeq, FcstStartSeq, FcstEndSeq
      INTEGER :: Start, Last, DlyFirst, DlyLast
      INTEGER :: NumHistDays, NumOtlkDays, NumHistMons, NumOtlkMons
      INTEGER :: DlyHistFirst, DlyHistLast, DlyOtlkFirst, DlyOtlkLast
      INTEGER :: MonHistFirst, MonHistLast, MonOtlkFirst, MonOtlkLast
      INTEGER :: NoSubbasins, Length
      INTEGER :: FileStart, FileProvEnd, FileOtlkEnd
      INTEGER :: FStart, PEnd, OEnd
      INTEGER :: DlyOutFirst, DlyOutLast, MonOutFirst, MonOutLast
      INTEGER :: YearMonHistFirst, YearMonHistLast
      INTEGER :: YearMonOtlkFirst, YearMonOtlkLast
      INTEGER :: YMH1, YMH2, YMO1, YMO2
      INTEGER, DIMENSION(30) :: MonCount

      REAL :: FlwValInCM
      REAL, DIMENSION(30) :: FlowVals, MonTotal
      REAL, DIMENSION(:,:), ALLOCATABLE :: DlyHistFlows
      REAL, DIMENSION(:,:), ALLOCATABLE :: DlyOtlkFlows
      REAL, DIMENSION(:,:), ALLOCATABLE :: MonHistFlows
      REAL, DIMENSION(:,:), ALLOCATABLE :: MonOtlkFlows

      CHARACTER (LEN=1)  :: Lf, Cr
      CHARACTER (LEN=2)  :: CrLf, SB
      CHARACTER (LEN=3)  :: BnS, Ext
      CHARACTER (LEN=4)  :: OverTheLake, OtlkYear
      CHARACTER (LEN=13) :: Name, S
      CHARACTER (LEN=99) :: File31, File41, File51, File61, FName, Line

      Lf = CHAR(10)
      Cr = CHAR(13)
      CrLf = Cr//Lf

      WinHandle = HandleIn
      ErrorLevel = 0
      ErrorMessage = ' '
      CALL ClearFilesOpen()

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Read BSSNM file to get basin name and Program Designator for 
!     the program run most recently.  (Note that this program does not
!     modify the PD value.)  Also get some dates for future use.
!
      File31 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=31, FILE=TRIM(File31), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM = 'FORMATTED', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899
      READ(31, 3103, REC=1, ERR=832) Name
      READ(31, 3105, REC=2, ERR=832) NoSubbasins, Pd, OverTheLake
      READ(31, 3104, REC=3, ERR=832) BaseDay, BaseMon, BaseYear     ! Base date 
      READ(31, 3104, REC=4, ERR=832) PEDay, PEMon, PEYear           ! Provisional data End date 
      READ(31, 3104, REC=5, ERR=832) FSDay, FSMon, FSYear           ! Forecast Start date 
      READ(31, 3104, REC=6, ERR=832) FEDay, FEMon, FEYear           ! Forecast End date 
      CLOSE (31)
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899
      BnS = Name(1:3)

      CALL DateSequence(BaseDay, BaseMon, BaseYear, BaseDateSeq);  IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(PEDay,   PEMon,   PEYear,   ProvEndSeq);   IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(FSDay,   FSMon,   FSYear,   FcstStartSeq); IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(FEDay,   FEMon,   FEYear,   FcstEndSeq);   IF (ErrorLevel .NE. 0) GOTO 899

!
!     Before assigning dates, get the period of record that actually 
!     exists in the ???nn.SUM files.
!     
      File51 = TRIM(ThisLakeDir) // BnS // '01.SUM'
      OPEN(UNIT=51, FILE=TRIM(File51), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
      CALL FileWasOpened(51); IF (ErrorLevel .NE. 0) GOTO 899
      READ(51, REC=1, ERR=852) Skip2, FileStart
      READ(51, REC=2, ERR=852) FileProvEnd, FileOtlkEnd
      CLOSE (51)
      CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899

      DO Sub = 2, NoSubbasins
         WRITE(SB, '(I2.2)') Sub      ! 01..nn as a 2-digit char string
         File51 = TRIM(ThisLakeDir) // BnS // SB // '.SUM'
         OPEN(UNIT=51, FILE=TRIM(File51), STATUS='OLD', RECL=12,             &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
         CALL FileWasOpened(51); IF (ErrorLevel .NE. 0) GOTO 899
         READ(51, REC=1, ERR=852) Skip2, FStart
         READ(51, REC=2, ERR=852) PEnd, OEnd
         CLOSE (51)
         CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899
         FileStart   = MAX(FileStart, FStart)
         FileProvEnd = MIN(FileProvEnd, PEnd)
         FileOtlkEnd = MIN(FileOtlkEnd, OEnd)
      END DO

!
!     Assign start/end dates for the daily data we will read into the arrays
!
!     1) The daily historical flows go from the base date to the end of the 
!        provisional data period.... or the end of the "valid" data in the 
!        SUM files - whichever comes first.
!       
      IF ((PD .EQ. 5) .OR. (PD .EQ. 8)) THEN
         DlyHistFirst = MAX(BaseDateSeq, FileStart)
         DlyHistLast  = MIN(ProvEndSeq, FileProvEnd)
      ELSE
         DlyHistFirst = FileStart
         DlyHistLast  = FileProvEnd
      END IF

!
!     2) The daily forecast flows go from the forecast start date to the end of the 
!        forecast period.
!       
      DlyOtlkFirst = FcstStartSeq
      DlyOtlkLast  = FcstEndSeq

!
!     Now assign desired start/end dates for daily output
!      
      IF (Pd .EQ. 5) THEN 
         !
         !  Daily date limits
         !
         DlyOutFirst  = MAX(BaseDateSeq, FileStart)
         DlyOutLast   = MIN(ProvEndSeq,  FileProvEnd)

         !
         !  Monthly date limits (truncated to *complete* months in provisional data period)
         !
         MonOutFirst = DlyOutFirst
         CALL SequenceDate(Day, Mon, Year, MonOutFirst); IF (ErrorLevel .NE. 0) GOTO 899
         IF (Day .NE. 1) THEN
            CALL AdjustToEndOfMonth(MonOutFirst)
            MonOutFirst = MonOutFirst + 1
         END IF

         MonOutLast = DlyOutLast
         CALL SequenceDate(Day, Mon, Year, MonOutLast); IF (ErrorLevel .NE. 0) GOTO 899
         IF (.NOT. IsLastDayOfMonth(Day, Mon, Year)) THEN
            CALL AdjustToStartOfMonth(MonOutLast)
            MonOutLast = MonOutLast - 1
         END IF
      END IF

      !
      !  Doing forecast period
      !
      IF (Pd .EQ. 8) THEN 
         !
         !  Daily date limits
         !
         DlyOutFirst = MAX(DlyOtlkFirst, FileStart)
         DlyOutLast  = MIN(DlyOtlkLast,  FileOtlkEnd)

         !
         !  Monthly date limits (start adjusted to first of month, end truncated)
         !
         MonOutFirst = DlyOutFirst
         CALL SequenceDate(Day, Mon, Year, MonOutFirst); IF (ErrorLevel .NE. 0) GOTO 899
         IF (Day .NE. 1) THEN
            Seq = MonOutFirst
            CALL AdjustToStartOfMonth(Seq)
            IF (Seq .GE. FileStart) THEN
               MonOutFirst = Seq
            ELSE
               CALL AdjustToEndOfMonth(MonOutFirst)
               MonOutFirst = MonOutFirst + 1
            END IF
         END IF

         MonOutLast = DlyOutLast
         CALL SequenceDate(Day, Mon, Year, MonOutLast); IF (ErrorLevel .NE. 0) GOTO 899
         IF (.NOT. IsLastDayOfMonth(Day, Mon, Year)) THEN
            CALL AdjustToStartOfMonth(MonOutLast)
            MonOutLast = MonOutLast - 1
         END IF
      END IF

      !
      !  Doing entire historical period
      !
      IF ((PD .NE. 5) .AND. (PD .NE. 8)) THEN          
         !
         !  Daily date limits
         !
         DlyOutFirst = FileStart
         DlyOutLast  = FileProvEnd

         !
         !  Monthly date limits (start/end truncated as necessary)
         !
         MonOutFirst = DlyOutFirst
         CALL SequenceDate(Day, Mon, Year, MonOutFirst); IF (ErrorLevel .NE. 0) GOTO 899
         IF (Day .NE. 1) THEN
            CALL AdjustToEndOfMonth(MonOutFirst)
            MonOutFirst = MonOutFirst + 1         ! first day of first full month
         END IF

         MonOutLast = DlyOutLast
         CALL SequenceDate(Day, Mon, Year, MonOutLast); IF (ErrorLevel .NE. 0) GOTO 899
         IF (.NOT. IsLastDayOfMonth(Day, Mon, Year)) THEN
            CALL AdjustToStartOfMonth(MonOutLast)
            MonOutLast = MonOutLast - 1           ! last day of final full month
         END IF
      END IF

!
!     Assign month designators for the historical and outlook periods based on
!     appropriate need for full months.
!
      Seq = DlyHistFirst
      CALL SequenceDate(Day, Mon, Year, Seq); IF (ErrorLevel .NE. 0) GOTO 899
      IF (Day .NE. 1) THEN
         CALL AdjustToEndOfMonth(Seq)
         Seq = Seq + 1              ! first day of first full month
      END IF
      CALL SequenceDate(Day, Mon, Year, Seq); IF (ErrorLevel .NE. 0) GOTO 899
      YearMonHistFirst = Year*100 + Mon           !  number like 200106

      Seq = DlyHistLast
      CALL SequenceDate(Day, Mon, Year, Seq); IF (ErrorLevel .NE. 0) GOTO 899
      IF (.NOT. IsLastDayOfMonth(Day, Mon, Year)) THEN
         CALL AdjustToStartOfMonth(Seq)
         Seq = Seq - 1              ! last day of last full month
      END IF
      CALL SequenceDate(Day, Mon, Year, Seq); IF (ErrorLevel .NE. 0) GOTO 899
      YearMonHistLast = Year*100 + Mon             !  number like 200211

      !
      !  Remember that the first outlook month may often include a few days of
      !  historical data.  That's why we treat this one differently.
      !
      Seq = DlyOtlkFirst
      CALL SequenceDate(Day, Mon, Year, Seq); IF (ErrorLevel .NE. 0) GOTO 899
      IF (Day .NE. 1) THEN
         CALL AdjustToStartOfMonth(Seq)
      END IF
      CALL SequenceDate(Day, Mon, Year, Seq); IF (ErrorLevel .NE. 0) GOTO 899
      YearMonOtlkFirst = Year*100 + Mon            !  number like 199701

      Seq = DlyOtlkLast
      CALL SequenceDate(Day, Mon, Year, Seq); IF (ErrorLevel .NE. 0) GOTO 899
      IF (.NOT. IsLastDayOfMonth(Day, Mon, Year)) THEN
         CALL AdjustToStartOfMonth(Seq)
         Seq = Seq - 1              ! last day of last full month
      END IF
      CALL SequenceDate(Day, Mon, Year, Seq); IF (ErrorLevel .NE. 0) GOTO 899
      YearMonOtlkLast = Year*100 + Mon             !  number like 200211
      
!
!     Allocate the arrays for holding the daily flow values
!
      NumHistDays = DlyHistLast - DlyHistFirst + 1
      ALLOCATE(DlyHistFlows(NoSubbasins, NumHistDays), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'ExtractSubFlw: Error allocating RAM.'
         CALL PassMsg; GOTO 898
      END IF
      DlyHistFlows(:,:) = -9.99

      NumOtlkDays = DlyOtlkLast - DlyOtlkFirst + 1
      ALLOCATE(DlyOtlkFlows(NoSubbasins, NumOtlkDays), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'ExtractSubFlw: Error allocating RAM.'
         CALL PassMsg; GOTO 898
      END IF
      DlyOtlkFlows(:,:) = -9.99

!
!     Allocate the array for holding the monthly flow values
!
      Year = YearMonHistFirst / 100          ! integer division - truncates month off
      Mon  = YearMonHistFirst - (Year*100)
      YMH1 = Year*12 + Mon
      Year = YearMonHistLast / 100           ! integer division - truncates month off
      Mon  = YearMonHistLast - (Year*100)
      YMH2 = Year*12 + Mon

      NumHistMons = YMH2 - YMH1 + 1
      ALLOCATE(MonHistFlows(NoSubbasins, NumHistMons), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'ExtractSubFlw: Error allocating RAM.'
         CALL PassMsg; GOTO 898
      END IF
      MonHistFlows(:,:) = -9.99

      Year = YearMonOtlkFirst / 100          ! integer division - truncates month off
      Mon  = YearMonOtlkFirst - (Year*100)
      YMO1 = Year*12 + Mon
      Year = YearMonOtlkLast / 100           ! integer division - truncates month off
      Mon  = YearMonOtlkLast - (Year*100)
      YMO2 = Year*12 + Mon

      NumOtlkMons = YMO2 - YMO1 + 1
      ALLOCATE(MonOtlkFlows(NoSubbasins, NumOtlkMons), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'ExtractSubFlw: Error allocating RAM.'
         CALL PassMsg; GOTO 898
      END IF
      MonOtlkFlows(:,:) = -9.99

!
!     For each subbasin....
!
      DO Sub = 1, NoSubbasins
         WRITE(SB, '(I2.2)') Sub      ! 01..nn as a 2-digit char string

!
!        Status message
!
         WRITE(ErrorMessage, 9001) BnS, SB 
         CALL PassMsg

!
!        Open subbasin model storage file (xxx??.SUM) and read header
!
         File51 = TRIM(ThisLakeDir) // BnS // SB // '.SUM'
         OPEN(UNIT=51, FILE=TRIM(File51), STATUS='OLD', RECL=12,             &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
         CALL FileWasOpened(51); IF (ErrorLevel .NE. 0) GOTO 899
         READ(51, REC=1, ERR=852) Skip2, FStart
         READ(51, REC=2, ERR=852) PEnd, OEnd

!
!        Read the daily data and store in the appropriate array
!
         DO Seq = FStart, PEnd
            RNum = (Seq - FStart) + Hdr + 1
            Ndx  = Seq-DlyHistFirst+1
            IF ((Ndx .GE. 1) .AND. (Ndx .LE. NumHistDays)) THEN
               READ(51, REC=RNum, ERR=852) Flw        ! flow in 100ths of cm
               FlwValInCM = (Flw * 1.0) / 100.0
               DlyHistFlows(Sub, Ndx) = FlwValInCM
            END IF
         END DO

         DO Seq = PEnd+1, OEnd
            RNum = (Seq - FStart) + Hdr + 1
            Ndx  = Seq-DlyOtlkFirst+1
            IF ((Ndx .GE. 1) .AND. (Ndx .LE. NumOtlkDays)) THEN
               READ(51, REC=RNum, ERR=852) Flw        ! flow in 100ths of cm
               FlwValInCM = (Flw * 1.0) / 100.0
               DlyOtlkFlows(Sub, Ndx) = FlwValInCM
            END IF
         END DO

         CLOSE (51)
         CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

!
!     Determine the output file name for daily data.
!
      IF ((Pd .NE. 5) .AND. (Pd .NE. 8)) WRITE(File61, 8103) TRIM(ThisLakeDir)
      IF (Pd .EQ. 5) WRITE(File61, 8101) TRIM(ThisLakeDir)
      IF (Pd .EQ. 8) THEN
         File41 = TRIM(ThisLakeDir) // 'DATENAME.LIS'
         OPEN(UNIT=41, FILE=TRIM(File41), STATUS='OLD', ERR=841)
         CALL FileWasOpened(41); IF (ErrorLevel .NE. 0) GOTO 899
         READ(41, 4100, ERR=842) Line
         READ(Line, 4101, IOSTAT=IOS) OtlkYear
         CLOSE (41)
         CALL FileWasClosed(41); IF (ErrorLevel .NE. 0) GOTO 899
         IF (IOS .NE. 0) THEN
            ErrorMessage = 'ExtractSubFlw: Bad year value in ' //TRIM(File41)
            CALL PassMsg
            ErrorMessage = 'Entire line is [' // TRIM(Line) // ']' 
            CALL PassMsg
            GOTO 898
         END IF
         WRITE(File61, 8102) TRIM(ThisLakeDir), OtlkYear
      END IF

!
!     Open/write/close the output file (daily data).
!
      OPEN(UNIT=61, FILE=TRIM(File61), STATUS='REPLACE', ERR=861)
      CALL FileWasOpened(61); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(61, 6101, ERR=863) BnS
      CALL SequenceDate(Day, Mon, Year, DlyOutFirst); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(61, 6102, ERR=863) Year, Mon, Day
      CALL SequenceDate(Day, Mon, Year, DlyOutLast); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(61, 6103, ERR=863) Year, Mon, Day
      WRITE(61, 6104, ERR=863)
      WRITE(61, 6105, ERR=863) (I, I=1, NoSubbasins)

      DO Seq = DlyOutFirst, DlyOutLast
         FlowVals(:) = -9.99
         Ndx = Seq - DlyHistFirst + 1
         IF ((Ndx .GE. 1) .AND. (Ndx .LE. NumHistDays)) THEN
            DO Sub = 1, NoSubbasins
               FlowVals(Sub) = DlyHistFlows(Sub,Ndx)
            END DO
         END IF
         Ndx = Seq - DlyOtlkFirst + 1
         IF ((Ndx .GE. 1) .AND. (Ndx .LE. NumOtlkDays)) THEN
            DO Sub = 1, NoSubbasins
               FlowVals(Sub) = DlyOtlkFlows(Sub,Ndx)
            END DO
         END IF
         CALL SequenceDate(Day, Mon, Year, Seq); IF (ErrorLevel .NE. 0) GOTO 899
         WRITE(61, 6110, ERR=863) Year, Mon, Day, (FlowVals(Sub), Sub=1,NoSubbasins)
      END DO
      CLOSE (61)
      CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Determine the output file name for monthly data.
!     Note that OtlkYear, if needed, was already filled when 
!     doing the daily file name.
!
      IF ((Pd .NE. 5) .AND. (Pd .NE. 8)) WRITE(File61, 8203) TRIM(ThisLakeDir)
      IF (Pd .EQ. 5) WRITE(File61, 8201) TRIM(ThisLakeDir)
      IF (Pd .EQ. 8) WRITE(File61, 8202) TRIM(ThisLakeDir), OtlkYear
      

!
!     Open/write/close the output file (monthly data).
!
      OPEN(UNIT=61, FILE=TRIM(File61), STATUS='REPLACE', ERR=861)
      CALL FileWasOpened(61); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(61, 6201, ERR=863) BnS
      CALL SequenceDate(Day, Mon, Year, MonOutFirst); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(61, 6202, ERR=863) Year, Mon
      CALL SequenceDate(Day, Mon, Year, MonOutLast); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(61, 6203, ERR=863) Year, Mon
      WRITE(61, 6204, ERR=863)
      WRITE(61, 6205, ERR=863) (I, I=1, NoSubbasins)

      MonTotal(:) = 0.0
      MonCount(:) = 0
      DO Seq = MonOutFirst, MonOutLast
         CALL SequenceDate(Day, Mon, Year, Seq); IF (ErrorLevel .NE. 0) GOTO 899
         Ndx = Seq - DlyHistFirst + 1
         IF ((Ndx .GE. 1) .AND. (Ndx .LE. NumHistDays)) THEN
            DO Sub = 1, NoSubbasins
               FlowVals(Sub) = DlyHistFlows(Sub,Ndx)
            END DO
         END IF
         Ndx = Seq - DlyOtlkFirst + 1
         IF ((Ndx .GE. 1) .AND. (Ndx .LE. NumOtlkDays)) THEN
            DO Sub = 1, NoSubbasins
               FlowVals(Sub) = DlyOtlkFlows(Sub,Ndx)
            END DO
         END IF
         DO Sub = 1, NoSubbasins
            IF (FlowVals(Sub) .GT. -0.01) THEN
               MonTotal(Sub) = MonTotal(Sub) + FlowVals(Sub)
               MonCount(Sub) = MonCount(Sub) + 1
            END IF
         END DO
         IF (IsLastDayOfMonth(Day, Mon, Year)) THEN
            FlowVals(:) = -9.99
            DO Sub = 1, NoSubbasins
               IF (MonCount(Sub) .EQ. DaysInMonth(Mon, Year)) FlowVals(Sub) = MonTotal(Sub)
            END DO
            WRITE(61, 6210, ERR=863) Year, Mon, (FlowVals(Sub), Sub=1,NoSubbasins)
            MonTotal(:) = 0.0
            MonCount(:) = 0
         END IF
      END DO
      CLOSE (61)
      CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Release the allocated RAM
!
      DEALLOCATE(DlyHistFlows, STAT=IOS)
      DEALLOCATE(DlyOtlkFlows, STAT=IOS)
      DEALLOCATE(MonHistFlows, STAT=IOS)
      DEALLOCATE(MonOtlkFlows, STAT=IOS)
      GOTO 999

!
!  Handlers for I/O errors
!
  831 ErrorMessage = 'ExtractSubFlw: Error opening file '   //TRIM(File31)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'ExtractSubFlw: Error reading file '   //TRIM(File31)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'ExtractSubFlw: Error writing to file '//TRIM(File31)
      CALL PassMsg; GOTO 898
  841 ErrorMessage = 'ExtractSubFlw: Error opening file '   //TRIM(File41)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'ExtractSubFlw: Error reading file '   //TRIM(File41)
      CALL PassMsg; GOTO 898
  851 ErrorMessage = 'ExtractSubFlw: Error opening file '   //TRIM(File51)
      CALL PassMsg; GOTO 898
  852 ErrorMessage = 'ExtractSubFlw: Error reading file '   //TRIM(File51)
      CALL PassMsg; GOTO 898
  861 ErrorMessage = 'ExtractSubFlw: Error opening file '   //TRIM(File61)
      CALL PassMsg; GOTO 898
  862 ErrorMessage = 'ExtractSubFlw: Error reading file '   //TRIM(File61)
      CALL PassMsg; GOTO 898
  863 ErrorMessage = 'ExtractSubFlw: Error writing to file '//TRIM(File61)
      CALL PassMsg; GOTO 898

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_ExtractSubFlw...'
      CALL PassMsg
      GOTO 999


!
!  Final cleanup and return
!
  999 CALL CloseAllFiles()
      IF (ALLOCATED(DlyHistFlows)) DEALLOCATE(DlyHistFlows, STAT=IOS)
      IF (ALLOCATED(DlyOtlkFlows)) DEALLOCATE(DlyOtlkFlows, STAT=IOS)
      IF (ALLOCATED(MonHistFlows)) DEALLOCATE(MonHistFlows, STAT=IOS)
      IF (ALLOCATED(MonOtlkFlows)) DEALLOCATE(MonOtlkFlows, STAT=IOS)
      ErrStatus = ErrorLevel
      RETURN
!
!  FORMAT statements
!
 2102 FORMAT(2I2, I4)

 3103 FORMAT(A10)
 3104 FORMAT(2I3, I5, A1)
 3105 FORMAT(2I3, A4, 1X, A1)
 4100 FORMAT(A)
 4101 FORMAT(17X, A4)

 6101 FORMAT('Daily subbasin outflow (expressed as a depth over each subbasin in centimeters) ',   &
     &       'from ', A3, '??.SUM')
 6102 FORMAT('FROM ', I4, 2I3, 5X, 'YR MO DY')
 6103 FORMAT('TO   ', I4, 2I3, 5X, 'YR MO DY')
 6104 FORMAT(14X, 'Subbasins...')
 6105 FORMAT('YEAR MO DY', 30I6)
 6110 FORMAT(I4, 2I3, 30F6.2)

 6201 FORMAT('Monthly total subbasin outflow (expressed as a depth over each subbasin in centimeters) ',   &
     &       'from ', A3, '??.SUM')
 6202 FORMAT('FROM ', I4, I3, 5X, 'YR MO')
 6203 FORMAT('TO   ', I4, I3, 5X, 'YR MO')
 6204 FORMAT(14X, 'Subbasins...')
 6205 FORMAT('YEAR MO', 30I6)
 6210 FORMAT(I4, I3, 30F6.2)

 8101 FORMAT(A, 'DLYFLOW_PRV.TXT') 
 8102 FORMAT(A, 'DLYFLOW_', A4, '.TXT') 
 8103 FORMAT(A, 'DLYFLOW_HIST.TXT')
 8201 FORMAT(A, 'MONFLOW_PRV.TXT') 
 8202 FORMAT(A, 'MONFLOW_', A4, '.TXT') 
 8203 FORMAT(A, 'MONFLOW_HIST.TXT') 

 9001 FORMAT('~^~Extracting outflow for subbasin ', A3, A2) 

      END SUBROUTINE DO_GetSubFlw

END MODULE ExtractSubFlwMain


