!------------------------------------------------------------------
!
!     Constructs a file that reflects the state of the AHPS files
!     in the current directory.  Used to determine whether we need
!     to re-run the outlooks when the AHPS is invoked.
!
!     Assumes that the file DIRLIST.INT has already been created
!     (either by an O/S operation or by another program).  This is
!     a file listing all of the files in the directory along with
!     their size and date (and maybe other info).
!
!------------------------------------------------------------------

MODULE MakeStateSubs

      USE FcstUtil

      IMPLICIT   NONE
      INTEGER, PARAMETER :: MaxNoOfLines = 1000
      INTEGER            :: NumSortLines
      CHARACTER (LEN=80) :: SortLines(MaxNoOfLines)

CONTAINS

!------------------------------------------------------------------
!  Strip off the file path and separate filename and extension
!
      SUBROUTINE ExtractFileName (Line, FName, Ext)
      IMPLICIT   NONE
      CHARACTER (LEN=*),  INTENT(IN)  :: Line
      CHARACTER (LEN=3) , INTENT(OUT) :: Ext
      CHARACTER (LEN=15), INTENT(OUT) :: FName

      INTEGER ::   I, J
      CHARACTER (LEN=120) :: S

!
      I = INDEX(Line, '\', .TRUE.)    ! find LAST instance of '\'
      S = Line
      IF (I .GT. 0) THEN
         J = LEN_TRIM(Line)
         S = Line(I+1:)
      END IF
      J = INDEX(S, '.')               ! find FIRST instance of '.'
      IF (J .EQ. 0) THEN
         FName = TRIM(S)
         Ext = ''
      ELSE
         FName = S(1:J-1)
         Ext = S(J:)
      END IF
      RETURN                                               

      END SUBROUTINE ExtractFileName

!---------------------------------------------------------------------
      SUBROUTINE Sort
      IMPLICIT NONE
      INTEGER     :: I, J, K, M, P
      CHARACTER (LEN=80) :: X

      IF (NumSortLines .GT. MaxNoOfLines) THEN
         ErrorMessage = 'MakeState:  Range Error in SORT'
         CALL PassMsg
         GOTO 898
      END IF

      P = NumSortLines
      DO WHILE (P .GT. 0)
        K = NumSortLines - P
        J = 1
        DO WHILE (J .LE. K)
          I = J
          M = I + P
          DO WHILE ((I .GT. 0) .AND. (SortLines(I) .GT. SortLines(M)))
            X = SortLines(I)
            SortLines(I) = SortLines(M)
            SortLines(M) = X
            M = I
            I = I - P
          END DO
          J = J + 1
        END DO
        P = P / 2
      END DO

      RETURN

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : SORT...'
      CALL PassMsg
      RETURN

      END SUBROUTINE SORT

END MODULE MakeStateSubs

!==============================================================================

MODULE MakeStateFile_Main

CONTAINS
      SUBROUTINE DO_MakeStateFile (ErrStatus, HandleIn, FileName, Lake)

      USE MakeStateSubs

      IMPLICIT NONE

      INTEGER, INTENT(INOUT) :: ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) ::  FileName, Lake

      INTEGER            :: I, J, K, IOS
      LOGICAL            :: Incl
      CHARACTER (LEN=3)  :: BnS, Ext
      CHARACTER (LEN=6)  :: IntFile(5)
      CHARACTER (LEN=8)  :: FileNm(16)
      CHARACTER (LEN=15) :: FName
      CHARACTER (LEN=80) :: S, SimLine
      CHARACTER (LEN=99) :: File1, File2, File3, File4


      DATA FileNm / 'BSNNM   ','EBSNNM  ','PARMTABL','PARAMEVP',          &
     &   'LENGTH  ','AREA    ','ZSEVAP  ','ZSEVAPPR','STOQUANT',          &
     &   'RUNQUANT','NBSQUANT','EVPQUANT','STQUANPR','EVQUANPR',          &
     &   'EPRVSNAL','PROVSNAL'/

      DATA IntFile/'FCSTBL','ESTOTL','STOOTL','DLYTBL','SUBSUM'/

!
      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen()
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
      File1 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=11, FILE=TRIM(File1), STATUS='OLD', RECL=12,              &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(11, 1001, REC=1, ERR=812) BnS
      CLOSE(11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

      File2 = TRIM(ThisLakeDir) // 'DIRLIST.INT'
      OPEN(UNIT=21, FILE=TRIM(File2), STATUS='OLD', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      K = 1
      READ(21, 1002, IOSTAT=IOS) SortLines(K)

      DO WHILE (IOS .EQ. 0)
         CALL ExtractFileName(SortLines(K), FName, Ext)
         IF (ErrorLevel .NE. 0) GOTO 899
         Incl = (FName(1:3) .EQ. BnS  .AND.  Ext .EQ. 'MAP') .OR.         &
     &          (FName(1:3) .EQ. BnS  .AND.  Ext .EQ. 'SUM') .OR.         &
     &          (FName(1:3) .EQ. BnS  .AND.  Ext .EQ. 'CLM') .OR.         &
     &          (FName(1:3) .EQ. BnS  .AND.  Ext .EQ. 'PRV') .OR.         &
     &          (FName(1:3) .EQ. BnS  .AND.  Ext .EQ. 'MET')
         DO I = 1, 16
           Incl = Incl .OR. (FName.EQ.FileNm(I) .AND. Ext.NE.'INT')
         END DO
         DO I = 1, 5
           Incl = Incl .OR. (FName(1:6) .EQ. IntFile(I) .AND. Ext .EQ. 'INT')
         END DO
         IF (Incl) K = K + 1
         IF (K .GT. MaxNoOfLines) THEN
            ErrorMessage = 'MakeState:  Too many lines in DIRLIST.INT.'
            GOTO 899
         END IF
         READ(21, 1002, IOSTAT=IOS) SortLines(K)
      END DO

      CLOSE(21, STATUS = 'DELETE')
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

      K = K - 1    ! number of lines actually included
      NumSortLines = K

      CALL Sort; IF (ErrorLevel .NE. 0) GOTO 899

      File3 = TRIM(ThisLakeDir) // TRIM(FileName)
      OPEN(UNIT=31, FILE=TRIM(File3), STATUS='OLD', ERR=31)
      CLOSE(31, STATUS='DELETE')
 31   OPEN(UNIT=31, FILE=TRIM(File3), STATUS='NEW', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899
      DO I = 1, K
         WRITE(31, 1003, ERR=833) TRIM(SortLines(I))
      END DO

      File4 = TRIM(ThisLakeDir) // 'SIMULATE'
      OPEN(UNIT=41, FILE=TRIM(File4), STATUS='OLD', ERR=841)
      CALL FileWasOpened(41); IF (ErrorLevel .NE. 0) GOTO 899
      READ(41, 1002, IOSTAT=IOS) SimLine
      DO WHILE (IOS .EQ. 0)
         WRITE(31, 1003, ERR=833) TRIM(SimLine)
         READ(41, 1002, IOSTAT=IOS) SimLine
      END DO

      CLOSE(41)
      CALL FileWasClosed(41); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE(31)
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Error handling
!
 811  ErrorMessage = 'MakeState:  Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
 812  ErrorMessage = 'MakeState:  Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
 821  ErrorMessage = 'MakeState:  Error opening file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
 822  ErrorMessage = 'MakeState:  Error reading file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
 831  ErrorMessage = 'MakeState:  Error opening file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
 832  ErrorMessage = 'MakeState:  Error reading file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
 833  ErrorMessage = 'MakeState:  Error writing file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
 841  ErrorMessage = 'MakeState:  Error opening file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
 842  ErrorMessage = 'MakeState:  Error reading file ' // TRIM(File4)
      CALL PassMsg; GOTO 898

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : MakeStateFile_Main...'
      CALL PassMsg
      GOTO 999

 999  CALL CloseAllFiles()
      ErrStatus = ErrorLevel
      RETURN

!
!  FORMATs
!
 1001 FORMAT(A3)
 1002 FORMAT(A80)
 1003 FORMAT(A)

      END SUBROUTINE DO_MakeStateFile

END MODULE MakeStateFile_Main

