!
!     Program to calculate daily overlake evaporation, for use in the FORECAST
!     PACKAGE, by using a zero-dimensional heat budget and a one-dimensional
!     heat storage model.  Version: 08feb2013.
!
!     Heat balance, heat storage function, and program by TEC II
!        development during 7/87-7/88 reported in WRR 25(5):781-792.
!        further developed 1/89-6/89, 1/90, 5/90, 11/90, 2/91.
!        development during 8/88-2/91 reported in WRR 28(1):69-81.
!        further developed 5/91, 6/91, 8/91-2/92 (daily ice computations).
!        development during 5/91-2/92 reported in WRR 30(3):625-639.
!
!     4-byte date handling and move to UNIX, TSH & TEC II 14oct92.
!     Missing data handling by TSH 31dec92, 11may93.
!     Unused parameters removed by TEC II 15jun94.
!     Updated by TEC II 21mar95 for change in BSNNM structure (addition of
!      forecast start date in record 5 and displacement of later records by
!      one to accomodate the addition) and to suppress writing of status info
!      if file: "QUIET.INT" exists in default directory.
!     Updated by TEC II 1aug95 to input/output ice cover as a decimal fraction
!      of the lake area rather than as an area; also use this to compute
!      "IceAmount" instead of reading in this (redundant) info.
!     Updated by TSH 23oct97 to handle ALLTEMPS.xxx file more cleanly and
!      correctly.
!     Translated to FORTRAN95 by TECII 30mar01.
!     Added capability to compute temperature-depth profiles by TEC 24apr01.
!     Restructured to improve modularity and allow integration into AHPS
!      by TSH 08feb2013.
!     StoredHeatAtEndOfDay calculation tweaked on 02Dec2013 by TSH.
!
!     Evaporation theory by Frank Quinn (WRR 15(4):935-940)
!        modified by TEC II from "JDEVAP.FOR", 3/87
!     Earlier versions revised, debugged, and/or tested
!        by TEC II 5/87 - 7/88
!        by TSH 12/87, 6/89, 7/90, 2/91
!        by Cynthia Sellinger 3/88
!        by Forrest McLaughlin 5/87 - 7/87
!        by Lauri Huff 6/90
!
!     Input files: eBSNNM, PARAMEVP.xxx, EVPPARAM.xxx, xxxMD.CLM, xxxMD.PRV,
!     ZSEVAPPR.xxx, ALLTEMPS.xxx, MET_DATA.xxx, and DATEFILE.INT
!     where "xxx" denotes the 3 letter lake abbrev. - SUP, ERI, ...
!
!     Output files: ZSEVAP.xxx, ZSEVAPPR.xxx, ZSTEMP.xxx, ZSEVyyyy.INT,
!     PARAMEVP.xxx, EVPPARAM.xxx, and eBSNNM.
!
!     This program is designed to be used in FOUR situations:
!
!     1) Program Designator (number in the 2nd record of file: eBSNNM) = -10.
!        The entire data file, MET_DATA.xxx (used in calibration), is used with
!        the initial conditions found in PARAMEVP.xxx to create ZSEVAP.xxx and
!        ZSTEMP.xxx.  For exact agreement with the calibration, initial
!        conditions and parameters in PARAMEVP.xxx must agree with the
!        parameters and conditions used in the calibration (imbedded in the
!        calibration code and in the calibration parameter file).
!
!     2) Program Designator = -9.
!        The data file, xxxMD.CLM, is used up to the base date (in file EBSNNM)
!        with the (arbitrary) initial conditions found in PARAMEVP.xxx to
!        recreate ZSEVAP.xxx.  The parameter file, PARAMEVP.xxx is REcreated
!        with conditions for the base date, to be used as initial conditions
!        for the provisional period and ZSEVAPPR.xxx is created also for a
!        period of LENGTH days up to the base date.
!
!     3) Program Designator = 4.
!        The data file, xxxMD.PRV, is used with the initial conditions and
!        start and finish dates found in EVPPARAM.xxx to update ZSEVAPPR.xxx.
!        The parameter file, EVPPARAM.xxx, is REcreated with final conditions
!        to be used as initial conditions for the outlook period.
!
!     4) Program Designator = 7.
!        The data file, xxxMD.CLM, is used with the initial conditions and
!        start and finish dates found in EVPPARAM.xxx to create ZSEVyyyy.INT.
!
!                                         Pd=-10  Pd=-9   Pd= 4   Pd= 7
!                                         ------  -----   -----   -----
!     Input files:  EBSNNM                  X       X       X       X
!                   PARAMEVP.xxx            X       X
!                   EVPPARAM.xxx                            X       X
!                   xxxMD.CLM                       X               X
!                   xxxMD.PRV                               X
!                   ZSEVAPPR.xxx                            X
!                   DATEFILE.INT                                    X
!                   MET_DATA.xxx            X
!
!     Output files: EBSNNM - updated        X       X       X       X
!                   ZSEVAP.xxx              X       X
!                   ZSEVAPPR.xxx                    X       X
!                   ZSEVyyyy.INT                                    X
!                   ZSTEMP.xxx              X
!                   PARAMEVP.xxx                    X
!                   EVPPARAM.xxx                            X
!
!     The eBSNNM file for this program is a formatted, direct access file with
!     a record length of 12 (11-byte records plus a line feed).
!     It has the following structure:
!
!      1st record      LakeName              FORMAT(A11)
!
!      2nd record      Program Designator    FORMAT(3X, I3, 5X)
!                      |                |
!                      |________________|___ This integer is used for
!                                            sequencing the programs in the
!                                            evaporation forecast package. In
!                                            this program the valid values are
!                                            -10, -9, 4, and 7.
!
!      3rd record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|_yr, climatic data end date + 1 d
!                      | | |___|____mo, climatic data end date + 1 d
!                      |_|________dy, climatic data end date + 1 d
!                                   This field is also called the BASE DATE.
!
!      4th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of provisional data end
!                      | | |___|______month of provisional data end
!                      |_|__________day of provisional data end
!
!      5th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of forecast end
!                      | | |___|______month of forecast end
!                      |_|__________day of forecast end
!
!      6th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of start for model stats output
!                      | | |___|______month of start for model stats output
!                      |_|__________day of start for model stats output
!
!      7th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of earliest changed data
!                      | | |___|______month of earliest changed data
!                      |_|__________day of earliest changed data
!
!     The parameter files for this program, PARAMEVP.xxx and EVPPARAM.xxx (xxx
!     = 'SUP', 'MIC', 'HUR', 'NHU', 'GEO', 'STC', 'ERI', 'ONT'), and
!     EVPPRMzz.INT (zz = 2-digit outlook year specifier, e.g., 48, 49, ...),
!     are formatted, sequential access files with the following structure:
!
!     1st record:      Ext Comment           FORMAT(A3, A77)
!                      | | |     |
!                      | | |_____|___documentation comment
!                      |_|_________string denoting provisional ('PRV') or
!                                  climatic ('CLM') data file to use
!
!     2nd through 14th records: evaporation parameters 1 - 13 respectively
!                               FORMAT(E10.4E1)
!
!     15th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of simulation start date
!                      | | |___|______month of simulation start date
!                      |_|__________day of simulation start date
!
!     16th record:     Day Month Year        FORMAT(2I3, I5)
!                      | | |   | |  |
!                      | | |   | |__|___year of simulation end date
!                      | | |___|______month of simulation end date
!                      |_|__________day of simulation end date
!
!     17th record:     SurfaceTemperature    FORMAT(E13.7E2)
!                      |
!                      |___initial water surface temperature, degrees Celcius
!
!     18th record:     IceAmount             FORMAT(E13.7E2)  NOT USED!!!
!                      |                                      NOT USED!!!
!                      |___initial ice cover, cubic meters    NOT USED!!!
!
!     19th record:     IceArea               FORMAT(E13.7E2)
!                      |
!                      |___initial ice area, decimal fraction of lake area
!
!     20th record:     IceDepth              FORMAT(E13.7E2)
!                      |
!                      |___initial ice depth, meters
!
!     21st record:     IceTemp               FORMAT(E13.7E2)
!                      |
!                      |___initial ice pack temperature, deg.C
!
!     22nd record:     NumberOfBlocks        FORMAT(E13.7E2)  NOT USED!!!
!                      |                                      NOT USED!!!
!                      |___initial number of ice blocks       NOT USED!!!
!
!     23rd record:     AgeOfCurrentLayer     FORMAT(E13.7E2)
!                      |
!                      |___wind history of the current layer (on first day of
!                          simulation), meters/sec
!
!     24th record:     NoOfLayers            FORMAT(I11)
!                      |
!                      |___number of layers in the heat superposition model on
!                          the first day of the simulation
!
!     25th through 24+3*NoOfLayers records: heat storage parameters for all
!                                           layers in the heat superposition
!                                           model on the first day of the
!                                           simulation; each record has the
!                                           FORMAT(E13.7E2)
!
!     NOTE THAT RECORDS 17 THROUGH 24 + 3 * NoOfLayers ARE FILLED IN BY A
!     PREVIOUS RUN OF THIS PROGRAM WITH AN END SIMULATION DATE ON THE DAY
!     BEFORE THE START SIMULATION DATE IN THIS PARAMETER FILE.  THESE REPRESENT
!     INITIAL CONDITIONS FOR THE PROGRAM SIMULATION AND ARE ARBITRARY FOR A
!     LONG SIMULATION WHERE THE INTEREST IS ONLY IN THE LATTER PART OF THE
!     SIMULATION.
!
!     The data file xxxMD.CLM contains the historical data gathered from
!     near-shore stations around the lake.  It is a direct access, unformatted
!     file with a record length of 8 bytes.  The structure of the file is given
!     below.
!
!     1st record:     Id StartSeqNum   (2, 4)
!                     || |         |
!                     || |_________|__ 4-byte sequence number corresponding
!                     ||               to date of first data record.
!                     ||____________ Lake identifier number (1=SUP, 2=MIC,
!                                    3=HUR, 4=STC, 5=ERI, 6=ONT, 7=CHA, 8=URA,
!                                    9=EMB, 10=VOL, 11=NHU, 12=GEO).
!
!     2nd record:     EndSeqNum   (4, 2x)
!                     |       |
!                     |_______|_____ 4-byte sequence number corresponding to
!                                    the date of the last record.
!
!     3rd through (NumDays+2) records:
!                     Each record contains four 2-byte integers.
!
!                     AirTemp DewPt Wind Cloud   (2, 2, 2, 2)
!                     |     | |   | |  | |   |
!                     |     | |   | |  | |___|__ Total cloud cover percentage
!                     |     | |   | |  |         (15 = 15.00%)
!                     |     | |   | |__|________ Wind speed in meters per sec
!                     |     | |   |              times 100 (1500 = 15.00 m/s)
!                     |     | |___|_____________ Dew point temperature in deg C
!                     |     |                    times 100 (1500 = 15.00 deg C)
!                     |_____|___________________ Air temperature in degrees C
!                                                times 100 (1500 = 15.00 deg C)
!
!     The data file xxxMD.PRV contains the provisional data from near-shore
!     stations. It has the same file structure as xxxMD.CLM.
!
!     The files, ZSEVAP.xxx, ZSEVAPPR.xxx, and ZSEVyyyy.INT, contain daily lake
!     evaporation variables. The period of record and source of data for each
!     file is as follows:
!
!       ZSEVAP.xxx   = Entire historical period from the beginning of available
!                      data in xxxMD.CLM through the base date if program
!                      designator = -9; if program designator = -10, period
!                      corresponds to that in MET_DATA.xxx.
!
!       ZSEVAPPR.xxx = Historical data for the LENGTH days immmediately prior
!                      to the BASE DATE and provisional historical data from
!                      the BASE DATE to the beginning of the forecast period.
!
!       ZSEVyyyy.INT = Data for the forecast period. This data results from
!                      running the evaporation model with data from year yyyy.
!                      The values for overlake air temperature, water vapor
!                      pressure, wind speed, and cloud cover come from year
!                      yyyy with overwater corrections applied. (Cloud cover
!                      will be unchanged.)
!
!     The files are direct access, unformatted files with 64-byte records. The
!     structure is as follows:
!
!     1st record:  StartSeq EndSeq   (4, 4)
!                  |      | |    |
!                  |      | |____|__ 4-byte integer ending sequence number
!                  |      |          corresponding to the last day in the file.
!                  |______|____ 4-byte integer starting sequence number
!                               corresponding to the first day in the file.
!
!     2nd through (NumDays+1) records:
!       Sixteen 4-byte real numbers as described:
!
!       R1 R2 R3 R4 R5 R6 R7 R8 R9 R10 R11 R12 R13 R14 R15 R16
!       || || || || || || || || || | | | | | | | | | | | | | |
!       || || || || || || || || || | | | | | | | | | | | | |_Stored Heat (cal)
!       || || || || || || || || || | | | | | | | | | | |_|_Ice Depth (m)
!       || || || || || || || || || | | | | | | | | |_|___Ice Area (decimal
!       || || || || || || || || || | | | | | | | |       fraction)
!       || || || || || || || || || | | | | | | |_|___Ice Temperature (C)
!       || || || || || || || || || | | | | |_|__ Advection (watts/m2)
!       || || || || || || || || || | | |_|______ Sensible Heat Flux (watts/m2)
!       || || || || || || || || || |_|__________ Latent Heat Flux (watts/m2)
!       || || || || || || || || ||______________ Net Long Wave Radiation
!       || || || || || || || ||                  Exchange (watts/m2)
!       || || || || || || || ||_________________ Reflection (watts/m2)
!       || || || || || || ||____________________ Incident Short-wave
!       || || || || || ||                        Radiation (watts/m2)
!       || || || || || ||_______________________ CloudCover (decimal fraction)
!       || || || || ||__________________________ Over-lake Wind Speed (m/s)
!       || || || ||_____________________________ Over-lake Water Vapor
!       || || ||                                 Pressure (mb)
!       || || ||________________________________ Over-lake Air Temperature (C)
!       || ||___________________________________ Over-lake Water Surface
!       ||                                       Temperature (C)
!       ||______________________________________ Lake Evaporation (cm)
!
!     The file ZSTEMP.xxx contains superposition information coincident to
!     ZSEVAP.xxx.  There is a block of 2-byte records for each day of
!     simulation in ZSEVAP.xxx and they are in sequential order.  The first
!     record in the block is the number of "layers" in the superposition, then
!     there are "2*NumberOfLayers" 2-byte records of SurfTempFromLayer and
!     relative volume of the lake, that each heat increment occupies,
!     alternating in reverse order.
!
!     MET_DATA.xxx contains meteorological variables selected especially for
!     evaporation calculations and used in the calibration of the evaporation
!     model (determination of parameters in PARAMEVP.xxx).  It is an
!     unformatted direct-access file with 40-byte records.  The structure is
!     as follows.
!
!     1st record:
!     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx SeqNum EndSeq  (32 4 4)
!     |                              | |    | |    |
!     |                              | |    |          ending sequence
!     |                              | |    | |____|__ 4-byte integer
!     |                              | |    |          number correspond-
!     |                              | |    |          ing to the last
!     |                              | |    |          day in the file.
!     |                              | |____|__ 4-byte integer starting
!     |                              |          sequence number
!     |                              |          corresponding to the
!     |                              |          first day in the file.
!     |______________________________|__identification/comment field.
!
!     2nd through ((EndSeq-SeqNum+1) + 1) records:
!        Ten 4-byte real numbers as described.
!        R1 R2 R3 R4 R5 R6 R7 R8 R9 R10     (4,4,4,4,4,4,4,4,4,4,4)
!        || || || || || || || || || | |
!        || || || || || || || || || |_|__Ice Concentration
!        || || || || || || || || ||      fraction of lake area in ice, %
!        || || || || || || || || ||__Outflow, daily outflows from lake, cubic
!        || || || || || || || ||     meters per day
!        || || || || || || || ||__Inflow, daily inflows other than runoff to
!        || || || || || || ||     lake, cubic meters per day
!        || || || || || || ||__Runoff, daily basin inflow to lake, cubic meters
!        || || || || || ||     per day
!        || || || || || ||__Precipitation, daily overlake precipitation to
!        || || || || ||     lake, cubic meters per day
!        || || || || ||__CloudCover, average daily fraction of sky covered
!        || || || ||__WindSpeed, average daily windspeed, meters per second
!        || || ||__DewPtTemp, average daily dew point temperature, degrees
!        || ||     Celcius
!        || ||__AirTemp, average daily air temperature, degrees Celcius
!        ||__WaterTemp, average daily water temperature, degrees Celcius
!
!     The file DATEFILE.INT is a formatted, sequential access file with the
!     following structure:
!
!     1st record:   StartDay    FORMAT(I4)
!                   |      |
!                   |______|__ The starting day for the historical period to
!                              be used as the forecast meteorology.
!
!     2nd record:   StartMon    FORMAT(I4)
!                   |      |
!                   |______|__ The starting month for the historical period to
!                              be used as the forecast meteorology.
!
!     3rd record:   StartYr     FORMAT(I4)
!                   |     |
!                   |_____|___ The starting year for the historical period to
!                              be used as the forecast meteorology.
!
!     4th record:   TodayDay    FORMAT(I4)
!                   |      |
!                   |______|__ The day the forecast is being run (when the
!                              program FRCST was run).
!
!     5th record:   TodayMon    FORMAT(I4)
!                   |      |
!                   |______|__ The month the forecast is being run (when the
!                              program FRCST was run).
!
!     6th record:   TodayYr     FORMAT(I4)
!                   |     |
!                   |_____|___ The year the forecast is being run (when the
!                              program FRCST was run).
!
!     7th record:   InitLevel   FORMAT(F10.3)
!                   |       |
!                   |_______|__ The initial lake level.
!
!------------------------------------------------------------------------------

MODULE XEvapSubs

      USE FcstUtil

      INTEGER, PARAMETER :: Length = 560  ! # of days in ZSEVAPPR prior to base date
      INTEGER, PARAMETER :: MHdr = 1      ! number of header records in MET_DATA.*
      INTEGER, PARAMETER :: CHdr = 2      ! number of header records in *.CLM
      INTEGER, PARAMETER :: ZHdr = 1      ! number of header records in ZSEVAP-type files
      INTEGER, PARAMETER :: MaxNoOfLayers = 500

      INTEGER  :: LakeNumber            ! number of the lake being processed
      
      LOGICAL  :: WriteBulkCoefficient  ! Do we want to output a file with the time-series 
                                        ! of the Bulk transfer Coefficient?
      LOGICAL  :: WriteTempDepthProfile ! Do we want to output a file with the time-series
                                        ! of the Temperature/Depth profile info?

      LOGICAL  :: PrintDebugInfo        ! If .TRUE. a lot of miscellaneous status and info
                                        ! messages will be printed for debugging purposes
      
      INTEGER  :: MissingDataUnitNo     ! unit number for filling in missing met data
      CHARACTER (LEN=99) :: MissingDataFileName

      REAL :: EvapUnitsCoeff, HeatUnitsCoeff, MonitoringDepth

      REAL, DIMENSION(13) :: AverageSurfaceArea

      !
      !  Please note well: The handling of the combined Huron+Georgian is
      !  messy, to say the least. AHPS does not do a combined lake, only
      !  doing the individual HUR and GEO basins. This is because the early
      !  versions of this LLTM model only applied to the individual basins.
      !  Tom Croley, however, has added the combined basin to this model, which
      !  "throws a monkey-wrench into" the basin numbering schemes, etc.
      !  What I am doing is, in the AHPS-wide modules, such as FcstUtil, I am
      !  adding a basin 13 at the end of the appropriate PARAMETER arrays. That
      !  will allow everything else in AHPS to continue working without change.
      !  In this model, I will assign DataIndex to 13 for HGB and fill in the
      !  entries 8-12 with dummy missing values.
      !
      ! Order of lakes here is:
      !   SUP, MIC, HUR, GEO, STC, ERI, ONT, nil, nil, nil, nil, nil, HGB
      !
      REAL, DIMENSION(13), PARAMETER :: AverageVolume =                   &
          (/ 0.1210e+14, 0.4920e+13, 0.2761e+13, 0.0779e+13,              &
             0.3400e+10, 0.4840e+12, 0.1640e+13, -9.999e+09,              &
             -9.999e+09, -9.999e+09, -9.999e+09, -9.999e+09,              &
             0.3540e+13/)

      REAL, DIMENSION(13), PARAMETER :: AtmosphericPressure =             &
          (/     991.80,     992.58,     992.58,     992.58,              &
                 992.75,     992.86,     1004.53,   -999.99,              &
                -999.99,    -999.99,    -999.09,    -999.99,              &
                 992.58/)

      REAL, DIMENSION(13), PARAMETER :: AirDensity =                      &
          (/   0.001204,   0.001205,   0.001205,   0.001205,              &
               0.001205,   0.001205,   0.001216,  -9.999999,              &
              -9.999999,  -9.999999,  -9.999999,  -9.999999,              &
               0.001205/)

      REAL, DIMENSION(8), PARAMETER :: DepthVolumeA =                   &
           (/  92666.,  26534.,  2622.0,  18937.,  3361.0,              &
             .56667e9,  600.34,  178275.0/)

      REAL, DIMENSION(8), PARAMETER :: DepthVolumeB =                   &
           (/ 3.11256, 3.37654, 3.86907, 3.45945, 3.77684,              &
                   1., 4.93110, 2.91689/)

      REAL, DIMENSION(8), PARAMETER :: DepthVolumeM =                   &
           (/    405.,    281.,    229.,    229.,    164.,              &
                   6.,     64.,    244./)

!
!   These are the variables containing the information read from the PARAMEVP
!   or EVPPARAM file.
!
      REAL    :: P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, AgeOfCurrentLayer
      REAL    :: SurfaceTemperature, IceAmount, IceArea, IceDepth, IceTemp
      INTEGER :: NoOfLayers, SequenceNumber, Finish
      REAL, DIMENSION(0:MaxNoOfLayers) :: SurfTempFromLayer, TotalHeat
      REAL, DIMENSION(1:MaxNoOfLayers) :: AgeOfLayer
      CHARACTER (LEN=3)  :: Ext
      CHARACTER (LEN=77) :: Comment

!
!     This structure holds the parameter information read from EVPPARAM.xxx or PARAMEVP.xxx
!
      TYPE EvapParmType
          CHARACTER (LEN=3)  :: Ext
          CHARACTER (LEN=77) :: Comment
          REAL    :: Parms(13)
          INTEGER :: SSeq, ESeq
          REAL    :: SurfTemp, IceAmount, IceArea, IceDepth, IceTemp, AgeOfCurrentLayer
          INTEGER :: NoOfLayers
          REAL, DIMENSION(0:MaxNoOfLayers) :: SurfTempFromLayer, TotalHeat
          REAL, DIMENSION(1:MaxNoOfLayers) :: AgeOfLayer
      END TYPE

CONTAINS


!-----------------------------------------------------------------------------
!     Support routines for evaporation computations, 15jun94
!
!     WARNING!  Do not change any of the following subroutines (for use with
!     the forecast package) independently of changes to these same routines for
!     use with the research version of the model (calibration).  These
!     subroutines are supposed to be identical between XCALIB and XEVAPOUT!
!     TECII
!------------------------------------------------------------------
!
! SUBROUTINE ReadMissing is used to fill in missing data that is encountered
! within the input data file.  It determines what the date is (Jan 1, etc.)
! and then reads and determines the average of all the values for that date
! from the CLM file.  This is the value that is used to fill in the missing
! data.
!
! e.g.  If the date is March 15, 1989, then we will read all of the March 15
! values (1948, 1949, 1950, ...).  We will then determine the average of all
! the valid data values.  This is the value to be used.
!
! The variable "Missing" is used to determine which values were missing.  It
! is assigned a value corresponding to a bit pattern so that if the first bit
! is set then I1 was missing, etc.  It can have a value from 1 (meaning only
! I4 was missing) to 15 (meaning all 4 values were missing).  When we return
! to the calling procedure it will have a value of 0 if all missing values were
! filled in.  A non-zero value means that we were unable to fill in one or more
! missing values.
!
! Assumptions:
!  The following assumptions are made and were true at the time this
!  routine was written.
!    1) Unit U1 is open and is the data file we are reading from. It is either
!       xxxMD.PRV or xxxMD.CLM and the PRV/CLM choice is reflected by the value
!       of the variable Ext.
!    2) If unit U1 is xxxMD.PRV, then xxxMD.CLM is available and is in the
!       current directory.
!    3) Using the standard 4-byte DateSequence routines.
!
! Written by Tim Hunter on 12/31/92.  Modified by TSH on 5/11/93 to handle case
! when we cannot fill in a missing value.
!
      SUBROUTINE ReadMissing (I1, I2, I3, I4, Missing, SeqIn, Ext)

      IMPLICIT   NONE

      INTEGER (KIND=INTEGER2) :: I1, I2, I3, I4, Missing, X1, X2, X3, X4
      INTEGER (KIND=INTEGER2) :: C1, C2, C3, C4, Temp

      INTEGER :: SeqIn, Day, Mon, Year, StartSeq, EndSeq, U1
      INTEGER :: UnitNo, A1, A2, A3, A4, SeqNum
      INTEGER :: Year1, Year2, RecNum

      CHARACTER (LEN=3) :: Ext

!
!  Assign unit number to local variable.  This unit is a file that was
!  already opened by another program unit (main) and is a "MD.CLM"-format
!  file from which we will read historical met data to fill in for
!  missing values.
!
      U1 = MissingDataUnitNo

!
!  Determine file limits
!
      READ(U1, REC=1, ERR=802) X1, StartSeq
      READ(U1, REC=2, ERR=802) EndSeq
      CALL SequenceDate(Day, Mon, Year1, StartSeq); IF (ErrorLevel .NE. 0) GOTO 899
      CALL SequenceDate(Day, Mon, Year2, EndSeq)  ; IF (ErrorLevel .NE. 0) GOTO 899

!
!  Determine date to be reading
!
      CALL SequenceDate(Day, Mon, Year, SeqIn) ; IF (ErrorLevel .NE. 0) GOTO 899

!
!  Read data and accumulate
!
      A1 = 0
      A2 = 0
      A3 = 0
      A4 = 0
      C1 = 0
      C2 = 0
      C3 = 0
      C4 = 0
      DO 30, Year = Year1, Year2
        CALL DateSequence(Day, Mon, Year, SeqNum); IF (ErrorLevel .NE. 0) GOTO 899
        IF (SeqNum .LT. StartSeq)   CYCLE
        IF (SeqNum .GT. EndSeq)     CYCLE
        RecNum = SeqNum - StartSeq + CHdr + 1
        READ(U1, REC=RecNum, ERR=802) X1, X2, X3, X4
        IF (X1 .GT. -9000) THEN
           A1 = A1 + X1
           C1 = C1 + 1
        END IF
        IF (X2 .GT. -9000) THEN
           A2 = A2 + X2
           C2 = C2 + 1
        END IF
        IF (X3 .GT. -9000) THEN
           A3 = A3 + X3
           C3 = C3 + 1
        END IF
        IF (X4 .GT. -9000) THEN
           A4 = A4 + X4
           C4 = C4 + 1
        END IF
  30  CONTINUE

!
!  Compute averages, if possible
!
      IF (C1 .GT. 0) X1 = A1 / C1
      IF (C2 .GT. 0) X2 = A2 / C2
      IF (C3 .GT. 0) X3 = A3 / C3
      IF (C4 .GT. 0) X4 = A4 / C4

!
!  Assign values to the missing data variables
!
      Temp = Missing
      IF (Temp .GE. 8) THEN
         IF (C1 .GT. 0) THEN
            I1 = X1
            Missing = Missing - 8
         END IF
         Temp = Temp - 8
      END IF
      IF (Temp .GE. 4) THEN
         IF (C2 .GT. 0) THEN
            I2 = X2
            Missing = Missing - 4
         END IF
         Temp = Temp - 4
      END IF
      IF (Temp .GE. 2) THEN
         IF (C3 .GT. 0) THEN
            I3 = X3
            Missing = Missing - 2
         END IF
         Temp = Temp - 2
      END IF
      IF (Temp .GE. 1) THEN
         IF (C4 .GT. 0) THEN
            I4 = X4
            Missing = Missing - 1
         END IF
         Temp = Temp - 1
      END IF

      RETURN

!
!  Error handlers for I/O problems
!
 802  ErrorMessage = 'XEVAPOUT: Error reading data from ' // TRIM(MissingDataFileName)
      CALL PassMsg;  GOTO 898

 898  ErrorLevel = 1
 899  RETURN

      END SUBROUTINE ReadMissing


!-----------------------------------------------------------------------------
      SUBROUTINE ReadParmFile(ParmFile, Pd, Ext, Comment,                  &
     &                        P1, P2, P3, P4, P5, P6, P7, P8, P9, P10,     &
     &                        StartSeq, EndSeq, SurfaceTemperature,        &
     &                        IceAmount, IceArea, IceDepth, IceTemp,       &
     &                        AgeOfCurrentLayer, NoOfLayers,               &
     &                        SurfTempFromLayer, TotalHeat, AgeOfLayer)

      IMPLICIT   NONE
      CHARACTER (LEN=*),  INTENT(IN)  :: ParmFile
      INTEGER,            INTENT(IN)  :: Pd
      CHARACTER (LEN=3),  INTENT(OUT) :: Ext
      CHARACTER (LEN=77), INTENT(OUT) :: Comment
      REAL,               INTENT(OUT) :: P1, P2, P3, P4, P5, P6, P7, P8, P9, P10
      INTEGER,            INTENT(OUT) :: StartSeq, EndSeq, NoOfLayers
      REAL,               INTENT(OUT) :: SurfaceTemperature, IceAmount, IceArea
      REAL,               INTENT(OUT) :: IceDepth, IceTemp, AgeOfCurrentLayer
      REAL, DIMENSION(0:MaxNoOfLayers), INTENT(OUT) :: SurfTempFromLayer, TotalHeat
      REAL, DIMENSION(1:MaxNoOfLayers), INTENT(OUT) :: AgeOfLayer

      INTEGER :: U1, Dy, Mn, Yr, I

      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(ParmFile), STATUS='OLD', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899

      READ(U1, 1001, ERR=812) Ext
      IF (Pd .EQ. -9) Ext = 'CLM'
      IF (Ext .NE. 'PRV' .AND. Ext .NE. 'CLM') THEN
         WRITE(ErrorMessage, 9015) Ext
         CALL PassMsg
         GOTO 898
      END IF

      READ(U1, 1005, ERR=812) P1
      READ(U1, 1005, ERR=812) P2
      READ(U1, 1005, ERR=812) P3
      READ(U1, 1005, ERR=812) P4
      READ(U1, 1005, ERR=812) P5
      READ(U1, 1005, ERR=812) P6
      READ(U1, 1005, ERR=812) P7
      READ(U1, 1005, ERR=812) P8
      READ(U1, 1005, ERR=812) P9
      READ(U1, 1005, ERR=812) P10
      READ(U1, 1005, ERR=812)                  ! unused as of 15jun94
      READ(U1, 1005, ERR=812)                  ! unused as of 15jun94
      READ(U1, 1005, ERR=812)                  ! unused as of 15jun94
      READ(U1, 1002, ERR=812) Dy, Mn, Yr
      CALL DateSequence(Dy, Mn, Yr, StartSeq); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U1, 1002, ERR=812) Dy, Mn, Yr
      CALL DateSequence(Dy, Mn, Yr, EndSeq); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U1, 1009, ERR=812) SurfaceTemperature       ! deg C
      READ(U1, 1009, ERR=812)                          ! unused as of 1aug95
      READ(U1, 1009, ERR=812) IceArea                  ! fraction (0-1.0)
      READ(U1, 1009, ERR=812) IceDepth                 ! meters
      READ(U1, 1009, ERR=812) IceTemp                  ! deg C
      READ(U1, 1009, ERR=812)                          ! unused as of 15jun94
      READ(U1, 1009, ERR=812) AgeOfCurrentLayer
      READ(U1, 1004, ERR=812) NoOfLayers

      IF (NoOfLayers .GT. MaxNoOfLayers) THEN
         WRITE(ErrorMessage, 9051) TRIM(ParmFile)
         CALL PassMsg
         WRITE(ErrorMessage, 9052) NoOfLayers, MaxNoOfLayers
         CALL PassMsg
         GOTO 898
      END IF

      DO I = 1, NoOfLayers
         READ(U1, 1009, ERR=812) AgeOfLayer(I)
      END DO
      DO I = 1, NoOfLayers
         READ(U1, 1009, ERR=812) SurfTempFromLayer(I)
      END DO
      DO I = 1, NoOfLayers
         READ(U1, 1009, ERR=812) TotalHeat(I)
      END DO

      IceArea   = IceArea * AverageSurfaceArea(LakeNumber)  ! fraction -> sq meters
      IceAmount = IceArea * IceDepth                        ! cubic meters

      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      RETURN

 811  ErrorMessage = 'XEVAPOUT: Error opening ' // TRIM(ParmFile)
      CALL PassMsg;  GOTO 898
 812  ErrorMessage = 'XEVAPOUT: Error reading ' // TRIM(ParmFile)
      CALL PassMsg;  GOTO 898

 898  ErrorLevel = 1
 899  RETURN

 1001 FORMAT(A3, 8X)
 1002 FORMAT(2I3, I5, A1)
 1004 FORMAT(I11, A1)
 1005 FORMAT(E10.4E1, 2X, I1)
 1009 FORMAT(E13.7E2)

 9015 FORMAT(' Illegal data file name extension, ', A3, ' given')
 9051 FORMAT(' Too many layers specified in ', A)
 9052 FORMAT(' NumberOfLayers = ', I0, '  MaxAllowed = ', I0)

      END SUBROUTINE ReadParmFile


!-----------------------------------------------------------------------------
      SUBROUTINE ParmToLocalVars(EvapParms, Ext, Comment,                  &
     &                        P1, P2, P3, P4, P5, P6, P7, P8, P9, P10,     &
     &                        StartSeq, EndSeq, SurfaceTemperature,        &
     &                        IceAmount, IceArea, IceDepth, IceTemp,       &
     &                        AgeOfCurrentLayer, NoOfLayers,               &
     &                        SurfTempFromLayer, TotalHeat, AgeOfLayer)

      IMPLICIT  NONE
      TYPE (EvapParmType), INTENT(IN) :: EvapParms
      CHARACTER (LEN=3)  :: Ext
      CHARACTER (LEN=77) :: Comment
      REAL    :: P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, AgeOfCurrentLayer
      REAL    :: SurfaceTemperature, IceAmount, IceArea, IceDepth, IceTemp
      INTEGER :: StartSeq, EndSeq, NoOfLayers
      REAL, DIMENSION(0:MaxNoOfLayers) :: SurfTempFromLayer, TotalHeat
      REAL, DIMENSION(1:MaxNoOfLayers) :: AgeOfLayer

      Ext     = EvapParms%Ext
      Comment = EvapParms%Comment
      P1 = EvapParms%Parms(1)
      P2 = EvapParms%Parms(2)
      P3 = EvapParms%Parms(3)
      P4 = EvapParms%Parms(4)
      P5 = EvapParms%Parms(5)
      P6 = EvapParms%Parms(6)
      P7 = EvapParms%Parms(7)
      P8 = EvapParms%Parms(8)
      P9 = EvapParms%Parms(9)
      P10 = EvapParms%Parms(10)
      StartSeq = EvapParms%SSeq
      EndSeq   = EvapParms%ESeq
      SurfaceTemperature = EvapParms%SurfTemp
      IceAmount = EvapParms%IceAmount
      IceArea   = EvapParms%IceArea
      IceDepth  = EvapParms%IceDepth
      IceTemp   = EvapParms%IceTemp
      AgeOfCurrentLayer = EvapParms%AgeOfCurrentLayer
      NoOfLayers = EvapParms%NoOfLayers
      SurfTempFromLayer(:) = EvapParms%SurfTempFromLayer(:)
      TotalHeat(:)  = EvapParms%TotalHeat(:)
      AgeOfLayer(:) = EvapParms%AgeOfLayer(:)

      END SUBROUTINE ParmToLocalVars

!-----------------------------------------------------------------------------
      SUBROUTINE LocalVarsToParm(EvapParms, Ext, Comment,                  &
     &                        P1, P2, P3, P4, P5, P6, P7, P8, P9, P10,     &
     &                        StartSeq, EndSeq, SurfaceTemperature,        &
     &                        IceAmount, IceArea, IceDepth, IceTemp,       &
     &                        AgeOfCurrentLayer, NoOfLayers,               &
     &                        SurfTempFromLayer, TotalHeat, AgeOfLayer)

      IMPLICIT  NONE
      TYPE (EvapParmType), INTENT(OUT) :: EvapParms
      CHARACTER (LEN=3)  :: Ext
      CHARACTER (LEN=77) :: Comment
      REAL    :: P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, AgeOfCurrentLayer
      REAL    :: SurfaceTemperature, IceAmount, IceArea, IceDepth, IceTemp
      INTEGER :: StartSeq, EndSeq, NoOfLayers
      REAL, DIMENSION(0:MaxNoOfLayers) :: SurfTempFromLayer, TotalHeat
      REAL, DIMENSION(1:MaxNoOfLayers) :: AgeOfLayer

      EvapParms%Ext     = Ext
      EvapParms%Comment = Comment
      EvapParms%Parms(1)  = P1
      EvapParms%Parms(2)  = P2
      EvapParms%Parms(3)  = P3
      EvapParms%Parms(4)  = P4
      EvapParms%Parms(5)  = P5
      EvapParms%Parms(6)  = P6
      EvapParms%Parms(7)  = P7
      EvapParms%Parms(8)  = P8
      EvapParms%Parms(9)  = P9
      EvapParms%Parms(10) = P10
      EvapParms%SSeq      = StartSeq
      EvapParms%ESeq      = EndSeq
      EvapParms%SurfTemp  = SurfaceTemperature
      EvapParms%IceAmount = IceAmount
      EvapParms%IceArea   = IceArea
      EvapParms%IceDepth  = IceDepth
      EvapParms%IceTemp   = IceTemp
      EvapParms%AgeOfCurrentLayer = AgeOfCurrentLayer
      EvapParms%NoOfLayers = NoOfLayers
      EvapParms%SurfTempFromLayer(:) = SurfTempFromLayer(:)
      EvapParms%TotalHeat(:)  = TotalHeat(:)
      EvapParms%AgeOfLayer(:) = AgeOfLayer(:)

      END SUBROUTINE LocalVarsToParm

!-----------------------------------------------------------------------------
!     Modified by TSH/TEC 10sep90 to keep NoOfLayers >= 0
!     Mixing corrected by TEC 02nov90 and truncation of aging function omitted.
!     Computation of ice pack added by TEC 01jul91, 22aug91, 10dec91.
!     Updated for 4-byte dates and move to UNIX by TSH & TEC 14oct92.
!     Simplified to eliminate unused parameters by TEC 15jun94.
!
!     Converted to Fortran 90 syntax and basin NHU removed and
!         renamed to HUR by TSH 03aug99.
!
!     Modified by TSH on 08Feb2013 to reinstate a HGB basin (Huron + Georgian combined).
!     It is considered lake basin 13 on input (input argument D=13), but I will
!     assign it to 8 for the duration of this routine in order to keep the arrays
!     relatively compact.
!
      SUBROUTINE OverWaterCorrection                                      &
     &   (WindSpeedOverLand, AirTempOverLand, DewPtTempOverLand,          &
     &    WaterTemp, WindRatio, AirTempCorrection, DewPtTempCorrection, D)

      IMPLICIT NONE

      INTEGER  ::  D, L, Stability
      INTEGER, DIMENSION(7), PARAMETER :: StabilityIndex= (/ 1, 2, 2, 3, 4, 4, 5 /)

      REAL  :: WindSpeedOverLand, AirTempOverLand, DewPtTempOverLand
      REAL  :: WaterTemp, WindRatio, AirTempCorrection, DewPtTempCorrection
      REAL  :: WindSpeedOverWater, AirTempOverWater, DewPtTempOverWater
      REAL  :: Ta, Tw

      REAL, DIMENSION(5,3,8) :: C

      REAL, DIMENSION(5,3), PARAMETER :: CSup = RESHAPE (                  &
     &             (/ 3.132,  2.795,  1.607,  2.740,  3.374,               &
     &               -1.333, -0.321,  0.290,  1.485,  1.822,               &
     &               -4.499,  0.484, -0.350, -0.160, -0.037 /) , (/5,3/) )


      REAL, DIMENSION(5,3), PARAMETER :: CMic = RESHAPE (                  &
     &             (/ 3.132,  2.795,  1.607,  2.740,  3.374,               &
     &               -1.333, -0.321,  0.290,  1.485,  1.822,               &
     &               -4.499,  0.484, -0.350, -0.160, -0.037 /) , (/5,3/) )


      REAL, DIMENSION(5,3), PARAMETER :: CHur = RESHAPE (                  &
     &             (/ 3.132,  2.795,  1.607,  2.740,  3.374,               &
     &               -1.333, -0.321,  0.290,  1.485,  1.822,               &
     &               -4.499,  0.484, -0.350, -0.160, -0.037 /) , (/5,3/) )

      REAL, DIMENSION(5,3), PARAMETER :: CGeo = RESHAPE (                  &
     &             (/ 3.132,  2.795,  1.607,  2.740,  3.374,               &
     &               -1.333, -0.321,  0.290,  1.485,  1.822,               &
     &               -4.499,  0.484, -0.350, -0.160, -0.037 /) , (/5,3/) )

      REAL, DIMENSION(5,3), PARAMETER :: CStc = RESHAPE (                  &
     &             (/ 2.640,  2.350,  1.141,  2.687,  3.317,               &
     &               -2.034, -0.696,  0.290,  2.110,  2.945,               &
     &               -5.115,  0.240, -0.350, -0.160,  0.790 /) , (/5,3/) )

      REAL, DIMENSION(5,3), PARAMETER :: CEri = RESHAPE (                  &
     &             (/ 3.132,  2.795,  1.607,  2.740,  3.374,               &
     &               -1.333, -0.321,  0.290,  1.485,  1.822,               &
     &               -4.499,  0.484, -0.350, -0.160, -0.037 /) , (/5,3/) )

      REAL, DIMENSION(5,3), PARAMETER :: COnt = RESHAPE (                  &
     &             (/ 3.132,  2.795,  1.607,  2.740,  3.374,               &
     &               -1.333, -0.321,  0.290,  1.485,  1.822,               &
     &               -4.499,  0.484, -0.350, -0.160, -0.037 /) , (/5,3/) )

      REAL, DIMENSION(5,3), PARAMETER :: CHGb = RESHAPE (                  &
     &             (/ 3.132,  2.795,  1.607,  2.740,  3.374,               &
     &               -1.333, -0.321,  0.290,  1.485,  1.822,               &
     &               -4.499,  0.484, -0.350, -0.160, -0.037 /) , (/5,3/) )



!
!  Since I replaced the EQUIVALENCE with regular (i.e. executable) assignments
!  this (inline/internal?) function had to be moved up above the assignments
!  in order to be compiled.
!
      Stability(Ta, Tw) = StabilityIndex(MIN(MAX(INT((Ta-Tw)/3.5),-3),3)+4)

!
!  Replace the Equivalence statement with Fortran 90 array section assignments.
!  Lahey's LF95 didn't like this Equivalence statement, presumably because
!  the Cxxx arrays were declared as PARAMETERs along with the fact that F90 does
!  not have the same sequential/contiguous storage assumptions the F77 had.
!    TSH, 06aug99
!
!      EQUIVALENCE (C(1,1,1), CSup), (C(1,1,2), CMic), (C(1,1,3), CHur),     &
!     &            (C(1,1,4), CGeo), (C(1,1,5), CStC), (C(1,1,6), CEri),     &
!     &            (C(1,1,7), COnt)
!
      C(:,:,1) = CSup
      C(:,:,2) = CMic
      C(:,:,3) = CHur
      C(:,:,4) = CGeo
      C(:,:,5) = CStc
      C(:,:,6) = CEri
      C(:,:,7) = COnt
      C(:,:,8) = CHGb

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'OverWaterCorrection'; CALL PassMsg
      END IF
      
!
!    Handle that special case of the combined Huron/GeorgianBay
!
      L = D
      if (D .EQ. 13) L = 8

!
!    Derived from Phillips and Irbe's IFYGL study on Lake Ontario, but
!    with the fetch (and derived quantities) replaced with averages
!    over their data set for each stability class in their equations.
!    (Fetch of 10.5 nautical miles assumed for St. Clair.)
!
!    19 January 1988, TEC II
!

      SELECT CASE (Stability(AirTempOverLand, WaterTemp))

         CASE (1)
           WindSpeedOverWater = C(1,1,L) + 1.05 * WindSpeedOverLand
           AirTempOverWater   = C(1,2,L) + 0.60 * AirTempOverLand         &
     &                                   + 0.54 * WaterTemp
           DewPtTempOverWater = C(1,3,L) + 0.56 * DewPtTempOverLand       &
     &                                   + 0.46 * WaterTemp

         CASE (2)
           WindSpeedOverWater = C(2,1,L) + 1.01 * WindSpeedOverLand
           AirTempOverWater   = C(2,2,L) + 0.67 * AirTempOverLand         &
     &                                   + 0.42 * WaterTemp
           DewPtTempOverWater = C(2,3,L) + 0.94 * DewPtTempOverLand       &
     &                                   + 0.11 * WaterTemp

         CASE (3)
           WindSpeedOverWater = C(3,1,L) + 0.92 * WindSpeedOverLand       &
     &                             - 0.28 * (AirTempOverLand-WaterTemp)
           AirTempOverWater   = C(3,2,L) + 0.47 * AirTempOverLand         &
     &                                   + 0.52 * WaterTemp
           DewPtTempOverWater = C(3,3,L) + 0.72 * DewPtTempOverLand       &
     &                                   + 0.31 * WaterTemp

         CASE (4)
           WindSpeedOverWater = C(4,1,L) + 0.49 * WindSpeedOverLand       &
     &                                   - 0.02 * AirTempOverLand
           AirTempOverWater   = C(4,2,L) + 0.29 * AirTempOverLand         &
     &                                   + 0.65 * WaterTemp
           DewPtTempOverWater = C(4,3,L) + 0.44 * DewPtTempOverLand       &
     &                                   + 0.55 * WaterTemp

         CASE (5)
           WindSpeedOverWater = C(5,1,L) + 0.32 * WindSpeedOverLand       &
     &                                   - 0.02 * AirTempOverLand
           AirTempOverWater   = C(5,2,L) + 0.30 * AirTempOverLand         &
     &                                   + 0.56 * WaterTemp
           DewPtTempOverWater = C(5,3,L) + 0.43 * DewPtTempOverLand       &
     &                                   + 0.53 * WaterTemp

      END SELECT

      WindRatio = WindSpeedOverWater / WindSpeedOverLand
      IF (WindRatio .LT. 0.01) WindRatio = 0.01
      AirTempCorrection = AirTempOverLand - AirTempOverWater
      DewPtTempCorrection = DewPtTempOverLand - DewPtTempOverWater
      
      IF (PrintDebugInfo) THEN
         ErrorMessage = 'OverWaterCorrection done'; CALL PassMsg
      END IF
      
      RETURN

      END SUBROUTINE OverWaterCorrection

!-----------------------------------------------------------------------------
!
!     Coefficient - BULK AERODYNAMIC COEFFICIENT FOR HEAT
!     Z0 - ROUGHNESS HEIGHT IN METERS
!     Al - STABILITY LENGTH IN METERS
!  FROM
!     U - WIND SPEED IN METERS/SEC
!     Z - REFERENCE HEIGHT IN METERS
!     Td - AIR-WATER TEMPERATURE DIFFERENCE IN DEGREES K
!     Ak - VON KARMANS COEFFICIENT# 0.41
!     Alph - FROM CHARNOCKS RELATIONSHIP# 0.0101
!
!-----------------------------------------------------------------------------
      REAL FUNCTION Coefficient (U, Td)

      IMPLICIT NONE

      REAL, INTENT(IN) :: U, Td
      REAL             :: U_tmp, Td_tmp

!
!  These next few variables are initialized, similar to using a DATA statement.
!  That is, they are assigned the first time, but after that they simply retain
!  any assigned values if changed.  That's the defined behavior in F77 and that's
!  how these were declared in the original source code.  I've simply made that
!  similar behavior in this F90 version of the code.
!
      REAL  :: A1 = 16.00
      REAL  :: A2 = 16.00
      REAL  :: A3 =  5.20
      REAL  :: A4 =  5.20
      REAL  :: Ak =  0.41
      REAL  :: B  =  1.00
      REAL  :: G  =  9.81
      REAL  :: Z  =  8.00
      REAL  :: Alph = 0.0101
      REAL  :: Tbar = 276.5

      REAL  :: Ustar, Z0, S, Al, X, X2, Sy2, Sy1, TeeDee
      INTEGER*4 K

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'Coefficient'; CALL PassMsg
      END IF
      
      Ustar = 0.1
!
!     Initial guess for Z0.
!
      Z0 = Alph * Ustar * Ustar / G
      TeeDee = Td
      IF (ABS(Td) .LT. 1.0E-7) TeeDee = SIGN(1.0E-7, Td)
      S = U * U * Tbar / (G * TeeDee)
!
!     Initial guess for L.
!
      Al = S / LOG(Z / Z0)
      IF (ABS(Al) .GT. 3.E6) Al = SIGN(3.E6, Al)
      DO K = 1, 10
        IF (Al .LE. 0.) THEN
!
!         Unstable section.
!
          X = (1.0 - A1 * Z / Al)**0.25
          X2 = (1.0 - A2 * Z / Al)**0.25
          Sy2 = 2. * LOG((1. + X2 * X2) / 2.)
          Sy1 = 2. * LOG((1. + X) / 2.) + LOG((1. + X * X) / 2.)          &
     &         - 2. * ATAN(X) + 1.570796
        ELSE
!
!         Stable section.
!
          Sy1 = -A3 * Z / Al
          Sy2 = -A4 * Z / Al
          IF (Z / Al .GT. 1.0) THEN
!
!           Strongly stable section.
!
            Sy1 = -A3 * (1.0 + LOG(Z / Al))
            Sy2 = -A4 * (1.0 + LOG(Z / Al))
          END IF
        END IF
!
!       Calculate Ustar, Z0, Coefficient, and Al.
!
        Ustar = U * Ak / (LOG(Z / Z0) - Sy1)
        Z0 = Alph * Ustar * Ustar / G
        Coefficient = Ak * Ustar / (U * (LOG(Z / Z0) - Sy2) * B)
        Al = S * (LOG(Z / Z0) - Sy2) / ((LOG(Z / Z0) - Sy1)**2) * B
      END DO
      IF (PrintDebugInfo) THEN
         ErrorMessage = 'Coefficient done'; CALL PassMsg
      END IF
      RETURN

      END FUNCTION Coefficient


!-----------------------------------------------------------------------------
      SUBROUTINE BalanceElevAndTempAndEvap(                               &
     & rCV, Mf1, Mf2, Mxf, Ms1, Ms2, Mxs, Hd,                             &   ! param.
     & P1, P2, P3, P4, P5, P6, P7, P8, P9, P10,                           &   ! param.
     & DataIndex, Area,                                                   &   ! cons.
     & Day, Month, Year,                                                  &   ! inputs
     & CloudCover, WindSpeed, AirTemperature, DewPtTemperature,           &   ! in.out.
     & Runoff, Precipitation, Inflow, Outflow,                            &   ! inputs
     & OtherTerms,                                                        &   ! input
     & IceAmount, IceTemp, IceArea, IceDepth, StoredHeat,                 &   ! in.out.
     & AgeOfCurrentLayer, SurfaceElevation, SurfaceTemperature,           &   ! in.out.
     & Evaporation, Count, Incident1, Reflect,                            &   ! outputs
     & NLWave, Latent, Sensible, Advection,                               &   ! outputs
     & Calibrating)                                                           ! input
!
!     By Thomas E. Croley II, using material developed by Jan Derecki, 6/87.
!
!     Determine water surface elevation and temperature at the end of the day
!     and evaporation amount during the day by balancing the heat budget, the
!     aerodynamic law (relationship of evaporation to water surface
!     temperature), the mass budget, average surface temperature during the
!     day, and average surface elevation during the day.
!
!     CONSTANTS:
!     DataIndex = index denoting location of lake, required by some functions
!     Area = Average area of the lake surface in square meters
!
!     INPUTS:
!     Day, Month, Year = calendar day, month, & year
!     CloudCover = average daily fraction of sky covered in clouds
!     WindSpeed = average daily windspeed in meters per second
!     AirTemperature = temperature of the air in degrees Celcius
!     DewPtTemperature = dew point temperature of the air in degrees Celcius
!     ThisAirTemp = this month's average air temperature in degrees Celcius
!     LastAirTemp = last month's average air temperature in degrees Celcius
!     Runoff = overland flow into the lake for the day in cubic meters
!     Precipitation = overlake precipitation for the day in cubic meters
!     Inflow = sum of other inflows and diversions into the lake for the day
!      in cubic meters
!     Outflow = sum of outflows from the lake for the day in cubic meters
!     OtherTerms = heat flux terms not otherwise considered in calories
!
!     SurfaceElevation = elevation of the lake surface at BEGINNING OF DAY in
!      meters
!     SurfaceTemperature = water surface temperature at BEGINNING OF DAY in
!      degrees Celcius
!
!     OUTPUTS:
!     SurfaceElevation = elevation of the lake surface at END OF DAY in meters
!     SurfaceTemperature = water surface temperature at END OF DAY in degrees
!      Celcius
!     IceAmount = Ice on lake at any given time.
!     Evaporation = Evaporation from lake for day in cubic meters
!     AirTemperature, DewPtTemperature, and WindSpeed are returned
!      corrected for overlake conditions; that is, overlake estimates
!      are returned.  -DISABLED FOR CALIBRATION PROGRAM
!
      IMPLICIT NONE

      REAL :: CloudCover, Hd, P1, P2, P3, P4, P5, P6, P7, P8, P9, P10
      REAL :: AirTemperature, DewPtTemperature, Runoff, Precipitation, Inflow
      REAL :: Outflow, OtherTerms, SurfaceElevation, SurfaceTemperature
      REAL :: Evaporation, WindSpeed, IceAmount, StoredHeat, Incident1, Latent
      REAL :: Sensible, EvapAdvect, Advection, NLWave, Reflect
      REAL :: AgeOfCurrentLayer, rCV, Mf1, Mf2, Ms1, Ms2, Mxf, Mxs

      INTEGER :: DataIndex, Day, Month, Year, Count

      REAL :: VaporPressureOverWater, Temperature, VaporPressureOverIce
      REAL :: EndSurfaceTemperature, Area, TrialValue, AvgSurfaceTemperature
      REAL :: WindRatio, AirTempAdjustment, DewPtAdjustment
      REAL :: VaporPressureOfAir, VaporPressureOnLake
      REAL :: FractionInIce, EvapOverIce, Incident
      REAL :: FractionInBareIce
      REAL :: FractionInNewSnow, FractionInOldSnow, FractionInMeltingSnow
      REAL :: FractionInOpenWater, StoredHeatAtEndOfDay, AtmosphereIceFlux
      REAL :: VaporPressureOnIce, VaporPressureOverIceSurface
      REAL :: TransferCoefficientW
      REAL :: TransferCoefficientI, OverLakeTemp, IceTemp, TIceTemp, AIceTemp
      REAL :: WindRatioI, AirTempAdjustmentI, DewPtAdjustmentI, OverLakeWind
      REAL :: TrialHeat, OverIceTemp, OverIceWind, IceArea
      REAL :: IceDepth, WaterIceFlux, TArea, TDepth, TAmount, ExcessFromIceMelt
      REAL :: DeltaStoredHeat
      
      LOGICAL Calibrating

      VaporPressureOverWater(Temperature) =                               &
     & 10.0**(-7.90298 * ((373.16 / (Temperature + 273.16)) - 1.0)        &
     & + 5.02808 * LOG10((373.16 / (Temperature + 273.16))) - 1.3816      &
     & * 0.0000001 * (10.0**(11.334 * (1.0 - (1.0 / (373.16               &
     & / (Temperature + 273.16))))) - 1.0) + 8.1328 * 0.001               &
     & * (10.0**(-3.49149 * ((373.16 / (Temperature + 273.16)) - 1.0))    &
     & - 1.0) + 3.0057149)

      VaporPressureOverIce(Temperature) =                                 &
     & 10.0**(-9.09718 * (273.16 / (Temperature + 273.16) - 1.0)          &
     & -3.56654 * LOG10(273.16 / (Temperature + 273.16)) + 0.876793       &
     & * (1.0 - 1.0 / (273.16 / (Temperature + 273.16))) + 0.785835)

!
!     as initial guesses:
!
      IF (PrintDebugInfo) THEN
         ErrorMessage = 'BalanceElevAndTemp'; CALL PassMsg
      END IF
      
      EndSurfaceTemperature = SurfaceTemperature
      StoredHeatAtEndOfDay = StoredHeat
      AvgSurfaceTemperature = SurfaceTemperature
      TArea = IceArea
      TAmount = IceAmount
      TIceTemp = IceTemp
      TrialHeat = -100. * (StoredHeatAtEndOfDay - TAmount*79.7*1000000.)
      Count = 0
!
      DO WHILE (ABS(((StoredHeatAtEndOfDay - TAmount * 79.7 * 1000000)    &
     &         - TrialHeat) / (StoredHeatAtEndOfDay - TAmount * 79.7      &
     &         * 1000000)) .GT. 0.001)
       TrialHeat = StoredHeatAtEndOfDay - TAmount * 79.7 * 1000000
       TrialValue = EndSurfaceTemperature
       CALL OverWaterCorrection(WindSpeed, AirTemperature,                &
     &      DewPtTemperature, AvgSurfaceTemperature, WindRatio,           &
     &      AirTempAdjustment, DewPtAdjustment, DataIndex)
       IF (ErrorLevel .NE. 0) RETURN
       OverLakeWind = WindSpeed * WindRatio
       OverLakeTemp = AirTemperature - AirTempAdjustment
       VaporPressureOfAir = VaporPressureOverWater(DewPtTemperature - DewPtAdjustment)
       VaporPressureOnLake = VaporPressureOverWater(AvgSurfaceTemperature)
       TransferCoefficientW = Coefficient(OverLakeWind,                   &
     &                        OverLakeTemp - AvgSurfaceTemperature)
       IF (ErrorLevel .NE. 0) RETURN
       Evaporation = EvapUnitsCoeff * TransferCoefficientW                &
     &               * (VaporPressureOnLake - VaporPressureOfAir)         &
     &               * OverLakeWind
       OverIceTemp = OverLakeTemp
       OverIceWind = OverIceTemp - 1.                                  ! dummy use

       DO WHILE (ABS(OverIceTemp - OverIceWind) .GT. 0.01)
        CALL OverWaterCorrection(WindSpeed, AirTemperature,               &
     &       DewPtTemperature, MIN(OverIceTemp, 0.), WindRatioI,          &
     &       AirTempAdjustmentI, DewPtAdjustmentI, DataIndex)
        IF (ErrorLevel .NE. 0) RETURN
        OverIceWind = OverIceTemp
        OverIceTemp = AirTemperature - AirTempAdjustmentI
       END DO
       TIceTemp = MIN(OverIceTemp, 0.)
       AIceTemp = (IceTemp + TIceTemp) / 2.
       OverIceWind = WindSpeed * WindRatioI
       VaporPressureOverIceSurface = VaporPressureOverWater(              &
     &                  DewPtTemperature - DewPtAdjustmentI)
       VaporPressureOnIce = VaporPressureOverIce(AIceTemp)
       TransferCoefficientI = Coefficient(OverIceWind, OverIceTemp-AIceTemp)
       IF (ErrorLevel .NE. 0) RETURN
       EvapOverIce = EvapUnitsCoeff * TransferCoefficientI                &
     &               * (VaporPressureOnIce - VaporPressureOverIceSurface) &
     &               * OverIceWind
       Incident = DailyIncidentSolarRadiation(Day,Month,Year, CloudCover, DataIndex)
       IF (ErrorLevel .NE. 0) RETURN
       FractionInBareIce = 1.0
       FractionInNewSnow = 0.
       FractionInOldSnow = 0.
       FractionInMeltingSnow = 0.
       FractionInIce = (IceArea + TArea) / 2. / Area
       FractionInOpenWater = 1.0 - FractionInIce
!
       DeltaStoredHeat = (Incident                                        &
     &  - Incident * 0.10) * (Area - (IceArea + TArea) / 2.) * 10000.     &
     &  + DailyNetLongwaveRadiation(AvgSurfaceTemperature,                &
     &   OverLakeTemp, CloudCover, VaporPressureOfAir, P9) * Area*10000.  &
     &  - (597.3 - 0.56 * AvgSurfaceTemperature                           &
     &   + AvgSurfaceTemperature * 1.0) * Evaporation / 10.               &
     &   * (Area - (IceArea + TArea) / 2.) * 10000. * 1.0                 &
     &  - HeatUnitsCoeff * TransferCoefficientW                           &
     &   * (AvgSurfaceTemperature - OverLakeTemp) * OverLakeWind          &
     &   * (Area - (IceArea + TArea) / 2.) * 10000.                       &
     &  + (Runoff + Inflow) * 1000000.                                    &
     &   * 1.0 * MAX(0., AvgSurfaceTemperature) * 1.0                     &
     &  + Precipitation * 1000000. * 1.0                                  &
     &   * (OverLakeTemp * 1.0 + MIN(SIGN(79.7, OverLakeTemp),0.))        &
     &   * FractionInOpenWater                                            &
     &  - Outflow * 1000000. * 1.0 * AvgSurfaceTemperature * 1.0          &
     &  + OtherTerms
       IF (ErrorLevel .NE. 0) RETURN

       StoredHeatAtEndOfDay = StoredHeat + DeltaStoredHeat
       !
       ! Somehow, StoredHeatAtEndofDay was going negative for StClair and
       ! crashing the program due to invalid floating point op.
       ! This is a kludge fix. Needs to be done better, but I just need 
       ! to get things working again. All I am trying to accomplish
       ! here is to keep the StoredHeatAtEndofDay "reasonable" when 
       ! things go bad. My assumption is that there was a fairly large
       ! heat loss calculated in order for this to trigger. I am gonna
       ! just call it a 3% loss of heat, which would seem to be fairly 
       ! significant to me.
       ! Tim Hunter  03Dec2013
       !
       IF (StoredHeatAtEndOfDay .LT. 0) THEN
           StoredHeatAtEndOfDay = StoredHeat * 0.97
       END IF

       AtmosphereIceFlux =                                                &
     &  (Incident                                                         &
     &  - DailyReflectedRadiationFromIce(Incident,                        &
     &   FractionInBareIce, FractionInNewSnow, FractionInOldSnow,         &
     &   FractionInMeltingSnow)) * (IceArea + TArea) / 2. * 10000.        &
     &  - (597.3 - 0.56 * AIceTemp + 79.7 + AIceTemp * 1.0)               &
     &   * EvapOverIce / 10. * (IceArea + TArea) / 2. * 10000. * 1.       &
     &  - HeatUnitsCoeff * TransferCoefficientI                           &
     &   * (AIceTemp - OverIceTemp) * OverIceWind * (IceArea + TArea)     &
     &   / 2. * 10000.                                                    &
     &  + Precipitation * 1000000. * 1.0                                  &
     &   * OverIceTemp * 1.0 * FractionInIce
       IF (ErrorLevel .NE. 0) RETURN

       CALL OverWaterCorrection(WindSpeed, AirTemperature,                &
     &      DewPtTemperature, AvgSurfaceTemperature, WindRatio,           &
     &      AirTempAdjustment, DewPtAdjustment, DataIndex)
       IF (ErrorLevel .NE. 0) RETURN
       OverLakeWind = WindSpeed * WindRatio
       AgeOfCurrentLayer = OverLakeWind * FractionInOpenWater
!
!      If negative, WaterIceFlux is the heat taken, by freezing the water,
!      to keep water temperatures from going below freezing, in calories.
!
!      If positive, WaterIceFlux is the heat available to melt ice by lowering
!      the water temperature toward freezing temperature, in calories.
!
       WaterIceFlux = StoredHeatAtEndOfDay - HeatInStorage(0.0,           &
     &                AgeOfCurrentLayer, Hd, Mf1, Mf2, Ms1, Ms2, rCV,     &
     &                P2, P3, P4, P5, P6, P7, P8)
       IF (ErrorLevel .NE. 0) RETURN
       
       StoredHeatAtEndOfDay = StoredHeatAtEndOfDay - WaterIceFlux
       CALL IceCover(IceArea, IceDepth, IceAmount, IceTemp,                 &
     &      WaterIceFlux, AtmosphereIceFlux, P1, P10, TArea, TDepth,        &
     &      TAmount, TIceTemp, ExcessFromIceMelt, Area,                     &
     &      EvapOverIce / 10 * (IceArea + TArea) / 2. * 10000. / 1000000.,  &
     &      Precipitation * MIN(SIGN(1., OverIceTemp), 0.) * FractionInIce)
       IF (ErrorLevel .NE. 0) RETURN
       StoredHeatAtEndOfDay = StoredHeatAtEndOfDay + ExcessFromIceMelt

       EndSurfaceTemperature = WaterSurfaceTemperature(                   &
     &               StoredHeatAtEndOfDay, AgeOfCurrentLayer, Hd, Mf1,    &
     &               Mf2, Ms1, Ms2, rCV, P2, P3, P4, P5, P6, P7, P8)
       Count = Count + 1
       AvgSurfaceTemperature = (SurfaceTemperature + EndSurfaceTemperature) / 2.
       IF (Count .EQ. 6) THEN
        EndSurfaceTemperature = (EndSurfaceTemperature + TrialValue) / 2.0
        StoredHeatAtEndOfDay = (StoredHeatAtEndOfDay + TrialHeat          &
     &                         + TAmount * 79.7 * 1000000) / 2.0
       END IF
       IF (Count .GT. 6) TrialHeat = StoredHeatAtEndOfDay  - TAmount * 79.7 * 1000000
      END DO

      IceArea = TArea
      IceDepth = TDepth
      IceTemp = TIceTemp
      IceAmount = TAmount
      IF (.NOT. Calibrating) THEN
!
!      Define heat flux variables (2.06346 cal/cm2/day = 1 watt/m2)
!
       Incident1 = Incident / 2.06346
       Reflect = -(Incident * 0.10 * FractionInOpenWater                  &
     &    + DailyReflectedRadiationFromIce(Incident,                      &
     &    FractionInBareIce, FractionInNewSnow, FractionInOldSnow,        &
     &    FractionInMeltingSnow) * FractionInIce) / 2.06346
       IF (ErrorLevel .NE. 0) RETURN
       NLWave = DailyNetLongwaveRadiation(AvgSurfaceTemperature,          &
     &    OverLakeTemp, CloudCover, VaporPressureOfAir, P9) / 2.06346
       IF (ErrorLevel .NE. 0) RETURN
       Latent = -((597.3 - 0.56 * AvgSurfaceTemperature)                  &
     &    * Evaporation / 10. * 1.0 * FractionInOpenWater                 &
     &    + (597.3 - 0.56 * AIceTemp + 79.7)                              &
     &    * EvapOverIce / 10. * 1.0 * FractionInIce) / 2.06346
       Sensible = -HeatUnitsCoeff * (TransferCoefficientW                 &
     &    * (AvgSurfaceTemperature - OverLakeTemp) * OverLakeWind         &
     &    * FractionInOpenWater + TransferCoefficientI                    &
     &    * (AIceTemp - OverIceTemp) * OverIceWind * FractionInIce)       &
     &    / 2.06346
       IF (ErrorLevel .NE. 0) RETURN
       EvapAdvect = -(AvgSurfaceTemperature * 1. * Evaporation / 10.      &
     &    * FractionInOpenWater + AIceTemp * 1.0 * EvapOverIce / 10.      &
     &    * FractionInIce) / 2.06346
       Advection = (((Runoff + Inflow) * MAX(0.,AvgSurfaceTemperature)    &
     &    * 1.0 - Outflow * AvgSurfaceTemperature * 1.0                   &
     &    + Precipitation * (OverLakeTemp * 1.0                           &
     &    + MIN(SIGN(79.7, OverLakeTemp), 0.)) * FractionInOpenWater)     &
     &    + Precipitation * OverIceTemp * 1.0 * FractionInIce)            &
     &    * 1000000. * 1.0 / Area / 10000. / 2.06346 + EvapAdvect
      END IF
!
      Evaporation = (Evaporation * FractionInOpenWater                    &
     &              + EvapOverIce * FractionInIce) / 1000. * Area
      SurfaceElevation = SurfaceElevation                                 &
     &      + (Precipitation + Runoff + Inflow - Outflow - Evaporation)   &
     &      / Area
      SurfaceTemperature = EndSurfaceTemperature
      StoredHeat = StoredHeatAtEndOfDay

!
!     Superimpose and age heat or heat deficiency layers.
!
      CALL Superposition(Hd, Mxf, Mxs, StoredHeat, AgeOfCurrentLayer)
      IF (ErrorLevel .NE. 0) RETURN
      IF (.NOT. Calibrating) THEN
        AirTemperature = (AirTemperature - AirTempAdjustment)             &
     &      * FractionInOpenWater                                         &
     &      + (AirTemperature - AirTempAdjustmentI) * FractionInIce
        DewPtTemperature = (DewPtTemperature - DewPtAdjustment)           &
     &      * FractionInOpenWater                                         &
     &      + (DewPtTemperature - DewPtAdjustmentI) * FractionInIce
        WindSpeed = WindSpeed * WindRatio * FractionInOpenWater           &
     &      + WindSpeed * WindRatioI * FractionInIce
      END IF

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'BalanceElevAndTemp done'; CALL PassMsg
      END IF

      RETURN

      END SUBROUTINE BalanceElevAndTempAndEvap

!----------------------------------------------------------------------------
      SUBROUTINE Superposition (Hd, Mxf, Mxs, StoredHeat, W)
      IMPLICIT NONE
      REAL :: Hd, Mxf, Mxs, StoredHeat, W

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'SuperPosition'; CALL PassMsg
      END IF
      
      IF (TotalHeat(NoOfLayers) .LE. Hd) THEN

!
!      Yesterday, lake was pre-spring-turnover...
!
       IF (StoredHeat / 1.E+20 .LE. Hd) THEN

!
!       Today, lake is still pre-spring-turnover...
!
!       Remove heat (deficiency) superposition layers if required...
!
        DO WHILE ((StoredHeat / 1.E+20 .GE. TotalHeat(NoOfLayers)) .AND.    &
     &            (NoOfLayers .GT. 0))
          NoOfLayers = NoOfLayers - 1
        END DO

!
!       Superimpose todays heat deficiency layer, "age" all layers, and
!       collapse earlier additions that are older than mixing function limit.
!
        CALL AddAgeAndCollapse(StoredHeat, W, Mxf)
        IF (ErrorLevel .NE. 0) RETURN

       ELSE

!
!        Today, lake is now just post-spring-turnover
!        (and pre-fall-turnover)...
!
         NoOfLayers = 1
         TotalHeat(1) = StoredHeat / 1.0E+20
         AgeOfLayer(1) = W
       END IF

      ELSE

!
!      Yesterday, lake was pre-fall-turnover...
!
       IF (StoredHeat / 1.E+20 .GT. Hd) THEN

!
!       Today, lake is still pre-fall-turnover...
!
!       Remove heat superposition layers if required...
!
        DO WHILE ((StoredHeat / 1.E+20 .LE. TotalHeat(NoOfLayers)) .AND.  &
     &            (NoOfLayers .GT. 0))
          NoOfLayers = NoOfLayers - 1
        END DO
!
!       Superimpose todays heat addition layer, "age" all layers, and
!       collapse earlier additions that are older than mixing function limit.
!
        CALL AddAgeAndCollapse(StoredHeat, W, Mxs)
        IF (ErrorLevel .NE. 0) RETURN
       ELSE
!
!       Today, lake is now just post-fall-turnover
!       (and pre-spring-turnover)...
!
        NoOfLayers = 1
        TotalHeat(1) = StoredHeat / 1.0E+20
        AgeOfLayer(1) = W
       END IF
      END IF

      AgeOfLayer(NoOfLayers + 1) = 0.
      TotalHeat(NoOfLayers + 1) = 0.

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'SuperPosition done'; CALL PassMsg
      END IF

      RETURN

      END SUBROUTINE Superposition

!----------------------------------------------------------------------------
      SUBROUTINE AddAgeAndCollapse (StoredHeat, W, AgeLimit)

      INTEGER :: LayerNo
      REAL    :: StoredHeat, W, AgeLimit

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'AddAgeCollapse'; CALL PassMsg
      END IF
      
!
!     Superimpose todays heat addition layer.
!
      NoOfLayers = NoOfLayers + 1
      IF (NoOfLayers .GT. MaxNoOfLayers - 1) THEN   ! allow one layer for zero on top!
        ErrorMessage = 'MaxNoOfLayers exceeded.'       ! See end of routine: SUPERIMPOSE.
        ErrorLevel = 1
        RETURN
      END IF
      TotalHeat(NoOfLayers) = StoredHeat / 1.0E+20

!
!     "Age" all layers...
!
      DO LayerNo = 1, NoOfLayers
        AgeOfLayer(LayerNo) = AgeOfLayer(LayerNo) + W
      END DO

!
!     Collapse earlier additions that are older than mixing function limit.
!
      IF (NoOfLayers .GT. 1 .AND. AgeOfLayer(1) .GE. AgeLimit) THEN
        DO WHILE (NoOfLayers .GT. 1 .AND. AgeOfLayer(2) .GE. AgeLimit)
          NoOfLayers = NoOfLayers - 1
          DO LayerNo = 1, NoOfLayers
            TotalHeat(LayerNo)  = TotalHeat(LayerNo + 1)
            AgeOfLayer(LayerNo) = AgeOfLayer(LayerNo + 1)
          END DO
        END DO
      END IF

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'AddAgeCollapse done'; CALL PassMsg
      END IF
      RETURN

      END SUBROUTINE AddAgeAndCollapse

!----------------------------------------------------------------------------
      REAL FUNCTION HeatInStorage (SurfaceTemperature, TodaysWind,        &
     &        Hd, Mf1, Mf2, Ms1, Ms2, rCV, P2, P3, P4, P5, P6, P7, P8)

      IMPLICIT NONE

      INTEGER ::  R
      REAL :: SurfaceTemperature, TodaysWind, Hd, Mf1, Mf2, Ms1, Ms2, rCV
      REAL :: P2, P3, P4, P5, P6, P7, P8, T, N

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'HeatInStorage'; CALL PassMsg
      END IF
      
      IF (TotalHeat(NoOfLayers) .LE. Hd) THEN

!
!      Yesterday, lake was pre-spring-turnover...
!
       IF (SurfaceTemperature .LE. 3.98) THEN

!
!       Today, lake is still pre-spring-turnover...
!
        N = AgeOfLayer(1) + TodaysWind
        T = 3.98 + (TotalHeat(1) - Hd)                                 &
     &           * MixingFunction(rCV, Mf1, Mf2, P2, P3, P4, P8, N)
        IF (ErrorLevel .NE. 0) RETURN
        R = 1
        DO WHILE (R .LE. NoOfLayers .AND. SurfaceTemperature .LT. T)
         R = R + 1
         N = AgeOfLayer(R) + TodaysWind
         T = T + (TotalHeat(R) - TotalHeat(R - 1))                        &
     &         * MixingFunction(rCV, Mf1, Mf2, P2, P3, P4, P8, N)
         IF (ErrorLevel .NE. 0) RETURN
        END DO
        T = T - (TotalHeat(R) - TotalHeat(R - 1))                         &
     &         * MixingFunction(rCV, Mf1, Mf2, P2, P3, P4, P8, N)
        IF (ErrorLevel .NE. 0) RETURN
        HeatInStorage = (TotalHeat(R-1) + (SurfaceTemperature - T)      &
     &      / MixingFunction(rCV, Mf1, Mf2, P2, P3, P4, P8, N)) * 1.0E+20
        IF (ErrorLevel .NE. 0) RETURN

       ELSE

!
!       Today, lake is just now post-spring-turnover...
!
        HeatInStorage = (Hd + (SurfaceTemperature - 3.98)                 &
     &     / MixingFunction(rCV, Ms1, Ms2, P5, P6, P7, P8, TodaysWind))   &
     &     * 1.0e+20
        IF (ErrorLevel .NE. 0) RETURN
       END IF

      ELSE

!
!      Yesterday, lake was pre-fall-turnover...
!
       IF (SurfaceTemperature .GE. 3.98) THEN
!
!       Today, lake is still pre-fall-turnover...
!
        N = AgeOfLayer(1) + TodaysWind
        T = 3.98 + (TotalHeat(1) - Hd)                                 &
     &           * MixingFunction(rCV, Ms1, Ms2, P5, P6, P7, P8, N)
        IF (ErrorLevel .NE. 0) RETURN
        R = 1
        DO WHILE (R .LE. NoOfLayers .AND. SurfaceTemperature .GT. T)
         R = R + 1
         N = AgeOfLayer(R) + TodaysWind
         T = T + (TotalHeat(R) - TotalHeat(R - 1))                  &
     &         * MixingFunction(rCV, Ms1, Ms2, P5, P6, P7, P8, N)
         IF (ErrorLevel .NE. 0) RETURN
        END DO
        T = T - (TotalHeat(R) - TotalHeat(R - 1))                   &
     &        * MixingFunction(rCV, Ms1, Ms2, P5, P6, P7, P8, N)
        IF (ErrorLevel .NE. 0) RETURN
        HeatInStorage = (TotalHeat(R - 1)                              &
     &      + (SurfaceTemperature - T)                                    &
     &      / MixingFunction(rCV, Ms1, Ms2, P5, P6, P7, P8, N)) * 1.0E+20
        IF (ErrorLevel .NE. 0) RETURN

       ELSE

!
!       Today, lake is just now post-fall-turnover...
!
        HeatInStorage = (Hd + (SurfaceTemperature - 3.98)                 &
     &     / MixingFunction(rCV, Mf1, Mf2, P2, P3, P4, P8, TodaysWind))   &
     &     * 1.0E+20
        IF (ErrorLevel .NE. 0) RETURN
       END IF
      END IF

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'HeatInStorage done'; CALL PassMsg
      END IF
      
      RETURN

      END FUNCTION HeatInStorage

!----------------------------------------------------------------------------
      REAL FUNCTION WaterSurfaceTemperature (StoredHeat, TodaysWind,      &
     &       Hd, Mf1, Mf2, Ms1, Ms2, rCV, P2, P3, P4, P5, P6, P7, P8)
!
!     WaterSurfaceTemperature in degrees Celcius
!     StoredHeat in Calories
!
      IMPLICIT NONE
      REAL :: StoredHeat, TodaysWind, Hd, Mf1, Mf2, Ms1, Ms2, rCV
      REAL :: P2, P3, P4, P5, P6, P7, P8

      INTEGER :: R
      REAL    :: T, N

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'WaterSurfaceTemp'; CALL PassMsg
      END IF
      
      IF (TotalHeat(NoOfLayers) .LE. Hd) THEN

!
!      Yesterday, lake was pre-spring-turnover...
!
       IF (StoredHeat / 1.E+20 .LE. Hd) THEN

!
!       Today, lake is still pre-spring-turnover...
!
        T = 3.98
        R = 1
        DO WHILE (R .LE. NoOfLayers .AND. StoredHeat / 1.E+20 .LT. TotalHeat(R))
          N = AgeOfLayer(R) + TodaysWind
          T = T + (TotalHeat(R) - TotalHeat(R - 1))                 &
     &          * MixingFunction(rCV, Mf1, Mf2, P2, P3, P4, P8, N)
          IF (ErrorLevel .NE. 0) RETURN
          R = R + 1
        END DO
        N = AgeOfLayer(R) + TodaysWind
        WaterSurfaceTemperature = T                                       &
     &      + (StoredHeat / 1.E+20 - TotalHeat(R - 1))                 &
     &      * MixingFunction(rCV, Mf1, Mf2, P2, P3, P4, P8, N)
        IF (ErrorLevel .NE. 0) RETURN

       ELSE
!
!       Today, lake is just now post-spring-turnover...
!
        WaterSurfaceTemperature = 3.98                                    &
     &       + (StoredHeat / 1.E+20 - Hd)                                 &
     &       * MixingFunction(rCV, Ms1, Ms2, P5, P6, P7, P8, TodaysWind)
        IF (ErrorLevel .NE. 0) RETURN
       END IF

      ELSE
!
!      Yesterday, lake was pre-fall-turnover...
!
       IF (StoredHeat / 1.E+20 .GE. Hd) THEN
!
!       Today, lake is still pre-fall-turnover...
!
        T = 3.98
        R = 1
        DO WHILE (R .LE. NoOfLayers .AND. StoredHeat / 1.E+20 .GT. TotalHeat(R))
          N = AgeOfLayer(R) + TodaysWind
          T = T + (TotalHeat(R) - TotalHeat(R - 1))                       &
     &          * MixingFunction(rCV, Ms1, Ms2, P5, P6, P7, P8, N)
          IF (ErrorLevel .NE. 0) RETURN
          R = R + 1
        END DO
        N = AgeOfLayer(R) + TodaysWind
        WaterSurfaceTemperature = T                                       &
     &     + (StoredHeat / 1.E+20 - TotalHeat(R - 1))                  &
     &     * MixingFunction(rCV, Ms1, Ms2, P5, P6, P7, P8, N)
        IF (ErrorLevel .NE. 0) RETURN
       ELSE
!
!       Today, lake is just now post-fall-turnover...
!
        WaterSurfaceTemperature = 3.98                                    &
     &      + (StoredHeat / 1.E+20 - Hd)                                  &
     &      * MixingFunction(rCV, Mf1, Mf2, P2, P3, P4, P8, TodaysWind)
        IF (ErrorLevel .NE. 0) RETURN
       END IF
      END IF

      IF (PrintDebugInfo) THEN
         ErrorMessage = 'WaterSurfaceTemp done'; CALL PassMsg
      END IF
      
      RETURN

      END FUNCTION WaterSurfaceTemperature

!------------------------------------------------------------------
      REAL FUNCTION MixingFunction(rCV, M1, M2, A, B, C, D, Age)
      IMPLICIT NONE
      REAL, INTENT(IN) :: rCV, M1, M2, A, B, C, D, Age
      REAL :: M

      M = D / (1.0 + A * EPN(-B * Age))

      IF (M .LT. M1) THEN
        MixingFunction = (2.0 - M / C) / M
      ELSE IF (M .LT. M2) THEN
        MixingFunction = 1.0 / M
      ELSE
        MixingFunction = 1.0 / rCV
      END IF
      RETURN
      END FUNCTION MixingFunction

!----------------------------------------------------------------------------
      REAL FUNCTION DailyIncidentSolarRadiation (Id, Im, Iy, CloudCover, Indeks)
!
!  DailyIncidentSolarRadiation = Daily total solar radiation incident to the
!     water surface in langleys (calories/square cm)
!  Id, Im, Iy = Calendar day, month, and year of the date.
!  CloudCover = average daily fraction of sky covered with clouds
!  Indeks = 1 (Superior), 2 (Michigan), 3 (Huron without Georgian Bay),
!           4 (Georgian Bay), 5 (St. Clair), 6 (Erie), 7 (Ontario),
!           8 (Champlain), or 13 (Huron with Georgian Bay).
!
      IMPLICIT NONE
      INTEGER :: Id, Im, Iy, INDPM(13), II, Indeks, LkNum
      REAL    :: CloudCover,  X, NDYS, NDY

!
!  R      = AVERAGE MID-MONTH CLOUDLESS-DAY INSOLATION in langleys
!               (note that before aug99 this was declared as a REAL array,
!                but the DATA statement contains integers.  This violates
!                good practice [and maybe the Fortran standard], and it
!                was not necessary that R be a REAL, so I've changed the
!                declaration to an INTEGER.   TSH 04aug99]
!
!              Dec,Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec,Jan
      INTEGER, DIMENSION(14,9), PARAMETER :: R = RESHAPE(                 &
     &      (/ 150,190,330,490,640,770,790,770,670,510,360,200,150,190,   &  ! SUP
     &         190,220,340,480,620,700,730,700,610,490,360,240,190,220,   &  ! MIC
     &         150,190,330,490,640,770,790,770,670,510,360,200,150,190,   &  ! HUR
     &         150,190,330,490,640,770,790,770,670,510,360,200,150,190,   &  ! GEO
     &         190,220,335,470,610,710,755,725,635,515,380,250,190,220,   &  ! STC
     &         190,220,335,470,610,710,755,725,635,515,380,250,190,220,   &  ! ERI
     &         180,200,320,460,595,695,740,705,615,495,360,230,180,200,   &  ! ONT
     &         200,225,345,510,665,770,810,765,680,535,375,245,200,225,   &  ! CHA
     &         150,190,330,490,640,770,790,770,670,510,360,200,150,190/), &  ! HGB   
     &         SHAPE(R))

      !
      !  INDPM  = number of days per month cumulative from the 1st of the year, days
      !
      DATA INDPM /0, 31, 59, 90,120,151,181,212,243,273,304,334,365/

      LkNum = Indeks
      IF (Indeks .EQ. 13) LkNum = 9      ! Huron & Georgian Bay combined

      IF (ID .GT. 15) THEN
        II = INDPM(IM + 1) - INDPM(IM)
        IF (II .EQ. 28 .AND. INT((FLOAT(IY) + .5) / 4.) * 4 .EQ. IY) II = 29
        NDYS = FLOAT(II)
        NDY = FLOAT(ID) - 15.
        X = (R(IM+2, LkNum) - R(IM+1, LkNum)) / NDYS * NDY + R(IM+1, LkNum)
      ELSE
        IF (IM .EQ. 1) THEN
          II = 31
        ELSE
          II = INDPM(IM) - INDPM(IM-1)
          IF (II .EQ. 28 .AND. INT((FLOAT(IY) + .5) / 4.) * 4 .EQ. IY) II = 29
        END IF
        NDYS = FLOAT(II)
        NDY = NDYS - 15. + FLOAT(ID)
        X = (R(IM+1,LkNum) - R(IM,LkNum)) / NDYS * NDY + R(IM,LkNum)
      END IF
      DailyIncidentSolarRadiation = X * (0.355 + 0.68 * (1. - CloudCover))
      RETURN

      END FUNCTION DailyIncidentSolarRadiation

!----------------------------------------------------------------------------
      REAL FUNCTION DailyReflectedRadiationFromIce                        &
     &       (Incident, FractionInBareIce, FractionInNewSnow,             &
     &       FractionInOldSnow, FractionInMeltingSnow)
!
!  DailyReflectedRadiationFromIce = Daily solar radiation reflected
!     from the ice surface in langleys (calories/square cm)
!  Incident = Daily total solar radiation incident to the
!     ice surface in langleys (calories/square cm)
!  FractionInBareIce = fraction of ice surface area that is bare
!  FractionInNewSnow = fraction of ice surface area covered with new snow
!  FractionInOldSnow = fraction of ice surface area covered with old snow
!  FractionInMeltingSnow = fraction of ice surface area covered with melting
!     snow
!
      IMPLICIT NONE
      REAL :: Incident, FractionInBareIce, FractionInNewSnow
      REAL :: FractionInOldSnow, FractionInMeltingSnow, Residual

      IF (FractionInBareIce .LT. 0.  .OR.                                 &
     &    FractionInNewSnow .LT. 0.  .OR.                                 &
     &    FractionInOldSnow .LT. 0.  .OR.                                 &
     &    FractionInMeltingSnow .LT. 0.) THEN
         ErrorMessage = 'XEVAPOUT: Error in DailyReflectedRadiationFromIce.'  &
     &                  // CHAR(13)//CHAR(10) //                              &
     &                  'A negative fraction was specified.'
         ErrorLevel = 1
         RETURN
      END IF

      Residual = 1.0 - FractionInBareIce - FractionInNewSnow              &
     &           - FractionInOldSnow - FractionInMeltingSnow
      IF (ABS(Residual) .GT. 0.0001) THEN
         ErrorMessage = 'XEVAPOUT: Error in DailyReflectedRadiationFromIce.'  &
     &                  // CHAR(13)//CHAR(10) //                              &
     &                  'Fractions are unbalanced.'
         ErrorLevel = 1
         RETURN
      END IF

      DailyReflectedRadiationFromIce = Incident                           &
     &    * (0.85 * FractionInNewSnow                                     &
     &     + 0.70 * FractionInOldSnow                                     &
     &     + 0.50 * FractionInMeltingSnow                                 &
     &     + 0.45 * FractionInBareIce)

      RETURN

      END FUNCTION DailyReflectedRadiationFromIce

!----------------------------------------------------------------------------
      REAL FUNCTION DailyNetLongwaveRadiation (WaterTemperature,          &
     &     AirTemperature, CloudCover, VaporPressureOfAir, P9)
!
!     DailyNetLongwaveRadiation = Daily net longwave radiation exchange between
!        the atmosphere and the body of water in langleys (calories/square cm)
!     AirTemperature = temperature of atmosphere above water body in degrees C
!     WaterTemperature = (surface) temperature of water body in degrees Celcius
!     CloudCover = average daily fraction of sky covered with clouds
!     VaporPressureOfAir = mean vapor pressure in the air (@ 2m) in millibars
!
      IMPLICIT NONE
      REAL :: WaterTemperature, AirTemperature, CloudCover, P9
      REAL :: VaporPressureOfAir, Emissivity, ATemperature, WTemperature

      Emissivity = 0.53 + 0.065 * SQRT(VaporPressureOfAir)
      ATemperature = AirTemperature + 273.16
      WTemperature = WaterTemperature + 273.16
      DailyNetLongwaveRadiation =                                         &
     &    1.17e-7 * ATemperature ** 4 * Emissivity                        &
     &    * (P9 + (1. - P9) * (1. - CloudCover))                          &
     &    - 0.97 * 1.17e-7 * WTemperature ** 4

      RETURN

      END FUNCTION DailyNetLongwaveRadiation


!----------------------------------------------------------------------------
      SUBROUTINE IceCover (Area1, Depth1, Volume1, T1, WaterIceFlux,      &
     &    AtmosphereIceFlux, Beta1, Beta2, Area2, Depth2, Volume2, T2,    &
     &    ExcessFromIceMelt, LakeArea, EvapOverIce, Snow)
!
!     Routine to iteratively solve finite difference approximations to the
!     differential equations describing ice growth as a function of the heat
!     fluxes to/from the ice pack.
!
!     Given:
!       parameters: Beta1, Beta2, (,and LakeArea)
!       beginning-of-time-interval values: Area1, Depth1, Volume1
!       ice pack fluxes and other: WaterIceFlux, AtmosphereIceFlux,
!                                  EvapOverIce (m3/day), Snow (m3/day)
!
!     => Estimate end-of-time-interval values: Area2, Depth2, Volume2
!     => Estimate residual heat if ice melts completely: ExcessFromIceMelt
!     => Correct WaterIceFlux & AtmosphereIceFlux: NOT CURRENTLY DONE.
!
!     Units: area (m2), depth (m), volume (m3), fluxes (cal/day),
!            temperature (deg.C)
!
      IMPLICIT NONE

      REAL, PARAMETER :: dT = 1.0          ! 1.0 day
      REAL, PARAMETER :: Rho = 1000000.0   ! density of water & ice, gm/m3
      REAL, PARAMETER :: Fusion = 79.7     ! heat of fusion of H2O, cal/gm

      REAL :: Area1, Depth1, Volume1, WaterIceFlux, T1, T2
      REAL :: AtmosphereIceFlux, Beta1, Beta2, Area2, Depth2, Volume2
      REAL :: ExcessFromIceMelt, Area, Depth, X1, X2, dA, dV, Old, Y1, Y2
      REAL :: LakeArea, Count, EvapOverIce, Snow, AIFafterTempRise

!
!     Volume2 is an equivalent volume of the ice pack, in m3, at end of dT
!     (computed by assuming that ice is at freezing temperature).
!
!     Simultaneous solution to these equations,
!
!     AIFafterTempRise = AtmosphereIceFlux
!    & - Rho * 1. / 2. * (Volume1 + Volume2) / 2. * (T2 - T1) / dT
!     dV = -(WaterIceFlux + AIFafterTempRise) / Fusion / Rho * dT
!    & - EvapOverIce + Snow
!     Volume2 = Volume1 + dV
!
!     is:
!
      AIFafterTempRise = (AtmosphereIceFlux - Rho * 1. / 2. * (T2 - T1)   &
     &      / dT * (Volume1 - WaterIceFlux / 2. * dT / Rho / Fusion -     &
     &      EvapOverIce / 2. + Snow / 2.)) / (1. - 1. / 4. / Fusion *     &
     &      (T2 - T1))
      dV = -(WaterIceFlux + AIFafterTempRise) / Fusion / Rho * dT         &
     &     - EvapOverIce + Snow
      Volume2 = Volume1 + dV

      IF (Volume2 .LT. 0.0001) THEN
!
!       Minimum icepack allowed has area of 1 m2 and depth of 1/10 mm.
!
        dV = 0.0001 - Volume1
        Volume2 = 0.0001
        AIFafterTempRise = AtmosphereIceFlux                              &
     &      - Rho * 1. / 2. * (Volume1 + Volume2) / 2. * (T2 - T1) / dT
        Area2 = 1.0
        Depth2 = 0.0001

      ELSE

!
!     Solve ice cover differential equations.
!
        Area2 = Area1
        Depth2 = Depth1
        Area = Area1
        Depth = Depth1
        Count = 0
        Old = Depth2 * 2.0
        DO WHILE (ABS(Depth2 - Old) / Depth2 .GT. 0.00001)
          Count = Count + 1
          Old = Depth2
          IF (Count .LE. 25) THEN
            X1 = Beta1 * SQRT(Area)
            X2 = Beta2 * SQRT(Area)
            Y1 = Area + Depth * X1
            Y2 = Area + Depth * X2
            dA = -(X2 / Y2 * MIN(AIFafterTempRise, 0.)                    &
     &           + X1 / Y1 * MAX(AIFafterTempRise, 0.)                    &
     &           + X2 / Y2 * WaterIceFlux) / Fusion / Rho * dT            &
     &           - X1 / Y1 * EvapOverIce * dT
            Area2 = MIN(LakeArea, MAX((Area2 + Area1 + dA) / 2.0, 1.0))
            Depth2 = Volume2 / Area2
            Area = (Area1 + Area2) / 2.0
            Depth = (Depth1 + Depth2) / 2.0
          END IF
        END DO
      END IF

      ExcessFromIceMelt = (dV + EvapOverIce - Snow) * Fusion * Rho / dT   &
     &                    + WaterIceFlux + AIFafterTempRise

      RETURN

      END SUBROUTINE IceCover

!----------------------------------------------------------------------------
!
!     This unit interprets the heat layer superposition information in terms
!     of a vertical temperature profile defined at various points along the
!     depth, where T(0) is temperature at the surface (depth = D(0) = 0) and
!     T(K) is temperature at the bottom (depth = D(K) = MaxDepth).
!
!     Added 24apr01 by TECII.  (Based on ExtDlyTD.FOR and T-Index.FOR.)
!
      SUBROUTINE GetTemperatureProfile                                  &
     & (rCV, Mf1, Mf2, Ms1, Ms2, Hd, P2, P3, P4, P5, P6, P7, P8,        &
     &       DataIndex, T, D, K)
      REAL,    INTENT(IN)  :: rCV, Mf1, Mf2, Ms1, Ms2, Hd
      REAL,    INTENT(IN)  :: P2, P3, P4, P5, P6, P7, P8
      INTEGER, INTENT(IN)  :: DataIndex
      REAL, DIMENSION(MaxNoOfLayers*2 + 1), INTENT(INOUT) :: T, D
      INTEGER, INTENT(OUT) :: K
      
      INTEGER :: I, M
      REAL    :: Temperature(1:MaxNoOfLayers)
      REAL    :: Depth(0:MaxNoOfLayers), V, M1, MaxDepth
      REAL    :: MaxVol, vF, Vol(0:MaxNoOfLayers), Slope(1:MaxNoOfLayers)

      !
      ! Compute surface temperature contributions...
      !
      IF (TotalHeat(NoOfLayers) .LE. Hd) THEN
        DO K = 1, NoOfLayers
          SurfTempFromLayer(K) = SurfTempFromLayer(K - 1)                 &
     &     + (TotalHeat(K) - TotalHeat(K - 1))                            &
     &     * MixingFunction(rCV, Mf1, Mf2, P2, P3, P4, P8, AgeOfLayer(K))
        END DO
      ELSE
        DO K = 1, NoOfLayers
          SurfTempFromLayer(K) = SurfTempFromLayer(K - 1)                 &
     &     + (TotalHeat(K) - TotalHeat(K - 1))                            &
     &     * MixingFunction(rCV, Ms1, Ms2, P5, P6, P7, P8, AgeOfLayer(K))
        END DO
      END IF
      
      !
      !  F = volume (measured down from the surface) times r times C, which
      !      when exceeded by rCVm yields a fully-mixed temperature profile;
      !      subscripted n for post-fall turnover but pre-spring turnover;
      !      subscripted p for post-spring turnover but pre-fall turnover.
      !
      !  F = 0 will produce rectangular temperature profiles (with volume) only.
      !
      V = CoordLakeVolume(DataIndex)
      Depth(0) = 0.0
      Vol(0) = 0.0
      MaxDepth = LakeDepth(0.0, DataIndex)  ! use actual maximum depth of lake
      MaxVol = V - LakeVolume(MaxDepth, DataIndex)
      DO K = 1, NoOfLayers
        Temperature(K) = SurfTempFromLayer(NoOfLayers - K + 1)
        IF (K .EQ. 1) THEN
          IF (Temperature(K) .LT. 3.98) THEN
            vF = P4 / 1000000. / 1.0 / 1.0 * 1.0e+20
          ELSE
            vF = P7 / 1000000. / 1.0 / 1.0 * 1.0E+20
          END IF
        END IF
        M1 = MIN(vF, V)
        Vol(K) = (TotalHeat(NoOfLayers-K+1) - TotalHeat(NoOfLayers-K))   &
     &          / (SurfTempFromLayer(NoOfLayers - K + 1)                        &
     &         - SurfTempFromLayer(NoOfLayers - K)) / rCV * V
        Slope(K) = 0.
        IF (Vol(K) .LT. M1) THEN
          Slope(K) = Vol(K) / vF
          Vol(K) = Vol(K) * 2.0 / (1.0 + Vol(K) / vF)
        END IF
        Depth(K) = LakeDepth(V - Vol(K), DataIndex)
      END DO
      Temperature(NoOfLayers + 1) = 3.98
      DO K = 1, NoOfLayers
        Temperature(K) = Temperature(K) - Temperature(K + 1)
        IF (Slope(K) .GT. 0)                                             &
     &     Slope(K) = (Slope(K) * Temperature(K) - Temperature(K)) / Vol(K)
      END DO
      K = 1
      DO WHILE (K .LE. 2 * NoOfLayers + 1 .AND. Depth(K/2) .LE. MaxDepth)
        T(K) = 3.98
        DO J = K / 2 + 1, NoOfLayers
          T(K) = T(K) + Temperature(J) + Slope(J) * Vol(K / 2)
        END DO
        IF (K/2*2 .EQ. K) T(K) = T(K) + Temperature(K / 2)               &
     &      + Slope(K / 2) * Vol(K / 2)
        IF (T(K) .LT. 0.) T(K) = 0.
        D(K) = Depth(K / 2)
        K = K + 1
      END DO
      K = K - 1
      IF (K .EQ. 2 * NoOfLayers + 1) THEN
        K = K + 1
        T(K) = 3.98
        D(K) = MaxDepth
      ELSE
        K = K + 1
        T(K) = 3.98
        DO J = K / 2 + 1, NoOfLayers
          T(K) = T(K) + Temperature(J) + Slope(J) * MaxVol
        END DO
        T(K) = T(K) + Temperature(K / 2) + Slope(K / 2) * MaxVol
        D(K) = MaxDepth
      END IF
      RETURN
      END SUBROUTINE GetTemperatureProfile

!--------------------------------------------------------------------
      REAL FUNCTION LakeDepth(Volume, I)
      REAL,    INTENT(IN) :: Volume
      INTEGER, INTENT(IN) :: I
      IF (Volume .LT. 0.) THEN
       LakeDepth = DepthVolumeM(I)
      ELSE
       LakeDepth = DepthVolumeM(I) - (Volume / DepthVolumeA(I)) ** (1. / DepthVolumeB(I))
      ENDIF
      RETURN
      END FUNCTION LakeDepth

!-------------------------------------------------------------------
      REAL FUNCTION LakeVolume(Depth, I)
      REAL,    INTENT(IN) :: Depth
      INTEGER, INTENT(IN) :: I
      IF (Depth .GT. DepthVolumeM(I)) THEN
       LakeVolume = 0.
      ELSE
       LakeVolume = DepthVolumeA(I) * (DepthVolumeM(I) - Depth) ** DepthVolumeB(I)
      ENDIF
      RETURN
      END FUNCTION LakeVolume



!----------------------------------------------------------------------------
      REAL FUNCTION EPN (X)
      IMPLICIT NONE
      REAL :: X
      IF (X .LT. -87.33) THEN
        EPN = 0.
      ELSE
        EPN = EXP(X)
      END IF
      RETURN

      END FUNCTION EPN


END MODULE XEvapSubs

!==============================================================================
!==============================================================================

MODULE XEvapMain

CONTAINS

!------------------------------------------------------------------------------

      SUBROUTINE DO_XEVAP(ErrStatus, HandleIn, Lake)

      USE XEvapSubs

      IMPLICIT NONE

      INTEGER, INTENT(INOUT) :: ErrStatus
      INTEGER, INTENT(IN)    :: HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER (KIND=INTEGER2) :: I1, I2, I3, I4, Id, Xk, Missing
      INTEGER (KIND=INTEGER2) :: LI1, LI2, LI3, LI4

      INTEGER :: Day, Month, Year, NumberOfDays, DataIndex, RecNo, N
      INTEGER :: BaseDate, I, J, K, M, RecNo1, NMonth
      INTEGER :: NDay, NYear, Pd, NoIterations, Hdr, SeqNum
      INTEGER :: SeqAllTemp, IOS, PrintInterval, SpinUp
      INTEGER :: U1, U3, U4, U5, U6, U7, U8a, U8b, ZSU1, ZSU2
      INTEGER :: UnitTD, MaxD, DayNum, Depth                            ! 10may2013
      INTEGER :: NumDaysInMetInputFile, RNum                            ! 13may2013

      LOGICAL :: Quiet, Flag

      REAL :: AirTemp, DewPtTemp, WindSpeed, CloudCover, Precipitation
      REAL :: Runoff, Inflow, Outflow, Hd, NWTemp
      REAL :: SurfaceElevation, StoredHeat
      REAL :: OtherTerms, LakeEvaporation, Area
      REAL :: VapPres, Temperature, Incident, Reflect, Latent, Sensible
      REAL :: Advection, NLWave, rCV, Mf1, Mf2, Ms1, Ms2, Mxf, Mxs
      REAL :: DD1, DD2, TT1, ST, A, B, Delta, Temp
      REAL, DIMENSION(0:999) :: TempAtDepth

      REAL, DIMENSION(MaxNoOfLayers*2 + 1) :: ProfileT, ProfileD

!      REAL    :: P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, AgeOfCurrentLayer
!      REAL    :: SurfaceTemperature, IceAmount, IceArea, IceDepth, IceTemp
!      INTEGER :: NoOfLayers, SequenceNumber, Finish
!      REAL, DIMENSION(0:MaxNoOfLayers) :: SurfTempFromLayer, TotalHeat
!      REAL, DIMENSION(1:MaxNoOfLayers) :: AgeOfLayer
!      CHARACTER (LEN=3)  :: Ext
!      CHARACTER (LEN=77) :: Comment

!      TYPE (EvapParmType) :: EvapParms

      CHARACTER (LEN=1), PARAMETER  :: Lf = CHAR(10)
      CHARACTER (LEN=2), PARAMETER  :: CrLf = CHAR(13)//CHAR(10)
      CHARACTER (LEN=3)   :: LakeName
      CHARACTER (LEN=4)   :: Yyyy
      CHARACTER (LEN=32)  :: Dummy, S
      CHARACTER (LEN=80)  :: Line, Line2
      CHARACTER (LEN=99)  :: ParmFileName, DataFileName
      CHARACTER (LEN=99)  :: File1, File3, File4, File5, File6, File7
      CHARACTER (LEN=99)  :: File8a, File8b, FileMiss, ZSFile1, ZSFile2
      CHARACTER (LEN=130) :: BulkFile, TDFile


      VapPres(Temperature) =                                              &
     &    10.0**(-7.90298 * ((373.16 / (Temperature + 273.16)) - 1.0)     &
     &    + 5.02808 * LOG10((373.16 / (Temperature + 273.16))) - 1.3816   &
     &    * 0.0000001 * (10.0**(11.334 * (1.0 - (1.0 / (373.16            &
     &    / (Temperature + 273.16))))) - 1.0) + 8.1328 * 0.001            &
     &    * (10.0**(-3.49149 * ((373.16 / (Temperature + 273.16)) - 1.0)) &
     &    - 1.0) + 3.0057149)


!
!     Assign the windows handle passed in (HandleIn) to the global variable
!     WinHandle which will be used throughout the program to pass messages
!     (error, status, etc) back to the calling windows application.
!
      WinHandle = HandleIn

!
!     If ErrStatus is 0, set PrintDebugInfo to .FALSE. otherwise set it .TRUE.
!
      PrintDebugInfo = .FALSE.
      IF (ErrStatus .NE. 0) PrintDebugInfo = .TRUE.

!
!     Initialize the array of open file unit numbers (global variables)
!
      CALL ClearFilesOpen()

!
!     Initialize the ErrorLevel variable to zero.  This is a global variable,
!     defined in GLERLUTIL, used to flag when an error has occured.  It is
!     set to a non-zero value if an error occurs  It will be used
!     throughout this program as the flag to indicate that an error occured
!     in a CALLed subroutine or function and that the program needs to be
!     terminated in a clean manner back to the Windows app.  When this was a
!     DOS app we didn't need to be this picky.  The OS would close the files
!     and do any other needed cleanup.  Now that this is a routine within a
!     DLL we have to handle all of that stuff ourselves.
!
      ErrorLevel = 0

      MonitoringDepth = 0.         ! depth of water-temperatures-to-report-in-simulation (24apr01)

!
!  Initialize array contents with values from array in FcstUtil module
!
      AverageSurfaceArea(:) = CoordLakeArea(:)

      IF (PrintDebugInfo) THEN
         WRITE(ErrorMessage, *) 'Printing Debug Info for XEvapOut'; CALL PassMsg
      END IF
      
!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Input and fill in parameter and initial values...
!
      U1 = 11
      File1 = TRIM(ThisLakeDir) // 'EBSNNM'
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,              &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U1, 1001, REC=1, ERR=812) LakeName
      READ(U1, 1010, REC=2, ERR=812) Pd
      READ(U1, 1002, REC=3, ERR=812) Day, Month, Year
      CLOSE(U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(Day, Month, Year, BaseDate); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Check Program Designator for valid value
!
      Flag = .FALSE.
      IF (Pd .EQ. -10) Flag = .TRUE.
      IF (Pd .EQ.  -9) Flag = .TRUE.
      IF (Pd .EQ.   4) Flag = .TRUE.
      IF (Pd .EQ.   7) Flag = .TRUE.
      IF (.NOT. Flag) THEN
         WRITE(ErrorMessage, 9014)
         CALL PassMsg
         GOTO 898
      END IF

!
!     Check Lakename for validity
!
!     Please note well: The handling of the combined Huron+Georgian is
!     messy, to say the least. AHPS does not do a combined lake, only
!     doing the individual HUR and GEO basins. This is because the early
!     versions of this LLTM model only applied to the individual basins.
!     Tom Croley, however, has added the combined basin to this model, which
!     "throws a monkey-wrench into" the basin numbering schemes, etc.
!     What I am doing is, in the AHPS-wide modules, such as FcstUtil, I am
!     adding a basin 13 at the end of the appropriate PARAMETER arrays. That
!     will allow everything else in AHPS to continue working without change.
!     In this model, I will assign DataIndex to 13 for HGB and fill in the
!     entries 8-12 with dummy missing values.
!
      DataIndex = -1
      IF ( Lakename .EQ. 'SUP' ) DataIndex = 1
      IF ( Lakename .EQ. 'MIC' ) DataIndex = 2
      IF ( Lakename .EQ. 'HGB' ) DataIndex = 13
      IF ( Lakename .EQ. 'HUR' ) DataIndex = 3
      IF ( Lakename .EQ. 'GEO' ) DataIndex = 4
      IF ( Lakename .EQ. 'STC' ) DataIndex = 5
      IF ( Lakename .EQ. 'ERI' ) DataIndex = 6
      IF ( Lakename .EQ. 'ONT' ) DataIndex = 7
      IF ( Lakename .EQ. 'MHU' ) DataIndex = 0
      IF ( Lakename .EQ. 'CHA' ) DataIndex = 0
      IF ( LakeName .EQ. 'EMB' ) DataIndex = 0
      IF ( LakeName .EQ. 'URA' ) DataIndex = 0
      IF ( LakeName .EQ. 'VOL' ) DataIndex = 0
      IF (DataIndex .EQ. -1) THEN
         WRITE(ErrorMessage, 9016)
         CALL PassMsg
         GOTO 898
      END IF
      IF (DataIndex .EQ. 0) THEN
         WRITE(ErrorMessage, 9017)
         CALL PassMsg
         GOTO 898
      END IF
      IF (DataIndex .GT. 7) THEN
         WRITE(ErrorMessage, 9018)
         CALL PassMsg
         GOTO 898
      END IF

!
!     More initialization
!
      Area = AverageSurfaceArea(DataIndex)
      rCV = AverageVolume(DataIndex) * 1000000. * 1.0 * 1.0 / 1.0e+20
      Hd = rCV * 3.98
      WRITE(Line(1:13), 1009) Hd      ! force same round-off as in
      READ(Line(1:13), 1009) Hd       ! write/read to/from EVPPARAM & PARAMEVP

!
!     E' = r Ce (qw - q) U' / rw
!     |    | |   |    |  |    |
!     |    | |   |    |  |    |__water density, gm/cm3
!     |    | |   |    |  |__wind speed, cm/s
!     |    | |   |    |__specific humidity of air, gm/gm
!     |    | |   |__spec. humid. of saturated air at water temperature, gm/gm
!     |    | |__evaporation mass transfer coefficient, dimensionless
!     |    |__air density, gm/cm3
!     |__evaporation, cm/s
!
!     E = (86400 * 10 * 100 * .622 / p * r / rw) Ce (ew - e) U
!     |      |     |     |      |    |               |    |  |
!     |      |     |     |      |    |               |    |  |__wind speed, m/s
!     |      |     |     |      |    |               |    |__vapor pressure, mb
!     |      |     |     |      |    |               |__vap. pres. @ surface,mb
!     |      |     |     |      |    |__atmospheric pressure, mb
!     |      |     |     |      |__(gm/cm3 water vapor)/(gm/cm3 dry air),
!     |      |     |     |__cm/m
!     |      |     |__mm/cm
!     |      |__sec/day
!     |__evaporation, mm/day
!
!     Q' = -r Cp Ch (Tw - T) U'
!     |       |  |   |    |
!     |       |  |   |    |__air temperature, deg. C
!     |       /  |   |__water surface temperature, deg. C
!     |      /   |__sensible heat mass transfer coefficient, dimensionless
!     |     |__specific heat of air at constant pressure = 0.22 cal/(gm deg.C)
!     |__sensible heat rate, cal/(cm2 s)
!
!     Q = -(86400 * 100 * 0.22 * r) Ch (Tw - T) U
!     |__sensible heat rate, cal/(cm2 d)
!
      EvapUnitsCoeff = 86400.0 * 10.0 * 100.0 * 0.622                     &
     &    / AtmosphericPressure(DataIndex) * AirDensity(DataIndex) / 1.0
      HeatUnitsCoeff = 86400.0 * 100.0 * 0.22 * AirDensity(DataIndex)

!
!  Read the data from the parameter file
!
      IF (PrintDebugInfo) THEN
         WRITE(ErrorMessage, *) 'XEVAPOUT: Read parameter file'; CALL PassMsg
      END IF
      IF (Pd .EQ. -9 .OR. Pd .EQ. -10) THEN
        ParmFileName = TRIM(ThisLakeDir) // 'PARAMEVP.'//LakeName
      ELSE
        ParmFileName = TRIM(ThisLakeDir) // 'EVPPARAM.'//LakeName
      END IF

      CALL ReadParmFile(ParmFileName, Pd, Ext, Comment,                    &
     &                  P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,                    &
     &                  SequenceNumber, Finish,                            &  ! start, end
     &                  SurfaceTemperature, IceAmount, IceArea, IceDepth,  &
     &                  IceTemp, AgeOfCurrentLayer, NoOfLayers,            &
     &                  SurfTempFromLayer, TotalHeat, AgeOfLayer)

      NumberOfDays = Finish - SequenceNumber + 1
      SurfTempFromLayer(0) = 3.98
      TotalHeat(0) = Hd
      I = NoOfLayers + 1
      TotalHeat(I)  = 0.0    ! dummy value avoids undefined use later
      AgeOfLayer(I) = 0.0    ! dummy value avoids undefined use later

!      CALL LocalVarsToParm(EvapParms, Ext, Comment,                        &
!     &       P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,                               &
!     &       SequenceNumber, Finish, SurfaceTemperature, IceAmount,        &
!     &       IceArea, IceDepth, IceTemp, AgeOfCurrentLayer, NoOfLayers,    &
!     &       SurfTempFromLayer, TotalHeat, AgeOfLayer)

      IF (PrintDebugInfo) THEN
         WRITE(ErrorMessage, *) 'XEVAPOUT: Preparing for daily data input'; CALL PassMsg
      END IF
      
!
!     Prepare for input of daily data.
!
      MissingDataUnitNo = -1
      IF (Pd .NE. -10) THEN
         U3 = 31
         File3 = TRIM(ThisLakeDir) // LakeName//'MD.'//Ext
         OPEN(UNIT=U3, FILE=TRIM(File3), STATUS='OLD', RECL=8,             &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=831)
         CALL FileWasOpened(U3); IF (ErrorLevel .NE. 0) GOTO 899
         READ(U3, REC=1, ERR=832) Id, RecNo
         READ(U3, REC=2, ERR=832) N
         N = N - RecNo + 1
         NumDaysInMetInputFile = N
         MissingDataUnitNo = U3
         MissingDataFileName = TRIM(File3)
         IF (Ext .NE. 'CLM') THEN
            MissingDataUnitNo = 99
            MissingDataFileName = TRIM(ThisLakeDir) // LakeName//'MD.CLM'
            OPEN(UNIT=MissingDataUnitNo, FILE=TRIM(MissingDataFileName),  &
                 STATUS='OLD', ACCESS='DIRECT', RECL=8,                   &
                 FORM='UNFORMATTED', ERR=891)
            CALL FileWasOpened(MissingDataUnitNo); IF (ErrorLevel .NE. 0) GOTO 899
         END IF
         DataFileName = File3
      ELSE  ! (IF Pd .EQ. -10)
         U4 = 41
         File4 = TRIM(ThisLakeDir) // 'MET_DATA.'//LakeName
         OPEN(UNIT=U4, FILE=TRIM(File4), STATUS='OLD', RECL=40,            &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=841)
         CALL FileWasOpened(U4); IF (ErrorLevel .NE. 0) GOTO 899
         READ(U4, REC=1, ERR=842) Dummy, RecNo, N
         N = N - RecNo + 1
         SequenceNumber = RecNo
         NumberOfDays = N
         DataFileName = File4
      END IF

      IF (Pd .EQ. -9) THEN
         SequenceNumber = RecNo
         Finish = BaseDate - 1
         NumberOfDays = Finish - RecNo + 1
         IF (NumberOfDays .LT. Length) THEN
            ErrorMessage = 'Cannot create ZSEVAPPR.'//LakeName//' with sufficient length'
            CALL PassMsg
            GOTO 898
         END IF
         SpinUp = NumberOfDays - Length
      END IF

      IF (SequenceNumber .LT. RecNo) THEN
         ErrorMessage = 'Requested start date in parameter file is before data start in: '//TRIM(DataFileName)
         CALL PassMsg
         GOTO 898
      END IF
      IF (SequenceNumber + NumberOfDays .GT. RecNo + N) THEN
         ErrorMessage = 'Requested end date in parameter file is after data end in: '//TRIM(DataFileName)
         CALL PassMsg
         GOTO 899
      END IF
      IF (Pd .EQ. -10) THEN
         Hdr = MHdr
      ELSE
         Hdr = CHdr
      END IF
      RecNo = Hdr + 1 + SequenceNumber - RecNo - 1

      
      IF (PrintDebugInfo) THEN
         WRITE(ErrorMessage, *) 'XEVAPOUT: Opening Bulk Coefficient and TempDepth files'; CALL PassMsg
      END IF
!
!     If requested, open file for the Bulk Transfer Coefficient values
!     Not allowed when Pd = 7 (for now, anyway).
!
      IF (Pd .EQ. 7) WriteBulkCoefficient = .FALSE.
      IF (WriteBulkCoefficient) THEN
         IF (Pd .EQ. -10) BulkFile = TRIM(ThisLakeDir) // 'BulkCoeff_'//LakeName//'.MET'
         IF (Pd .EQ.  -9) BulkFile = TRIM(ThisLakeDir) // 'BulkCoeff_'//LakeName//'.CLM'
         IF (Pd .EQ.   4) BulkFile = TRIM(ThisLakeDir) // 'BulkCoeff_'//LakeName//'.PRV'
         OPEN(UNIT=47, FILE=TRIM(BulkFile), STATUS='REPLACE')
         CALL FileWasOpened(47); IF (ErrorLevel .NE. 0) GOTO 899
         WRITE(47, 4701)
      END IF
  
!
!     If requested, open file for the Temperature/Depth Profile data.
!     Not allowed when Pd = 7 (for now, anyway).
!
      IF (Pd .EQ. 7) WriteTempDepthProfile = .FALSE.
      IF (WriteTempDepthProfile) THEN
         MonitoringDepth = LakeDepthMaxMeters(DataIndex)                              ! 10may13
         IF (Pd .EQ. -10) TDFile = TRIM(ThisLakeDir) // 'TempDepth_'//LakeName//'.MET'
         IF (Pd .EQ.  -9) TDFile = TRIM(ThisLakeDir) // 'TempDepth_'//LakeName//'.CLM'
         IF (Pd .EQ.   4) TDFile = TRIM(ThisLakeDir) // 'TempDepth_'//LakeName//'.PRV'
         UnitTD = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
         OPEN(UNIT=UnitTD, FILE=TRIM(TDFile), STATUS='REPLACE')
         CALL FileWasOpened(UnitTD); IF (ErrorLevel .NE. 0) GOTO 899
         WRITE(UnitTD, 4801)
      END IF
  
!
!     Calculate evaporation for these parameter and data sets.
!
      IF (PrintDebugInfo) THEN
         WRITE(ErrorMessage, *) 'XEVAPOUT: Calculate evap...'; CALL PassMsg
      END IF
      ZSU1 = 51
      ZSU2 = 52
      IF (Pd .EQ. -9 .OR. Pd .EQ. -10) THEN
        ZSFile1 = TRIM(ThisLakeDir) // 'ZSEVAP.'//LakeName
        OPEN(UNIT=ZSU1, FILE=TRIM(ZSFile1), STATUS='NEW', RECL=64,        &
     &       ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
        CALL FileWasOpened(ZSU1); IF (ErrorLevel .NE. 0) GOTO 899

        IF (Pd .EQ. -10) THEN
          ZSFile2 = TRIM(ThisLakeDir) // 'ZSTEMP.'//LakeName
          OPEN(UNIT=ZSU2, FILE=TRIM(ZSFile2), STATUS='NEW', RECL=2,       &
     &         ACCESS='DIRECT', FORM='UNFORMATTED', ERR=856)
          CALL FileWasOpened(ZSU2); IF (ErrorLevel .NE. 0) GOTO 899
        ELSE
          ZSFile2 = TRIM(ThisLakeDir) // 'ZSEVAPPR.'//LakeName
          OPEN(UNIT=ZSU2, FILE=TRIM(ZSFile2), STATUS='NEW', RECL=64,      &
     &         ACCESS='DIRECT', FORM='UNFORMATTED', ERR=856)
          CALL FileWasOpened(ZSU2); IF (ErrorLevel .NE. 0) GOTO 899
          N = Finish - Length + 1
          WRITE(ZSU2, REC=1, ERR=858) N, Finish
        END IF
        N = 0
        RecNo1 = SequenceNumber
      ELSE
        IF (Pd .EQ. 4) THEN
           ZSFile1 = TRIM(ThisLakeDir) // 'ZSEVAPPR.'//LakeName
           ZSU1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
           OPEN(UNIT=ZSU1, FILE=TRIM(ZSFile1), STATUS='OLD', RECL=64,     &
     &          ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
           CALL FileWasOpened(ZSU1); IF (ErrorLevel .NE. 0) GOTO 899
        ELSE
           U6 = 61
           File6 = TRIM(ThisLakeDir) // 'DATEFILE.INT'
           OPEN(UNIT=U6, FILE=TRIM(File6), STATUS='OLD', ERR=861)
           CALL FileWasOpened(U6); IF (ErrorLevel .NE. 0) GOTO 899
           READ(U6, 1073, ERR=862)
           READ(U6, 1073, ERR=862)
           READ(U6, 1073, ERR=862) Yyyy
           CLOSE(U6)
           CALL FileWasClosed(U6); IF (ErrorLevel .NE. 0) GOTO 899
           ZSFile1 = TRIM(ThisLakeDir) // 'ZSEV'//Yyyy//'.INT'
           ZSU1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
           OPEN(UNIT=ZSU1, FILE=TRIM(ZSFile1), STATUS='OLD', RECL=64,      &
     &          ACCESS='DIRECT', FORM='UNFORMATTED', ERR=75)
           CLOSE(ZSU1, STATUS='DELETE')
 75        OPEN(UNIT=ZSU1, FILE=TRIM(ZSFile1), STATUS='NEW', RECL=64,      &
     &          ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
           CALL FileWasOpened(ZSU1); IF (ErrorLevel .NE. 0) GOTO 899
           N = SequenceNumber - 1
           WRITE(ZSU1, REC=1, ERR=853) SequenceNumber, N
        END IF

        READ(ZSU1, REC=1, ERR=852) RecNo1, N
        N = N - RecNo1 + 1
        IF (SequenceNumber .LT. RecNo1) THEN
          ErrorMessage = 'Cannot place results before ZSEVAPPR start.'
          CALL PassMsg
          WRITE(ErrorMessage, 9274) SequenceNumber, RecNo1
     9274 FORMAT(' Intended date=', I0, '; ZSEVAPPR start=', I0)
          CALL PassMsg
          GOTO 898
        END IF
      END IF

      I = SequenceNumber + NumberOfDays - RecNo1
      IF (I .GT. N) N = I
      M = RecNo1
      RecNo1 = ZHdr + 1 + SequenceNumber - RecNo1 - 1

!      CALL ParmToLocalVars(EvapParms, P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,     &
!     &     SurfaceTemperature, IceAmount, IceArea, IceDepth, IceTemp,     &
!     &     AgeOfCurrentLayer, NoOfLayers, SurfTempFromLayer,              &
!     &     TotalHeat, AgeOfLayer)

!
!     Call heat balance routine for each days data, yielding daily evaporation.
!
      SequenceNumber = SequenceNumber - 1
      IF (P4 .EQ. 0) THEN
        Mf1 = 0.0
        Mf2 = rCV
      ELSE
        Mf1 = MIN(P4, 2.0 * rCV / (1.0 + rCV / P4))
        Mf2 = MAX(rCV, 2.0 * rCV / (1.0 + rCV / P4))
      END IF

      IF (P8 .GT. rCV .AND. P8 .GT. Mf2) THEN
        Mxf = -LOG((P8 - Mf2) / P2 / Mf2) / P3
      ELSE
        Mxf = 3.40E+38
      END IF
      
      IF (P7 .EQ. 0) THEN
        Ms1 = 0.0
        Ms2 = rCV
      ELSE
        Ms1 = MIN(P7, 2.0 * rCV / (1.0 + rCV / P7))
        Ms2 = MAX(rCV, 2.0 * rCV / (1.0 + rCV / P7))
      END IF

      IF (P8 .GT. rCV .AND. P8 .GT. Ms2) THEN
        Mxs = -LOG((P8 - Ms2) / P5 / Ms2) / P6
      ELSE
        Mxs = 3.40E+38
      END IF

      IF (PrintDebugInfo) THEN
         WRITE(ErrorMessage, *) 'Compute StoredHeat'; CALL PassMsg
      END IF
      
      StoredHeat = HeatInStorage(SurfaceTemperature, AgeOfCurrentLayer,   &
     &         Hd, Mf1, Mf2, Ms1, Ms2, rCV, P2, P3, P4, P5, P6, P7, P8)
      IF (ErrorLevel .NE. 0) GOTO 899
      Runoff = 0.            ! unknown but small term (for heat budget)
      Precipitation = 0.     ! unknown but small term (for heat budget)
      Inflow = 0.            ! unknown but small term (for heat budget)
      Outflow = 0.           ! unknown but small term (for heat budget)
      OtherTerms = 0.        ! unknown but small (?) term (for heat budget)
      SurfaceElevation = 0.  ! unknown but unimportant term (for heat budget)

      CALL DateSequence(1, 1, 9999, SeqAllTemp); IF (ErrorLevel .NE. 0) GOTO 899
      IF ((Pd .NE. -10) .AND. (Pd .NE. 7)) THEN
        U7 = 71
        File7 = TRIM(ThisLakeDir) // 'ALLTEMPS.'//LakeName
        OPEN(UNIT=U7, FILE=TRIM(File7), STATUS='OLD', IOSTAT=IOS)
        IF (IOS .EQ. 0) THEN
          CALL FileWasOpened(U7); IF (ErrorLevel .NE. 0) GOTO 899
          READ(U7, 1996, ERR=872)
          READ(U7, 1997, ERR=872) NMonth, NDay, NYear, NWTemp
          CALL DateSequence(NDay, NMonth, NYear, SeqAllTemp); IF (ErrorLevel .NE. 0) GOTO 899
        ELSE
          NDay = 0
          NMonth = 0
          NYear = 0
          NWTemp = 0.0
        END IF
      END IF

      IF (PrintDebugInfo) THEN
         WRITE(ErrorMessage, *) 'Set up to handle missing data'; CALL PassMsg
      END IF
      
!
!  Initialize variables for handling missing data
!
      LI1 = -9999
      LI2 = -9999
      LI3 = -9999
      LI4 = -9999

!
!     Call heat balance routine for each days data, yielding daily evaporation.
!
      WRITE(ErrorMessage, 9701) 0
      CALL PassMsg
      PrintInterval = MIN(NINT(NumberOfDays / 10.0), 400)
      IF (PrintInterval .LT. 1) PrintInterval = 1

      IF (PrintDebugInfo) THEN
         WRITE(ErrorMessage, '(A,I0)') 'XEVAPOUT: NumberOfDays=', NumberOfDays; CALL PassMsg
      END IF
      
      DO DayNum = 1, NumberOfDays
         IF (PrintDebugInfo) THEN
            WRITE(ErrorMessage, '(A,I0)') 'XEVAPOUT: DayNum=', DayNum; CALL PassMsg            
         END IF
        I = MOD(DayNum,PrintInterval)
        SeqNum = SequenceNumber + DayNum
        CALL SequenceDate(Day, Month, Year, SeqNum); IF (ErrorLevel .NE. 0) GOTO 899
        IF (Pd .NE. -10) THEN
          Missing = 0
          RNum = RecNo + DayNum
          IF ((RNum .LT. 3) .OR. (RNum .GT. NumDaysInMetInputFile+2)) THEN
             WRITE(ErrorMessage, 9022) RNum, Month, Day, Year
             CALL PassMsg
             GOTO 898
          END IF
          READ(U3, REC=RNum, ERR=832) I1, I2, I3, I4
          IF (I1 .LE. -9000) Missing = 8
          IF (I2 .LE. -9000) Missing = Missing + 4
          IF (I3 .LE. -9000) Missing = Missing + 2
          IF (I4 .LE. -9000) Missing = Missing + 1
          IF (Missing .NE. 0) THEN
            CALL ReadMissing(I1, I2, I3, I4, Missing, SeqNum, Ext)
            IF (ErrorLevel .NE. 0) GOTO 899
          END IF
          IF (Missing .NE. 0) THEN
            IF (I1 .LE. -9000) I1 = LI1
            IF (I2 .LE. -9000) I2 = LI2
            IF (I3 .LE. -9000) I3 = LI3
            IF (I4 .LE. -9000) I4 = LI4
          END IF
          IF ((I1 .LT. -9000) .OR. (I2 .LT. -9000) .OR.                   &
     &        (I3 .LT. -9000) .OR. (I4 .LT. -9000)) THEN
             WRITE(ErrorMessage, 9021) Month, Day, Year
             CALL PassMsg
             GOTO 898
          END IF
        
          LI1 = I1
          LI2 = I2
          LI3 = I3
          LI4 = I4
          AirTemp = I1 / 100.
          DewPtTemp = I2 / 100.
          WindSpeed = I3 / 100.
          CloudCover = I4 / 100.
!
          DO WHILE (SeqAllTemp .LT. SeqNum)
            READ(U7, 1997, IOSTAT=IOS) NMonth, NDay, NYear, NWTemp
            IF (IOS .EQ. 0) THEN
               CALL DateSequence(NDay, NMonth, NYear, SeqAllTemp)
               IF (ErrorLevel .NE. 0) GOTO 899
            ELSE
               CALL DateSequence(1, 1, 9999, SeqAllTemp)
               IF (ErrorLevel .NE. 0) GOTO 899
            END IF
          END DO
          
          IF (SeqAllTemp .EQ. SeqNum) THEN
            IF (NWTemp .GE. -90.0) THEN
              StoredHeat = HeatInStorage(NWTemp, AgeOfCurrentLayer,       &
     &          Hd, Mf1, Mf2, Ms1, Ms2, rCV, P2, P3, P4, P5, P6, P7, P8)
              IF (ErrorLevel .NE. 0) GOTO 899
              SurfaceTemperature = NWTemp
            END IF
          END IF
        ELSE
          READ(U4, REC=RecNo+I, ERR=842) AirTemp, AirTemp, DewPtTemp,     &
     &       WindSpeed, CloudCover, Precipitation, Runoff, Inflow, Outflow
        END IF
        IF (WindSpeed .LT. 0.01) WindSpeed = 0.01

!        CALL ParmToLocalVars(EvapParms, Ext, Comment,                        &
!     &         P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,                               &
!     &         SequenceNumber, Finish, SurfaceTemperature, IceAmount,        &
!     &         IceArea, IceDepth, IceTemp, AgeOfCurrentLayer, NoOfLayers,    &
!     &         SurfTempFromLayer, TotalHeat, AgeOfLayer)

        IF (PrintDebugInfo) THEN
           ErrorMessage = 'XEVAPOUT: CALL BalanceElevAndTempAndEvap'; CALL PassMsg
        END IF
      
        CALL BalanceElevAndTempAndEvap(                               &
     &    rCV, Mf1,Mf2,Mxf,Ms1,Ms2,Mxs,Hd,                            &  ! param.
     &    P1, P2, P3, P4, P5, P6, P7, P8, P9, P10,                    &  ! param.
     &    DataIndex, Area,                                            &  ! cons.
     &    Day, Month, Year,                                           &  ! inputs
     &    CloudCover, WindSpeed, AirTemp, DewPtTemp,                  &  ! in.out.
     &    Runoff, Precipitation, Inflow, Outflow,                     &  ! inputs
     &    OtherTerms,                                                 &  ! input
     &    IceAmount, IceTemp, IceArea, IceDepth, StoredHeat,          &  ! in.out.
     &    AgeOfCurrentLayer, SurfaceElevation, SurfaceTemperature,    &  ! in.out.
     &    LakeEvaporation, NoIterations, Incident,                    &  ! outputs
     &    Reflect, NLWave, Latent, Sensible, Advection,               &  ! outputs
     &    .FALSE.)                                                       ! input
        IF (ErrorLevel .NE. 0) GOTO 899

        IF (PrintDebugInfo) THEN
           ErrorMessage = 'XEVAPOUT: Back from BalanceElevAndTempAndEvap'; CALL PassMsg
        END IF
      
        IF (MonitoringDepth .GT. 0.0) THEN                                    ! 24apr01
           IF (PrintDebugInfo) THEN
              ErrorMessage = 'XEVAPOUT: Do TempDepth stuff'; CALL PassMsg
           END IF
           !
           !  GetTemperatureProfile interprets the heat layer superposition 
           !  information in terms of a vertical temperature profile defined 
           !  at various points along the depth, where ProfileT(1) is the 
           !  temperature at the surface (depth = ProfileD(1) = 0) and
           !  ProfileT(K) is the water temperature at the bottom 
           !  (depth = ProfileD(K) = MaxDepth).
           !  On return, K is the number of valid entries in the ProfileT
           !  and ProfileD arrays.
           !
           CALL GetTemperatureProfile(rCV, Mf1, Mf2, Ms1, Ms2, Hd, P2, P3, &  ! 24apr01
     &        P4, P5, P6, P7, P8, DataIndex, ProfileT, ProfileD, K)           ! 24apr01
           M = 1                                                              ! 24apr01
           DO WHILE ((M .LE. K) .AND. (ProfileD(M) .LE. MonitoringDepth))     ! 24apr01
              M = M + 1                                                       ! 24apr01
           END DO                                                             ! 24apr01
           IF ((M .GT. K) .OR. (M .LE. 1)) THEN
              ErrorMessage = 'Temperature-Depth Interpolation Error'          ! 24apr01
              CALL PassMsg
              GOTO 898
           ELSE
              IF (M .EQ. K) THEN
                 MaxD = INT(ProfileD(M))
              ELSE
                 MaxD = INT(ProfileD(M-1))
              END IF
              MaxD = INT(MonitoringDepth)
              TempAtDepth(:) = -999.9
              DO I = 1, K
                 Temp  = ProfileT(I)           ! temperature of the entry, deg C
                 Depth = NINT(ProfileD(I))     ! depth of the entry, in meters
                 TempAtDepth(Depth) = Temp
              END DO
              
              !
              !  Fill any missing entries by interpolation
              !  Assumption is made that no more than 1 entry per depth
              !  exists (i.e. each ProfileD() entry is unique).
              !
              DO I = 2, MaxD-1
                 IF (TempAtDepth(I) .LT. 0.0) THEN
                    A = TempAtDepth(I-1)
                    J = I + 1
                    DO WHILE (TempAtDepth(J) .LT. 0) 
                       J = J + 1
                    END DO
                    IF (J .LE. MaxD) THEN
                       B = TempAtDepth(J)
                       Delta = (B-A) / (ProfileD(J)-ProfileD(I))    ! degrees per meter
                       TempAtDepth(I) = TempAtDepth(I-1) + Delta
                    END IF
                 END IF
              END DO
              
              !
              !  Write the entries to output file
              !
              DO I = 0, MaxD
                 WRITE(UnitTD, 4802) Year, Month, Day, I, TempAtDepth(I)
              END DO
              WRITE(UnitTD, *)
              
              !
              !  Computation for continuing
              !
              DD1 = MonitoringDepth - ProfileD(M-1)
              DD2 = ProfileD(M)     - ProfileD(M-1)
              TT1 = ProfileT(M)     - ProfileT(M-1)
              ST  = ProfileT(M-1)   + (DD1/DD2 * TT1)
           END IF
        ELSE                                                                 ! 24apr01
           ST = SurfaceTemperature                                           ! 24apr01
        END IF                                                               ! 24apr01
        SurfaceTemperature = ST                                              ! 08feb2013

        IF (PrintDebugInfo) THEN
           ErrorMessage = 'XEVAPOUT: Write to output file'; CALL PassMsg
        END IF
!        CALL LocalVarsToParm(EvapParms, Ext, Comment,                        &
!     &         P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,                               &
!     &         SequenceNumber, Finish, SurfaceTemperature, IceAmount,        &
!     &         IceArea, IceDepth, IceTemp, AgeOfCurrentLayer, NoOfLayers,    &
!     &         SurfTempFromLayer, TotalHeat, AgeOfLayer)

        WRITE(ZSU1, REC=RecNo1+DayNum, ERR=853)           &
     &    LakeEvaporation / Area *100.,                   & ! cm
     &    SurfaceTemperature,                             & ! deg. C.
     &    AirTemp,                                        & ! deg. C.
     &    VapPres(DewPtTemp),                             & ! mb
     &    WindSpeed,                                      & ! m/s
     &    CloudCover,                                     & ! frac.
     &    Incident,                                       & ! watt/m2
     &    Reflect,                                        & ! watt/m2
     &    NLWave,                                         & ! watt/m2
     &    Latent,                                         & ! watt/m2
     &    Sensible,                                       & ! watt/m2
     &    Advection,                                      & ! watt/m2
     &    IceTemp,                                        & ! deg. C.
     &    IceArea / Area,                                 & ! frac.
     &    IceDepth,                                       & ! m
     &    StoredHeat                                        ! cal

        IF (Pd .EQ. -10) THEN
!
!         Compute surface temperature contributions...
!
          IF (TotalHeat(NoOfLayers) .LE. Hd) THEN
            DO K = 1, NoOfLayers
              SurfTempFromLayer(K) = SurfTempFromLayer(K - 1)     &
     &             + (TotalHeat(K) - TotalHeat(K - 1))            &
     &             * MixingFunction(rCV, Mf1,Mf2, P2,P3,P4,P8, AgeOfLayer(K))
              IF (ErrorLevel .NE. 0) GOTO 899
            END DO
          ELSE
            DO K = 1, NoOfLayers
              SurfTempFromLayer(K) = SurfTempFromLayer(K - 1)             &
     &                + (TotalHeat(K) - TotalHeat(K - 1))                 &
     &                * MixingFunction(rCV, Ms1,Ms2, P5,P6,P7,P8, AgeOfLayer(K))
              IF (ErrorLevel .NE. 0) GOTO 899
            END DO
          END IF
          Xk = NoOfLayers
          WRITE(ZSU2, ERR=858) Xk
          DO K = NoOfLayers, 1, -1
            Xk = NINT(SurfTempFromLayer(K) * 100.0)
            WRITE(ZSU2, ERR=858) Xk
            Xk = NINT((TotalHeat(K) - TotalHeat(K - 1))                   &
     &           / (SurfTempFromLayer(K) - SurfTempFromLayer(K - 1))      &
     &           / rCV * 10000.)
            WRITE(ZSU2, ERR=858) Xk
          END DO
        END IF

        IF ((Pd .EQ. -9) .AND. (DayNum .GT. SpinUp)) THEN
          WRITE(ZSU2, REC=DayNum-SpinUp+ZHdr, ERR=858)     &
     &       LakeEvaporation / Area * 100.,            &    ! cm
     &       SurfaceTemperature,                       &    ! deg. C.
     &       AirTemp,                                  &    ! deg. C.
     &       VapPres(DewPtTemp),                       &    ! mb
     &       WindSpeed,                                &    ! m/s
     &       CloudCover,                               &    ! frac.
     &       Incident,                                 &    ! watt/m2
     &       Reflect,                                  &    ! watt/m2
     &       NLWave,                                   &    ! watt/m2
     &       Latent,                                   &    ! watt/m2
     &       Sensible,                                 &    ! watt/m2
     &       Advection,                                &    ! watt/m2
     &       IceTemp,                        &    ! deg. C.
     &       IceArea / Area,                 &    ! frac.
     &       IceDepth,                       &    ! m
     &       StoredHeat                                     ! cal
        END IF

      END DO

      SequenceNumber = SequenceNumber + 1
      WRITE(ZSU1, REC=1, ERR=853) M, M + N - 1

!
!     Close a bunch of files
!
      IF (Pd .NE. -10) THEN
        CLOSE (U3)
        CALL FileWasClosed(U3); IF (ErrorLevel .NE. 0) GOTO 899
      ELSE
        CLOSE (U4)
        CALL FileWasClosed(U4); IF (ErrorLevel .NE. 0) GOTO 899
      END IF

      IF (ZSU1 .GT. -1) THEN
        CLOSE (ZSU1)
        CALL FileWasClosed(ZSU1); IF (ErrorLevel .NE. 0) GOTO 899
      END IF

      IF (ZSU2 .GT. -1) THEN
        CLOSE (ZSU2)
        CALL FileWasClosed(ZSU2); IF (ErrorLevel .NE. 0) GOTO 899
      END IF

      IF (FileIsOpen(U7)) THEN
        CLOSE (U7)
        CALL FileWasClosed(U7);  IF (ErrorLevel .NE. 0) GOTO 899
      END IF

      IF (FileIsOpen(UnitTD)) THEN
         CLOSE(UnitTD)
         CALL FileWasClosed(UnitTD); IF (ErrorLevel .NE. 0) GOTO 899
      END IF
      
      IF (Pd .EQ. -10) THEN
         WRITE(ErrorMessage, 9006) 'ZSEVAP.' // LakeName, NumberOfDays
         CALL PassMsg
         WRITE(ErrorMessage, 9006) 'ZSTEMP.' // LakeName, NumberOfDays
         CALL PassMsg
      ELSE IF (Pd .EQ. -9) THEN
         WRITE(ErrorMessage, 9006) 'ZSEVAP.' // LakeName, NumberOfDays
         CALL PassMsg
         WRITE(ErrorMessage, 9006) 'ZSEVAPPR.' // LakeName, Length
         CALL PassMsg
      ELSE IF (Pd .EQ. 4) THEN
         WRITE(ErrorMessage, 9006) 'ZSEVAPPR.' // LakeName, NumberOfDays
         CALL PassMsg
      ELSE
         WRITE(ErrorMessage, 9006) 'ZSEV' // Yyyy // '.INT', NumberOfDays
         CALL PassMsg
      END IF
      SequenceNumber = SequenceNumber + NumberOfDays
      IF (Pd .EQ. -9 .AND. SequenceNumber .NE. BaseDate) THEN
         WRITE(ErrorMessage, 9019)
         CALL PassMsg
         GOTO 898
      END IF
      CALL SequenceDate(Day, Month, Year, SequenceNumber); IF (ErrorLevel .NE. 0) GOTO 899

      IF (Pd .EQ. -9 .OR. Pd .EQ. 4 .OR. Pd .EQ. 7) THEN
!
!        Write a new parameter file for use in continuing simulation.
!
         U8a = 81
         IF (Pd .EQ. -9) THEN
            File8a = TRIM(ThisLakeDir) // 'PARAMEVP.'//LakeName
         ELSE
            File8a = TRIM(ThisLakeDir) // 'EVPPARAM.'//LakeName
         END IF
         OPEN(UNIT=U8a, FILE=TRIM(File8a), STATUS='OLD', ERR=881)
         CALL FileWasOpened(12); IF (ErrorLevel .NE. 0) GOTO 899

         U8b = 82
         File8b = TRIM(ThisLakeDir) // 'NEWPARAM.'//LakeName
         OPEN(UNIT=U8b, FILE=TRIM(File8b), STATUS='NEW', ERR=886)
         CALL FileWasOpened(13); IF (ErrorLevel .NE. 0) GOTO 899

         IF (Pd .EQ. -9) Ext = 'PRV'               ! Prepare for subsequent uses
         IF (Pd .EQ. 4 .OR. Pd .EQ. 7) Ext = 'CLM'
         READ(U8a, 1011, ERR=882) Line
         I = LEN_TRIM(Line)
         Line2 = Ext // Line(4:I)                  ! so it will work with
         Line = Line2                              ! USACE 'af77' compiler
         WRITE(U8b, 1012, ERR=888) (Line(J:J), J = 1, I)
         DO Id = 1, 13
            READ(U8a, 1011, ERR=882) Line
            I = LEN_TRIM(Line)
            WRITE(U8b, 1012, ERR=888) (Line(J:J), J = 1, I)
         END DO
         WRITE(U8b, 1002, ERR=888) Day, Month, Year
         WRITE(U8b, 1002, ERR=888) Day, Month, Year      ! Dummy repeat for unknown end date

!
!        Since layers have been remixed since "SurfaceTemperature" was computed
!        from "StoredHeat", recompute SurfaceTemperature so that when read in
!        in a subsequent invocation of "XEVAPOUT", the same value for StoredHeat
!        will be computed.  Must preserve heat conservation across runs of this
!        program as is done during each run!
!
!        CALL ParmToLocalVars(EvapParms, P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,     &
!     &       SurfaceTemperature, IceAmount, IceArea, IceDepth, IceTemp,     &
!     &       AgeOfCurrentLayer, NoOfLayers, SurfTempFromLayer,              &
!     &       TotalHeat, AgeOfLayer)

         SurfaceTemperature = WaterSurfaceTemperature(StoredHeat,           &
     &            AgeOfCurrentLayer, Hd, Mf1, Mf2, Ms1, Ms2, rCV,           &
     &            P2, P3, P4, P5, P6, P7, P8)
         IF (ErrorLevel .NE. 0) GOTO 899

!         CALL LocalVarsToParm(EvapParms, P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,    &
!     &        SurfaceTemperature, IceAmount, IceArea, IceDepth, IceTemp,    &
!     &        AgeOfCurrentLayer, NoOfLayers, SurfTempFromLayer,             &
!     &        TotalHeat, AgeOfLayer)

         WRITE(U8b, 1009, ERR=888) SurfaceTemperature
         WRITE(U8b, 1009, ERR=888) IceAmount                 ! unused
         WRITE(U8b, 1009, ERR=888) IceArea / Area
         WRITE(U8b, 1009, ERR=888) IceDepth
         WRITE(U8b, 1009, ERR=888) IceTemp
         WRITE(U8b, 1009, ERR=888) 1.0                       ! unused (NumberOfBlocks)
         WRITE(U8b, 1009, ERR=888) AgeOfCurrentLayer
         WRITE(U8b, 1004, ERR=888) NoOfLayers
         DO I = 1, NoOfLayers
            WRITE(U8b, 1009, ERR=888) AgeOfLayer(I)
         END DO
         DO I = 1, NoOfLayers
            WRITE(U8b, 1009, ERR=888) SurfTempFromLayer(I)
         END DO
         DO I = 1, NoOfLayers
            WRITE(U8b, 1009, ERR=888) TotalHeat(I)
         END DO
         CLOSE (U8a)
         CALL FileWasClosed(U8a); IF (ErrorLevel .NE. 0) GOTO 899
         CLOSE (U8b)
         CALL FileWasClosed(U8b); IF (ErrorLevel .NE. 0) GOTO 899

         IF (Pd .EQ. -9) THEN
            CALL DeleteFile('PARAMEVP.' // LakeName)
            IF (ErrorLevel .NE. 0) GOTO 899
            CALL Rename('NEWPARAM.' // LakeName, 'PARAMEVP.' // LakeName)
            IF (ErrorLevel .NE. 0) GOTO 899
         ELSE IF (Pd .EQ. 7) THEN
            CALL DeleteFile('EVPPRM' // Yyyy // '.INT')
            IF (ErrorLevel .NE. 0) GOTO 899
            CALL Rename('NEWPARAM.'//LakeName,'EVPPRM'//Yyyy//'.INT')
            IF (ErrorLevel .NE. 0) GOTO 899
         ELSE
            CALL DeleteFile('EVPPARAM.' // LakeName)
            IF (ErrorLevel .NE. 0) GOTO 899
            CALL Rename('NEWPARAM.' // LakeName, 'EVPPARAM.' // LakeName)
            IF (ErrorLevel .NE. 0) GOTO 899
         END IF
      END IF

      IF (FileIsOpen(MissingDataUnitNo)) THEN
         CLOSE(MissingDataUnitNo)
         CALL FileWasClosed(MissingDataUnitNo); IF (ErrorLevel .NE. 0) GOTO 899
      END IF

!
!     Update the EBSNNM file
!
      U1 = 11
      File1 = TRIM(ThisLakeDir) // 'EBSNNM'
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', RECL=12,              &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      Pd = Pd + 1
      IF (Pd .EQ. -8) Pd = 0
      WRITE(U1, 1010, REC=2, ERR=813) Pd, Lf

!
!     Reset earliest date for data update to 1 day after this simulation.
!
      IF (Pd .EQ. 5) WRITE(U1, 1002, REC=8, ERR=813) Day, Month, Year, Lf
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(File1)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(File1)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'XEVAPOUT: Error writing to file ' // TRIM(File1)
      CALL PassMsg; GOTO 898

  831 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(File3)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(File3)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'XEVAPOUT: Error writing to file ' // TRIM(File3)
      CALL PassMsg; GOTO 898

  841 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(File4)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(File4)
      CALL PassMsg; GOTO 898
  843 ErrorMessage = 'XEVAPOUT: Error writing to file ' // TRIM(File4)
      CALL PassMsg; GOTO 898

  851 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(ZSFile1)
      CALL PassMsg; GOTO 898
  852 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(ZSFile1)
      CALL PassMsg; GOTO 898
  853 ErrorMessage = 'XEVAPOUT: Error writing to file ' // TRIM(ZSFile1)
      CALL PassMsg; GOTO 898

  856 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(ZSFile2)
      CALL PassMsg; GOTO 898
  857 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(ZSFile2)
      CALL PassMsg; GOTO 898
  858 ErrorMessage = 'XEVAPOUT: Error writing to file ' // TRIM(ZSFile2)
      CALL PassMsg; GOTO 898

  861 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(File6)
      CALL PassMsg; GOTO 898
  862 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(File6)
      CALL PassMsg; GOTO 898
  863 ErrorMessage = 'XEVAPOUT: Error writing to file ' // TRIM(File6)
      CALL PassMsg; GOTO 898

  871 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(File7)
      CALL PassMsg; GOTO 898
  872 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(File7)
      CALL PassMsg; GOTO 898
  873 ErrorMessage = 'XEVAPOUT: Error writing to file ' // TRIM(File7)
      CALL PassMsg; GOTO 898

  881 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(File8a)
      CALL PassMsg; GOTO 898
  882 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(File8a)
      CALL PassMsg; GOTO 898
  883 ErrorMessage = 'XEVAPOUT: Error writing to file ' // TRIM(File8a)
      CALL PassMsg; GOTO 898

  886 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(File8b)
      CALL PassMsg; GOTO 898
  887 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(File8b)
      CALL PassMsg; GOTO 898
  888 ErrorMessage = 'XEVAPOUT: Error writing to file ' // TRIM(File8b)
      CALL PassMsg; GOTO 898

  891 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(MissingDataFileName)
      CALL PassMsg; GOTO 898
  892 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(MissingDataFileName)
      CALL PassMsg; GOTO 898


!  806 ErrorMessage = 'XEVAPOUT: Error opening file '    // TRIM(File2)
!      CALL PassMsg; GOTO 898
!  807 ErrorMessage = 'XEVAPOUT: Error reading file '    // TRIM(File2)
!      CALL PassMsg; GOTO 898
!  808 ErrorMessage = 'XEVAPOUT: Error writing to file ' // TRIM(File2)
!      CALL PassMsg; GOTO 898




  898 ErrorLevel = 1
  899 CALL MakeErrorInt()
      GOTO 999

!
!  Force the closure of all open files then return.
!
 999  CALL CloseAllFiles()
      ErrStatus = ErrorLevel
      RETURN

!
!  FORMATs
!
 1001 FORMAT(A3, 8X)
 1002 FORMAT(2I3, I5, A1)
 1004 FORMAT(I11, A1)
 1005 FORMAT(E10.4E1, 2X, I1)
 1009 FORMAT(E13.7E2)
 1010 FORMAT(3X, I3, 5X, A1)
 1011 FORMAT(A80)
 1012 FORMAT(80A1)
 1073 FORMAT(A4)
 1996 FORMAT(/,/,/)
 1997 FORMAT(1X, I2, 6X, I2, 3X, I4, F10.3)

 4701 FORMAT('Bulk Transfer Coefficient values calculated within the Lake Thermodynamics Model.', /,   &
     &       'The coefficient is always calculated for both overWater and overIce.', /,                &
     &       '              OverWater    OverIce', /,                                                  &
     &       ' Year Mn Dy  Coefficient Coefficient')
 
 4801 FORMAT('Temperature/Depth values calculated within the Lake Thermodynamics Model.', /,   &
     &       ' Year Mn Dy Depth(m)  Temperature')
 4802 FORMAT(I5, 2I3, I9, F9.2) 
 
 9006 FORMAT('~^~File: ', A12, ' updated;', I6, ' days added.')
 9014 FORMAT('XEVAPOUT: Program run in invalid sequence.')
 9015 FORMAT('Illegal data file name extension, ', A3, ' given.')
 9016 FORMAT('Bad lake name in file eBSNNM; program aborted')
 9017 FORMAT('Lake evaporation data are unavailable; program aborted')
 9018 FORMAT('Lake heat balance data are unavailable; program aborted')
 9019 FORMAT('Internal program error; program aborted')
 9021 FORMAT('Unable to fill in missing data value(s) on ',                     &
     &       I2, '/', I2, '/', I4, ' (MM/DD/YYYY)!')
 9022 FORMAT('Attempt to read bad record number (',I0,') from input met file on ',       &
     &       I2, '/', I2, '/', I4, ' (MM/DD/YYYY)!')
 9700 FORMAT('~^~XEVAPOUT is ', I3, '% finished')
 9701 FORMAT('XEVAPOUT is ', I3, '% finished')

      END SUBROUTINE DO_XEVAP

END MODULE XEvapMain
