!
!     Written by TSH 9oct2007
!
!     This routine reads the monthly subbasin outflow timeseries data for the
!     Maumee River. Then we use the months specified by Stuart Ludsin in his
!     multiple regression equations to estimate yellow perch recruitment.  
!
!     We are producing 4 types of "forecasts" of perch recruitment based on outflows.
!     Stuart has defined the forecasts as follows:
!
!      #1 = millions of YP expected 2 years later in western Lake Erie (from Interagency ADMB)
!      #2 = millions of YP expected 2 years later in central Lake Erie (from Interagency ADMB)
!      #3 = catch per unit effort of age-0 YP (# individuals per 10 minute trawling) during
!           October in western Lake Erie (ODNR sites; all sites)
!      #4 = catch per unit effort of age-0 YP (# individuals per 10 minute trawling) during
!           October in central Lake Erie (ODNR sites; all sites)
!
!     The equations for each forecast are:
!       #1) YP = 10^[0.002180*FAM + 0.51878*LOG10(FM) - 1.014520] - 1
!       #2) YP = 10^[0.002248*FAM + 0.39248*LOG10(FM) - 0.681813] - 1
!       #3) YP = 10^[0.002990*FAM + 0.021355]
!       #4) YP = 10^[0.003759*FAM - 0.820126]
!
!       FAM = Mean Flow from Maumee River (cms) during April 1-30  +
!             Mean Flow from Maumee River (cms) during May 1-31.
!             Note that this is *NOT* the mean flow for April 1 - May 31.
!
!       FM  = Mean Flow from Maumee River (cms) during March 1-31.
!
!
!     We will produce the 4 forecasts as a deterministic value for each year contained 
!     in the historical data set (MONFLOW_PRV.TXT).  These can be used as verification/hindcasts.
!
!     We will also produce the 4 forecasts as a probabilistic outlook for the forecast period.  
!     Since each equation uses the outflows in April/May or March/April/May, the number of 
!     forecasts we can produce will depend on what is contained in the forecast period.  We will
!     only be able to produce forecasts when the required months are there, obviously.
!
!     We will read the data from:
!        MONFLOW_PRV.TXT    (monthly flows for the provisional data period)
!        MONFLOW_yyyy.TXT   (monthly flows for the outlook data period using met from year yyyy)
!
!     We will output 4 files, one per forecast type:
!        YP_Age2_WB.txt
!        YP_Age2_CB.txt
!        YP_Age0_WB.txt
!        YP_Age0_CB.txt
!
MODULE YellowPerchMaumee

   USE FcstUtil
   USE GlerlUtil

   INTEGER, PARAMETER :: Quantiles = 9      ! # of probabilistic values for each forecast
   INTEGER, PARAMETER :: MaxOtlks = 100     ! max # of outlook years (scenarios)

   INTEGER :: LogFileUnit
   REAL    :: MaumeeArea = 1.68060e10          ! subbasin area in square meters
   CHARACTER (LEN=50), DIMENSION(:), ALLOCATABLE :: FcstFileNames

   !
   !  Info from the OTLKWGTS.??? file is stored in these 2 arrays
   !   
   INTEGER, DIMENSION(MaxOtlks)  :: OtlkYr
   REAL,    DIMENSION(MaxOtlks)  :: OtlkWght

   !
   !  The percentages to use
   !
   REAL, DIMENSION(Quantiles), PARAMETER :: QntlPrcnt = (/0.05, 0.10, 0.20, 0.30, 0.50, 0.70, 0.80, 0.90, 0.95/)
   
   
CONTAINS

      SUBROUTINE DO_YP_Maumee(ErrStatus, HandleIn, StringMXD)
      IMPLICIT NONE
      INTEGER, INTENT(INOUT) :: ErrStatus
      INTEGER, INTENT(IN)    :: HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: StringMXD

      !
      !   iNSbsns = number of subbasins.
      !
      INTEGER (KIND=INTEGER2), PARAMETER :: iNSbsns = 21

      INTEGER (KIND=INTEGER2)  :: Skip2, Flw
      INTEGER :: I, J, F, Y, IOS, Mon, Year
      INTEGER :: SYr, SMn, EYr, EMn
      INTEGER :: HistSYr, HistSMn, HistEYr, HistEMn
      INTEGER :: OtlkSYr, OtlkSMn, OtlkEYr, OtlkEMn
      INTEGER :: NumOtlkFiles, NumYears, NumFcsts
      INTEGER :: DtValues(8)

      LOGICAL :: OK, DoMixed

      REAL :: FlowCM, FlowCms
      REAL :: FlowMar, FlowApr, FlowMay, FlowAprMay

      REAL, DIMENSION(:),     ALLOCATABLE :: ProbOtlks             !  Forecast corresponding to QntlPrcnt values
      REAL, DIMENSION(:,:),   ALLOCATABLE :: HistFlows
      REAL, DIMENSION(:,:,:), ALLOCATABLE :: FcstFlows

      !
      !  Perch???(Y) is the forecast for each input flow
      !     Y = 1..Number of input flow time-series 
      !            (for historical period, this is the number of years in the historical period)
      !            (for outlook period, this is the number of scenario years)
      !
      !  PerchOtlksIndividual(Otlk, I, 4) stores all of the individual perch forecasts from a particular flow set. 
      !     Otlk = 1..NumOtlkYears;  i.e. 1948, 1949, 1950, ..., 2005  (so there will likely be about 50 of them)
      !     I    = 1..NumForecastsFromEachFlowSeries
      !            (e.g. if we do an AHPS forecast starting 2/1/2007 and ending 1/31/2009
      !             then there are 2 perch forecasts [2007 & 2008] from each scenario year.)
      !     N = 1..4     1=PerchWB0, 2=PerchCB0, 3=PerchWB2, 4=PerchCB2
      !
      !  PerchOtlksProbabilistic(P, I, 4) is the forecast for a non-exceedence prob (P), Year, and Forecast type
      !     P = 1..9 corresponding to the ProbOtlks array
      !     I = 1..Number of forecasts from a particular input flow time-series 
      !         (e.g. if we do an AHPS forecast starting 2/1/2007 and ending 1/31/2009
      !          then there are 2 perch forecasts [2007 & 2008] from each scenario year.)
      !     N = 1..4     1=PerchWB0, 2=PerchCB0, 3=PerchWB2, 4=PerchCB2
      !
      REAL, DIMENSION(:),     ALLOCATABLE :: PerchWB0, PerchCB0, PerchWB2, PerchCB2  
      REAL, DIMENSION(:,:,:), ALLOCATABLE :: PerchOtlksIndividual, PerchOtlksProbabilistic
      
      CHARACTER (LEN=1)  :: Lf, Cr
      CHARACTER (LEN=2)  :: CrLf, SB
      CHARACTER (LEN=3)  :: BnS, Ext
      CHARACTER (LEN=4)  :: OtlkYear
      CHARACTER (LEN=9)  :: DateStr       ! e.g. 16Oct2007
      CHARACTER (LEN=13) :: Name, S
      CHARACTER (LEN=99) :: File21, File31, File41, FName
      CHARACTER (LEN=200) :: Line

      Lf = CHAR(10)
      Cr = CHAR(13)
      CrLf = Cr//Lf

      WinHandle = HandleIn
      ErrorLevel = 0
      ErrorMessage = ' '
      CALL ClearFilesOpen()


      LogFileUnit = GetFreeUnitNumber()
      OPEN(LogFileUnit, FILE='PerchDebug.txt', STATUS='REPLACE', IOSTAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = ' Unable to open the Perch forecast debug file. Aborting!'
         CALL PassMsg
         GOTO 898
      ENDIF
      CALL FileWasOpened(LogFileUnit)
      
!
!     Which weights file will we use?
!     By default we will use OTLKWGTS.ERI, but if either string
!     MIX or LEVEL (case-insensitive) is passed in StringArray, then we
!     will use OTLKWGTS.MXD instead.
!
      DoMixed = .FALSE.
      Line = TRIM(StringMXD)
      CALL Caps(Line); IF (ErrorLevel .NE. 0) GOTO 899
      IF (INDEX(Line, 'MIX') .GT. 0) DoMixed = .TRUE.
      IF (INDEX(Line, 'LEVEL') .GT. 0) DoMixed = .TRUE.
      
!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. 'ERI') ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for ERIE!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Build a list of the forecast flow files.
!     The filenames (without path info) will be stored in the global array FcstFileNames.
!
      WRITE(LogFileUnit, *) 'BuildListOfForecastFiles...'
      CALL FLUSH(LogFileUnit)
      CALL BuildListOfForecastFiles(NumOtlkFiles)
      IF (NumOtlkFiles .EQ. 0) THEN
         ErrorMessage = ' No outlook data flow files found!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      END IF

!
!     Read header info from the first forecast flow file.  It's assumed that they all have
!     the same period of record.  This will be used when allocating the RAM needed to store
!     the forecast flow values.
!
      WRITE(LogFileUnit, *) 'Read header from first forecast flow file...'
      CALL FLUSH(LogFileUnit)
      File21 = TRIM(ThisLakeDir) // TRIM(FcstFileNames(1))
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      READ(21,    *, ERR=822)
      READ(21, 2101, ERR=822) OtlkSYr, OtlkSMn
      READ(21, 2101, ERR=822) OtlkEYr, OtlkEMn
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

      
!
!     Read the header info for the historical period.  We only need the values
!     for subbasin 6 (Maumee River).  The values are in centimeters over the subbasin.
!
      WRITE(LogFileUnit, *) 'Read MONFLOW_PRV.TXT...'
      CALL FLUSH(LogFileUnit)
      File21 = TRIM(ThisLakeDir) // 'MONFLOW_PRV.TXT'
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      READ(21,    *, ERR=822)
      READ(21, 2101, ERR=822) HistSYr, HistSMn
      READ(21, 2101, ERR=822) HistEYr, HistEMn
      READ(21,    *, ERR=822)
      READ(21,    *, ERR=822)
      
!
!     Allocate the monthly HistFlows array to the correct size.  
!     For simplicity, just go ahead and allocate full years.
!
      WRITE(LogFileUnit, *) 'Allocating HistFlows...'
      CALL FLUSH(LogFileUnit)
      ALLOCATE(HistFlows(HistSYr:HistEYr, 12), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = ' Error allocating RAM for historical monthly flow values!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      END IF
      HistFlows(:,:) = -9.99

!
!     Allocate the monthly FcstFlows array to the correct size.  
!     For simplicity, just go ahead and allocate full years again.
!
      WRITE(LogFileUnit, *) 'Allocating FcstFlows:'
      CALL FLUSH(LogFileUnit)
      ALLOCATE(FcstFlows(NumOtlkFiles, OtlkSYr:OtlkEYr, 12), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = ' Error allocating RAM for forecast monthly flow values!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      END IF
      FcstFlows(:,:,:) = -9.99
      
!
!     Read/store the historical flow data.  We only need the values for subbasin 6 (Maumee River).
!     If the date happens to be within the period of record for the forecast flows, then go ahead
!     and store it there as well.  That way we have it when doing the forecast for the perch.
!     The values are in centimeters over the subbasin.
!     
      WRITE(LogFileUnit, *) 'Historical flows:'
      CALL FLUSH(LogFileUnit)
      READ(21, 2105, IOSTAT=IOS) Year, Mon, FlowCM
      DO WHILE (IOS .EQ. 0)
         IF ((Year .LT. HistSYr) .OR. (Year .GT. HistEYr)) THEN
            ErrorMessage = ' Year value inconsistent with header info!  Forecast aborted!'
            CALL PassMsg
            GOTO 898
         END IF
         IF (((Mon .LT. 1) .OR. (Mon .GT. 12)) .OR.                         &
     &       ((Year .EQ. HistSYr) .AND. (Mon .LT. HistSMn)) .OR.            &
     &       ((Year .EQ. HistEYr) .AND. (Mon .GT. HistEMn))) THEN
            ErrorMessage = ' Month value inconsistent with header info!  Forecast aborted!'
            CALL PassMsg
            GOTO 898
         END IF
         HistFlows(Year, Mon) = FlowCM
         IF ((Year .GE. OtlkSYr) .AND. (Year .LE. OtlkEYr)) THEN
            WRITE(LogFileUnit, *) '   Assigning flow for year,mon=',Year,Mon,' to the FcstFlows array'
            DO I=1, NumOtlkFiles
               FcstFlows(I, Year, Mon) = FlowCM
            END DO
         END IF
         READ(21, 2105, IOSTAT=IOS) Year, Mon, FlowCM
      END DO
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Now read/store the forecast flows.
!
      DO I=1, NumOtlkFiles
         File21 = TRIM(ThisLakeDir) // TRIM(FcstFileNames(I))
         OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', ERR=821)
         CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
         READ(21,    *, ERR=822)
         READ(21, 2101, ERR=822) SYr, SMn
         READ(21, 2101, ERR=822) EYr, EMn
         READ(21,    *, ERR=822)
         READ(21,    *, ERR=822)

         READ(21, 2105, IOSTAT=IOS) Year, Mon, FlowCM
         DO WHILE (IOS .EQ. 0)
            IF ((Year .LT. SYr) .OR. (Year .GT. EYr)) THEN
               ErrorMessage = ' Year value inconsistent with header info (outlook)!  Forecast aborted!'
               CALL PassMsg
               GOTO 898
            END IF
            IF (((Mon .LT. 1) .OR. (Mon .GT. 12)) .OR.                           &
     &          ((Year .EQ. SYr) .AND. (Mon .LT. SMn)) .OR.                      &
     &          ((Year .EQ. EYr) .AND. (Mon .GT. EMn))) THEN
               ErrorMessage = ' Month value inconsistent with header info (outlook)!  Forecast aborted!'
               CALL PassMsg
               GOTO 898
            END IF
            FcstFlows(I, Year, Mon) = FlowCM
            READ(21, 2105, IOSTAT=IOS) Year, Mon, FlowCM
         END DO
         CLOSE (21)
         CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899
      END DO         
         
!
!     Allocate the RAM for the arrays that will store the perch forecasts for the historical period.
!
      NumYears = HistEYr - HistSYr + 1
      ALLOCATE(PerchWB0(HistSYr:HistEYr), PerchCB0(HistSYr:HistEYr), PerchWB2(HistSYr:HistEYr), PerchCB2(HistSYr:HistEYr), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = ' Error allocating RAM for historical perch forecast values!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      END IF
      PerchWB0(:) = -9.99
      PerchCB0(:) = -9.99
      PerchWB2(:) = -9.99
      PerchCB2(:) = -9.99
      
!
!     For each year of the historical period, generate a deterministic perch forecast
!
      DO Y = HistSYr, HistEYr
         !
         !  Calculate the flow in cms for each month of interest
         !
         IF (HistFlows(Y,3) .GT. -1) THEN
            FlowMar = (HistFlows(Y,3) / 100.0) * MaumeeArea / (86400*31)    ! cm -> m3/s
         ELSE
            FlowMar = -99.9
         END IF
         IF ((HistFlows(Y,4) .GT. -1) .AND. (HistFlows(Y,5) .GT. -1)) THEN
            FlowApr = (HistFlows(Y,4) / 100.0) * MaumeeArea / (86400*30)    ! cm -> m3/s
            FlowMay = (HistFlows(Y,5) / 100.0) * MaumeeArea / (86400*31)    ! cm -> m3/s
            FlowAprMay = FlowApr + FlowMay                                  ! sum of the 2 flow rates
         ELSE
            FlowAprMay = -99.9
         END IF

         WRITE(LogFileUnit, 9901) Y, (HistFlows(Y,I), I=3,5), FlowMar, FlowApr, FlowMay
         
         !
         !  Calculate the perch forecast values
         !
         PerchWB0(Y) = ComputePerchWestBasinAgeZero(FlowAprMay)
         PerchCB0(Y) = ComputePerchCentralBasinAgeZero(FlowAprMay)
         PerchWB2(Y) = ComputePerchWestBasinAgeTwo(FlowMar, FlowAprMay)
         PerchCB2(Y) = ComputePerchCentralBasinAgeTwo(FlowMar, FlowAprMay)
      END DO
      
!
!     Write the deterministic forecast to the output file
!
      File31 = TRIM(ThisLakeDir) // 'YellowPerch_Hist.txt'
      OPEN(UNIT=31, FILE=TRIM(File31), STATUS='REPLACE', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899
      CALL DATE_AND_TIME(VALUES=DtValues)
      WRITE(DateStr, '(I2.2, A3, I4.4)') DtValues(3), MonthName3(DtValues(2)), DtValues(1)
      WRITE(31, 3101, ERR=833) DateStr
      WRITE(31, 3102, ERR=833)
      WRITE(31, 3111, ERR=833)
      WRITE(31, 3112, ERR=833)
      WRITE(31, 3113, ERR=833)
      WRITE(31, 3114, ERR=833)
      WRITE(31, 3119, ERR=833)
      DO Year = HistSYr, HistEYr
         WRITE(LogFileUnit, 9901) Year, (HistFlows(Year,I), I=3,5), FlowMar, FlowApr, FlowMay
         WRITE(LogFileUnit, 9902) Year, PerchWB0(Year), PerchCB0(Year), PerchWB2(Year), PerchCB2(Year)
         WRITE(31, 3120, ERR=833) Year, PerchWB0(Year), PerchCB0(Year), PerchWB2(Year), PerchCB2(Year)
      END DO
      CLOSE (31)
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899      

!
!     Free the RAM allocated for holding the historical perch flow forecasts
!      
      IF (ALLOCATED(PerchWB0))  DEALLOCATE(PerchWB0,  STAT=IOS)
      IF (ALLOCATED(PerchCB0))  DEALLOCATE(PerchCB0,  STAT=IOS)
      IF (ALLOCATED(PerchWB2))  DEALLOCATE(PerchWB2,  STAT=IOS)
      IF (ALLOCATED(PerchCB2))  DEALLOCATE(PerchCB2,  STAT=IOS)
            
!
!     Allocate the RAM for the arrays that will store the set of perch 
!     forecasts for a single year.  i.e. All of the forecasts for a particular
!     year (e.g. 2007) based on the N input flow series.
!
      F = NumOtlkFiles
      ALLOCATE(PerchWB0(F), PerchCB0(F), PerchWB2(F), PerchCB2(F), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = ' Error allocating RAM for forecast perch forecast values!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      END IF
      PerchWB0(:) = -9.99
      PerchCB0(:) = -9.99
      PerchWB2(:) = -9.99
      PerchCB2(:) = -9.99
      
      !
      !  This array is for each of the forecasts.  The final index is:
      !    1 = Western Basin year 0
      !    2 = Central Basin year 0
      !    3 = Western Basin year 2
      !    4 = Central Basin year 2
      !
      ALLOCATE(PerchOtlksIndividual(NumOtlkFiles, OtlkSYr:OtlkEYr, 4), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = ' Error allocating RAM for individual perch forecast values!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      END IF
      PerchOtlksIndividual(:,:,:) = -9.99

      !
      !  This array holds the probabilistic values for each of the forecasts
      !
      ALLOCATE(PerchOtlksProbabilistic(Quantiles, OtlkSYr:OtlkEYr, 4), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = ' Error allocating RAM for probabilistic perch forecast values!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      END IF
      PerchOtlksProbabilistic(:,:,:) = -9.99

      !
      !  This array holds the values for a single forecast
      !
      ALLOCATE(ProbOtlks(Quantiles), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = ' Error allocating RAM for perch forecast samples!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      END IF
      ProbOtlks(:) = -9.99
      
!
!     Now compute the perch forecasts for the outlook period.
!
      DO Y = OtlkSYr, OtlkEYr                ! date being forecasted
         DO I = 1, NumOtlkFiles                  ! i.e. 1 to the # of forecast scenarios
            IF (FcstFlows(I, Y, 3) .GT. -1) THEN
               FlowMar = (FcstFlows(I,Y,3) / 100.0) * MaumeeArea / (86400*31)    ! cm -> m3/s
            ELSE
               FlowMar = -99.9
            END IF
            IF ((FcstFlows(I,Y,4) .GT. -1) .AND. (FcstFlows(I,Y,5) .GT. -1)) THEN
               FlowApr = (FcstFlows(I,Y,4) / 100.0) * MaumeeArea / (86400*30)    ! cm -> m3/s
               FlowMay = (FcstFlows(I,Y,5) / 100.0) * MaumeeArea / (86400*31)    ! cm -> m3/s
               FlowAprMay = FlowApr + FlowMay                                    ! sum of the 2 flow rates
            ELSE
               FlowAprMay = -99.9
            END IF
            PerchWB0(I) = ComputePerchWestBasinAgeZero(FlowAprMay)
            PerchCB0(I) = ComputePerchCentralBasinAgeZero(FlowAprMay)
            PerchWB2(I) = ComputePerchWestBasinAgeTwo(FlowMar, FlowAprMay)
            PerchCB2(I) = ComputePerchCentralBasinAgeTwo(FlowMar, FlowAprMay)
         END DO

         !
         !  Generate the probabilistic forecasts from the collected set of perch forecasts.
         !  Save the results in PerchOtlksProbabilistic()
         !
         CALL ComputeProbabilisticOutlook(PerchWB0, DoMixed, ProbOtlks)
         PerchOtlksProbabilistic(:,Y,1) = ProbOtlks(:)
         CALL ComputeProbabilisticOutlook(PerchCB0, DoMixed, ProbOtlks)
         PerchOtlksProbabilistic(:,Y,2) = ProbOtlks(:)
         CALL ComputeProbabilisticOutlook(PerchWB2, DoMixed, ProbOtlks)
         PerchOtlksProbabilistic(:,Y,3) = ProbOtlks(:)
         CALL ComputeProbabilisticOutlook(PerchCB2, DoMixed, ProbOtlks)
         PerchOtlksProbabilistic(:,Y,4) = ProbOtlks(:)
      END DO
      
!
!     Open the output file and write the header info
!
      File41 = TRIM(ThisLakeDir) // 'YellowPerch_Otlk.txt'
      OPEN(UNIT=41, FILE=TRIM(File41), STATUS='REPLACE', ERR=841)
      CALL FileWasOpened(41); IF (ErrorLevel .NE. 0) GOTO 899
      CALL DATE_AND_TIME(VALUES=DtValues)
      WRITE(DateStr, '(I2.2, A3, I4.4)') DtValues(3), MonthName3(DtValues(2)), DtValues(1)
      WRITE(41, 4101, ERR=843) DateStr
      WRITE(41, 4102, ERR=843)
      WRITE(41, 4111, ERR=843)
      WRITE(41, 4112, ERR=843)
      WRITE(41, 4113, ERR=843)
      WRITE(41, 4114, ERR=843)
      WRITE(41, 4115, ERR=843)
      WRITE(41, 4116, ERR=843)
      WRITE(41, 4119, ERR=843) (NINT(QntlPrcnt(I)*100), I=1,Quantiles)

!
!     Write out the probabilistic perch forecasts
!
      DO Year = OtlkSYr, OtlkEYr
         IF (PerchOtlksProbabilistic(1,Year,1) .GT. -1) THEN
            WRITE(41, 4120, ERR=843) 'WB_Age_0', Year, (PerchOtlksProbabilistic(I,Year,1), I=1,Quantiles)
         END IF
      END DO
      DO Year = OtlkSYr, OtlkEYr
         IF (PerchOtlksProbabilistic(1,Year,2) .GT. -1) THEN
            WRITE(41, 4120, ERR=843) 'CB_Age_0', Year, (PerchOtlksProbabilistic(I,Year,2), I=1,Quantiles)
         END IF
      END DO
      DO Year = OtlkSYr, OtlkEYr
         IF (PerchOtlksProbabilistic(1,Year,3) .GT. -1) THEN
            WRITE(41, 4120, ERR=843) 'WB_Age_2', Year, (PerchOtlksProbabilistic(I,Year,3), I=1,Quantiles)
         END IF
      END DO
      DO Year = OtlkSYr, OtlkEYr
         IF (PerchOtlksProbabilistic(1,Year,4) .GT. -1) THEN
            WRITE(41, 4120, ERR=843) 'CB_Age_2', Year, (PerchOtlksProbabilistic(I,Year,4), I=1,Quantiles)
         END IF
      END DO
      CLOSE(41)      

!
!     Release the allocated RAM
!
      GOTO 999

!
!  Handlers for I/O errors
!
  821 ErrorMessage = 'YP_Maumee: Error opening file '   //TRIM(File21)
      CALL PassMsg; GOTO 898
  822 ErrorMessage = 'YP_Maumee: Error reading file '   //TRIM(File21)
      CALL PassMsg; GOTO 898
  823 ErrorMessage = 'YP_Maumee: Error writing to file '//TRIM(File21)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'YP_Maumee: Error opening file '   //TRIM(File31)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'YP_Maumee: Error reading file '   //TRIM(File31)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'YP_Maumee: Error writing to file '//TRIM(File31)
      CALL PassMsg; GOTO 898
  841 ErrorMessage = 'YP_Maumee: Error opening file '   //TRIM(File41)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'YP_Maumee: Error reading file '   //TRIM(File41)
      CALL PassMsg; GOTO 898
  843 ErrorMessage = 'YP_Maumee: Error writing to file '//TRIM(File41)
      CALL PassMsg; GOTO 898

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_YP_Maumee...'
      CALL PassMsg
      GOTO 999


!
!  Final cleanup and return
!
  999 CALL CloseAllFiles()
      IF (ALLOCATED(PerchWB0))   DEALLOCATE(PerchWB0,  STAT=IOS)
      IF (ALLOCATED(PerchCB0))   DEALLOCATE(PerchCB0,  STAT=IOS)
      IF (ALLOCATED(PerchWB2))   DEALLOCATE(PerchWB2,  STAT=IOS)
      IF (ALLOCATED(PerchCB2))   DEALLOCATE(PerchCB2,  STAT=IOS)
      IF (ALLOCATED(HistFlows))  DEALLOCATE(HistFlows, STAT=IOS)
      IF (ALLOCATED(FcstFlows))  DEALLOCATE(FcstFlows, STAT=IOS)
      IF (ALLOCATED(PerchOtlksIndividual))    DEALLOCATE(PerchOtlksIndividual, STAT=IOS)
      IF (ALLOCATED(PerchOtlksProbabilistic)) DEALLOCATE(PerchOtlksProbabilistic, STAT=IOS)

      ErrStatus = ErrorLevel
      RETURN
!
!  FORMAT statements
!
 2101 FORMAT(5X, I4, I3)
 2105 FORMAT(I4, I3, 30X, F6.2)
      
 3101 FORMAT('Deterministic Yellow Perch forecast for Lake Erie based on Maumee River', /, &
     &       'flows as modeled by the GLERL AHPS package on ', A9)
 3102 FORMAT('The perch forecast methodology is developed by Stuart Ludsin. The forecasts are defined as follows:')
 3111 FORMAT(' WB_Age_0 = catch per unit effort of age-0 YP (# individuals per 10 minutes trawling)', /,  &
     &       '            during October in western Lake Erie (ODNR sites; all sites)')
 3112 FORMAT(' CB_Age_0 = catch per unit effort of age-0 YP (# individuals per 10 minutes trawling)', /,  &
     &       '            during October in central Lake Erie (ODNR sites; all sites)')
 3113 FORMAT(' WB_Age_2 = millions of YP expected 2 years later in western Lake Erie (from Interagency ADMB)')
 3114 FORMAT(' CB_Age_2 = millions of YP expected 2 years later in central Lake Erie (from Interagency ADMB)')
 3119 FORMAT('Year   WB_Age_0   CB_Age_0   WB_Age_2   CB_Age_2')
 3120 FORMAT(I4, 4F11.2)
 
 4101 FORMAT('Probabilistic Yellow Perch forecast for Lake Erie based on Maumee River', /, &
     &       'flows as modeled by the GLERL AHPS package on ', A9)
 4102 FORMAT('The perch forecast methodology is developed by Stuart Ludsin. The forecasts are defined as follows:')
 4111 FORMAT(' WB_Age_0 = catch per unit effort of age-0 YP (# individuals per 10 minutes trawling)', /,  &
     &       '            during October in western Lake Erie (ODNR sites; all sites)')
 4112 FORMAT(' CB_Age_0 = catch per unit effort of age-0 YP (# individuals per 10 minutes trawling)', /,  &
     &       '            during October in central Lake Erie (ODNR sites; all sites)')
 4113 FORMAT(' WB_Age_2 = millions of YP expected 2 years later in western Lake Erie (from Interagency ADMB)')
 4114 FORMAT(' CB_Age_2 = millions of YP expected 2 years later in central Lake Erie (from Interagency ADMB)')
 4115 FORMAT('Each line contains the probabilistic forecast for a particular forecast type and year.')
 4116 FORMAT('The probabilistic values are the non-exceedance probabilities.')
 4119 FORMAT('Type     Year ', 9(I8,'%'))
 4120 FORMAT(A8, 1X, I4, 1X, 9F9.3)

 9001 FORMAT('~^~Extracting outflow for subbasin ', A3, A2) 
 
 9901 FORMAT('Year=',I4, '  HistFlows(cm; M,A,M)=', 3F9.4, '  Flows(cms; M,A,M)=', 3F9.4)
 9902 FORMAT('Year=',I4, '  PerchVals:  WB0(c/e)=', F7.3, '  CB0(c/e)=', F7.3, '  WB2(mil)=', F7.3, '  CB2(mil)=', F7.3)
 9910 FORMAT('IOS, Y,M,Flw=', 3I5, F7.2)
         

      END SUBROUTINE DO_YP_Maumee

      
!------------------------------------------------------------
!  Note that this routine allocates RAM for the list of file names.
!  That RAM must be released by the calling routine.
!
      SUBROUTINE BuildListOfForecastFiles(NumFiles)
      IMPLICIT NONE
      INTEGER, INTENT(INOUT) :: NumFiles
      INTEGER :: I, IOS, UNum, Y
      LOGICAL :: FileThere
      CHARACTER (LEN=50) :: FName

      !
      !  Build a list of the files that exist in the current directory and
      !  are named MONFLOW_yyyy.TXT.
      !  This would usually be done with something like:
      !      CALL SYSTEM('DIR/B/ON MONFLOW_????.TXT > filename.txt')
      !  but I want to avoid using 'CALL SYSTEM'.
      !
      UNum = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=UNum, FILE='_M_F_O.TXT', STATUS='REPLACE', ERR=811)
      CALL FileWasOpened(UNum); IF (ErrorLevel .NE. 0) GOTO 899

      NumFiles = 0
      DO Y = 1800, 2199
         WRITE(FName, 9001) Y
         INQUIRE(FILE=TRIM(FName), EXIST=FileThere)
         IF (FileThere) THEN
            WRITE(UNum, 9002) TRIM(FName)
            NumFiles = NumFiles + 1
         END IF
      END DO
      CLOSE(UNum, IOSTAT=IOS)
      CALL FileWasClosed(UNum); IF (ErrorLevel .NE. 0) GOTO 899

      !
      !  Allocate the RAM for the list of names
      !
      ALLOCATE(FcstFileNames(NumFiles), STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = ' Error allocating RAM for forecast file names!  Forecast aborted!'
         CALL PassMsg
         GOTO 898
      END IF
      FcstFileNames(:) = ''
      
      !
      !  Now process the file we just created in order to store the 
      !  file names in the array.
      !      
      UNum = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=UNum, FILE='_M_F_O.TXT', STATUS='OLD', IOSTAT=IOS)
      IF (IOS .NE. 0) RETURN
      CALL FileWasOpened(UNum); IF (ErrorLevel .NE. 0) GOTO 899

      I = 0
      READ(UNum, 9002, IOSTAT=IOS) FName
!      WRITE(ErrorMessage, 9008) UNum, I, IOS;  CALL PassMsg
      DO WHILE (IOS .EQ. 0)
         I = I + 1
!         WRITE(ErrorMessage, 9009) UNum, I, IOS;  CALL PassMsg
         FcstFileNames(I) = FName
         READ(UNum, 9002, IOSTAT=IOS) FName
      END DO
      CLOSE(UNum, STATUS='DELETE')
      CALL FileWasClosed(UNum); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Handlers for I/O errors
!
 811  ErrorMessage = 'Error opening file _M_F_O.TXT'
      CALL PassMsg; GOTO 898
 812  ErrorMessage = 'Error reading file _M_F_O.TXT'   
      CALL PassMsg; GOTO 898
 813  ErrorMessage = 'Error writing to file _M_F_O.TXT'
      CALL PassMsg; GOTO 898

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : BuildListOfForecastFiles...'
      CALL PassMsg

 999  RETURN

 8800 FORMAT(A)
 9001 FORMAT('MONFLOW_', I4.4, '.TXT')
 9002 FORMAT(A16)
 9008 FORMAT('YP: BuildList;  UNum, I, IOS=', 3I5, '  - (initial read)')
 9009 FORMAT('YP: BuildList;  UNum, I, IOS=', 3I5)
 
      END SUBROUTINE BuildListOfForecastFiles

!------------------------------------------------------------
!  Return the "catch per unit effort of age-0 yellow perch (#individuals per 10 minute trawling)
!  during October in Western Basin of Lake Erie (ODNR sites; all sites)" based on Maumee River
!  flow during April and May of that same year (i.e. 5-6 months prior).
!
!  YP = 10^[0.002990*FAM + 0.021355]
!
      REAL FUNCTION ComputePerchWestBasinAgeZero(FlowAprMay)
      IMPLICIT   NONE
      REAL, INTENT(IN) :: FlowAprMay      
      REAL  :: Tmp
      
      IF (FlowAprMay .LT. 0) THEN
         ComputePerchWestBasinAgeZero = -9.99
         RETURN
      END IF
      
      WRITE(LogFileUnit, *) 'ComputePerchWestBasinAgeZero:     Flow=', FlowAprMay
      
      Tmp = 0.002990 * FlowAprMay + 0.021355
      ComputePerchWestBasinAgeZero = (10 ** Tmp)
      
      END FUNCTION ComputePerchWestBasinAgeZero      
      
!------------------------------------------------------------
!  Return the "catch per unit effort of age-0 yellow perch (#individuals per 10 minute trawling)
!  during October in Central Basin of Lake Erie (ODNR sites; all sites)" based on Maumee River
!  flow during April and May of that same year (i.e. 5-6 months prior).
!
!     YP = 10^[0.003759*FAM - 0.820126]
!
      REAL FUNCTION ComputePerchCentralBasinAgeZero(FlowAprMay)
      IMPLICIT   NONE
      REAL, INTENT(IN) :: FlowAprMay      
      REAL  :: Tmp

      IF (FlowAprMay .LT. 0) THEN
         ComputePerchCentralBasinAgeZero = -9.99
         RETURN
      END IF

      WRITE(LogFileUnit, *) 'ComputePerchCentralBasinAgeZero:  Flow=', FlowAprMay

      Tmp = 0.003759 * FlowAprMay - 0.820126
      ComputePerchCentralBasinAgeZero = (10 ** Tmp)
      
      END FUNCTION ComputePerchCentralBasinAgeZero
      
!------------------------------------------------------------
!  Return the "millions of YP expected two years later in western Lake Erie (from Interagency ADMB)"
!  based on Maumee River flow during March, April and May.
!
!   P = 10^[0.002180*FAM + 0.51878*LOG10(FM) - 1.014520]
!
      REAL FUNCTION ComputePerchWestBasinAgeTwo(FlowMar, FlowAprMay)
      IMPLICIT   NONE
      REAL, INTENT(IN) :: FlowMar, FlowAprMay
      REAL  :: Tmp

      IF ((FlowMar .LT. 0) .OR. (FlowAprMay .LT. 0)) THEN
         ComputePerchWestBasinAgeTwo = -9.99
         RETURN
      END IF

      Tmp = 0.002180*FlowAprMay + 0.51878*LOG10(FlowMar) - 1.01452
      ComputePerchWestBasinAgeTwo = (10 ** Tmp)

      END FUNCTION ComputePerchWestBasinAgeTwo

!------------------------------------------------------------
!
!     YP = 10^[0.002248*FAM + 0.39248*LOG10(FM) - 0.681813]
!
      
      REAL FUNCTION ComputePerchCentralBasinAgeTwo(FlowMar, FlowAprMay)
      IMPLICIT   NONE
      REAL, INTENT(IN) :: FlowMar, FlowAprMay
      REAL  :: Tmp

      IF ((FlowMar .LT. 0) .OR. (FlowAprMay .LT. 0)) THEN
         ComputePerchCentralBasinAgeTwo = -9.99
         RETURN
      END IF
      
      Tmp = 0.002248*FlowAprMay + 0.39248*LOG10(FlowMar) - 0.681813
      ComputePerchCentralBasinAgeTwo = (10 ** Tmp)

      END FUNCTION ComputePerchCentralBasinAgeTwo
      
!------------------------------------------------------------
      SUBROUTINE ComputeProbabilisticOutlook(PerchFcsts, DoMixed, ProbOtlks)
      IMPLICIT  NONE
      REAL, DIMENSION(:), INTENT(IN)  :: PerchFcsts
      LOGICAL,            INTENT(IN)  :: DoMixed
      REAL, DIMENSION(:), INTENT(OUT) :: ProbOtlks

      INTEGER :: I, J
      INTEGER :: OutLks, WeightsCount, PFCount
      LOGICAL :: OK
      REAL    :: R4
      CHARACTER (LEN=3) :: Ext
      
      INTEGER, DIMENSION(MaxOtlks)  :: B
      REAL,    DIMENSION(MaxOtlks)  :: A, Y
      
      !
      !  Get the outlook weights from the OTLKWGTS.??? file
      !
      Ext = 'ERI'
      IF (DoMixed) Ext = 'MXD'
      OK = GetOtlkWghts(Ext, WeightsCount); IF (ErrorLevel .NE. 0) GOTO 899

      !
      !  Verify that the number of perch forecasts in the sample is compatible
      !  with the weights read from the file.
      !
      PFCount = UBOUND(PerchFcsts, 1)
      IF (WeightsCount .NE. PFCount) THEN
         ErrorMessage = 'Incompatibility between weights file and number of perch forecasts.'; CALL PassMsg
         WRITE(ErrorMessage, 9001) WeightsCount;  CALL PassMsg
         WRITE(ErrorMessage, 9002) PFCount;       CALL PassMsg
         GOTO 898
      END IF
      
      !
      !  Assign the valid data (for each of the outlooks) to temporary 
      !  array for processing.
      !  PerchFcsts will have one entry for each possible outlook year, but some
      !  of the entries may be missing/invalid (-9.99).  For example, if the first
      !  outlook year was 2005, but the forecast doesn't start until June of
      !  2005, then the Perch forecast for 2005 will be missing.  In this case,
      !  PerchFcsts(1,n) will be -9.99, and since we don't want to use this value
      !  in our sample, we will skip it and not include it in the arrays.  This
      !  also means that the number of valid outlooks will be reduced by one.
      !  We need to ensure that the B array has the right year so that we use the right
      !  weight.  That's why we assign "I", not "Outlks".  B will be used as an index
      !  into the Outlks array
      !
      Outlks = 0
      DO I = 1, PFCount
         IF (PerchFcsts(I) .GT. -1) THEN
            Outlks = Outlks + 1
            A(Outlks) = PerchFcsts(I)
            B(Outlks) = I
         END IF
      END DO
            
      !
      !  Sort sample A; B(I) will become the original array index of the Ith
      !  value in the reordered array.
      !
      CALL SortTwo(A, B, Outlks); IF (ErrorLevel .NE. 0) GOTO 899

      !
      !  Compute sample cumulative frequency distribution.
      !
      R4 = 0
      DO I = 1, Outlks
         IF (OtlkWght(B(I)) .GE. 0.000001) THEN                   ! TSH 25apr03
            R4 = R4 + OtlkWght(B(I))
            Y(I) = R4 / Outlks
         END IF
      END DO

      !
      !  Interpolate c.d.f. for selected quantiles.
      !  ProbOtlks() will contain the weighted results for output.
      !
      I = 1
      DO J = 1, Quantiles
         DO WHILE ((I .LE. Outlks) .AND. (Y(I) .LT. QntlPrcnt(J)))
            I = I + 1
         END DO
         I = I - 1
         IF (I .EQ. 0) THEN    ! Take A(0) = 2 * A(1) - A(2)
            ProbOtlks(J) = (A(2) - A(1)) / Y(1) * QntlPrcnt(J) + 2 * A(1) - A(2)
            I = 1
         ELSE
            ProbOtlks(J) = (A(I+1)-A(I)) / (Y(I+1)-Y(I)) * (QntlPrcnt(J)-Y(I)) + A(I)
         ENDIF
         IF (ProbOtlks(J) .LT. 0) ProbOtlks(J)=0
      END DO      !  {DO J = 1, Quantiles}

      GOTO 999

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : ComputeProbabilisticOutlook...'
      CALL PassMsg

 999  RETURN

 9001 FORMAT('Weights File contains ', I0, ' entries')
 9002 FORMAT('Perch Forecast Count = ', I0)
      
      END SUBROUTINE ComputeProbabilisticOutlook


!------------------------------------------------------------------------
      LOGICAL FUNCTION GetOtlkWghts (Ext, NumYrs)
!
!     Routine to input the outlook weights from file OTLKWGTS.xxx.
!
      IMPLICIT NONE
      CHARACTER (LEN=3), INTENT(IN) :: Ext
      INTEGER, INTENT(OUT) :: NumYrs

      INTEGER :: K, L, U1, IOS, Yr
      REAL    :: Wt
      CHARACTER (LEN=99)  :: Line
      CHARACTER (LEN=150) :: FileName

      FileName = TRIM(ThisLakeDir) // 'OTLKWGTS.' // Ext
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(FileName), STATUS='OLD', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899

      READ(U1, 9180, ERR=812) Line
      K = INDEX(Line, 'Maximum)')
      IF (K .GT. 0) THEN
         ErrorMessage = '     Error!!! When the outlook weights file,'; CALL PassMsg
         ErrorMessage = '     '//TRIM(FileName); CALL PassMsg
         ErrorMessage = '     was created, the program was unable to find a minimum'; CALL PassMsg
         ErrorMessage = '     solution to the problem.  It indicated this in the'; CALL PassMsg
         ErrorMessage = '     header of the file by putting MAXIMUM at the end of the first line.'; CALL PassMsg
         ErrorMessage = '     The weights in the file are not valid.'; CALL PassMsg
         ErrorMessage = '     Please create a new OTLKWGTS.* file and rerun the forecast.'; CALL PassMsg
         CLOSE(U1)
         CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
         GOTO 898
      END IF
      READ(U1, *, ERR=812)

      K = 0
      READ(U1, 9192, IOSTAT=IOS) Yr, Wt
      DO WHILE ((IOS .EQ. 0) .AND. (K .LT. MaxOtlks))
         IF ((Yr .GE. 1000) .AND. (Yr .LE. 2999) .AND. (Wt .GE. 0.0)) THEN      ! TSH 25apr2003
            K = K + 1
            OtlkYr(K)   = Yr
            OtlkWght(K) = Wt
         END IF
         READ(U1, 9192, IOSTAT=IOS) Yr, Wt
      END DO
      CLOSE (U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      NumYrs = K
      GetOtlkWghts = .TRUE.
      RETURN

 811  ErrorMessage = 'POUTLOOK: Error opening file ' // TRIM(FileName)
      CALL PassMsg;  GOTO 898
 812  ErrorMessage = 'POUTLOOK: Error reading file ' // TRIM(FileName)
      CALL PassMsg;  GOTO 898

 898  GetOtlkWghts = .FALSE.
      ErrorLevel = 1
 899  ErrorMessage = '[traceback] : GetOtlkWghts...'
      CALL PassMsg
      RETURN

 9180 FORMAT(A)
 9190 FORMAT(24X, I3, I5)
 9191 FORMAT(32X, A280)
 9192 FORMAT(I6, F10.6)

      END FUNCTION GetOtlkWghts

!------------------------------------------------------------------------
!     This routine sorts the values in array A from smallest to largest and
!     also arranges the corresponding values in array B in the same order.
!     Thus, B is not sorted based on its own contents but rather by the 
!     contents of A.  Each element of B just "tags along" with it's partner
!     element in A and the final state of B will, in general, appear to
!     be unsorted.
!
      SUBROUTINE SortTwo (A, B, Number)
      IMPLICIT NONE
      REAL   , INTENT(INOUT) :: A(:)
      INTEGER, INTENT(INOUT) :: B(:), Number

      INTEGER ::  I, J, K, L, M, P
      REAL    ::  X

      P = Number
      DO WHILE (P .GT. 0)
        K = Number - P
        J = 1
        DO WHILE (J .LE. K)
          I = J
          M = I + P
          DO WHILE ((I .GT. 0) .AND. (A(I) .GT. A(M)))
            X = A(I)
            A(I) = A(M)
            A(M) = X

            L = B(I)
            B(I) = B(M)
            B(M) = L

            M = I
            I = I - P
          END DO
          J = J + 1
        END DO
        P = P / 2
      END DO
      RETURN

      END SUBROUTINE SortTwo

      
      
END MODULE YellowPerchMaumee

