!
!     Adapted from LUMPOUT1.NI4, TEC II, Sep88, 26oct88 (for PC)
!         updated 17jan90 to include variable forecast length, TEC II
!         updated 23oct92 for 4-byte date and move to UNIX, TSH, TEC II
!         updated 01apr93 for expansion of ESTOTLyy.INT file,  TEC II
!         updated 21mar95 for change in structure of EBSNNM file and use
!          of forecast start date instead of provisional end date, TEC II
!         updated 01aug95 for change in units of ice area from m2 to decimal
!          fraction (documentation change only)
!
!     This routine reads the evaporation model output file for evaporation
!     outlooks and computes a file of evaporation parameters, eSTOTLzz.INT,
!     where zz is the two-digit year of the scenario under consideration.
!
!     ZSEVyyyy.INT is a direct access unformatted file of daily lake
!     evaporation variables.  The first record contains two 4-byte integers:
!     the starting sequence number and the ending sequence number respectively.
!     These correspond to the date of the first day in the file and the date of
!     the last day in the file, respectively.  The remaining records contain
!     daily values, in chronological order, as 4-byte reals; 16 values per day
!     to each record, respectively: Lake Evaporation (cm), Water Surface
!     Temperature (deg. C), Over-lake Air Temperature (deg. C), Over-lake Water
!     Vapor Pressure (mb), Over-lake Wind Speed (m/s), Cloud Cover (decimal
!     fraction), Incident Short-wave Radiation (watts/m2), Reflection
!     (watts/m2), Net Long Wave Radiation Exchange (watts/m2), Latent Heat Flux
!     (watts/m2), Sensible Heat Flux (watts/m2), Advection (expressed
!     equivalently as watts/m2), Ice Temperature (deg. C), Ice Area (decimal
!     fraction), Ice Depth (m), and Stored Heat in Lake (calories).
!


MODULE ELMPOUT1_main

      USE FcstUtil
      INTEGER, PARAMETER :: ZHdr = 1            ! header records in ZS*.*

CONTAINS
      SUBROUTINE DO_ELMP1(ErrStatus, HandleIn, Lake)
      IMPLICIT NONE

      INTEGER, INTENT(INOUT) :: ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER (KIND=INTEGER2) :: iD2, iM2, iY2
      INTEGER :: Start_Day, Start_Month, Hist_Year, I, HiStrt, FcEnd
      INTEGER :: FcStrt, FrcstLngth, J, Srne, iD, iM, iY

      REAL :: qE, qTw, qTa, qVa, qW, qC, q07, q08, q09
      REAL :: q10, q11, q12, q13, q14, q15, q16

      CHARACTER (LEN=1), PARAMETER  :: Lf = CHAR(10)
      CHARACTER (LEN=4)  :: Hist_YearC
      CHARACTER (LEN=20) :: S
      CHARACTER (LEN=99) :: File1, File2, File3, File4


      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen()

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Determine application-specific information
!
      File1 = TRIM(ThisLakeDir) // 'DATEFILE.INT'
      OPEN(UNIT=11, FILE=TRIM(File1), STATUS='OLD', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(11, 1102, ERR=812) Start_Day
      READ(11, 1102, ERR=812) Start_Month
      READ(11, 1103, ERR=812) Hist_YearC
      READ(Hist_YearC, 1002) Hist_Year
      CALL DateSequence(Start_Day, Start_Month, Hist_Year, HiStrt)
      IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

      File2 = TRIM(ThisLakeDir) // 'EBSNNM'
      OPEN(UNIT=21, FILE=TRIM(File2), STATUS='OLD', RECL=12,              &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      READ(21, 2101, REC=2, ERR=822) I
      IF (I .NE. 8) THEN
         ErrorMessage = 'ELMPOUT1: Invalid sequencing of forecast programs.'
         CALL PassMsg
         GOTO 898
      END IF

!
!     Find out when forecast begins.         
!
      READ(21, 2105, REC=5, ERR=822) iD, iM, iY 
      CALL DateSequence(iD, iM, iY, FcStrt); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Find out when model output files end (all are assumed
!     to end on the same date).
!
      READ(21, 2105, REC=6, ERR=822) iD, iM, iY 
      CALL DateSequence(iD, iM, iY, FcEnd); IF (ErrorLevel .NE. 0) GOTO 899
      FrcstLngth = FcEnd - FcStrt + 1       
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Input/Output lake evaporation, surface temperature, air temperature,
!     vapor pressure, windspeed, cloudcover, short-wave radiation, reflection,
!     net long-wave radiation exchange, latent heat flux, sensible heat flux,
!     advection, ice temperature, ice area, ice depth, & heat in storage.
!
      File3 = TRIM(ThisLakeDir) // 'ZSEV' // Hist_YearC // '.INT'
      OPEN(UNIT=31, FILE=TRIM(File3), STATUS='OLD', RECL=64,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899

      READ(31, REC=1, ERR=832) J, I
      IF ((HiStrt .LT. J) .OR. ((HiStrt + FrcstLngth - 1) .GT. I)) THEN
         ErrorMessage = 'ELMPOUT1: The historic meteorologic scenario selected'
         CALL PassMsg
         ErrorMessage = '          for the forecast begins or ends outside'
         CALL PassMsg
         ErrorMessage = '          the available period.'
         CALL PassMsg
         GOTO 898
      END IF
      Srne = HiStrt - J + ZHdr + 1 - 1

      File4 = TRIM(ThisLakeDir) // 'ESTOTL' // Hist_YearC // '.INT'
      OPEN(UNIT=41, FILE=TRIM(File4), STATUS='OLD', RECL=70,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=41)
      CLOSE (41, STATUS='DELETE')
 41   OPEN(UNIT=41, FILE=TRIM(File4), STATUS='NEW', RECL=70,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=841)
      CALL FileWasOpened(41); IF (ErrorLevel .NE. 0) GOTO 899

      DO I = FcStrt, FcEnd
        J = I - FcStrt + 1
        CALL SequenceDate(iD, iM, iY, I); IF (ErrorLevel .NE. 0) GOTO 899
        READ(31, REC=Srne+J, ERR=832) qE, qTw, qTa, qVa, qW, qC, q07,     &
     &                   q08, q09, q10, q11, q12, q13, q14, q15, q16
        iD2 = iD
        iM2 = iM
        iY2 = iY
        WRITE(41, REC=J, ERR=843) iD2, iM2, iY2, qE, qTw, qTa, qVa,       &
     &        qW, qC, q07, q08, q09, q10, q11, q12, q13, q14, q15, q16
      END DO

      CLOSE (41)
      CALL FileWasClosed(41); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (31)
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Update 'EBSNNM' file with new program designator.
!
      File2 = TRIM(ThisLakeDir) // 'EBSNNM'
      OPEN(UNIT=21, FILE=TRIM(File2), STATUS='OLD', RECL=12,              &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(21, 2101, REC=2, ERR=823) 10, Lf
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Handlers for I/O errors
!
 811  ErrorMessage = 'ELMPOUT1: Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
 812  ErrorMessage = 'ELMPOUT1: Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
 813  ErrorMessage = 'ELMPOUT1: Error writing file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
 821  ErrorMessage = 'ELMPOUT1: Error opening file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
 822  ErrorMessage = 'ELMPOUT1: Error reading file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
 823  ErrorMessage = 'ELMPOUT1: Error writing file ' // TRIM(File2)
      CALL PassMsg; GOTO 898
 831  ErrorMessage = 'ELMPOUT1: Error opening file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
 832  ErrorMessage = 'ELMPOUT1: Error reading file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
 833  ErrorMessage = 'ELMPOUT1: Error writing file ' // TRIM(File3)
      CALL PassMsg; GOTO 898
 841  ErrorMessage = 'ELMPOUT1: Error opening file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
 842  ErrorMessage = 'ELMPOUT1: Error reading file ' // TRIM(File4)
      CALL PassMsg; GOTO 898
 843  ErrorMessage = 'ELMPOUT1: Error writing file ' // TRIM(File4)
      CALL PassMsg; GOTO 898

 898  ErrorLevel = 1
 899  CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_ELMP1...'
      CALL PassMsg
      GOTO 999

!  -----------------------

 999  CALL CloseAllFiles()
      ErrStatus = ErrorLevel

      RETURN

!
! FORMATs
!
 1002 FORMAT(I4)
 1102 FORMAT(I4)
 1103 FORMAT(A4)
 2101 FORMAT(3X, I3, 5X, A1)
 2105 FORMAT(2I3, I5)

      END SUBROUTINE DO_ELMP1

END MODULE ELMPOUT1_main



