!
!     Modified 10mar88, TEC II, to use internationally coordinated lake areas.
!     Updated by TEC II 28sep88, 26oct88 (for PC)
!     Updated 17jan90, TEC II, to include variable forecast length.
!     Updated 23apr92, TSH, to handle missing data.
!     Updated 28oct92, TSH, TEC II, for 4-byte dates and move to UNIX
!     Updated 12may93, TSH - corrected error in reading SFCSTTBL.INT
!             (direct access file was being read sequentially - a No-No!)
!     Updated 06dec93, TSH, to build FCSTBL??.INT files from the beginning of
!             the forecast month even when the forecast starts in the middle
!             of the month, rather than from the beginning of the following
!             month when the forecast does not start on the first.
!     Updated 17nov94, TSH, to also create DLYTBL??.INT files which contain
!             data similar to the FCSTBL??.INT files, but DAILY values.
!             They are also direct-access unformatted files.
!     Updated 21mar95 for change in structure of BSNNM file and use
!             of forecast start date instead of provisional end date, TEC II.
!     Updated 23jun00, TSH, to add air temperatures to DLYTBL??.INT
!
!     This routine reads the model output file for runoff and storage outlooks
!     for the entire basin (from xxxALL.SUM), daily lumped over-lake
!     precipitation (from either xxx00.CLM or SFCSTTBL.INT), and daily lake
!     evaporation (from ZSEVyyyy.INT) and computes daily basin storages (in
!     centimeters over the basin in file STOOTLzz.INT), daily net basin
!     supplies (in file NBSOTL.zz.INT) and monthly values of runoff,
!     precipitation, lake evaporation, and net basin supply (all in millimeters
!     over the lake in file FCSTBLzz.INT), where zz is the two-digit year of
!     the scenario under consideration.
!
!     ZSEVyyyy.INT is a direct access unformatted file of daily lake
!     evaporation in centimeters over the lake.  The first record contains two
!     4-byte integers: the starting sequence number corresponding to the date
!     of the first day in the file and the ending sequence number corresponding
!     to the last day in the file.  Remaining records each contain sixteen (16)
!     daily values, in chronological order, as 4-byte reals of the following
!     (respectively):
!
!           LakeEvaporation        cm
!           SurfaceTemperature     deg. C.
!           AirTemp                deg. C.
!           VapPres                millibars
!           WindSpeed              m/s
!           CloudCover             fraction
!           Incident               watts/m2
!           Reflect                watts/m2
!           NLWave                 watts/m2
!           Latent                 watts/m2
!           Sensible               watts/m2
!           Advection              watts/m2
!           Ice Temperature        deg. C.
!           Ice Area               m2
!           Ice Depth              m
!           No. Ice Blocks
!
!--------------------------------------------------------------------------

MODULE LUMP1_Subs

      USE FcstUtil
      USE ReadData

      INTEGER, PARAMETER ::  NoDays = 36526      ! 100 yrs maximum allowed
      INTEGER, PARAMETER ::  NoMonths = 1200     ! 100 yrs maximum allowed
      INTEGER, PARAMETER ::  ZHdr = 1            ! # of header records in ZSEV*.*

CONTAINS

!--------------------------------------------------------------------------
!  SUBROUTINE GetHPrecip() computes the "over-area" precipitation for the
!  specified subbasin range for the days in the forecast month before the forecast
!  start day.  e.g. If the forecast starts on April 7, 1993 then this routine
!  will compute the precip for 4/1/93 - 4/6/93.  The xxxnn.PRV files are
!  used for this data, unless we have to go back into the CLM files for
!  some reason.
!
!  Subs = total # of subbasins for this lake (e.g. 22 for SUP, 27 for MIC, etc)
!  SSub = first subbasin to use
!  ESub = last subbasin to use
!
!  The data passed back in the "Prc" array is in centimeters over the
!  surface area of the combined subbasins.
!
!  Written by Tim Hunter  18aug2000
!
      SUBROUTINE GetHPrecip(FcStrt, Prc, Bsn, Subs, SSub, ESub)
      IMPLICIT   NONE
      INTEGER,             INTENT(IN)  :: FcStrt, Subs, SSub, ESub
      CHARACTER (LEN=3),   INTENT(IN)  :: Bsn
      REAL, DIMENSION(31), INTENT(OUT) :: Prc

      INTEGER :: I, Seq, Sub, SSeq, ESeq, Day, Mon, Year, U1
      REAL    :: P, PArea
      REAL    :: Area(0:30), MetData(31,3), SubPrc(0:30, 31)
      CHARACTER (LEN=99) :: File1

!
!  Initialize variables
!
      Prc(:) = -999.9
      ESeq = FcStrt - 1
      CALL SequenceDate(Day, Mon, Year, FcStrt); IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(  1, Mon, Year, SSeq);   IF (ErrorLevel .NE. 0) GOTO 899
      IF (SSeq .GT. ESeq) RETURN

!
!  Read subbasin areas
!
      File1 = TRIM(ThisLakeDir) // 'AREA.' // Bsn
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      DO I = 1, Subs
        READ(U1, 1101, ERR=812) Area(I)
      END DO
      READ(U1, 1101, ERR=812) Area(0)
      CLOSE(U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

!
!  Read the data from the subbasin files.  The precipitation values
!  are expressed in mm over the subbasin.
!
      DO Sub = SSub, ESub
         CALL ReadMetData(SSeq, ESeq, MetData, Bsn, Sub, ThisLakeDir)
         DO Seq = SSeq, ESeq
            I = Seq - SSeq + 1
            P = MetData(I, 3)                        ! precip is 3rd entry
            SubPrc(Sub, I) = P * Area(Sub) / 1000.   ! mm -> cubic meters
         END DO
      END DO

!
!  Combine subbasin values into an overbasin value
!
      DO Seq = SSeq, ESeq
         I = Seq - SSeq + 1
         Prc(I) = 0.0
         P = 0
         PArea = 0.0
         DO Sub = SSub, ESub
            IF (SubPrc(Sub,I) .GE. 0) THEN     
               PArea = PArea + Area(Sub)       ! accumulate surface area (sq meters)
               P = P + SubPrc(Sub, I)          ! accumulate water volume (cu meters)
            END IF
         END DO
         IF (PArea .GT. 1) THEN
            Prc(I) = P / PArea * 100           ! P is cubic meters, result is in cm
         ELSE
            Prc(I) = -999.9
         END IF
      END DO
      RETURN

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : GetHPrecip...'
      CALL PassMsg
      RETURN

!
 1101 FORMAT(E13.6E2)

      END SUBROUTINE GetHPrecip

!--------------------------------------------------------------------------
!  SUBROUTINE HBasnPrc() uses subroutine GetHPrecip to compute the 
!  overbasin (lake and land combined) precipitation for the days in the 
!  forecast month before the forecast start day.  e.g. If the forecast 
!  starts on April 7, 1993 then this routine will compute the precip for 
!  4/1/93 - 4/6/93.  The xxxnn.PRV files (and possibly xxxnn.CLM files)
!  are used for this data.
!
!  The data passed back in the "Prc" array is in centimeters over the
!  basin surface (lake & land combined).
!
!  Written by Tim Hunter  18aug2000
!
      SUBROUTINE HBasnPrc(FcStrt, Prc, Bsn, Subs)
      IMPLICIT   NONE
      INTEGER,             INTENT(IN)  :: FcStrt, Subs
      CHARACTER (LEN=3),   INTENT(IN)  :: Bsn
      REAL, DIMENSION(31), INTENT(OUT) :: Prc

      INTEGER :: Sub1, Sub2

      Sub1 = 0
      Sub2 = Subs
      CALL GetHPrecip(FcStrt, Prc, Bsn, Subs, Sub1, Sub2)
      IF (ErrorLevel .NE. 0) GOTO 899
      RETURN

  899 ErrorMessage = '[traceback] : HBasnPrc...'
      CALL PassMsg
      RETURN

      END SUBROUTINE HBasnPrc

!--------------------------------------------------------------------------
!  SUBROUTINE HLandPrc() uses subroutine GetHPrecip to compute the 
!  overland precipitation for the days in the 
!  forecast month before the forecast start day.  e.g. If the forecast 
!  starts on April 7, 1993 then this routine will compute the precip for 
!  4/1/93 - 4/6/93.  The xxxnn.PRV files (and possibly xxxnn.CLM files)
!  are used for this data.
!
!  The data passed back in the "Prc" array is in centimeters over the
!  land surface.
!
!  Written by Tim Hunter  18aug2000
!
      SUBROUTINE HLandPrc(FcStrt, Prc, Bsn, Subs)
      IMPLICIT   NONE
      INTEGER,             INTENT(IN)  :: FcStrt, Subs
      CHARACTER (LEN=3),   INTENT(IN)  :: Bsn
      REAL, DIMENSION(31), INTENT(OUT) :: Prc

      INTEGER :: Sub1, Sub2

      Sub1 = 1
      Sub2 = Subs
      CALL GetHPrecip(FcStrt, Prc, Bsn, Subs, Sub1, Sub2)
      IF (ErrorLevel .NE. 0) GOTO 899
      RETURN

  899 ErrorMessage = '[traceback] : HLandPrc...'
      CALL PassMsg
      RETURN

      END SUBROUTINE HLandPrc

!--------------------------------------------------------------------------
!  SUBROUTINE HLakePrc() uses subroutine GetHPrecip to compute the 
!  overlake precipitation for the days in the 
!  forecast month before the forecast start day.  e.g. If the forecast 
!  starts on April 7, 1993 then this routine will compute the precip for 
!  4/1/93 - 4/6/93.  The xxx00.PRV file (and possibly xxx00.CLM file)
!  is used for this data.
!
!  The data passed back in the "Prc" array is in centimeters over the
!  lake surface.
!
!  Written by Tim Hunter  18aug2000
!
      SUBROUTINE HLakePrc(FcStrt, Prc, Bsn, Subs)
      IMPLICIT   NONE
      INTEGER,             INTENT(IN)  :: FcStrt, Subs
      CHARACTER (LEN=3),   INTENT(IN)  :: Bsn
      REAL, DIMENSION(31), INTENT(OUT) :: Prc

      INTEGER :: Sub1, Sub2

      Sub1 = 0
      Sub2 = 0
      CALL GetHPrecip(FcStrt, Prc, Bsn, Subs, Sub1, Sub2)
      IF (ErrorLevel .NE. 0) GOTO 899
      RETURN

  899 ErrorMessage = '[traceback] : HLakePrc...'
      CALL PassMsg
      RETURN

      END SUBROUTINE HLakePrc

!--------------------------------------------------------------------------
!  SUBROUTINE GetHAir() reads the air temperatures for the specified subbasin
!  range for the days in the forecast month before the forecast start day.
!  e.g. If the forecast starts on April 7, 1993 then this routine
!  will read the air temps for 4/1/93 - 4/6/93.  The xxxnn.PRV file(s) and/or
!  the xxxnn.CLM files(s) will be used for this data.
!
!  The data passed back in the "Tmx" and "Tmn" arrays is in degrees C
!
!  Written by Tim Hunter  18aug2000
!
      SUBROUTINE GetHAir(FcStrt, Tmx, Tmn, Bsn, Subs, SSub, ESub)
      IMPLICIT   NONE
      INTEGER,             INTENT(IN)  :: FcStrt, Subs, SSub, ESub
      CHARACTER (LEN=3),   INTENT(IN)  :: Bsn
      REAL, DIMENSION(31), INTENT(OUT) :: Tmx, Tmn

      INTEGER :: I, Seq, Sub, SSeq, ESeq, Day, Mon, Year, U1
      REAL    :: Mn, Mx, MnArea, MxArea
      REAL    :: Area(0:30), MetData(31,3), SubMn(0:30,31), SubMx(0:30,31)
      CHARACTER (LEN=99) :: File1

!
!  Initialize variables
!
      Tmx(:) = -999.9
      Tmn(:) = -999.9
      ESeq = FcStrt - 1
      CALL SequenceDate(Day, Mon, Year, FcStrt); IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(  1, Mon, Year, SSeq);   IF (ErrorLevel .NE. 0) GOTO 899
      IF (SSeq .GT. ESeq) RETURN

!
!  Read subbasin areas
!
      File1 = TRIM(ThisLakeDir) // 'AREA.' // Bsn
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(File1), STATUS='OLD', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      DO I = 1, Subs
        READ(U1, 1101, ERR=812) Area(I)
      END DO
      READ(U1, 1101, ERR=812) Area(0)
      CLOSE(U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

!
!  Read the data from the subbasin files.  The air temp values
!  are expressed in degrees C
!
      DO Sub = SSub, ESub
         CALL ReadMetData(SSeq, ESeq, MetData, Bsn, Sub, ThisLakeDir)
         DO Seq = SSeq, ESeq
            I = Seq - SSeq + 1
            Mn = MetData(I, 1)
            Mx = MetData(I, 2)
            SubMn(Sub, I) = Mn
            SubMx(Sub, I) = Mx
         END DO
      END DO

!
!  Combine subbasin values into an overbasin value
!
      DO Seq = SSeq, ESeq
         I = Seq - SSeq + 1
         Tmn(I) = 0.0
         Tmx(I) = 0.0
         Mn = 0
         Mx = 0
         MnArea = 0.0
         MxArea = 0.0
         DO Sub = SSub, ESub
            IF (SubMn(Sub,I) .GE. 0) THEN     
               MnArea = MnArea + Area(Sub)       ! accumulate surface area (sq meters)
               Mn = Mn + SubMn(Sub, I)           ! accumulate water volume (cu meters)
            END IF
            IF (SubMx(Sub,I) .GE. 0) THEN     
               MxArea = MxArea + Area(Sub)       ! accumulate surface area (sq meters)
               Mx = Mx + SubMx(Sub, I)           ! accumulate water volume (cu meters)
            END IF
         END DO
         IF (MnArea .GT. 1) THEN
            Tmn(I) = Mn / MnArea * 100
         ELSE
            Tmn(I) = -999.9
         END IF
         IF (MxArea .GT. 1) THEN
            Tmx(I) = Mx / MxArea * 100
         ELSE
            Tmx(I) = -999.9
         END IF
      END DO
      RETURN

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File1)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File1)
      CALL PassMsg; GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : GetHAir...'
      CALL PassMsg
      RETURN

!
 1101 FORMAT(E13.6E2)

      END SUBROUTINE GetHAir

!--------------------------------------------------------------------------
!  SUBROUTINE HBasnAir() uses subroutine GetHAir to compute the 
!  overbasin (lake and land combined) air temps for the days in the 
!  forecast month before the forecast start day.  e.g. If the forecast 
!  starts on April 7, 1993 then this routine will compute the air temps for 
!  4/1/93 - 4/6/93.  The xxxnn.PRV files (and possibly xxxnn.CLM files)
!  are used for this data.
!
!  The data passed back in the "Tmx" & "Tmn" arrays is in degrees C.
!
!  Written by Tim Hunter  18aug2000
!
      SUBROUTINE HBasnAir(FcStrt, Tmx, Tmn, Bsn, Subs)
      IMPLICIT   NONE
      INTEGER,             INTENT(IN)  :: FcStrt, Subs
      CHARACTER (LEN=3),   INTENT(IN)  :: Bsn
      REAL, DIMENSION(31), INTENT(OUT) :: Tmx, Tmn

      INTEGER :: Sub1, Sub2

      Sub1 = 0
      Sub2 = Subs
      CALL GetHAir(FcStrt, Tmx, Tmn, Bsn, Subs, Sub1, Sub2)
      IF (ErrorLevel .NE. 0) GOTO 899
      RETURN

  899 ErrorMessage = '[traceback] : HBasnAir...'
      CALL PassMsg
      RETURN

      END SUBROUTINE HBasnAir

!--------------------------------------------------------------------------
!  SUBROUTINE HLandAir() uses subroutine GetHAir to compute the 
!  overland air temps for the days in the 
!  forecast month before the forecast start day.  e.g. If the forecast 
!  starts on April 7, 1993 then this routine will compute the air temps for 
!  4/1/93 - 4/6/93.  The xxxnn.PRV files (and possibly xxxnn.CLM files)
!  are used for this data.
!
!  The data passed back in the "Tmx" & "Tmn" arrays is in degrees C.
!
!  Written by Tim Hunter  18aug2000
!
      SUBROUTINE HLandAir(FcStrt, Tmx, Tmn, Bsn, Subs)
      IMPLICIT   NONE
      INTEGER,             INTENT(IN)  :: FcStrt, Subs
      CHARACTER (LEN=3),   INTENT(IN)  :: Bsn
      REAL, DIMENSION(31), INTENT(OUT) :: Tmx, Tmn

      INTEGER :: Sub1, Sub2

      Sub1 = 1
      Sub2 = Subs
      CALL GetHAir(FcStrt, Tmx, Tmn, Bsn, Subs, Sub1, Sub2)
      IF (ErrorLevel .NE. 0) GOTO 899
      RETURN

  899 ErrorMessage = '[traceback] : HLandAir...'
      CALL PassMsg
      RETURN

      END SUBROUTINE HLandAir

!--------------------------------------------------------------------------
!  SUBROUTINE HLakeAir() uses subroutine GetHAir to compute the 
!  overlake air temps for the days in the 
!  forecast month before the forecast start day.  e.g. If the forecast 
!  starts on April 7, 1993 then this routine will compute the air temps for 
!  4/1/93 - 4/6/93.  The xxxnn.PRV files (and possibly xxxnn.CLM files)
!  are used for this data.
!
!  The data passed back in the "Tmx" & "Tmn" arrays is in degrees C.
!
!  Written by Tim Hunter  18aug2000
!
      SUBROUTINE HLakeAir(FcStrt, Tmx, Tmn, Bsn, Subs)
      IMPLICIT   NONE
      INTEGER,             INTENT(IN)  :: FcStrt, Subs
      CHARACTER (LEN=3),   INTENT(IN)  :: Bsn
      REAL, DIMENSION(31), INTENT(OUT) :: Tmx, Tmn

      INTEGER :: Sub1, Sub2

      Sub1 = 0
      Sub2 = 0
      CALL GetHAir(FcStrt, Tmx, Tmn, Bsn, Subs, Sub1, Sub2)
      IF (ErrorLevel .NE. 0) GOTO 899
      RETURN

  899 ErrorMessage = '[traceback] : HLakeAir...'
      CALL PassMsg
      RETURN

      END SUBROUTINE HLakeAir

!--------------------------------------------------------------------------
!  SUBROUTINE ReadLandPrecip() computes the overland precipitation for the
!  specified days.
!
!  Subs = total # of subbasins for this lake (e.g. 22 for SUP, 27 for MIC, etc)
!  SSub = first subbasin to use
!  ESub = last subbasin to use
!
!  The data passed back in the "Prc" array is in centimeters over the land surface.
!
!  Written by Tim Hunter  28 sep 2000
!
      SUBROUTINE ReadLandPrecip(SSeq, ESeq, Bsn, Subs, Prc)
      IMPLICIT  NONE
      INTEGER,             INTENT(IN)    :: SSeq, ESeq, Subs
      CHARACTER (LEN=3),   INTENT(IN)    :: Bsn
      REAL, DIMENSION(:),  INTENT(OUT)   :: Prc

      INTEGER :: I, J, Sub, U1
      REAL    :: Area(1:30)
      REAL, DIMENSION(RD_MaxDays)    :: TPrc, TArea
      REAL, DIMENSION(RD_MaxDays, 4) :: MData
      CHARACTER (LEN=3)  :: Lk
      CHARACTER (LEN=99) :: FName 

      FName = TRIM(ThisLakeDir) // 'AREA.' // Bsn
      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(FName), STATUS='OLD', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      DO Sub = 1, Subs
         READ(U1, 1101, ERR=812) Area(Sub)
      END DO
      CLOSE(U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      TPrc(:) = 0.0
      TArea(:) = 0.0;
      Lk = Bsn
      DO Sub = 1, Subs
         CALL ReadMetData(SSeq, ESeq, MData, Lk, Sub, ThisLakeDir)
         IF (ErrorLevel .NE. 0) GOTO 899
         DO I = SSeq, ESeq
            J = I - SSeq + 1
            IF (MData(J,3) .GE. 0) THEN
               TPrc(J) = TPrc(J) + MData(J,3)*Area(Sub)
               TArea(J) = TArea(J) + Area(Sub)
            END IF
         END DO
      END DO

      Prc(:) = -999.99
      DO I = SSeq, ESeq
         J = I - SSeq + 1
         IF (TArea(J) .GT. 0) Prc(J) = TPrc(J) / TArea(J)
      END DO
      RETURN


  811 ErrorMessage = 'LUMPOUT1 : Error opening file ' // TRIM(FName)
      CALL PassMsg;  GOTO 898
  812 ErrorMessage = 'LUMPOUT1 : Error reading file ' // TRIM(FName)
      CALL PassMsg;  GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : ReadLandPrecip...'
      CALL PassMsg
      RETURN

 1101 FORMAT(E13.6E2)

      END SUBROUTINE ReadLandPrecip

!--------------------------------------------------------------------------
!  SUBROUTINE ReadLandAirTemps() computes the overland Tmin & Tmax for the
!  specified days.
!
!  Subs = total # of subbasins for this lake (e.g. 22 for SUP, 27 for MIC, etc)
!  SSub = first subbasin to use
!  ESub = last subbasin to use
!
!  The data passed back in the Tmin and Tmax arrays is in degrees C.
!
!  Written by Tim Hunter  18 oct 2000
!
      SUBROUTINE ReadLandAirTemps(SSeq, ESeq, Bsn, Subs, Tmin, Tmax)
      IMPLICIT  NONE
      INTEGER,             INTENT(IN)    :: SSeq, ESeq, Subs
      CHARACTER (LEN=3),   INTENT(IN)    :: Bsn
      REAL, DIMENSION(:),  INTENT(OUT)   :: Tmin, Tmax

      INTEGER :: I, J, Sub, U1
      REAL    :: Area(1:30)
      REAL, DIMENSION(RD_MaxDays)    :: Tn, Tx, TArea
      REAL, DIMENSION(RD_MaxDays, 4) :: MData
      CHARACTER (LEN=3)  :: Lk
      CHARACTER (LEN=99) :: FName 

      FName = TRIM(ThisLakeDir) // 'AREA.' // Bsn
      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(FName), STATUS='OLD', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      DO Sub = 1, Subs
         READ(U1, 1101, ERR=812) Area(Sub)
      END DO
      CLOSE(U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899

      Tn(:) = 0.0
      Tx(:) = 0.0
      TArea(:) = 0.0;
      Lk = Bsn
      DO Sub = 1, Subs
         CALL ReadMetData(SSeq, ESeq, MData, Lk, Sub, ThisLakeDir)
         IF (ErrorLevel .NE. 0) GOTO 899
         DO I = SSeq, ESeq
            J = I - SSeq + 1
            IF ((MData(J,1) .GE. -99.99) .AND. (MData(J,2) .GE. -99.99)) THEN
               Tn(J) = Tn(J) + MData(J,1)*Area(Sub)
               Tx(J) = Tx(J) + MData(J,2)*Area(Sub)
               TArea(J) = TArea(J) + Area(Sub)
            END IF
         END DO
      END DO

      Tmin(:) = -999.99
      Tmax(:) = -999.99
      DO I = SSeq, ESeq
         J = I - SSeq + 1
         IF (TArea(J) .GT. 0) THEN
            Tmin(J) = Tn(J) / TArea(J)
            Tmax(J) = Tx(J) / TArea(J)
         END IF
      END DO
      RETURN


  811 ErrorMessage = 'LUMPOUT1 : Error opening file ' // TRIM(FName)
      CALL PassMsg;  GOTO 898
  812 ErrorMessage = 'LUMPOUT1 : Error reading file ' // TRIM(FName)
      CALL PassMsg;  GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : ReadLandAirTemps...'
      CALL PassMsg
      RETURN

 1101 FORMAT(E13.6E2)

      END SUBROUTINE ReadLandAirTemps

!--------------------------------------------------------------------------
!  SUBROUTINE HistEvp() gets the overlake evaporation for the days in
!  the forecast month before the forecast start day.
!  e.g. If the forecast starts on April 7, 1993 then this routine
!  will retrieve the evap for 4/1/93 - 4/6/93.  The ZSEVAPPR.xxx file is used
!  for this data, unless we have to go back into the ZSEVAP.xxx files for
!  some reason.
!
!  The data passed back in the "Evp" array is in centimeters over the
!  lake surface.
!
!  Written by Tim Hunter  06Dec93
!
      SUBROUTINE HistEvp(FcStrt, Evp, Bsn)

      IMPLICIT   NONE

      INTEGER,             INTENT(IN)    :: FcStrt
      CHARACTER (LEN=3),   INTENT(IN)    :: Bsn
      REAL, DIMENSION(31), INTENT(INOUT) :: Evp

      INTEGER :: SSeq, ESeq, I, J, Day, Mon, Year
      REAL, DIMENSION(31,16) :: AllDat

!
!  Initialize variables
!
      Evp(:) = -999.9
      ESeq = FcStrt - 1
      CALL SequenceDate(Day, Mon, Year, FcStrt); IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(  1, Mon, Year, SSeq);   IF (ErrorLevel .NE. 0) GOTO 899
      IF (SSeq .GT. ESeq) RETURN

!
!  Get the data.  It is in centimeters over the lake surface.
!
      CALL ReadEvapData(SSeq, ESeq, AllDat, Bsn, ThisLakeDir); IF (ErrorLevel .NE. 0) GOTO 899
      DO I = SSeq, ESeq
         J = I - SSeq + 1
         Evp(J) = AllDat(J,1)    ! assign evap value only
      END DO

      IF (ErrorLevel .NE. 0) GOTO 899
      RETURN

  899 ErrorMessage = '[traceback] : HistEvp...'
      CALL PassMsg
      RETURN

      END SUBROUTINE HistEvp

END MODULE LUMP1_Subs

!==========================================================================
!==========================================================================

MODULE LUMPOUT1_Main

CONTAINS

      SUBROUTINE DO_LMP1(ErrStatus, HandleIn, Lake)
      USE LUMP1_Subs

      IMPLICIT NONE
      INTEGER, INTENT(INOUT) :: ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER (KIND=INTEGER2) :: ScaleNum, Runoff, Uszm, Lszm, Gzm, Ss
      INTEGER (KIND=INTEGER2) :: Snw, iD2, iM2, iY2, I2, Tn2, Tx2, Pc2

      INTEGER :: StartDay, StartMonth, HistYear, I, HiStrt, StopMonth
      INTEGER :: NoSubbasins, LastNo, FcStrt, Start, Month
      INTEGER :: ForecastPeriodLength, ActualYear, Subbasin, Srn
      INTEGER :: iMl, J, Srne, K, iD, iM, iY, Length, LkNum
      INTEGER :: FcDay, FcMon, FcYear, Seq1, Seq2
      INTEGER :: Counts(3, NoMonths)

      REAL :: qTotArea, qArea, qLakeArea, qRunoff
      REAL :: qUszm, qLszm, qGzm, qSs, qSnw, qEvp, qPc, qTx, qTn
      REAL :: oTmn, oTmx, oPrc, oEvp, oRun, oNbs

      REAL, DIMENSION(31)          :: qHPre, qHEvp, qHTx, qHTn
      REAL, DIMENSION(NoMonths)    :: qRun, qPre, qEvap, qNBS
      REAL, DIMENSION(RD_MaxDays)  :: qDlyTmin, qDlyTmax, qDlyPc

      CHARACTER (LEN=1)  :: Lf
      CHARACTER (LEN=3)  :: Bsn
      CHARACTER (LEN=4)  :: OverTheLake, HistYearC, Yyyy
      CHARACTER (LEN=10) :: LakeName
      CHARACTER (LEN=20) :: S
      CHARACTER (LEN=99) :: File11, File21, File26, File31, File41
      CHARACTER (LEN=99) :: File51, File61, File71, File76, File81, File86

      Lf = CHAR(10)

      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen(); IF (ErrorLevel .NE. 0) GOTO 899

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array -- as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Determine application-specific information
!
      File11 = TRIM(ThisLakeDir) // 'DATEFILE.INT'
      OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(11, 1102, ERR=812) StartDay
      READ(11, 1102, ERR=812) StartMonth
      READ(11, 1103, ERR=812) HistYearC
      READ(HistYearC, 1002) HistYear
      Yyyy = HistYearC
      CALL DateSequence(StartDay, StartMonth, HistYear, HiStrt); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

      File21 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', RECL=12,                  &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      READ(21, 2101, REC=1, ERR=822) LakeName
      Bsn = LakeName(1:3)
      READ(21, 2102, REC=2, ERR=822) NoSubbasins, I, OverTheLake
      IF (I .NE. 9) THEN
         WRITE(ErrorMessage, 9001)
         CALL PassMsg
         GOTO 898
      END IF

      IF (OverTheLake .NE. 'BASN' .AND. OverTheLake .NE. 'LAKE') THEN
        ErrorMessage = 'LUMPOUT1: Unknown specification for computation of lake precipitation.'
        CALL PassMsg
        ErrorMessage = '          Place "BASN" or "LAKE" in file BSNNM and rerun.'
        CALL PassMsg
        GOTO 898
      END IF

!
!     Find out when forecast begins.                 
!
      READ(21, 2105, REC=5, ERR=822) FcDay, FcMon, FcYear
      CALL DateSequence(FcDay, FcMon, FcYear, FcStrt); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Find out when model output files end (all are assumed
!     to end on the same date).
!
      READ(21, 2105, REC=6, ERR=822) iD, iM, iY  
      CALL DateSequence(iD, iM, iY, LastNo); IF (ErrorLevel .NE. 0) GOTO 899
      ActualYear = iY
      StopMonth = iM
      ForecastPeriodLength = LastNo - FcStrt + 1
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Determine the start of the historical and actual forecast periods
!     (first full month).
!
      File26 = TRIM(ThisLakeDir) // 'LENGTH.'//Bsn
      OPEN(UNIT=26, FILE=TRIM(File26), STATUS='OLD', RECL=52,             &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=826)
      CALL FileWasOpened(26); IF (ErrorLevel .NE. 0) GOTO 899
      READ(26, 2601, REC=1, ERR=827) Length
      CLOSE (26)
      CALL FileWasClosed(26); IF (ErrorLevel .NE. 0) GOTO 899

      IF (Length .GT. NoMonths .OR. ForecastPeriodLength .GT. NoDays) THEN
        WRITE(ErrorMessage, 9005)
        CALL PassMsg
        GOTO 898
      END IF

      StartMonth = StopMonth - Length + 1        ! first FULL forecast month
      DO WHILE (StartMonth .LT. 1)
        StartMonth = StartMonth + 12
        ActualYear = ActualYear - 1
      END DO

!
!     Determine total basin area and lake area.
!
      qTotArea = 0.
      File31 = TRIM(ThisLakeDir) // 'AREA.'//Bsn
      OPEN(UNIT=31, FILE=TRIM(File31), STATUS='OLD', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899
      DO Subbasin = 1, NoSubbasins
        READ(31, 3101, ERR=832) qArea
        qTotArea = qTotArea + qArea
      END DO
      CLOSE (31)
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899

      I = 1
      DO WHILE (Bsn .NE. LakeName3(I))
         I = I + 1
      END DO

      IF (I .GT. 12 .OR. CoordLakeArea(I) .LT. 0.) THEN
         WRITE(ErrorMessage, 9002)
         CALL PassMsg
         GOTO 898
      END IF
      LkNum = I
      qLakeArea = CoordLakeArea(I)

!
!     Read daily lake precipitation estimate and convert to centimeters
!     over the lake.  Also read the overland air temperature values.
!
!     Note that if OverTheLake = 'BASN' we will use:
!         Precipitation = overland    (not overBASIN)
!         Air temp      = overland    (not overBASIN)
!
!     If OverTheLake = 'LAKE' we will use:
!         Precipitation = overlake    (from 00.CLM file)   
!         Air temp      = overland    (not overBASIN)
!
!    Does that seem inconsistent to you?  Me too.  I think we need to revisit
!    this question in the future.  Right now, though, I just need to get it done.
!    TSH - 28 sep 2000
!
      IF (OverTheLake .EQ. 'BASN') THEN
         Seq1 = HiStrt
         Seq2 = HiStrt + ForecastPeriodLength - 1
         CALL ReadLandPrecip(Seq1, Seq2, Bsn, NoSubbasins, qDlyPc); IF (ErrorLevel .NE. 0) GOTO 899
         DO I = Seq1, Seq2
            J = I - Seq1 + 1
            qDlyPc(J) = qDlyPc(J) / 10.0      ! mm -> cm
         END DO
         IF (FcDay .NE. 1) THEN
            CALL HLandPrc(FcStrt, qHPre, Bsn, NoSubbasins); IF (ErrorLevel .NE. 0) GOTO 899
         END IF
      ELSE
         File41 = TRIM(ThisLakeDir) // Bsn // '00.CLM'
         OPEN(UNIT=41, FILE=TRIM(File41), STATUS='OLD', RECL=6,           &
     &        ACCESS='DIRECT', FORM='UNFORMATTED', ERR=841)
         CALL FileWasOpened(41); IF (ErrorLevel .NE. 0) GOTO 899
         READ(41, REC=1, ERR=842) ScaleNum, Srn
         Srn = (HiStrt - Srn + 1) + ClmHdr - 1
         DO I = 1, ForecastPeriodLength
           READ(41, REC = Srn + I, ERR=842) Tn2, Tx2, Pc2
           qDlyPc(I) = Pc2 / 1000.     ! hundreths of mm -> cm
         END DO
         CLOSE (41)
         CALL FileWasClosed(41); IF (ErrorLevel .NE. 0) GOTO 899
         IF (FcDay .NE. 1) THEN
            CALL HLakePrc(FcStrt, qHPre, Bsn, NoSubbasins); IF (ErrorLevel .NE. 0) GOTO 899
         END IF
      ENDIF

      Seq1 = HiStrt
      Seq2 = HiStrt + ForecastPeriodLength - 1
      CALL ReadLandAirTemps(Seq1, Seq2, Bsn, NoSubbasins, qDlyTmin, qDlyTmax); IF (ErrorLevel .NE. 0) GOTO 899
      IF (FcDay .NE. 1) THEN
         CALL HLandAir(FcStrt, qHTx, qHTn, Bsn, NoSubbasins); IF (ErrorLevel .NE. 0) GOTO 899
      END IF

!
!     Open entire-basin model output file: xxxALL.SUM.
!
      File51 = TRIM(ThisLakeDir) // Bsn // 'ALL.SUM'
      OPEN(UNIT=51, FILE=TRIM(File51), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=851)
      CALL FileWasOpened(51); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Srn = Sequence number for first day in xxxALL.SUM file.
!
      READ(51, REC=1, ERR=852) I2, Srn, ScaleNum

!
!     Record number for first day of forecast minus one.
!
      Srn = (FcStrt - Srn + 1) + SumHdr - 1

!
!     Input basin runoff, basin storages, and lake evaporation, output,
!     storages, accumulate runoff, lake precipitation, and lake evaporation
!     over the month, and accumulate daily net basin supply.
!
      CALL DateSequence(1, StartMonth, ActualYear, Start); IF (ErrorLevel .NE. 0) GOTO 899

!
!     If FcStrt > Start then summations of dailies over the month to calculate
!     monthlies will not begin on the first day of a month, as is assumed
!     (since we only initialize first month arrays here and the rest as
!     needed).
!
      IF (FcStrt .GT. Start) THEN
        WRITE(ErrorMessage, 9010)
        CALL PassMsg
        GOTO 898
      ENDIF

!
!     Initialize variables and first elements of the arrays
!
      Month = 1
      qRun(1) = 0.
      qPre(1) = 0.
      qEvap(1) = 0.
      DO K = 1, 3
         Counts(K, 1) = 0
      END DO
      iMl = FcMon

!                                                                        
!     If the forecast period does not start on the first of a month      
!     then we read the data for the first few days of the forecast       
!     start month in order to "backfill" the monthly runoff, evap and    
!     precip accumulation arrays. The daily precip array was filled in   
!     earlier in the program.                                            
!     It is not necessary to change any of the other initialized         
!     variables except iMl since there will still be data to be read     
!     and accumulated in the first month.                                
!                                                                        
      IF (FcDay .NE. 1) THEN                                             
        Month = 1                                                        
        CALL HistEvp(FcStrt, qHEvp, Bsn); IF (ErrorLevel .NE. 0) GOTO 899
        DO K = 1, FcDay-1                                                
          J = K - FcDay                                                  
          READ(51, REC=Srn+J, ERR=852) Runoff                                  
          IF (Runoff .GE. 0) THEN                                        
            qRunoff = Runoff / 100.                                      
            qRunoff = qRunoff * qTotArea / qLakeArea                     
            qRun(Month) = qRun(Month) + qRunoff                          
            Counts(1, Month) = Counts(1, Month) + 1                      
          END IF                                                          
          IF (qHPre(K) .GE. 0) THEN                                      
            qPre(Month) = qPre(Month) + qHPre(K)                         
            Counts(2, Month) = Counts(2, Month) + 1                      
          END IF                                                          
          IF (qHEvp(K) .GE. 0) THEN                                      
            qEvap(Month) = qEvap(Month) + qHEvp(K)                       
            Counts(3, Month) = Counts(3, Month) + 1                      
          END IF                                                          
        END DO                                                            
      END IF                                                              

!
!     Read header info from ZSEVyyyy.INT file
!
      File61 = TRIM(ThisLakeDir) // 'ZSEV' // HistYearC // '.INT'
      OPEN(UNIT=61, FILE=TRIM(File61), STATUS='OLD', RECL=64,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=861)
      CALL FileWasOpened(61); IF (ErrorLevel .NE. 0) GOTO 899
      READ(61, REC=1, ERR=862) J, I
      Srne = (HiStrt - J + 1) + Zhdr - 1
      IF (HiStrt .LT. J  .OR.  HiStrt+ForecastPeriodLength-1 .GT. I) THEN
        ErrorMessage = 'LUMPOUT1: The historic meteorologic scenario selected begins or'
        CALL PassMsg
        ErrorMessage = '          ends on a date outside the period available for '
        CALL PassMsg
        ErrorMessage = '          historic evaporation and groundwater flux.'
        CALL PassMsg
        CLOSE(51)
        CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899
        CLOSE(61)
        CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899
        GOTO 899
      ENDIF

!
!     OPEN the STOOTLyyyy.INT and DLYTBLyyyy.INT files
!
      File71 = TRIM(ThisLakeDir) // 'STOOTL' // Yyyy // '.INT'
      OPEN(UNIT=71, FILE=TRIM(File71), STATUS='OLD', RECL=26,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=71)
      CLOSE (71, STATUS = 'DELETE')
  71  OPEN(UNIT=71, FILE=TRIM(File71), STATUS='NEW', RECL=26,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=871)
      CALL FileWasOpened(71); IF (ErrorLevel .NE. 0) GOTO 899

      File76 = TRIM(ThisLakeDir) // 'DLYTBL' // Yyyy // '.INT'
      OPEN(UNIT=76, FILE=TRIM(File76), STATUS='OLD', RECL=24,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=76)
      CLOSE (76, STATUS = 'DELETE')
  76  OPEN(UNIT=76, FILE=TRIM(File76), STATUS='NEW', RECL=24,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=876)
      CALL FileWasOpened(76); IF (ErrorLevel .NE. 0) GOTO 899

!
!     For every day in the actual forecast period (starting on the day
!     after the provisional model update), read the data from the
!     respective files, and write it to the output files.        
!
      WRITE(76, REC=1, ERR=878) FcStrt, LastNo
      DO K = FcStrt, LastNo
        J = K - FcStrt + 1
        CALL SequenceDate(iD, iM, iY, K); IF (ErrorLevel .NE. 0) GOTO 899
        READ(51, REC=Srn+J, ERR=852) Runoff, Uszm, Lszm, Gzm, Ss, Snw   ! hundredths of cm
        qRunoff = Runoff / 100.
        qUszm = Uszm / 100.
        qLszm = Lszm / 100.
        qGzm = (Gzm / 100.) * ScaleNum
        qSs = Ss / 100.
        qSnw = Snw / 100.
        iD2 = iD
        iM2 = iM
        iY2 = iY
        WRITE(71, REC=J, ERR=873) iD2, iM2, iY2, qUszm + qLszm,            &
     &             qGzm, qSs, qSnw, qUszm + qLszm + qGzm + qSs + qSnw         ! centimeters
        qRunoff = qRunoff * qTotArea / qLakeArea             ! cm over land -> cm over lake
        READ(61, REC=Srne+J, ERR=862) qEvp

        IF (iM .EQ. iMl) THEN
           IF (qRunoff .GT. -99.) THEN
             qRun(Month) = qRun(Month) + qRunoff
             Counts(1, Month) = Counts(1, Month) + 1
           END IF
           IF (qDlyPc(J) .GT. -99.) THEN
             qPre(Month) = qPre(Month) + qDlyPc(J)
             Counts(2, Month) = Counts(2, Month) + 1
           END IF
           IF (qEvp .GT. -99.) THEN
             qEvap(Month) = qEvap(Month) + qEvp
             Counts(3, Month) = Counts(3, Month) + 1
           END IF
        ELSE
           Month = Month + 1
           IF (qRunoff .GT. -99.) THEN
             qRun(Month) = qRunoff
             Counts(1, Month) = 1
           ELSE
             qRun(Month) = 0.
             Counts(1, Month) = 0
           END IF
           IF (qDlyPc(J) .GT. -99.) THEN
             qPre(Month) = qDlyPc(J)
             Counts(2, Month) = 1
           ELSE
             qPre(Month) = 0.
             Counts(2, Month) = 0
           END IF
           IF (qEvp .GT. -99.) THEN
             qEvap(Month) = qEvp
             Counts(3, Month) = 1
           ELSE
             qEvap(Month) = 0.
             Counts(3, Month) = 0
           ENDIF
           iMl = iM
        END IF

        IF (qDlyPc(J) .GT. -99. .AND. qRunoff .GT. -99. .AND. qEvp .GT. -99.) THEN
           qNBS(J) = qDlyPc(J) + qRunoff - qEvp
        ELSE
           qNBS(J) = -999.9
        ENDIF
        oTmn = qDlyTmin(J)      ! degrees C
        oTmx = qDlyTmax(J)      ! degrees C
        oRun = qRunoff*10       ! millimeters (over lake)
        oPrc = qDlyPc(J)*10     ! millimeters (over lake)
        oEvp = qEvp*10          ! millimeters (over lake)
        oNbs = qNBS(J)*10       ! millimeters (over lake)
        WRITE(76, REC=J+1, ERR=878) oTmn, oTmx, oRun, oPrc, oEvp, oNbs
      END DO

      CLOSE (51)
      CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (61, STATUS = 'DELETE')
      CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (71)
      CALL FileWasClosed(71); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (76)
      CALL FileWasClosed(76); IF (ErrorLevel .NE. 0) GOTO 899

!
! Build the NBSOTLyyyy.INT file of monthly NBS values
!   (units are centimeters over the lake)
!
      File81 = TRIM(ThisLakeDir) // 'NBSOTL' // Yyyy // '.INT'
      OPEN(UNIT=81, FILE=TRIM(File81), STATUS='OLD', RECL=4,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=81)
      CLOSE (81, STATUS = 'DELETE')
  81  OPEN(UNIT=81, FILE=TRIM(File81), STATUS='NEW', RECL=4,              &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=881)
      CALL FileWasOpened(81); IF (ErrorLevel .NE. 0) GOTO 899
      DO I = 1, ForecastPeriodLength
         WRITE(81, REC=I, ERR=883) qNBS(I)
      END DO
      CLOSE(81)
      CALL FileWasClosed(81); IF (ErrorLevel .NE. 0) GOTO 899

!
! Build the FCSTBLyyyy.INT file of monthly values
!   (units are millimeters over the lake)
!
      CALL SequenceDate(iD, iM, iY, FcStrt); IF (ErrorLevel .NE. 0) GOTO 899
      File86 = TRIM(ThisLakeDir) // 'FCSTBL' // Yyyy // '.INT'
      OPEN(UNIT=86, FILE=TRIM(File86), STATUS='OLD', ERR=86)
      CLOSE (86, STATUS = 'DELETE')
  86  OPEN(UNIT=86, FILE=TRIM(File86), STATUS='NEW', ERR=886)
      CALL FileWasOpened(86); IF (ErrorLevel .NE. 0) GOTO 899
      J = Length
      IF (FcDay .NE. 1) J = Length + 1
      DO I = 1, J
        WRITE(86, 8601, ERR=888) qRun(I)*10., qPre(I)*10., qEvap(I)*10.,  &
     &                  (qRun(I) + qPre(I) - qEvap(I)) * 10., iM, iY
        iM = iM + 1
        IF (iM .GT. 12) THEN
           iM = iM - 12
           iY = iY + 1
        END IF
      END DO
      CLOSE (86)
      CALL FileWasClosed(86); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Update 'BSNNM' file with new program designator.
!
      File21 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(21, 2102, REC=2, ERR=823) NoSubbasins, 10, OverTheLake, Lf
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File11)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File11)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File11)
      CALL PassMsg; GOTO 898
  821 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File21)
      CALL PassMsg; GOTO 898
  822 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File21)
      CALL PassMsg; GOTO 898
  823 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File21)
      CALL PassMsg; GOTO 898
  826 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File26)
      CALL PassMsg; GOTO 898
  827 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File26)
      CALL PassMsg; GOTO 898
  828 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File26)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File31)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File31)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File31)
      CALL PassMsg; GOTO 898
  841 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File41)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File41)
      CALL PassMsg; GOTO 898
  843 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File41)
      CALL PassMsg; GOTO 898
  851 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File51)
      CALL PassMsg; GOTO 898
  852 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File51)
      CALL PassMsg; GOTO 898
  853 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File51)
      CALL PassMsg; GOTO 898
  861 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File61)
      CALL PassMsg; GOTO 898
  862 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File61)
      CALL PassMsg; GOTO 898
  863 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File61)
      CALL PassMsg; GOTO 898
  871 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File71)
      CALL PassMsg; GOTO 898
  872 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File71)
      CALL PassMsg; GOTO 898
  873 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File71)
      CALL PassMsg; GOTO 898
  876 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File76)
      CALL PassMsg; GOTO 898
  877 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File76)
      CALL PassMsg; GOTO 898
  878 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File76)
      CALL PassMsg; GOTO 898
  881 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File81)
      CALL PassMsg; GOTO 898
  882 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File81)
      CALL PassMsg; GOTO 898
  883 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File81)
      CALL PassMsg; GOTO 898
  886 ErrorMessage = 'LUMPOUT1: Error opening file ' // TRIM(File86)
      CALL PassMsg; GOTO 898
  887 ErrorMessage = 'LUMPOUT1: Error reading file ' // TRIM(File86)
      CALL PassMsg; GOTO 898
  888 ErrorMessage = 'LUMPOUT1: Error writing file ' // TRIM(File86)
      CALL PassMsg; GOTO 898


  898 ErrorLevel = 1
  899 CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_LMP1...'
      CALL PassMsg
      GOTO 999

!  -----------------------

  999 CALL CloseAllFiles()
      ErrStatus = ErrorLevel

      RETURN

!
! FORMATs
!
 1002 FORMAT(I4)

 1102 FORMAT(I4)
 1103 FORMAT(A4)
 2101 FORMAT(A10)
 2102 FORMAT(2I3, A4, 1X, A1)
 2105 FORMAT(2I3, I5)
 2601 FORMAT(45X, I5)
 3101 FORMAT(E13.6E2)
 8601 FORMAT(4F8.2, 5X, I2, '/', I4)

 9001 FORMAT('LUMPOUT1: Invalid sequencing of forecast programs.')
 9002 FORMAT('LUMPOUT1: Incorrect basin name in BSNNM.')
 9005 FORMAT('LUMPOUT1: Forecast period length exceeds program limits.')
 9010 FORMAT('LUMPOUT1: Internal error which should never happen!')

      END SUBROUTINE DO_LMP1


END MODULE LUMPOUT1_Main
