!----------------------------------------------------------------------------
!
!     This routine reads all STOOTLzz.INT and FCSTBLzz.INT files and determines
!     the average (daily or monthly) value for every storage in STOOTLzz.INT
!     and for every variable in FCSTBLzz.INT.  It places this information into
!     STOOTLKS.lNT and MFCSTTBL.INT, respectively.  It places simulated storage
!     results preceding the forecast period into SIMULSTO.INT.
!
!     Updated by TEC II 28sep88, 26oct88 (for PC)
!     Updated by TEC II 17jan90 to allow for variable (long) plotting period.
!        Note that while there is no limit to simulation length with regard to
!        processing of daily data (STOOTLzz.INT -> STOOTLKS.INT) since the
!        program processes it in pieces, there is a limit to simulation length
!        with regard to processing of monthly data (FCSTBLzz.INT, LVLOTLzz.INT
!        -> MFCSTTBL.INT) since the program processes it as one piece.
!     Updated by TSH on 23apr92 to handle missing data in FCSTBLzz.INT
!     Updated 27oct92, TSH, TEC II for 4-byte dates and move to UNIX
!     Updated 14apr94, TSH, changed file "SFCSTTBL.INT" to "MFCSTTBL.INT"
!     Updated by TEC II 21mar95 for change in BSNNM structure (addition of
!      forecast start date in record 5 and displacement of later records by
!      one to accomodate the addition); also removed reference to lake level
!      computations since they will (eventually) be done in an entirely
!      different manner than that planned in the past and to suppress writing
!      of status info if file: "QUIET.INT" exists in default directory.
!
!----------------------------------------------------------------------------

MODULE LUMPOUT2_Main

      INTEGER, PARAMETER :: BufLen = 1200    ! 100 yrs of monthly data
      INTEGER, PARAMETER :: SumHdr =    2    ! # of header records in *.SUM

CONTAINS

      SUBROUTINE DO_LMP2(ErrStatus, HandleIn, Lake)
      USE FcstUtil
      IMPLICIT NONE
      INTEGER, INTENT(INOUT) :: ErrStatus, HandleIn
      CHARACTER (LEN=*), INTENT(IN) :: Lake

      INTEGER (KIND=INTEGER2) :: ScaleNum, Run, Uszm, Lszm, Gzm, Ss, Snw

      INTEGER :: I, J, K, sD, sM, sY, LastNo, FcStrt, ForecastPeriodLength
      INTEGER :: StartNo, Srn, iD, iM, iY, iQ, St, En, PieceNo, Length
      INTEGER :: NumberOfScenarios, NoSubbasins, StopDay, StopMonth, StopYear

      INTEGER, DIMENSION(4, Buflen) :: Counts

      REAL :: qUszm, qLszm, qGzm, qSs, qSnw, qRunoff, qPre, qEvap, qNBS                                       
      REAL :: Q1, Q2, Q3, Q4, Q5

      REAL, DIMENSION(13)     :: qMeanLevel
      REAL, DIMENSION(Buflen) :: q1Mean, q2Mean, q3Mean, q4Mean, q5Mean, q6Mean
      REAL, DIMENSION(12, 9)  :: Quantile, QuantNBS

      CHARACTER (LEN=1)  :: Cr, Lf
      CHARACTER (LEN=3)  :: Bsn
      CHARACTER (LEN=4)  :: OverTheLake, Yyyy, YearFile(200)
      CHARACTER (LEN=6)  :: Skip6
      CHARACTER (LEN=10) :: LakeName
      CHARACTER (LEN=20) :: S
      CHARACTER (LEN=99) :: File11, File21, File31, File41, File51
      CHARACTER (LEN=99) :: File61, File66, File71, File76, File81, File86

      Lf = CHAR(10)
      Cr = CHAR(13)

      WinHandle = HandleIn
      ErrorLevel = 0
      CALL ClearFilesOpen(); IF (ErrorLevel .NE. 0) GOTO 899

!
!  Find out which directory each lake is in.  Values are assigned to
!  the module array CfgDirs -- a 9-element character array -- as follows:
!    CfgDirs(1) = Lake Superior
!    CfgDirs(2) = Lake Michigan
!    CfgDirs(3) = Lake Huron
!    CfgDirs(4) = Georgian Bay
!    CfgDirs(5) = Lake StClair
!    CfgDirs(6) = Lake Erie
!    CfgDirs(7) = Lake Ontario
!    CfgDirs(8) = Lake Michigan-Huron
!    CfgDirs(9) = Level (CGLRRM stuff)
!
!  Each valid entry will end with the requisite (for directories) backslash.
!
      CALL ReadCfgFile('..\FORECAST.CFG')
      IF (ErrorLevel .NE. 0) GOTO 899

!
!  Make sure that there was an entry in FORECAST.CFG for the specified lake.
!
      S = TRIM(ADJUSTL(Lake))
      CALL Caps(S); IF (ErrorLevel .NE. 0) GOTO 899
      ThisLakeDir = ''
      DO I = 1, 8
         IF (LakeName3(I) .EQ. S(1:3)) ThisLakeDir = TRIM(CfgDirs(I))
      END DO
      IF (ThisLakeDir .EQ. '') THEN
         ErrorMessage = ' No entry in FORECAST.CFG for '//S//'!'
         CALL PassMsg
         GOTO 898
      ENDIF

!
!     Input basin name and other info.
!
      File11 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', RECL=12,               &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(11, 1101, REC=1, ERR=812) LakeName
      Bsn = LakeName(1:3)
      READ(11, 1102, REC=2, ERR=812) NoSubbasins, I, OverTheLake
      IF (I .NE. 10) THEN
         ErrorMessage = 'LUMPOUT2: Invalid sequencing of forecast programs.'
         CALL PassMsg
         GOTO 898
      END IF

!
!     Find out when forecast begins.                                    
!
      READ(11, 1105, REC=5, ERR=812) sD, sM, sY
      CALL DateSequence(sD, sM, sY, FcStrt);  IF (ErrorLevel .NE. 0) GOTO 899

!
!     Find out when model output files end (all are assumed
!     to end on the same date).
!
      READ(11, 1105, REC=6, ERR=812) StopDay, StopMonth, StopYear          
      CALL DateSequence(StopDay, StopMonth, StopYear, LastNo); IF (ErrorLevel .NE. 0) GOTO 899
      ForecastPeriodLength = LastNo - FcStrt + 1

!
!     Determine starting date for model statistics output.
!
      READ(11, 1105, REC=7, ERR=812) sD, sM, sY
      CALL DateSequence(sD, sM, sY, StartNo); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Read scenario years from YEARFILE.INT
!
      File21 = TRIM(ThisLakeDir) // 'YEARFILE.INT'
      OPEN(UNIT=21, FILE=TRIM(File21), STATUS='OLD', ERR=821)
      CALL FileWasOpened(21); IF (ErrorLevel .NE. 0) GOTO 899
      NumberOfScenarios = 0
      READ(21, 2101, IOSTAT=K) I
      DO WHILE (K .EQ. 0)
        NumberOfScenarios = NumberOfScenarios + 1
        WRITE(Yyyy, 9901) I
        YearFile(NumberOfScenarios) = Yyyy
        READ(21, 2101, IOSTAT=K) I
      ENDDO
      CLOSE (21)
      CALL FileWasClosed(21); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Open entire-basin model output file: xxxALL.SUM.
!
      File31 = TRIM(ThisLakeDir) // Bsn // 'ALL.SUM'
      OPEN(UNIT=31, FILE=TRIM(File31), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM='UNFORMATTED', ERR=831)
      CALL FileWasOpened(31); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Sequence number for first day in file.
!
      READ(31, REC=1, ERR=832) ScaleNum, Srn, ScaleNum

!
!     Record number for first day of output minus one.
!
      Srn = (StartNo - Srn + 1) + SumHdr - 1

!
!     Input and output basin storages for simulation period.
!
      File41 = TRIM(ThisLakeDir) // 'SIMULSTO.INT'
      OPEN(UNIT=41, FILE=TRIM(File41), STATUS='OLD', ERR=41)
      CLOSE (41, STATUS = 'DELETE')
 41   OPEN(UNIT=41, FILE=TRIM(File41), STATUS='NEW', ERR=841)
      CALL FileWasOpened(41); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(41, 4101, ERR=843) ForecastPeriodLength
      WRITE(41, 4102, ERR=843) sD, sM, sY
      DO K = StartNo, FcStrt - 1
         CALL SequenceDate(iD, iM, iY, K); IF (ErrorLevel .NE. 0) GOTO 899
         J = Srn + K - StartNo + 1
         READ(31, REC=J, ERR=832) Run, Uszm, Lszm, Gzm, Ss, Snw
         qUszm = Uszm / 100.                      ! hundredths of cm -> cm
         qLszm = Lszm / 100.                      ! hundredths of cm -> cm
         qGzm = (Gzm / 100.) * ScaleNum           ! hundredths of cm -> cm
         qSs = Ss / 100.                          ! hundredths of cm -> cm
         qSnw = Snw / 100.                        ! hundredths of cm -> cm
         WRITE(41, 4103, ERR=843) iD, iM, iY, qUszm + qLszm, qGzm, qSs,   &
     &                         qSnw, qUszm + qLszm + qGzm + qSs + qSnw
      END DO
      CLOSE (31)
      CALL FileWasClosed(31); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE (41)
      CALL FileWasClosed(41); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Determine average storages for the forecast period.
!
      File51 = TRIM(ThisLakeDir) // 'STOOTLKS.INT'
      OPEN(UNIT=51, FILE=TRIM(File51), STATUS='OLD', ERR=51)
      CLOSE (51, STATUS='DELETE')
   51 OPEN(UNIT=51, FILE=TRIM(File51), STATUS='NEW', ERR=851)
      CALL FileWasOpened(51); IF (ErrorLevel .NE. 0) GOTO 899
      PieceNo = 0
      St = 1
      DO WHILE (St .LE. ForecastPeriodLength)
         PieceNo = PieceNo + 1
!         WRITE(ErrorMessage, 9009) PieceNo; CALL PassMsg
         En = St + BufLen - 1
         IF (En .GT. ForecastPeriodLength) En = ForecastPeriodLength
         q1Mean(1:BufLen) = 0.
         q2Mean(1:BufLen) = 0.
         q3Mean(1:BufLen) = 0.
         q4Mean(1:BufLen) = 0.
         q5Mean(1:BufLen) = 0.
         DO I = 1, NumberOfScenarios
            Yyyy = YearFile(I)
            File61 = TRIM(ThisLakeDir) // 'STOOTL' // Yyyy // '.INT'
            OPEN(UNIT=61, FILE=TRIM(File61), STATUS='OLD', RECL=26,       &
     &           ACCESS='DIRECT', FORM='UNFORMATTED', ERR=861)
            CALL FileWasOpened(61); IF (ErrorLevel .NE. 0) GOTO 899
            DO K = St, En
               READ(61, REC=K, ERR=862) Skip6, Q1, Q2, Q3, Q4, Q5
               J = K - St + 1
               q1Mean(J) = q1Mean(J) + Q1 / NumberOfScenarios
               q2Mean(J) = q2Mean(J) + Q2 / NumberOfScenarios
               q3Mean(J) = q3Mean(J) + Q3 / NumberOfScenarios
               q4Mean(J) = q4Mean(J) + Q4 / NumberOfScenarios
               q5Mean(J) = q5Mean(J) + Q5 / NumberOfScenarios
            END DO
            CLOSE (61)
            CALL FileWasClosed(61); IF (ErrorLevel .NE. 0) GOTO 899
         END DO
         DO K = St, En
            CALL SequenceDate(iD, iM, iY, FcStrt+K-1); IF (ErrorLevel .NE. 0) GOTO 899
            J = K - St + 1
            WRITE(51, 5101, ERR=853) iD, iM, iY, q1Mean(J), q2Mean(J),    &
     &                                q3Mean(J), q4Mean(J), q5Mean(J)
         END DO
         St = En + 1
      END DO
      CLOSE (51)
      CALL FileWasClosed(51); IF (ErrorLevel .NE. 0) GOTO 899

!      ErrorMessage = '~^~   '  ! blank out the line
!      CALL PassMsg

!
!      Now write monthly outputs.
!
      File66 = TRIM(ThisLakeDir) // 'LENGTH.' // Bsn
      OPEN(UNIT=66, FILE=TRIM(File66), STATUS='OLD', RECL=52,           &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=866)
      CALL FileWasOpened(66); IF (ErrorLevel .NE. 0) GOTO 899
      READ(66, 6601, REC=1, ERR=867) Length
      CLOSE (66)
      CALL FileWasClosed(66); IF (ErrorLevel .NE. 0) GOTO 899
      IF (Length .GT. BufLen) THEN
         ErrorMessage = 'LUMPOUT2: Forecast period length exceeds program limits.'
         CALL PassMsg
         GOTO 898
      END IF

!
!     Lake level quantiles forever taken as unavailable now.            
!
      DO K = 1, Length
        q1Mean(K) = -999.
        q2Mean(K) = -999.
      END DO
      DO K =  1, 13
        qMeanLevel(K) = -999.
      END DO

!
!     Compute averages of monthly variables.
!
      q3Mean(1:K) = 0.       ! q3 = runoff 
      q4Mean(1:K) = 0.       ! q4 = net basin supply  
      q5Mean(1:K) = 0.       ! q5 = precipitation
      q6Mean(1:K) = 0.       ! q6 = evaporation
      Counts(:, 1:K) = 0

      DO I = 1, NumberOfScenarios
         Yyyy = YearFile(I)
         File71 = TRIM(ThisLakeDir) // 'FCSTBL' // Yyyy // '.INT'
         OPEN(UNIT=71, FILE=TRIM(File71), STATUS='OLD', ERR=871)
         CALL FileWasOpened(71); IF (ErrorLevel .NE. 0) GOTO 899
         DO K = 1, Length
           READ(71, 7101, ERR=872) qRunoff, qPre, qEvap, qNBS    ! mm over the lake
           IF (qRunoff .GT. -999.) THEN
             q3Mean(K) = q3Mean(K) + qRunoff
             Counts(1, K) = Counts(1, K) + 1
           END IF
           IF (qNBS .GT. -999.) THEN
             q4Mean(K) = q4Mean(K) + qNBS
             Counts(2, K) = Counts(2, K) + 1
           END IF
           IF (qPre .GT. -999.) THEN
             q5Mean(K) = q5Mean(K) + qPre
             Counts(3, K) = Counts(3, K) + 1
           END IF
           IF (qEvap .GT. -999.) THEN
             q6Mean(K) = q6Mean(K) + qEvap
             Counts(4, K) = Counts(4, K) + 1
           END IF
         END DO
         CLOSE (71)
         CALL FileWasClosed(71); IF (ErrorLevel .NE. 0) GOTO 899
      END DO

      DO K = 1, Length
        IF (Counts(1, K) .GT. 0) THEN
          q3Mean(K) = q3Mean(K) / Counts(1, K)
        ELSE
          q3Mean(K) = -999.99
        ENDIF
        IF (Counts(2, K) .GT. 0) THEN
          q4Mean(K) = q4Mean(K) / Counts(2, K)
        ELSE
          q4Mean(K) = -999.99
        ENDIF
        IF (Counts(3, K) .GT. 0) THEN
          q5Mean(K) = q5Mean(K) / Counts(3, K)
        ELSE
          q5Mean(K) = -999.99
        ENDIF
        IF (Counts(4, K) .GT. 0) THEN
          q6Mean(K) = q6Mean(K) / Counts(4, K)
        ELSE
          q6Mean(K) = -999.99
        ENDIF
      END DO

!
!     Read runoff quantiles (centimeters over the lake).
!
      File76 = TRIM(ThisLakeDir) // 'RUNQUANT.' // Bsn
      OPEN(UNIT=76, FILE=TRIM(File76), STATUS='OLD', ERR=876)
      CALL FileWasOpened(76); IF (ErrorLevel .NE. 0) GOTO 899
      READ(76, 7601, ERR=877) ((Quantile(K, iQ), iQ = 1,9), K = 1,12)
      CLOSE (76)
      CALL FileWasClosed(76); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Read net basin supply quantiles (centimeters over the lake).
!
      File81 = TRIM(ThisLakeDir) // 'NBSQUANT.' // Bsn
      OPEN(UNIT=81, FILE=TRIM(File81), STATUS='OLD', ERR=881)
      CALL FileWasOpened(81); IF (ErrorLevel .NE. 0) GOTO 899
      READ(81, 8101, ERR=882) ((QuantNBS(K, iQ), iQ = 1,9), K = 1,12)
      CLOSE (81)
      CALL FileWasClosed(81); IF (ErrorLevel .NE. 0) GOTO 899

!
!     Write table of month, year, runoff, median runoff, precipitation,
!     median lake evaporation, and median net basin supply using units
!     of millimeters over the lake.
!
      File86 = TRIM(ThisLakeDir) // 'MFCSTTBL.INT'
      OPEN(UNIT=86, FILE=TRIM(File86), STATUS='OLD', ERR=86)
      CLOSE (86, STATUS = 'DELETE')
  86  OPEN(UNIT=86, FILE=TRIM(File86), STATUS='NEW', ERR=886)
      CALL FileWasOpened(86); IF (ErrorLevel .NE. 0) GOTO 899
      iY = StopYear
      iM = StopMonth - Length

!
!     iM = starting month of outlook minus one and iY = corresponding year
!
      DO WHILE (iM .LT. 1)
        iM = iM + 12
        iY = iY - 1
      END DO
      DO I = 1, Length
        iM = iM + 1
        IF (iM .GT. 12) THEN
          iM = iM - 12
          iY = iY + 1
        ENDIF
        WRITE(86, 8601, ERR=888) MonthName10(iM), iY, q3Mean(I),          &
     &       Quantile(iM, 5) * 10., q4Mean(I), QuantNBS(iM, 5) * 10.,     &
     &       q1Mean(I), qMeanLevel(iM), q2Mean(I),                        &
     &       qMeanLevel(iM + 1), q5Mean(I), q6Mean(I)
      END DO
      CLOSE (86)
      CALL FileWasClosed(86); IF (ErrorLevel .NE. 0) GOTO 899

!
!      Update 'BSNNM' file with new program designator.
!
      File11 = TRIM(ThisLakeDir) // 'BSNNM'
      OPEN(UNIT=11, FILE=TRIM(File11), STATUS='OLD', RECL=12,             &
     &     ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(11); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(11, 1102, REC=2, ERR=813) NoSubbasins, 0, OverTheLake, Lf
      CLOSE (11)
      CALL FileWasClosed(11); IF (ErrorLevel .NE. 0) GOTO 899

      GOTO 999

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File11)
      CALL PassMsg; GOTO 898
  812 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File11)
      CALL PassMsg; GOTO 898
  813 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File11)
      CALL PassMsg; GOTO 898
  821 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File21)
      CALL PassMsg; GOTO 898
  822 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File21)
      CALL PassMsg; GOTO 898
  823 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File21)
      CALL PassMsg; GOTO 898
  831 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File31)
      CALL PassMsg; GOTO 898
  832 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File31)
      CALL PassMsg; GOTO 898
  833 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File31)
      CALL PassMsg; GOTO 898
  841 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File41)
      CALL PassMsg; GOTO 898
  842 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File41)
      CALL PassMsg; GOTO 898
  843 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File41)
      CALL PassMsg; GOTO 898
  851 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File51)
      CALL PassMsg; GOTO 898
  852 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File51)
      CALL PassMsg; GOTO 898
  853 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File51)
      CALL PassMsg; GOTO 898
  861 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File61)
      CALL PassMsg; GOTO 898
  862 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File61)
      CALL PassMsg; GOTO 898
  863 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File61)
      CALL PassMsg; GOTO 898
  866 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File66)
      CALL PassMsg; GOTO 898
  867 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File66)
      CALL PassMsg; GOTO 898
  868 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File66)
      CALL PassMsg; GOTO 898
  871 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File71)
      CALL PassMsg; GOTO 898
  872 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File71)
      CALL PassMsg; GOTO 898
  873 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File71)
      CALL PassMsg; GOTO 898
  876 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File76)
      CALL PassMsg; GOTO 898
  877 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File76)
      CALL PassMsg; GOTO 898
  878 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File76)
      CALL PassMsg; GOTO 898
  881 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File81)
      CALL PassMsg; GOTO 898
  882 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File81)
      CALL PassMsg; GOTO 898
  883 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File81)
      CALL PassMsg; GOTO 898
  886 ErrorMessage = 'LUMPOUT2: Error opening file ' // TRIM(File86)
      CALL PassMsg; GOTO 898
  887 ErrorMessage = 'LUMPOUT2: Error reading file ' // TRIM(File86)
      CALL PassMsg; GOTO 898
  888 ErrorMessage = 'LUMPOUT2: Error writing file ' // TRIM(File86)
      CALL PassMsg; GOTO 898


  898 ErrorLevel = 1
  899 CALL MakeErrorInt
      ErrorMessage = '[traceback] : DO_LMP2...'
      CALL PassMsg
      GOTO 999

!------------------

  999 CALL CloseAllFiles()
      ErrStatus = ErrorLevel
      RETURN

!
!  FORMATs
!
 1101 FORMAT(A10)
 1102 FORMAT(2I3, A4, 1X, A1)
 1105 FORMAT(2I3, I5, A1)
 2101 FORMAT(I4)
 4101 FORMAT(I5)
 4102 FORMAT(I5, I3, I5)
 4103 FORMAT(I5, I3, I5, 5F13.4)
 5101 FORMAT(I5, I3, I5, 5F13.4)
 6601 FORMAT(45X, I5)
 7101 FORMAT(4F8.2)
 7601 FORMAT(9X, 9E13.6E2)
 8101 FORMAT(9X, 9E13.6E2)
 8601 FORMAT(A10, I5, 10F8.2)

 9009 FORMAT('~^~Processing piece number: ', I4)
 9901 FORMAT(I4)

      END SUBROUTINE DO_LMP2


END MODULE LUMPOUT2_Main



