! ******************************************************************************
!-------------------------------------------------------------------------------
!   ErrorChk is a subroutine intended to be compiled as a DLL for use
!   with a GUI or other main program. It is the first step in processing
!   station data for use within GLERL's AHPS package.
!
!   This version is an adaptation of the earlier programs which processed
!   station data in *.STN format.  In late 2013, I determined that the
!   time had come to revamp GLERL's internal format for meteorological 
!   station data. The old formats were built on fixed-width fields and
!   relied on the station ID (specifically the first character of the ID)
!   to designate the units (English or metric) of the data contained in 
!   the file. That station ID was also limited to seven characters in length.
!
!   As the data collection agencies change their systems for assigning 
!   station IDs the old system is no longer functional. Specifically, NCDC
!   has folded the old cooperative station network (TD3200) into their GHCND
!   dataset, along with other station datasets. This new GHCDND dataset has
!   longer station IDs (11 characters) that completely break the old versions
!   of these data processing programs used in AHPS.
!
!   Since we need to revise our internal format, I could have gone with a
!   minor incremental change (just widen some fields, etc.). But I chose to
!   do some "future-proofing" at the same time. So the met station files are
!   now a CSV format (much more compatible with software in general use).
!   I am also separating the unit designation from the station ID. The units
!   are now explicitly defined, and not implied by the station ID.
!
!   I am also removing the concept of a "base date" from this program. That
!   base date concept is scheduled to be removed in the next major revision
!   of AHPS, and it is not really necessary for the station data anyway.
!   So I might as well eliminate it now.  Note that it still exists in the
!   BSNNM/EBSNNM files; I just ignore it for any processing in this program.
!
!   Tim Hunter - 16 Dec 2013
!
!   Modified by Tim Hunter, 26 Oct 2001
!       Added the "TruncToBase" parameter in the main procedure.  If the
!       user supplies station files with data before the base date then this
!       parameter specifies the desired behavior.  See top of main routine
!       for more information.
!
!   If the program completes normally, the variables ErrCount and WarnCount
!   will contain the numbers of errors and warnings encountered in the
!   station files (obviously).  If an "abort"-type error occurs, i.e. an
!   error opening a file or something else causing the program to terminate
!   prematurely, then ErrCount will have a value of -1 and the variable
!   CrashString will contain the appropriate error message.
!
!   This program checks all specified met files for obvious errors.
!   Messages are printed as errors are found.  All data are tested and,
!   at the end, if any uncorrectable errors have been found, the
!   procedure terminates with an error condition.
!
!   This program reads the file: EBSNNM/BSNNM and updates it; all forecast
!   programs do the same with the exception of optional modules.
!   This file is not to be disturbed by the user as it exists only for
!   the purpose of each program keeping track of what other programs have
!   been run as well as to reference specific basin information.
!   It is a formatted direct access file with 12-byte records (the last
!   byte of each record is a line feed) and its contents are as follows:
!
!   -------------------
!   (EBSNNM)
!
!   1st record:   (FORMAT: A10, 1X)
!
!     Name  -  basin name, left adjusted, in all upper case.
!
!   2nd record:   (FORMAT: 3X, I3, 5X)
!
!     Program designator number.
!
!   The remaining records are filled in during a forecast session and are
!   rewritten by the programs only.  They have no meaning after the
!   session is over.  Each record is a date (day, month, year) in the
!   format: 2I3, I5.
!
!   3rd through 8th records:   (FORMAT: 2I3, I5)
!
!     3rd record:  base date (see PRVSNAL.f90)
!     4th record:  provisional data end date
!     5th record:  forecast start date
!     6th record:  forecast end date
!     7th record:  plot start date
!     8th record:  date of earliest new data since XEVAPOUT.f90 last run
!   -------------------
!   (BSNNM)
!
!   1st record:   (FORMAT: A10, 1X)
!
!     Name  -  basin name, left adjusted, in all upper case.
!
!   2nd record:   (FORMAT: 2I3, A4, 1X)
!
!     # # switch
!     | | |    |
!     | | |____|____lake precipitation computation switch:
!     | |           "BASN" => estimate over-the-lake precipitation with
!     | |                     land depths
!     | |           "LAKE" => estimate over-the-lake precipitation
!     | |                     directly from nearby gages (subbasin "0"
!     | |                     file)
!     | |_________program designator number; each program has its own and
!     |           places it in BSNNM so programs can check for proper
!     |           sequence
!     |_________number of subbasins in basin
!
!   The remaining records are filled in during a forecast session and are
!   rewritten by the programs only.  They have no meaning after the
!   session is over.  Each record is a date (day, month, year) in the
!   format: 2I3, I5.
!
!   3rd through 9th records:   (FORMAT: 2I3, I5)
!
!     3rd record:  base date (see PROVSNAL.f and REPACK.f)
!     4th record:  provisional data end date
!     5th record:  forecast start date
!     6th record:  forecast end date
!     7th record:  plot start date
!     8th record:  date of earliest new data since WATOUT.f last run
!     9th record:  date of earliest new data since LUMPSTOR.f last run
!    10th record:  date of earliest new data since CMOFPOSR.f last run
!   -------------------
!
!   This program reads the list of files to check as PROVSTNS.LIS; this contains
!   the full file specification of each file to be opened and read by this program.  
!   Each record of PROVSTNS.LIS should contain one file specification (filename) 
!   left-adjusted in the record, not to exceed 150 characters.
!
!-------------------------------------------------------------------------------
! ******************************************************************************


MODULE ErrorChkSubs

!-------------------------------------------------------------------------------
!
!  Module GLERLUTIL contains the following "global" variables that are relevant
!  for this program:
!
!   CHARACTER (LEN=128) :: ErrorMessage
!   INTEGER             :: ErrorLevel
!   INTEGER             :: WinHandle    ! Windows handle used to pass messages back to the GUI
!   INTEGER             :: OpenedFiles
!   INTEGER, PARAMETER  :: MaxOpenedFiles = 20
!   INTEGER, DIMENSION(MaxOpenedFiles)  :: OpenedFileUnits
!
!  It also contains the following relevant subroutines/functions:
!
!   SUBROUTINE DateSequence(Day, Month, Year, SequenceNumber)
!   SUBROUTINE SequenceDate(iD, iM, iY, SequenceNumber)
!   SUBROUTINE DeleteFile (Name)
!   SUBROUTINE Caps (StringToConvert)
!
!  Within GLERLUTIL, the type or "kind" definitions are set:
!
!   INTEGER, PARAMETER :: INTEGER2 = SELECTED_INT_KIND(R=4)
!   INTEGER, PARAMETER :: INTEGER4 = SELECTED_INT_KIND(R=8)
!   INTEGER, PARAMETER :: REAL4    = SELECTED_REAL_KIND(P=6,R=37)
!   INTEGER, PARAMETER :: REAL8    = SELECTED_REAL_KIND(P=15,R=100)
!
!-------------------------------------------------------------------------------
!
      USE GLERLUTIL
      USE ReadMetCsv

      !
      !  Various global variables that keep subroutine parameter lists from
      !  being "ugly long".
      !
      CHARACTER (LEN=3)  ::  Bsn        ! 3-char basin code (SUP, MIC, etc)

      INTEGER  :: ErrFatal
      CHARACTER (LEN=80) :: File_ID

      INTEGER  :: Warnings, Removed   ! accumulators for a single station
      
      INTEGER  :: TotalNumRemovals    ! how many total errors were removed (all files)
      INTEGER  :: TotalNumWarnings    ! how many total warnings were given (all files)

      
CONTAINS

!-------------------------------------------------------------------------
!  This routine checks that all of the data values are within reasonable limits.
!
!  Temperature values must be in degrees C.
!  Precipitation must be in millimeters.
!  Wind Speed must be in meters per second.
!  Cloud Cover must be in percent (0-100).
!
!  Note that I do adjust the valid temperature ranges based on the date.
!  That's because 35 C is possible in July, but not in January.
!  And -30 C is possible in January, but not July.
!  In the earlier code I also had some adjustment for latitude, but I
!  am removing that. It was a bit too subjective, in my opinion, and
!  just added processing overhead.  I am just setting the ranges to 
!  encompass the entire Great Lakes region. 
!  International Falls, MN was used as the standard for the minimums. 
!  Fort Wayne, IN was used as the standard for the maximums.
!
!    Month    AirTemps (Max / Min)
!    Jan      25 / -50
!    Feb      25 / -50
!    Mar      30 / -40
!    Apr      35 / -30
!    May      40 / -20
!    Jun      45 / -10
!    Jul      50 /  -5
!    Aug      45 /  -5
!    Sep      40 / -15
!    Oct      35 / -25
!    Nov      30 / -40
!    Dec      25 / -50
!
!  The DataVals array contains the 7 daily data values of interest, in 
!  the following order:
!    DataVals(1) = Max Air Temp (deg C)
!    DataVals(2) = Min Air Temp (deg C)
!    DataVals(3) = Total Precipitation (mm)
!    DataVals(4) = Mean Air Temp (deg C)
!    DataVals(5) = Mean Dewpoint Temp (deg C)
!    DataVals(6) = Mean Wind Speed (m/s)
!    DataVals(7) = Mean Cloud Cover (%)
!--------------------------------------------------------------------------
      SUBROUTINE DataCheck(DataVals, DataUnits, SeqNum)
      IMPLICIT   NONE
      REAL,    DIMENSION(7), INTENT(INOUT) :: DataVals
      INTEGER, DIMENSION(7), INTENT(IN)    :: DataUnits
      INTEGER, INTENT(IN)    :: SeqNum

      INTEGER :: DegC, MM, MPS, Pct, Day, Mon, Year, ErrStat
      REAL    :: TMax, TMin, Prec, Air, Dew, Wind, Cloud
      REAL, DIMENSION(12), PARAMETER :: MinTemps =         &
              (/-50.0, -50.0, -40.0, -30.0, -20.0, -10.0,  &
                 -5.0,  -5.0, -15.0, -25.0, -40.0, -50.0 /)
      REAL, DIMENSION(12), PARAMETER :: MaxTemps =         &
              (/ 25.0,  25.0,  30.0,  35.0,  40.0,  45.0,  &
                 50.0,  45.0,  40.0,  35.0,  30.0,  25.0 /)
               
      !
      !  Assign index values for the units needed in this routine.
      !  These index values are used for unit conversions.
      !
      DegC = DataUnitIndexFromString('Celsius')
      MM   = DataUnitIndexFromString('Millimeters')
      MPS  = DataUnitIndexFromString('MetersPerSecond')
      Pct  = DataUnitIndexFromString('Percent')

      !
      !  Assign data values to temporary working variables.
      !  Convert values to the correct units for this routine.
      !
      TMax  = ConvertDataUnits(DataVals(1), DataUnits(1), DegC)
      TMin  = ConvertDataUnits(DataVals(2), DataUnits(2), DegC)
      Prec  = ConvertDataUnits(DataVals(3), DataUnits(3), MM)
      Air   = ConvertDataUnits(DataVals(4), DataUnits(4), DegC)
      Dew   = ConvertDataUnits(DataVals(5), DataUnits(5), DegC)
      Wind  = ConvertDataUnits(DataVals(6), DataUnits(6), MPS)
      Cloud = ConvertDataUnits(DataVals(7), DataUnits(7), Pct)
      
      !
      !  Get year, month, day for use in any error messages
      !
      CALL SequenceDate(Day, Mon, Year, SeqNum)
      
      !
      !  Check validity of TMax/Tmin
      !
      IF (.NOT. IsMissing(TMax)) THEN
         CALL RangeCheck(SeqNum, 1, DataUnits(1), TMax, MinTemps(Mon), MaxTemps(Mon), ErrStat)
         IF (ErrStat .NE. 0) THEN
            TMax = MissingData_Real
            Removed = Removed + 1
         END IF
      END IF
      IF (.NOT. IsMissing(TMin)) THEN
         CALL RangeCheck(SeqNum, 2, DataUnits(2), TMin, MinTemps(Mon), MaxTemps(Mon), ErrStat)
         IF (ErrStat .NE. 0) THEN
            TMin = MissingData_Real
            Removed = Removed + 1
         END IF
      END IF
      IF ((.NOT. IsMissing(TMax)) .AND. (.NOT. IsMissing(TMin))) THEN
         CALL CompareValues(SeqNum, 1, 2, TMax, TMin, ErrStat)
         IF (ErrStat .NE. 0) THEN
            TMax = MissingData_Real
            TMin = MissingData_Real
            Removed = Removed + 1
         END IF
      END IF
      
      !
      !  Check validity of precipitation
      !
      IF (.NOT. IsMissing(Prec)) THEN
         CALL RangeCheck(SeqNum, 3, DataUnits(3), Prec, 0.0, 250.0, ErrStat)
         IF (ErrStat .NE. 0) THEN
            Prec = MissingData_Real
            Removed = Removed + 1
         END IF
      END IF
      
      !
      !  Check validity of dry bulb air temp
      !
      IF (.NOT. IsMissing(Air)) THEN
         CALL RangeCheck(SeqNum, 4, DataUnits(4), Air, MinTemps(Mon), MaxTemps(Mon), ErrStat)
         IF (ErrStat .NE. 0) THEN
            Air = MissingData_Real
            Removed = Removed + 1
         END IF
      END IF
      
      !
      !  Check validity of dew point temp
      !
      IF (.NOT. IsMissing(Dew)) THEN
         CALL RangeCheck(SeqNum, 5, DataUnits(5), Dew, MinTemps(Mon), MaxTemps(Mon), ErrStat)
         IF (ErrStat .NE. 0) THEN
            Dew = MissingData_Real
            Removed = Removed + 1
         END IF
      END IF

      !
      !  Check validity of wind speed
      !
      IF (.NOT. IsMissing(Wind)) THEN
         CALL RangeCheck(SeqNum, 6, DataUnits(6), Wind, 0.0, 50.0, ErrStat)
         IF (ErrStat .NE. 0) THEN
            Wind = MissingData_Real
            Removed = Removed + 1
         END IF
      END IF

      !
      !  Check validity of cloud cover
      !
      IF (.NOT. IsMissing(Cloud)) THEN
         CALL RangeCheck(SeqNum, 7, DataUnits(7), Cloud, 0.0, 100.0, ErrStat)
         IF (ErrStat .NE. 0) THEN
            Cloud = MissingData_Real
            Removed = Removed + 1
         END IF
      END IF

      !
      !  Assign (possibly modified) data values back into the passed array.
      !  Convert back to the correct units.
      !
      DataVals(1) = ConvertDataUnits(TMax,  DegC,  DataUnits(1))
      DataVals(2) = ConvertDataUnits(TMin,  DegC,  DataUnits(2))
      DataVals(3) = ConvertDataUnits(Prec,    MM,  DataUnits(3))
      DataVals(4) = ConvertDataUnits(Air,   DegC,  DataUnits(4))
      DataVals(5) = ConvertDataUnits(Dew,   DegC,  DataUnits(5))
      DataVals(6) = ConvertDataUnits(Wind,   MPS,  DataUnits(6))
      DataVals(7) = ConvertDataUnits(Cloud,  Pct,  DataUnits(7))
    
      RETURN

      END SUBROUTINE DataCheck

      
!-------------------------------------------------------------------------
      SUBROUTINE RangeCheck(SeqNum, DType, DUnits, DVal, RMin, RMax, ErrStat)
      IMPLICIT NONE
      INTEGER, INTENT(IN)  :: SeqNum, DType, DUnits
      REAL,    INTENT(IN)  :: DVal, RMin, RMax
      INTEGER, INTENT(OUT) :: ErrStat
      
      INTEGER :: DD, MM, YY, IOS
      CHARACTER(LEN=15)  :: DT, DU
      CHARACTER(LEN=200) :: EMsg
      
      IF (IsMissing(DVal)) RETURN
      ErrStat = 0
      
      IF (DVal .LT. RMin) THEN
         CALL SequenceDate(DD, MM, YY, SeqNum)
         DT = DataTypeStringFromIndex(DType)
         DU = DataUnitStringFromIndex(DUnits)
         WRITE(EMsg, 1001, IOSTAT=IOS) TRIM(File_ID), YY, MM, DD, TRIM(DT), RMin, TRIM(DU), DVal
         CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
         ErrStat = -1
      END IF
      
      IF (DVal .GT. RMax) THEN
         CALL SequenceDate(DD, MM, YY, SeqNum)
         DT = DataTypeStringFromIndex(DType)
         DU = DataUnitStringFromIndex(DUnits)
         WRITE(EMsg, 1002, IOSTAT=IOS) TRIM(File_ID), YY, MM, DD, TRIM(DT), RMax, TRIM(DU), DVal
         CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
         ErrStat = -1
      END IF
      
      
 1001 FORMAT(1X, A, ' (YMD=',I4,'/',I2.2,'/',I2.2,'): ', A, ' < ', F0.1, 1X, A, ' : value=', F0.1, ' **REMOVED**')
 1002 FORMAT(1X, A, ' (YMD=',I4,'/',I2.2,'/',I2.2,'): ', A, ' > ', F0.1, 1X, A, ' : value=', F0.1, ' **REMOVED**')
      
      END SUBROUTINE RangeCheck
      
!-------------------------------------------------------------------------
      SUBROUTINE CompareValues(SeqNum, DType1, DType2, DVal1, DVal2, ErrStat)
      IMPLICIT NONE
      INTEGER, INTENT(IN)  :: SeqNum, DType1, DType2
      REAL,    INTENT(IN)  :: DVal1, DVal2
      INTEGER, INTENT(OUT) :: ErrStat
      
      INTEGER :: DD, MM, YY, IOS
      CHARACTER(LEN=15)  :: DT1, DT2
      CHARACTER(LEN=200) :: EMsg
      
      
      IF (DVal1 .LT. DVal2) THEN
         CALL SequenceDate(DD, MM, YY, SeqNum)
         DT1 = DataTypeStringFromIndex(DType1)
         DT2 = DataTypeStringFromIndex(DType2)
         WRITE(EMsg, 1001, IOSTAT=IOS) TRIM(File_ID), YY, MM, DD, TRIM(DT1), DVal1, TRIM(DT2), DVal2
         CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
         ErrStat = -1
      END IF      
      
 1001 FORMAT(1X, A, ' (YMD=',I4,'/',I2.2,'/',I2.2,'): ', A, '(',F0.3,') < ', A, '(',F0.3,') **REMOVED**')
      
      END SUBROUTINE CompareValues
      
!-------------------------------------------------------------------------
      SUBROUTINE ProcessOneStation(FileIn, ErrStatus)      
      IMPLICIT NONE      
      CHARACTER(LEN=*),  INTENT(IN)    :: FileIn
      INTEGER,           INTENT(OUT)   :: ErrStatus
      
      TYPE (MetCsvData)  :: StnData
      INTEGER :: I, IOS, NumberOfDays, DaysWritten, SeqNum
      INTEGER :: DayNum
      INTEGER :: LinesRemovedDuringRead
      LOGICAL :: Flag
      CHARACTER(LEN=80)  :: EMsg
      REAL, DIMENSION(7) :: Data_Vals

      Warnings = 0
      Removed = 0
      
      !
      !  Get the short file name for use in messaging
      ! (strip off any leading path info)
      !
      CALL FileNameNoPath(FileIn, File_ID)
      
      !
      !  Attempt to read the station data file
      !
      StnData%FileName = File_ID
      CALL ReadMetCsvFile(FileIn, StnData, LinesRemovedDuringRead, ErrStatus)
      IF (ErrStatus .NE. 0) THEN
         WRITE(EMsg, *, IOSTAT=IOS) TRIM(File_ID), ' had a fatal error.'
         CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
         GOTO 995
      ENDIF
      Removed = Removed + LinesRemovedDuringRead

      !
      !  Fill out the data arrays (and update header info) so the data array
      !  will contain all 7 data types that are of possible interest.
      !  By doing this now, it simplifies the remaining processing.
      !
      CALL MakeDataGlerlStandard(StnData, ErrStatus)
      IF (ErrStatus .GT. 0) RETURN
      
      !
      !  Now check the data values, testing for range errors, etc.
      !  Update the StnData%DataValues array with (potentially) modified data.
      !  Warnings encountered and errors removed are tracked via the global
      !  variables:  Warnings, Removed
      !
      NumberOfDays = StnData%EDate - StnData%SDate + 1
      DO DayNum = 1, NumberOfDays
         SeqNum = StnData%SDate + DayNum - 1
         Data_Vals(:) = MissingData_Real
         DO I = 1, 7
            Data_Vals(I) = StnData%DataValues(I,DayNum)
         END DO
         CALL DataCheck(Data_Vals, StnData%DataUnitsForColumn, SeqNum)
         DO I = 1, 7
            StnData%DataValues(I,DayNum) = Data_Vals(I)
         END DO
      END DO

      !
      !  Update the accumulators                                              
      !
      TotalNumWarnings = TotalNumWarnings + Warnings
      TotalNumRemovals = TotalNumRemovals + Removed

      !
      !  "Squeeze" the header date info to match the actual data extents
      !
      CALL SqueezeHeaderDates(StnData, ErrStatus)
      IF (ErrStatus .NE. 0) RETURN

      !
      !  Now write a new temporary output file with updated data values.
      !  Note that DaysWritten = the number of days with actual data. It is
      !  not just 'end-start+1'.
      !
      CALL WriteMetCsvFile('TEMP.STA', StnData, DaysWritten, ErrStatus)
      IF (ErrStatus .GT. 0) RETURN
      
      !
      !  Write the number of errors                                           
      !
      IF (Warnings .GT. 0) THEN
         WRITE(EMsg, 1005) TRIM(File_ID), Warnings
         CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
      END IF
      IF (Removed .GT. 0) THEN
         WRITE(EMsg, 1006) TRIM(File_ID), Removed
         CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
      END IF
      IF (Warnings+Removed .EQ. 0) THEN
         WRITE(EMsg, 1007) TRIM(File_ID)
         CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
      END IF
    
      !
      !  If there were any valid data records written into the temporary
      !  station file, then delete the old station file and rename the temporary
      !  file to take its place. If no valid data was written, then just
      !  delete both files (original and temporary).  We don't want 
      !  empty station data files.
      !
      IF (DaysWritten .GT. 0) THEN
         CALL RenameFile(FileIn, '$$Backup.tmp', Flag)
         IF (.NOT. Flag) THEN
            EMsg = 'Error while attempting to replace old file.'
            CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
         END IF

         CALL RenameFile('Temp.STA', FileIn, Flag)
         IF (Flag) THEN
            CALL DeleteFile('$$Backup.tmp')
         ELSE
            CALL DeleteFile('Temp.STA')
            CALL RenameFile('$$Backup.tmp', FileIn, Flag)
         END IF
      ELSE
         CALL DeleteFile(FileIn)
         CALL DeleteFile('Temp.STA')
      END IF
      RETURN

      !
      !  Error handling
      !      

      !
      !  2) There was an error
      !
 995  EMsg = '[Traceback] ProcessOneStation...'
      CALL WriteErrorMessage(EMsg, '', -1, .TRUE.)   ! only write to user, not log
      RETURN

      !
      ! FORMATs
      !      
 1005 FORMAT(A, ' has', I6, ' warning(s).')
 1006 FORMAT(A, ' has', I6, ' removed error(s).')
 1007 FORMAT(A, ' has no errors or warnings.')
      
      END SUBROUTINE ProcessOneStation
      
      
      
END MODULE ErrorChkSubs


! ******************************************************************************
! ******************************************************************************
! ******************************************************************************
! ******************************************************************************
! ******************************************************************************


MODULE ErrChkMain

CONTAINS

! ******************************************************************************

      SUBROUTINE DO_ERRCHK(ErrStatus, HandleIn)

      USE ErrorChkSubs
      IMPLICIT    NONE

      !
      !   The parameter RunEvp is an integer (0 or 1) interpreted as follows:
      !     0 = Check the station data files E*.STN used as input to the evap model
      !         (mean airtemp, mean dewpoint, mean windspeed, mean cloudcover)
      !     1 = Check the station data files M*.STN used as input to the LBRM
      !         (max airtemp, min airtemp, total precipitation)
      !
      !   ErrStatus will be used as a flag to the calling procedure.
      !     0 = Everything proceeded normally and the program completed successfully.
      !   <>0 = Bad things happened, and the calling procedure should abort.
      !
      INTEGER, INTENT(INOUT) :: ErrStatus
      INTEGER, INTENT(IN)    :: HandleIn

      INTEGER ::  StationsToProcess, StationsDone
      INTEGER ::  iDc, iMc, iYc, iBase_Date, iNoFiles
      INTEGER ::  IOS, PD
      INTEGER ::  J, DateTimeVals(8)
      LOGICAL ::  Found
      LOGICAL ::  Flag
      CHARACTER (LEN=1)   ::  Lf
      CHARACTER (LEN=2)   ::  DTDay, DTHour, DTMinute
      CHARACTER (LEN=3)   ::  Dum3, DTMon
      CHARACTER (LEN=4)   ::  DTYear
      CHARACTER (LEN=5)   ::  Dum5
      CHARACTER (LEN=10)  ::  Lake, Date, Time, Zone
      CHARACTER (LEN=15)  ::  FileName
      CHARACTER (LEN=80)  ::  NameS, DateTimeString
      CHARACTER (LEN=100) ::  EMsg

      CHARACTER (LEN=3), PARAMETER, DIMENSION(12) :: MonthNames =        &
                (/'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',              &
                  'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'/)


      !
      !  Initialize some variables
      !
      CALL ClearFilesOpen()
      WinHandle = HandleIn
      ErrorMessage = ''
      ErrorLevel = 0
      ErrStatus  = 0
      LogUnit = 99      ! defined in ReadMetCsv

      !
      !  Does the user want to cancel already?
      !
      IF (UserWantsToCancel()) GOTO 999

      !
      !  More initialization
      !
      Lf = CHAR(10)

!      DoEvp = .FALSE.
!      DoRun = .FALSE.
!      IF (RunEvp .EQ. 0) DoEvp = .TRUE.
!      IF (RunEvp .EQ. 1) DoRun = .TRUE.
!      IF (DoEvp) FileName = 'EBSNNM'
!      IF (DoRun) FileName = 'BSNNM'

      EMsg = 'Checking station files for problems.'
      CALL WriteErrorMessage(EMsg, '', -1, .TRUE.)

      !
      !  Open the error check log file.  It is the responsibility of the
      !  calling program to ensure that *really* old versions of this file are
      !  deleted.  We will simply append to it here, so that errors in the
      !  evap inputs and runoff inputs can be combined (2 successive runs).
      !  Write a timestamp line into the log file.
      !
      INQUIRE(FILE='ERRORCHK.LOG', EXIST=Flag)
      IF (Flag) THEN
         OPEN(99, FILE='ERRORCHK.LOG', STATUS='OLD', POSITION='APPEND', IOSTAT=IOS)
         IF (IOS .NE. 0) THEN
            EMsg = 'ERRORCHK: Could not open ERRORCHK.LOG for output.'
            CALL WriteErrorMessage(EMsg, '~!~', -1, .TRUE.) 
            GOTO 995
         ENDIF
         CALL FileWasOpened(99)
      ELSE
         OPEN(UNIT=99, FILE='ERRORCHK.LOG', STATUS='NEW', ERR=891)
         CALL FileWasOpened(99)
      ENDIF

      CALL DATE_AND_TIME(Date, Time, Zone, DateTimeVals)
      IF (ErrorLevel .NE. 0) GOTO 995
      DTYear = '    '
      IF (DateTimeVals(1) .GT. 0) WRITE(DTYear, '(I4)') DateTimeVals(1)
      DTMon = '   '
      IF (DateTimeVals(2) .GT. 0) DTMon =  MonthNames(DateTimeVals(2))
      DTDay = '  '
      IF (DateTimeVals(3) .GT. 0) WRITE(DTDay, '(I2.2)') DateTimeVals(3)
      DTHour = '  '
      IF (DateTimeVals(5) .GT. 0) WRITE(DTHour, '(I2.2)') DateTimeVals(5)
      DTMinute = '  '
      IF (DateTimeVals(6) .GT. 0) WRITE(DTMinute, '(I2.2)') DateTimeVals(6)
      DateTimeString = DTMon//' '//DTDay//', '//DTYear//' @ '//DTHour//':'//DTMinute
      WRITE(99, *) ' '
      WRITE(99, *) DateTimeString

      !
      !  Read information from BSNNM or EBSNNM file
      !
      FileName = 'BSNNM'
      OPEN(UNIT=10, FILE=TRIM(FileName), STATUS='OLD', RECL=12,         &
           ACCESS = 'DIRECT', FORM = 'FORMATTED', ERR=811)
      CALL FileWasOpened(10)
      IF (ErrorLevel .NE. 0) GOTO 995

      READ(10, 1000, REC=1, ERR=812) Lake
      Bsn = Lake(1:3)
      READ(10, 3000, REC=3, ERR=812) iDc, iMc, iYc
      CLOSE (10)
      CALL FileWasClosed(10)
      IF (ErrorLevel .NE. 0) GOTO 995
      CALL DateSequence(iDc, iMc, iYc, iBase_Date)
      IF (ErrorLevel .NE. 0) THEN
         EMsg = 'ERRORCHK: Problem with base date in '//TRIM(FileName)
         CALL WriteErrorMessage(EMsg, '~!~', 99, .TRUE.)
         GOTO 995
      END IF

      !
      !  Initialize some variables used for flagging errors
      !
      iNoFiles = 0
      TotalNumRemovals = 0           ! Removed errors total
      TotalNumWarnings = 0           ! Warning total

      IF (UserWantsToCancel()) GOTO 999    ! test for user CANCEL

      !
      !  Open the list of files to process
      !
      OPEN(UNIT=20, FILE='PROVSTNS.LIS', STATUS='OLD', ERR=821)
      CALL FileWasOpened(20)
      IF (ErrorLevel .NE. 0) GOTO 995

      !
      !  Read the list to determine how many entries there are.  Then REWIND it.
      !
      StationsToProcess = 0
      READ(20, 1002, IOSTAT=IOS) NameS
      DO WHILE (IOS .EQ. 0)
        StationsToProcess = StationsToProcess + 1
        READ(20, 1002, IOSTAT=IOS) NameS
      END DO
      REWIND(20)

      !
      !  Write a blank line (to screen only) which will be overwritten.
      !  Do the initializing READ.
      !
      CALL WriteErrorMessage('', '~*~', -1, .TRUE.)
      StationsDone = 0
      NameS='               '
      READ(20, 1002, IOSTAT=IOS) NameS

      !
      !
      !  Main loop
      !
      DO WHILE (IOS .EQ. 0)
         IF (UserWantsToCancel()) GOTO 999    ! test for user CANCEL

         !
         !  Make sure the input file exists
         !
         INQUIRE(FILE=TRIM(NameS), EXIST=Flag)
         IF (Flag) THEN
            CALL ProcessOneStation(NameS, ErrStatus)
            IF (ErrStatus .NE. 0) GOTO 995
            StationsDone = StationsDone + 1
         END IF
         
         NameS='               '
         READ(20, 1002, IOSTAT=IOS) NameS
      END DO

      !
      !  Close the station list file
      !
      CLOSE (20)
      CALL FileWasClosed(20)
      IF (ErrorLevel .NE. 0) GOTO 995


      !
      !  Write status to the user's screen and log file.
      !  
      IF ((TotalNumWarnings .EQ. 0) .AND. (TotalNumRemovals .EQ. 0)) THEN
         EMsg = 'No errors, warnings, or removed items.'
         CALL WriteErrorMessage(EMsg, '~^~', 99, .TRUE.)
      ELSE
         WRITE(EMsg, 1027) TotalNumWarnings
         CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
         WRITE(EMsg, 1029) TotalNumRemovals
         CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
      END IF

      !
      !  Update the BSNNM or EBSNNM file
      !  ProgramDesignator needs to be set to 1 in order to continue.
      !
      OPEN (UNIT=10, FILE=FileName, STATUS='OLD', RECL=12,             &
            ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(10); IF (ErrorLevel .NE. 0) GOTO 995
      READ (10, 2000, REC=2, ERR=812) Dum3, PD, Dum5
      PD = 1
      WRITE(10, 2000, REC=2, ERR=812) Dum3, PD, Dum5, Lf
      CLOSE (10)
      CALL FileWasClosed(10); IF (ErrorLevel .NE. 0) GOTO 995
      GOTO 999

      !
      !  Handlers for I/O errors
      !
  811 EMsg = 'ERRORCHK: Error opening file '//TRIM(FileName)//'!'
      CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
      GOTO 995
  812 EMsg = 'ERRORCHK: Error reading file '//TRIM(FileName)//'!'
      CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
      GOTO 995
  821 EMsg = 'ERRORCHK: Error opening file PROVSTNS.LIS!'
      CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
      GOTO 995
  891 EMsg = 'ERRORCHK: Error opening file ERRORCHK.LOG!'
      CALL WriteErrorMessage(EMsg, '', 99, .TRUE.)
      GOTO 995


      !      
      !
 995  EMsg = 'ERRORCHK: Fatal error(s) encountered in station data.'
      CALL WriteErrorMessage(EMsg, '~!~', 99, .TRUE.)
      GOTO 997
    
!
!     There were some errors, so write an appropriate message to the
!     user and write a 0 as the PD into the EBSNNM or BSNNM file.
!
!     If unit 10 is already in use that means that the execution path 
!     which got us to here had an error while opening/reading/writing the
!     BSNNM/EBSNNM file.  Don't try to open it again!
!
 997  INQUIRE(10, OPENED=Found)           ! if already open, Found = .TRUE.
      IF (Found) GOTO 998

      J = GetFreeUnitNumber()
      OPEN (UNIT=J, FILE=FileName, STATUS='OLD', RECL=12,                &
            ACCESS='DIRECT', FORM='FORMATTED', IOSTAT=IOS)
      IF (IOS .NE. 0) GOTO 998
      CALL FileWasOpened(J); IF (ErrorLevel .NE. 0) GOTO 998
      READ (J, 2000, REC=2, ERR=998) Dum3, PD, Dum5
      PD = 0
      WRITE(J, 2000, REC=2, ERR=998) Dum3, PD, Dum5, Lf
      CLOSE (J)
      CALL FileWasClosed(J); IF (ErrorLevel .NE. 0) GOTO 998

 998  ErrorLevel = 1

 999  ErrStatus = ErrorLevel
      CALL CloseAllFiles()
      CALL WriteErrorMessage('000', '~%~', -1, .TRUE.)
      RETURN
      
!
!  FORMAT statements
!
 1000 FORMAT(A10)
 1002 FORMAT(A)
 1027 FORMAT(X,'There were a total of', I6, ' warning(s).')
 1029 FORMAT(X,'There were a total of', I6, ' removed error(s).')
 2000 FORMAT(A3, I3, A5, A1)
 3000 FORMAT(2I3, I5)

      END SUBROUTINE DO_ERRCHK


END MODULE ErrChkMain


