!----------------------------------------------------------------------------
      CHARACTER (LEN=4) FUNCTION IDString (Lt, Ln)
!
!     This routine returns the concatenated binary image of the two 2-byte
!     integers as a 4-byte string.
!     An effort is made to enable subsequent ordering by latitude first and
!     then longitude by arranging the byte order so that the high order byte
!     of each integer is stored before the low order byte.  The byte order
!     in the resultant string is as follows:
!        INT1(hi) INT1(lo) INT2(hi) INT2(lo)
!
!     Revised TSH, 09jan2001, to use the Fortran 90 TRANSFER intrinsic rather
!     than CHAR.  CHAR cannot handle negative values, while TRANSFER doesn't
!     care.  I use byte-swapping to retain the desired order, since Intel
!     processors are "little-endian' -- i.e. the low order byte precedes the 
!     high order byte.  By swapping them we get the desired effect.
!
!     Rewritten TECII, 21mar95, as a FUNCTION rather than a SUBROUTINE
!     which required the use of a COMMON block, after changes made to PROVSNAL
!     by TSH 18jan94.
!
!     TECII, 26jun92
!
      IMPLICIT NONE
      INTEGER (KIND=INTEGER2), INTENT(IN) :: Lt, Ln
      CHARACTER (LEN=2)  :: LtStr, LnStr, Tmp

      Tmp = TRANSFER(Lt, LtStr)     ! copy bytes of Lt into Tmp
      LtStr = Tmp(2:2) // Tmp(1:1)  ! swap order (hi/lo)
      Tmp = TRANSFER(Ln, LnStr)     ! copy bytes of Ln into Tmp
      LnStr = Tmp(2:2) // Tmp(1:1)  ! swap order (hi/lo)

      IDString = LtStr // LnStr
      RETURN
      END FUNCTION IDString

!---------------------------------------------------------------------------
      INTEGER FUNCTION Hash (Symbol, K, HashSize)
!
!     This function returns the hash code for a particular symbol given the
!     symbol length (K) and hash size.
!
      IMPLICIT NONE
      CHARACTER (LEN=NwStLg)  :: Symbol
      INTEGER (KIND=INTEGER2) :: K
      INTEGER (KIND=INTEGER2) :: J, I, BitNo, ByteNo, M, Eight
      INTEGER                 :: J4, K4, HashSize

      Eight = 8

      K4 = K        ! protect against overflow in calculation of BitNo -- maybe unnecessary
      J = 0
      DO I = 0, 14
        BitNo = I * (K4 * 8 / 15)
        ByteNo = BitNo / 8 + 1
        BitNo = MOD(BitNo, Eight)
        M = IAND(ICHAR(Symbol(ByteNo:ByteNo)), ISHFT(128, -BitNo))
        IF (M .NE. 0) J = IOR(J, ISHFT(1, I))
      END DO
      J4 = J
      Hash = MOD(J4, HashSize) + 1
      RETURN
      END FUNCTION Hash

!---------------------------------------------------------------------------
      SUBROUTINE FindSymbol (S, Ss, Found, Indecs)
!
!     This routine searches for a given symbol in the hashed symbol table
!     and, if it is found, returns Found = .TRUE. and its index in the symbol
!     table.  S is the symbol to store.  HashTable is the table to hash to;
!     its contents point to positions in the SymbolTable (on disk) and in the
!     ChainTable.  If a symbol hashes to the same location in the HashTable,
!     then it will exist in the symbol table along the links established by the
!     ChainTable.
!
      IMPLICIT NONE
      CHARACTER (LEN=NwStLg)  :: S
      INTEGER (KIND=INTEGER2) :: Ss, Indecs, ValFromHash
      LOGICAL                 :: Found
      CHARACTER (LEN=NwStLg)  :: Ste

      ValFromHash = Hash(S, Ss, HashSize)

      Indecs = HashTable(ValFromHash)
      IF (Indecs .EQ. 0) THEN
        Found = .FALSE.
      ELSE
        READ(41, REC=Indecs, ERR=800) Ste
        DO WHILE (S .NE. Ste .AND. ChainTable(Indecs) .NE. 0)
          Indecs = ChainTable(Indecs)
          READ(41, REC=Indecs, ERR=800) Ste
        END DO
        Found = S .EQ. Ste
      END IF
      RETURN

  800 ErrorMessage = 'FindSymbol...'
      CALL PassMsg
      CALL ReinitThiessen()
      ErrorLevel = 50
      Found = .FALSE.
      RETURN

      END SUBROUTINE FindSymbol

!---------------------------------------------------------------------------
      SUBROUTINE StoreSymbol (Symbol, SymbolSize, Indecs)
!
!     This routine stores a symbol in the given hashed symbol table.  See
!     comments in the procedure: FindSymbol.  Indecs is the place in the
!     tables to store the symbol.  THIS ROUTINE DOES NOT CHECK TO SEE IF
!     SYMBOL IS ALREADY IN THE TABLES NOR IF THE TABLES OVERFLOW!
!
      IMPLICIT NONE
      CHARACTER (LEN=NwStLg)  :: Symbol
      INTEGER (KIND=INTEGER2) :: SymbolSize, Indecs, I

      IF (Indecs .LE. 0) GOTO 800
      WRITE(41, REC=Indecs, ERR=800) Symbol             ! Store in symbol table
      NtwrkSymSz(Indecs) = SymbolSize
      I = Hash(Symbol, SymbolSize, HashSize)
      IF (HashTable(I) .EQ. 0) THEN
       ChainTable(Indecs) = 0                           ! Start new chain
      ELSE
       ChainTable(Indecs) = HashTable(I)                ! Insert in chain
      END IF
      HashTable(I) = Indecs                             ! Store in hash table
      RETURN

  800 ErrorMessage = 'StoreSymbol...'
      CALL PassMsg
      CALL ReinitThiessen()
      ErrorLevel = 50
      RETURN

      END SUBROUTINE StoreSymbol

!---------------------------------------------------------------------------
      SUBROUTINE DeleteSymbol (S, Ss, Indecs)
!
!     This routine finds and deletes a given symbol from the hashed symbol
!     table and returns the index in the symbol table of the new vacancy.  See
!     comments in the procedure: FindSymbol.  ASSUMES THAT SYMBOL EXISTS IN
!     THE HASHED SYMBOL TABLE; IT IS UNCHECKED HERE!!!
!
      IMPLICIT NONE
      CHARACTER (LEN=NwStLg)  :: S
      INTEGER (KIND=INTEGER2) :: Ss, Indecs, IndexSave
      CHARACTER (LEN=NwStLg)  :: Ste

      IndexSave = Hash(S, Ss, HashSize)
      Indecs = HashTable(IndexSave)
      READ(41, REC=Indecs, ERR=800) Ste
      IF (S .EQ. Ste) THEN
       HashTable(IndexSave) = ChainTable(Indecs)
      ELSE
       DO WHILE (S .NE. Ste)
        IndexSave = Indecs
        Indecs = ChainTable(Indecs)
        READ(41, REC=Indecs, ERR=800) Ste
       END DO
       ChainTable(IndexSave) = ChainTable(Indecs)
      END IF

      RETURN

  800 ErrorMessage = 'DeleteSymbol...'
      CALL PassMsg
      CALL ReinitThiessen()
      ErrorLevel = 50
      RETURN

      END SUBROUTINE DeleteSymbol


