!-----------------------------------------------------------------------------------------
!
!   New version, 2014, by Tim Hunter
!
!   Previously, we maintained two separate met data "tracks". One for tmax/tmin/precip and
!   one for air/dew/wind/cloud data. There were parallel programs and files maintained for 
!   each of these data sets, and they were kept separated. But both were ultimately used
!   for running models and the division between them was a consistent source of confusion
!   for anyone to whom I supplied the station data. When it became obvious that I needed
!   to re-write all of the station data processing software, and use a new CSV format for
!   the data, it was an obvious time to also eliminate that artificial division in the
!   station met data sets. The new MET_*.CSV files can contain any combination of those
!   data values, and I will now also combine them into a single provisional data base.
!   Note that I am not changing the philosophical "structure" of how this is all handled.
!   I am simply combining the two data sets into a single combined set.
!
!   Notes carried over (condensed) from previous version of this program:
!   
!   1) Note that we allow multiple stations at the same location. 
!      DISAVMET will handle the situation by averaging the values of
!      any stations at the same location and treating the average as
!      if it were the value for the only station at that location.
!
!
!  -------------------- PROGRAM DESCRIPTION ---------------------
!
!  This routine is the interface between the user and the system for entering
!  new provisional meteorological data into the linked-list file: ProvMetData.xxx.
!  xxx is read from the file: BSNNM and is defined as follows:
!
!       xxx = SUP  (Lake Superior Basin)
!             MIC  (Lake Michigan Basin)
!             HUR  (Lake Huron Basin)
!             GEO  (Georgian Bay Basin)
!             STC  (Lake St. Clair Basin)
!             ERI  (Lake Erie Basin)
!             ONT  (Lake Ontario Basin)
!             CHA  (Lake Champlain Basin)
!             URA  (Ural River Basin in USSR)
!             EMB  (Embe River Basin in USSR)
!             VOL  (Volga River Basin in USSR)
!
!  This program is to be used in a "batch" mode; that is, the user must
!  prepare input files for station information to be added and then
!  execute this program.
!
!  NOTE:  THE DATA BASE (ProvMetData.xxx) SHOULD BE CREATED, MODIFIED,
!         OR UPDATED ONLY WITH THIS PROGRAM!!!!!!!
!
!  The input file: MetStationList.txt must have each record containing the name
!  of a file to be incorporated into ProvMetData.xxx.  Each file named must
!  follow the following name convention:
!
!        MET_*.CSV
!        | | |   
!        | | |____station identification string 
!        |_!______standard prefix denoting met station data.
!
!  Each station file is a comma-separated file (CSV) and has the following structure:
!
!  Line 1: Station header information.
!    Field #   Description    Notes
!      1       Station ID     Alphanumeric (letters and numbers only, no imbedded spaces).  
!      2       Station Name   Optional field.  Alphanumeric – any characters except comma
!  
!  Line 2: Station location information.
!    Field #   Description    Notes
!      1   “Lat & Long”   Indicator that this is latitude and longitude
!      2   Latitude   Latitude in decimal degrees north of the equator (e.g. 42.123)
!      3   Longitude   Longitude in decimal degrees east of the prime meridian (e.g. a station in Michigan might be -87.340)
!  
!  Line 3: Station data start date
!    Field #   Description    Notes
!      1     “Starts (YMD):”  Indicator to reader that this is the start date
!      2       Year           Year of the first data line
!      3       Month          Month of the first data line
!      4       Day            Day of the first data line
!  
!  Line 4: Station data end date
!    Field #   Description    Notes
!      1      “Ends (YMD):”   Indicator to reader that this is the end date
!      2       Year           Year of the first data line
!      3       Month          Month of the first data line
!      4       Day            Day of the first data line
!  
!  Line 5: Column Header 1
!    Field#   Description   Notes
!      1      Blank         Blank; used in header line 2
!     2-n     Data Type     Type of data to be found in this column; Valid entries are:
!                             AIRTEMPMAX = maximum air temperature for the day
!                             AIRTEMPMIN = minimum air temperature for the day
!                             PRECIPITATION = total precipitation for the day
!                             AIRTEMP = mean air temperature for the day
!                             DEWPOINT = mean dew point temp for the day
!                             WINDSPEED = mean wind speed for the day
!                             CLOUD = mean cloud cover for the day
!   
!  Line 6: Column Header 2
!    Field#   Description   Notes
!      1      “YYYYMMDD”    Explains column 1 is date in YYYYMMDD format
!     2-n      Data Units   Units for the data found in this column; Valid entries are:
!                             CELSIUS         = degrees Celsius
!                             FAHRENHEIT      = degrees Fahrenheit
!                             INCHES          = inches
!                             CENTIMETERS     = centimeters
!                             MILLIMETERS     = millimeters
!                             METERSPERSECOND = meters per second
!                             PERCENT         = percent
!                             FRACTION        = decimal fraction; 0.0 to 1.0
!  
!  Lines 7-end: Station data
!    Field #   Description    Notes
!      1       YearMonthDay   Date of the observation (YYYYMMDD)
!     2-n      Data values    Data values corresponding to this date and placed in 
!                             the correct column to match header lines. Missing data 
!                             values are denoted by any of the following:  
!                             Blank entry, -9.9e9, “N/A”
!
!
!
!  This program creates or updates the file: ProvMetData.xxx which is
!  composed of "blocks", one for each day of record, arranged in a linked
!  sequential order with every day represented by one block.  
!  Every day between the beginning and end dates must be represented even if 
!  there are no data for that day.  Each block must have the individual station 
!  data arranged in an order that is unique for that group of stations; 
!  i. e., every time the same set of stations reoccurs, they must be listed 
!  in exactly the same order. In order to accomplish this, I will sort the
!  entries by Latitude, then Longitude; both in ascending order.
!
!  The file is a direct access file with a fixed record length of 62 bytes; 
!  therefore, it must not be edited normally since an editor will create 
!  sequential access files which are unusable in DISAVMET.  Besides that, 
!  all records are unformatted, so it would be pretty difficult to do 
!  anything meaningful within a text editor.  Each block has the following 
!  structure:
!
!  First record of the file:
!
!    date linkNH PP MaxCount unused link (unformatted: 4, 4, 4, 4, 42, 4)
!    |  | |    | || |      | |    | |  |
!    |  | |    | || |      | |    | |__|__Pointer to next logical record (header or data)
!    |  | |    | || |      | |____|_______This field is filled with nulls. It is used simply
!    |  | |    | || |      |              to fill all the records to 62 bytes.
!    |  | |    | || |______|_______This is a special field in record 1. It contains the
!    |  | |    | ||                maximum number of stations reporting on any day.
!    |  | |    | ||_____Special interpretation for record 1...
!    |  | |    |        The absolute value of this field is a pointer to the last 
!    |  | |    |        physical data record. The positive/negative state is still 
!    |  | |    |        interpreted as in the other block headers.
!    |  | |____|_____pointer to header record of next logical data block
!    |  |            In the final block, this value is 0, indicating no more data blocks
!    |__|_________date sequence number
!
!  First record of each "block":
!
!    date linkNH PP unused link (unformatted: 4, 4, 4, 46, 4)
!    |  | |    | || |    | |  |
!    |  | |    | || |    | |__|__Pointer to next logical record (header or data)
!    |  | |    | || |____|_______This field is filled with nulls. It is used simply
!    |  | |    | ||              to fill all the records to 62 bytes.
!    |  | |    | ||_____This field is just used as an indicator of whether or not 
!    |  | |    |        the block has been processed by DISAVMET.  
!    |  | |    |          Negative = the block has been processed by DISAVMET
!    |  | |    |          Positive = new data has been put into the block since the 
!    |  | |    |                     last time DISAVMET was run and DISAVMET needs to 
!    |  | |    |                     process/reprocess this block.  The actual value 
!    |  | |    |                     is irrelevant.
!    |  | |____|_____pointer to header record of next logical data block
!    |  |            In the final block, this value is 0, indicating no more data blocks
!    |__|_________date sequence number
!
!   Other records in the block  (i.e. the data records):
!
!    Lt Ln Tmx Tmn Prc Air Dew Wnd Cld ID link (unformatted: 2,2,2,2,2,2,2,2,2,40,4)
!    || || | | | | | | | | | | | | | | || |  |
!    || || | | | | | | | | | | | | | | || |__|__pointer to next logical record. In the
!    || || | | | | | | | | | | | | | | ||       final record, this is set to zero.
!    || || | | | | | | | | | | | | | | ||_______station ID (40 characters, left-justified)
!    || || | | | | | | | | | | | | |_|_____cloudcover in percent (0-100)
!    || || | | | | | | | | | | |_|_________windspeed in hundredths of meters / second
!    || || | | | | | | | | |_|_____________daily dewpoint air temperature in
!    || || | | | | | | | |                 hundredths of degrees Centigrade
!    || || | | | | | | |_|_________________daily dry-bulb temperature in 
!    || || | | | | | |                     hundredths of degrees Centigrade
!    || || | | | | |_|_____________________daily total precipitation in 
!    || || | | | |                         hundredths of millimeters
!    || || | | |_|_________________________daily minimum air temperature in 
!    || || | |                             hundredths of degrees Centigrade
!    || || |_|_____________________________daily maximum air temperature in 
!    || ||                                 hundredths of degrees Centigrade
!    || ||_________________________________longitude in hundredths of decimal degrees
!    ||____________________________________latitude in hundredths of decimal degrees
!
!
!    Note that the very first record of the file is special.  It is a block header, 
!    as would be expected, but the third field indicates the record number of the 
!    last data record in the file and the 4th field indicates the maximum number of 
!    stations reporting in a single day.
!
!    The last block header record has some special values as well.  Field
!    number 2 (pointer to the next block header) is set to zero.
!    Field number 5 (pointer to the next logical record) in the last data
!    record (which could possibly be the last block header record as well)
!    is also set to zero.  Both of these zero "flag values" are used to 
!    indicate the end of logical records.
!
!
!-----------------------------------------------------------------------------------------

MODULE ProvMetDataSubs
      USE GlerlUtil
      USE FcstUtil
      USE ReadMetCsv

      IMPLICIT NONE


      !
      !  Define a big box around the entire Great Lakes.  We will throw away
      !  any stations that fall outside this box.
      !
      REAL, PARAMETER :: BigBoxLeft   = -100.0      ! westernmost longitude
      REAL, PARAMETER :: BigBoxRight  =  -70.0      ! easternmost longitude
      REAL, PARAMETER :: BigBoxTop    =   60.0      ! northernmost latitude
      REAL, PARAMETER :: BigBoxBottom =   35.0      ! southernmost latitude

      !
      !  Provisional data base data structures
      !
      TYPE StationRecordType
         CHARACTER (LEN=40)      :: StnID
         INTEGER (KIND=INTEGER2) :: Lat, Long, Dat(7)
         TYPE (StationRecordType), POINTER :: NxtSta
      END TYPE StationRecordType

      TYPE BlockRecordType
         INTEGER (KIND=INTEGER4) :: SeqNum, NumStations
         LOGICAL                 :: Processed
         TYPE (StationRecordType), POINTER :: Stations
      END TYPE BlockRecordType

      INTEGER, PARAMETER :: StnRecBlkSz = 1000
      TYPE StnRecList
         INTEGER  :: Used
         TYPE (StationRecordType), DIMENSION(StnRecBlkSz) :: Arr
         TYPE (StnRecList), POINTER  :: Next
      END TYPE StnRecList

      TYPE (StnRecList), POINTER :: StnListHead    ! root pointer to the list

      !
      !  Global variables
      !
      INTEGER, PARAMETER :: Max_No_Days = 55000   ! ~ 150 years
      INTEGER  :: EarliestNewData, EndOfPrvData, LatestNewData
      INTEGER  :: MaxEntriesOnOneDay
 
      CHARACTER (LEN=40) :: Skip40, Null40
      CHARACTER (LEN=42) :: Skip42, Null42
      CHARACTER (LEN=46) :: Skip46, Null46
      CHARACTER (LEN=58) :: Skip58, Null58
 
      TYPE (BlockRecordType) :: BlockRec(Max_No_Days)

      CHARACTER (LEN=10) :: BsnFile                    ! 'EBSNNM' or 'BSNNM'

CONTAINS

!-----------------------------------------------------------------------------
      SUBROUTINE CreateInitialFile(FileName, StartSeq)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: FileName
      INTEGER, INTENT(IN) :: StartSeq
      
      INTEGER :: LinkBlk, LinkRec, PP, MaxSta, UNum

      ErrorLevel = 0
      UNum = GetFreeUnitNumber()
      OPEN(UNIT=UNum, FILE=TRIM(FileName), STATUS='NEW', ACCESS='DIRECT',   &
           RECL=62, FORM='UNFORMATTED', ERR=811)
      CALL FileWasOpened(UNum); IF (ErrorLevel .NE. 0) GOTO 899
      LinkBlk = 0
      LinkRec = 0
      PP      = -1
      MaxSta  = 0
      WRITE(UNum, REC=1, ERR=813) StartSeq, LinkBlk, PP, MaxSta, Null42, LinkRec 
      CLOSE(UNum)
      CALL FileWasClosed(UNum); IF (ErrorLevel .NE. 0) GOTO 899
      RETURN
      
      !      
  811 ErrorMessage = 'ProvMetData: Error opening file '//TRIM(FileName)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  813 ErrorMessage = 'ProvMetData: Error writing file '//TRIM(FileName)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : CreateInitialFile...'; 
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
      
      END SUBROUTINE CreateInitialFile

!-----------------------------------------------------------------------------
      SUBROUTINE DeleteRecord(StnSeq, StID)
      IMPLICIT NONE
      INTEGER,            INTENT(IN) ::  StnSeq
      CHARACTER (LEN=40), INTENT(IN) ::  StID

      INTEGER            :: I, Seq
      CHARACTER (LEN=40) :: ID
      CHARACTER (LEN=100) :: EMsg

      TYPE (StationRecordType), Pointer :: PrvSRT, DelSRT, SRT

      CHARACTER (LEN=80) :: DebugString

      !
      !  Go to the right date for this bit of station data.  "I" will be the 
      !  index into the BlockRec array for the right date.
      !
      I = 1
      DO WHILE (BlockRec(I)%SeqNum .NE. StnSeq)
         I = I + 1
         IF (I .GT. Max_No_Days) THEN
            EMsg = 'ProvMetData: Maximum number of days exceeded!'
            CALL WriteErrorMessage(EMsg, '', LogUnit, .TRUE.)
            GOTO 898
         END IF
      END DO
      BlockRec(I)%Processed = .FALSE.     ! this block now has NEW data

      !
      !  Now find the right station record
      !
      PrvSRT => BlockRec(I)%Stations
      DelSRT => BlockRec(I)%Stations
      DO WHILE ((ASSOCIATED(DelSRT)) .AND. (DelSRT%StnID .NE. StID))
         PrvSRT => DelSrt
         DelSRT => DelSRT%NxtSta
      END DO

      IF (.NOT. ASSOCIATED(DelSRT)) THEN
         EMsg = 'ProvMetData: Error deleting old station entry.'
         CALL WriteErrorMessage(EMsg, '', LogUnit, .TRUE.)
         GOTO 898
      END IF

      !
      !  If the entry to be deleted is the root entry we have to
      !  manipulate the entry in BlockRec(I).
      !
      IF (ASSOCIATED(DelSRT, PrvSRT)) THEN
         BlockRec(I)%NumStations = BlockRec(I)%NumStations - 1
         BlockRec(I)%Stations => DelSRT%NxtSta
         RETURN
      END IF

      !
      !  Otherwise, we just have to remove that one entry and fix the link.
      !
      BlockRec(I)%NumStations = BlockRec(I)%NumStations - 1
      PrvSRT%NxtSta => DelSRT%NxtSta
      RETURN

  898 ErrorLevel = 1
  899 EMsg = '[traceback] : DeleteRecord...'
      CALL WriteErrorMessage(EMsg, '', LogUnit, .TRUE.)

      END SUBROUTINE DeleteRecord

!-----------------------------------------------------------------------------
      SUBROUTINE AddRecord(StnSeq, Lat, Long, TheData, StID, IsNewData)
      IMPLICIT NONE
      INTEGER,                 INTENT(IN) :: StnSeq
      INTEGER (KIND=INTEGER2), INTENT(IN) :: Lat, Long
      INTEGER (KIND=INTEGER2), INTENT(IN) :: TheData(7)
      CHARACTER (LEN=40),      INTENT(IN) :: StID
      LOGICAL,                 INTENT(IN) :: IsNewData
      
      INTEGER (KIND=INTEGER2)  :: Lt, Ln
      INTEGER                  :: I, J, Seq 
      LOGICAL                  :: Found, Later
      CHARACTER (LEN=40)       :: ID
      CHARACTER (LEN=80)       :: DebugString, EMsg
      TYPE (StationRecordType) :: NewRec
      TYPE (StationRecordType), Pointer :: CurStn, TmpRec, SRT, NStn

      !
      !  Fill in blank BlockRec entries if necessary
      !
      IF (StnSeq .GT. EndOfPrvData) THEN
         I = 1
         DO WHILE (BlockRec(I)%SeqNum .GT. 0)
            Seq = BlockRec(I)%SeqNum
            I = I + 1
            IF (I .GT. Max_No_Days) THEN
               ErrorMessage = 'ProvMetData: Maximum number of days exceeded (1)!'
               CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
               GOTO 898
            END IF
         END DO
         Seq = Seq + 1
         DO WHILE (Seq .LE. StnSeq)
            BlockRec(I)%SeqNum = Seq
            I = I + 1
            IF (I .GT. Max_No_Days) THEN
               ErrorMessage = 'ProvMetData: Maximum number of days exceeded (2)!'
               CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
               GOTO 898
            END IF
            Seq = Seq + 1
         END DO
         EndOfPrvData = StnSeq
      END IF

      !
      !  Go to the right date for this bit of station data.  "I" will be the 
      !  index into the BlockRec array for the right date.
      !
      I = 1
      DO WHILE (BlockRec(I)%SeqNum .NE. StnSeq)
         I = I + 1
         IF (I .GT. Max_No_Days) THEN
            ErrorMessage = 'ProvMetData: Maximum number of days exceeded!'
            CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
            WRITE(ErrorMessage, 5001) I, Max_No_Days
            CALL PassMsg
            GOTO 898
         END IF
      END DO
      IF (IsNewData) BlockRec(I)%Processed = .FALSE.     ! this block now has NEW data

      !
      !  Assign values to a temporary station record.
      !
      NewRec%Lat  = Lat
      NewRec%Long = Long
      DO J = 1, 7
         NewRec%Dat(J) = TheData(J)
      END DO
      NewRec%StnID  = StID
      NULLIFY(NewRec%NxtSta)

      !
      !  Add the station record to the linked list of stations.  It must be
      !  added in the correct sorted position based on lat/long/StID
      !
      !  1) Was the list empty?  If so, then this is the only station (so far)
      !     for this date.  Allocate the RAM, assign the values, and return.
      !
      IF (.NOT. ASSOCIATED(BlockRec(I)%Stations)) THEN
         CALL StnAlloc(SRT); IF (ErrorLevel .NE. 0) GOTO 899
         SRT = NewRec
         BlockRec(I)%Stations => SRT
         BlockRec(I)%NumStations = 1
         RETURN
      END IF

      !
      !  2) Is the new entry a replacement for one of the existing entries?
      !      i.e. Is it the same station, but new data?
      !
      SRT => BlockRec(I)%Stations
      DO WHILE (ASSOCIATED(SRT))
         Lt = SRT%Lat
         Ln = SRT%Long
         ID = SRT%StnID
         IF ((Lat .EQ. Lt) .AND. (Long .EQ. Ln) .AND. (StID .EQ. ID)) THEN
            SRT%Dat = NewRec%Dat   ! we only need to update the data fields
            RETURN
         END IF
         SRT => SRT%NxtSta
      END DO
        
      !
      !  3) Assign CurStn to point at the first entry. This will always be 
      !     a valid one, since we already eliminated the case of an empty list.
      !
      CurStn => BlockRec(I)%Stations
      Lt = CurStn%Lat
      Ln = CurStn%Long
      ID = CurStn%StnID

      !
      !  4) Does the new entry need to be inserted at the 
      !     head of the non-empty list?
      !
      !     Found = the correct spot is right here. 
      !
      Found = .FALSE.
      IF (Lat .LT. Lt) Found = .TRUE.
      IF ((Lat .EQ. Lt) .AND. (Long .LT. Ln)) Found = .TRUE.
      IF ((Lat .EQ. Lt) .AND. (Long .EQ. Ln) .AND. (StID .LT. ID)) Found = .TRUE.
      IF (Found) THEN
          CALL StnAlloc(SRT); IF (ErrorLevel .NE. 0) GOTO 899   ! allocate RAM
          SRT = NewRec                                          ! assign values
          SRT%NxtSta => CurStn                                  ! point to current record
          BlockRec(I)%Stations => SRT                           ! replace root entry
          BlockRec(I)%NumStations = BlockRec(I)%NumStations + 1
          RETURN
      END IF

      !
      !  5) We haven't found the spot yet, so it goes in the list somewhere
      !     within records 2 to the end.
      !
      !
      !  Does the new entry go later in the sort order than the "current" entry?
      !  If so, then "Later" will be TRUE.  (Remember that duplicate stations
      !  have already been handled in step 2.)
      !
      Later = .TRUE.                       ! We already know this from step 4
      DO WHILE (ASSOCIATED(CurStn) .AND. Later)

         !
         !  If we get to here, then by default (since "Later" was true and they weren't 
         !  the same station) we know that the CurStn entry is earlier in the sort
         !  order than the new station.  The question is, does the new entry go
         !  *immediately* after the CurStn entry.  The answer is YES if either of
         !  these conditions is met:
         !    1) CurStn is the tail entry.
         !    2) The station after CurStn sorts out AFTER the new entry.
         !  So check for these conditions.
         !
         NStn => CurStn%NxtSta

         !
         !   1) Is CurStn the tail entry?
         !
         IF (.NOT. ASSOCIATED(NStn)) THEN
            CALL StnAlloc(SRT); IF (ErrorLevel .NE. 0) GOTO 899   ! allocate RAM
            SRT = NewRec                                          ! assign values
            CurStn%NxtSta => SRT                                  ! make current record point to new one
            BlockRec(I)%NumStations = BlockRec(I)%NumStations + 1
            RETURN
         END IF
          
         !
         !   2) Is the new entry later than NStn?
         !      If FALSE then the new entry goes between CurStn and NStn.
         !      We will re-use Lt, Ln, ID.
         !
         Lt = NStn%Lat
         Ln = NStn%Long
         ID = NStn%StnID
         Later = .FALSE.
         IF (Lat .GT. Lt) Later = .TRUE.
         IF ((Lat .EQ. Lt) .AND. (Long .GT. Ln)) Later = .TRUE.
         IF ((Lat .EQ. Lt) .AND. (Long .EQ. Ln) .AND. (StID .GT. ID)) Later = .TRUE.
         IF (.NOT. Later) THEN
            CALL StnAlloc(SRT); IF (ErrorLevel .NE. 0) GOTO 899    ! allocate RAM
            SRT = NewRec                                           ! assign values
            SRT%NxtSta => CurStn%NxtSta                            ! make new record point to NStn entry
            CurStn%NxtSta => SRT                                   ! make current record point to new one
            BlockRec(I)%NumStations = BlockRec(I)%NumStations + 1
            RETURN
         END IF

         !
         !  The new entry belongs somewhere after "current" entry, and there are
         !  still more items in the list, so change the "current" entry to be the
         !  next one in the list.
         !
         CurStn => CurStn%NxtSta
      END DO

      !
      !  If we get to here, then something didn't work right.  We should
      !  have already assigned the new entry somewhere before exiting the loop.
      !
      ErrorMessage = 'ProvMetData: Error adding new station entry to linked list!'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : AddRecord...'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)

      
 5001 FORMAT(2X, 'Attempted to add entry at day ', I0, ', but maximum is ', I0)      
      
      
      END SUBROUTINE AddRecord


!--------------------------------------------------------------------------
      SUBROUTINE StnAlloc(StnRec)
      IMPLICIT   NONE
      TYPE (StationRecordType), POINTER  :: StnRec
      TYPE (StnRecList),        POINTER  :: SL, TmpSL
      INTEGER  :: I, IOS
      LOGICAL  :: Done


      !
      !  Cycle through linked list until we get to the last list item.
      !  This is where we will grab a station record from (or find out
      !  that none are left so we have to allocate another).
      !
      SL => StnListHead
      Done = .FALSE.

      DO WHILE (ASSOCIATED(SL) .AND. (.NOT. Done))
        IF (SL%Used .EQ. StnRecBlkSz) THEN
           SL => SL%Next
        ELSE
           Done = .TRUE.
        END IF
      END DO

      !
      !  Do we need to allocate another item array?  (i.e. StnRecList item)
      !
      IF (.NOT. ASSOCIATED(SL)) THEN
         SL => StnListHead
         DO WHILE (ASSOCIATED(SL%Next))
            SL => SL%Next
         END DO
         ALLOCATE(TmpSL, STAT=IOS)
         IF (IOS .NE. 0) THEN
            ErrorMessage = 'ProvMetData: Error allocating RAM!'
            CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
            GOTO 898
         END IF
         NULLIFY(TmpSL%Next)
         TmpSL%Used = 0
         SL%Next => TmpSL
         SL => TmpSL
      END IF

      !
      !  Assign the next available item (inside one of those 100-element
      !  arrays) to the dummy argument passed in.
      !
      I = SL%Used + 1
      SL%Used = I
      StnRec => SL%Arr(I)
      RETURN

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : StnAlloc...'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)

      END SUBROUTINE StnAlloc


!--------------------------------------------------------------------------
      SUBROUTINE MyDealloc(L1)
      IMPLICIT   NONE
      TYPE (StnRecList), POINTER  :: L1, L2

      DO WHILE (ASSOCIATED(L1))
         L2 => L1
         L1 => L1%Next
         DEALLOCATE(L2)
      END DO

      END SUBROUTINE MyDealloc


      !--------------------------------------------------------------------------
      !  This routine reads an existing ProvMetData file, storing the data into a
      !  linked list in RAM.  The parameters are:
      !   FileName  = Name of the ProvMetData file
      !   NumDays   = Number of days expected. Set to negative value if no idea. 
      !   PctOfProc = What percent of the total processing is this step assumed to be?
      !               Used to provide user feedback. If no idea, just set to negative.
      !   EarliestAllowed = Sequence number of the earliest allowed data to be stored.
      !                     Any data prior to this date will be ignored. Set to
      !                     a large negative number to allow anything.
      !--------------------------------------------------------------------------
      SUBROUTINE ReadExistingData(FileName, NumDays, PctOfTotal, EarliestAllowed)  
      IMPLICIT NONE
      CHARACTER (LEN=*), INTENT(IN) :: FileName
      INTEGER,           INTENT(IN) :: NumDays      ! total # of days, based on BSNNM
      INTEGER,           INTENT(IN) :: PctOfTotal
      INTEGER,           INTENT(IN) :: EarliestAllowed
      
      INTEGER (KIND=INTEGER2) :: ILat, ILon
      INTEGER ::  I, J, K, L, D, M, Y, BD, PD, LastRec, Seq, SeqNum, RNum
      INTEGER ::  Base_Day, Base_Month, Base_Year, Base, Start
      INTEGER ::  Prov_Day, Prov_Month, Prov_Year, PrvEndDate, PrvDays, PrvPct
      INTEGER ::  LinkBlk, LinkRec, LinkEof, PP, CurRec, MaxSta
      INTEGER ::  TotalRecs, StationsToProcess, StnsToday
      INTEGER ::  IOS, IOS11, IOS22, LinesRemoved, Ndx
      INTEGER ::  TotPct, UNum
      LOGICAL ::  AllMissing, FileExist, OkToContinue
      LOGICAL ::  NeedToSkipMore, IsNewData, LoopDone
      REAL    ::  Lat, Lon

      INTEGER (KIND=INTEGER2), DIMENSION(7) :: DatVals
      
      CHARACTER (LEN=1)  :: Lf
      CHARACTER (LEN=3)  :: Bsn, Dum3
      CHARACTER (LEN=4)  :: OTLCode
      CHARACTER (LEN=10) :: LkName
      CHARACTER (LEN=20) :: PRVName
      CHARACTER (LEN=40) :: StNumber
      CHARACTER (LEN=80) :: StnName, Line
      CHARACTER (LEN=100):: EMsg

      TYPE (StationRecordType), POINTER :: CurStation, SRT

      !
      !  user feedback, if appropriate
      !
      IF (NumDays .GT. 0) THEN
         WRITE(EMsg, 9120) 0; CALL WriteErrorMessage(EMsg, '', LogUnit, .TRUE.)
      END IF
      
      !
      ! Open the file
      !     
      UNum = GetFreeUnitNumber()
      OPEN(UNIT=UNum, FILE=TRIM(FileName), STATUS='OLD', ACCESS='DIRECT',      &
           RECL=62, FORM='UNFORMATTED', ERR=811)
      CALL FileWasOpened(UNum); IF (ErrorLevel .NE. 0) GOTO 899
      
      READ(UNum, REC=1, ERR=812) Seq, LinkBlk, PP, MaxSta, Skip42, LinkRec
      LinkEof = ABS(PP)    ! record number of the last entry in the file
      IF (PP .GT. 0) THEN
         IsNewData = .TRUE.
      ELSE
         IsNewData = .FALSE.
      END IF

      !
      ! main read loop
      !
      MaxEntriesOnOneDay = 0
      LoopDone = (LinkBlk .EQ. 0) .AND. (LinkRec .EQ. 0)
      DO WHILE  (.NOT. LoopDone)
        !
        ! User feedback
        !
        IF (MOD(BD,100) .EQ. 0) THEN
           IF (NumDays .GT. 0) THEN
              PrvPct = (BD*100) / NumDays
              TotPct = PrvPct * (PctOfTotal / 100.0)
              WRITE(EMsg, 9121) PrvPct;  CALL WriteErrorMessage(EMsg, '~^~', LogUnit, .TRUE.)
              WRITE(EMsg, 9125) TotPct;  CALL WriteErrorMessage(EMsg, '~%~', LogUnit, .TRUE.)
           END IF
        END IF
        
        !
        ! block header record
        !
        READ(UNum, REC=LinkBlk, ERR=812) Seq, LinkBlk, PP, Skip46, LinkRec
        IF (PP .GT. 0) THEN
           IsNewData = .TRUE.
        ELSE
           IsNewData = .FALSE.
        END IF
        
        !
        ! station data in the block
        !
        StnsToday = 0                         ! count of stations reporting for this one day
        DO WHILE (LinkRec .NE. LinkBlk)
           READ(UNum, REC=LinkRec, ERR=812) ILat,ILon, (DatVals(K),K=1,7), StNumber, LinkRec
           IF (Seq .GE. EarliestAllowed) THEN
              CALL AddRecord(Seq, ILat, ILon, DatVals, StNumber, IsNewData)
              IF (ErrorLevel .NE. 0) GOTO 898
           END IF
        END DO
        LoopDone = (LinkBlk .EQ. 0) .AND. (LinkRec .EQ. 0)
      END DO

      
      IF (PctOfTotal .GT. 0) THEN
         WRITE(EMsg, 9121) 100;         CALL WriteErrorMessage(EMsg, '~^~', LogUnit, .TRUE.)
         WRITE(EMsg, 9125) PctOfTotal;  CALL WriteErrorMessage(EMsg, '~%~', LogUnit, .TRUE.)
      END IF

      !
      !  MaxEntriesOnOneDay (computed as we read the file) is the correct value for the 
      !  active set of data. But note that the early (ignored) period may possibly
      !  have included even more stations on a particular day. Since we exclude those 
      !  data, though, I will leave MaxEntriesOnOneDay set to the computed value. If we
      !  were to change our mind, the following line would need to be enabled.
      !
      !  MaxEntriesOnOneDay = MAX(MaxEntriesOnOneDay, MaxSta)
      
      GOTO 999

      !
      !  Error Handling
      !
  811 ErrorMessage = 'ProvMetData: Error opening file '//TRIM(FileName)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  812 ErrorMessage = 'ProvMetData: Error reading file '//TRIM(FileName)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : ReadExistingData...'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)

      !
      !  Return to calling procedure. 
      !
  999 CLOSE(UNum, IOSTAT=IOS)
      CALL FileWasClosed(UNum)
  
 9120 FORMAT('Reading existing data into RAM: ', I0, '%')
 9121 FORMAT('Reading existing data into RAM: ', I0, '%')
 9125 FORMAT(I0)
  
      END SUBROUTINE ReadExistingData
 
!--------------------------------------------------------------------------
      SUBROUTINE WriteBinaryFile(FileName)
      IMPLICIT NONE
      CHARACTER (LEN=*), INTENT(IN) :: FileName
      INTEGER (KIND=INTEGER2) :: TheData(7), ILat, ILon
      INTEGER                 :: I, J, K, IOS, RNum, Seq, UNum, PP
      INTEGER                 :: NxtBlk, NxtRec, TotalRecs
      CHARACTER (LEN=40)      :: StNumber
      CHARACTER (LEN=99)      :: DebugString

      TYPE (StationRecordType), POINTER :: CurStation

      DO I = 1, 40
        Skip40(I:I) = ' '
        Null40(I:I) = CHAR(0)
      END DO

      UNum = GetFreeUnitNumber()
      OPEN(UNIT=UNum, FILE=TRIM(FileName), STATUS='REPLACE', RECL=62,      &
           ACCESS='DIRECT', FORM='UNFORMATTED')
      CALL FileWasOpened(UNum)

      !
      !  Write the information stored in RAM into the file.  The "LastRec" field of
      !  record 1 will be written incorrectly at first, but fixed at the end.
      !
      RNum = 1
      DO I = 1, Max_No_Days
         Seq = BlockRec(I)%SeqNum
         IF (Seq .LT. 1) CYCLE

         NxtBlk = RNum + BlockRec(I)%NumStations + 1
         PP = 1                                       ! flag = NEW
         IF (BlockRec(I)%Processed) PP = -1           ! flag = OLD
         NxtRec = RNum + 1

         !
         !  Is this the last block to be written?  If so, then the
         !  NxtBlk and NxtRec (maybe) fields need to be handled special.
         !
         IF (Seq .EQ. EndOfPrvData) THEN
            NxtBlk = 0
            IF (BlockRec(I)%NumStations .EQ. 0) NxtRec = 0
         END IF
         WRITE(UNum, REC=RNum, IOSTAT=IOS) Seq, NxtBlk, PP, Null46, NxtRec
         RNum = RNum + 1

         IF (ASSOCIATED(BlockRec(I)%Stations)) THEN 
            CurStation => BlockRec(I)%Stations     ! first item in list
            DO J = 1, BlockRec(I)%NumStations
              IF (.NOT. ASSOCIATED(CurStation)) THEN
                 ErrorMessage = 'ProvMetData: Invalid pointer error while writing binary file.'
                 CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
                 WRITE(ErrorMessage,*) 'Seq, CurStation # = ', Seq, J
                 CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
                 GOTO 898
              END IF
              ILat = CurStation%Lat
              ILon = CurStation%Long
              DO K=1,7
                 TheData(K) = CurStation%Dat(K)
              END DO
              StNumber = CurStation%StnID

              NxtRec = RNum + 1
              IF ((NxtBlk .EQ. 0) .AND. (J .EQ. BlockRec(I)%NumStations)) NxtRec = 0
              WRITE(UNum, REC=RNum, IOSTAT=IOS) ILat,ILon, TheData(:), StNumber, NxtRec
              RNum = RNum + 1
              CurStation => CurStation%NxtSta
            END DO 
         END IF
      END DO
      TotalRecs = RNum - 1

      !
      !  Fix record 1
      !
      READ(UNum, REC=1, IOSTAT=IOS) Seq, NxtBlk, J, Skip46, NxtRec
      J = TotalRecs
      IF (BlockRec(1)%Processed) J = 0 - TotalRecs
      WRITE(UNum, REC=1, IOSTAT=IOS) Seq, NxtBlk, J, MaxEntriesOnOneDay, Skip42, NxtRec
      CLOSE(UNum)
      CALL FileWasClosed(UNum)
      RETURN

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : WriteBinaryFile...'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)

      END SUBROUTINE WriteBinaryFile


!--------------------------------------------------------------------------
      SUBROUTINE WriteASCIIFile(PRVName)

      IMPLICIT NONE
      INTEGER   ::  SeqNum, NxtBlk, Rc, NxtRec
      INTEGER   ::  I, K, IOS, Y, M, D, FNum
      INTEGER   ::  UNumI, UNumO, MaxStns
      INTEGER (KIND=INTEGER2) :: Lt, Ln, IVals(7)
      LOGICAL :: Flag

      CHARACTER (LEN=*)  :: PRVName

      CHARACTER (LEN=4)  :: FN
      CHARACTER (LEN=40) :: StID
      CHARACTER (LEN=80) :: File11, File21

      File11 = TRIM(PrvName)
      UNumI = GetFreeUnitNumber()
      OPEN(UNIT=UNumI, FILE=TRIM(File11), STATUS='OLD', ACCESS='DIRECT',      &
           RECL=62, FORM='UNFORMATTED', IOSTAT=IOS)
      IF (IOS .NE. 0) RETURN
      CALL FileWasOpened(UNumI); IF (ErrorLevel .NE. 0) GOTO 899

      !
      !  Find out name of next TmpPrv????.TSH file (next higher number).  Open it.
      !
      Flag = .TRUE.
      FNum = 0
      DO WHILE (Flag)
         FNum = FNum  + 1
         IF (FNum .GT. 9999) THEN
            ErrorMessage = 'Unable to open a TmpPrv????.TSH file.  9999 already exists.'
            CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
            GOTO 898
         END IF
         WRITE(FN, '(I4.4)') FNum
         File21 = 'TmpPrv'//FN//'.TSH'
         INQUIRE(FILE=TRIM(File21), EXIST=Flag)
      END DO
      UNumO = GetFreeUnitNumber()
      OPEN(UNIT=UNumO, FILE=TRIM(File21), STATUS='NEW', ERR=821)
      CALL FileWasOpened(UNumO); IF (ErrorLevel .NE. 0) GOTO 899

      I = 1
      READ(UNumI, REC=1, IOSTAT=IOS) SeqNum, NxtBlk, Rc, MaxStns, Skip42, NxtRec 
      DO WHILE (IOS .EQ. 0)
         IF (SeqNum .GT. 0) THEN
            CALL SequenceDate(D, M, Y, SeqNum); IF (ErrorLevel .NE. 0) GOTO 899
            IF ((Y .GT. 1900) .AND. (Y .LT. 2050)) THEN
               READ(UNumI, REC=I, ERR=822) SeqNum, NxtBlk, Rc, Skip46, NxtRec
               WRITE(UNumO, 2321, ERR=813) I, Y,M,D, NxtBlk, Rc, NxtRec
            ELSE
               READ(UNumI, REC=I, ERR=822)    Lt,Ln, (IVals(K),K=1,7), StID, NxtRec
               WRITE(UNumO, 2322, ERR=813) I, Lt,Ln, (IVals(K),K=1,7), StID, NxtRec
            END IF
         ELSE
            READ(UNumI, REC=I, ERR=822)    Lt,Ln, (IVals(K),K=1,7), StID, NxtRec
            WRITE(UNumO, 2322, ERR=813) I, Lt,Ln, (IVals(K),K=1,7), StID, NxtRec
         END IF
         I = I + 1
         READ(UNumI, REC=I, IOSTAT=IOS) SeqNum
      END DO

      CLOSE(UNumI)
      CALL FileWasClosed(UNumI); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE(UNumO)
      CALL FileWasClosed(UNumO); IF (ErrorLevel .NE. 0) GOTO 899
      RETURN

  811 ErrorMessage = 'ProvMetData: Error opening file '//TRIM(File11)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  812 ErrorMessage = 'ProvMetData: Error reading file '//TRIM(File11)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  813 ErrorMessage = 'ProvMetData: Error writing file '//TRIM(File11)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  821 ErrorMessage = 'ProvMetData: Error opening file '//TRIM(File21)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  822 ErrorMessage = 'ProvMetData: Error reading file '//TRIM(File21)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  823 ErrorMessage = 'ProvMetData: Error writing file '//TRIM(File21)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : WriteAsciiFile...'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)

 2321 FORMAT(I6, ':', I4,'/',I2,'/',I2, 3I9)
 2322 FORMAT(I6, ':', 2I6, 7I6, 2X, A40, I9)

      END SUBROUTINE WriteASCIIFile

      !--------------------------------------------------------------------------
      !  Convert from units in the station data file into the units that are
      !  stored in the binary database file. It is assumed that when this routine
      !  is called, the DataVals data is already in the following units:
      !    DataType        Incoming units       Database units
      !    Air Temps       Degrees C            Hundredths of Deg C
      !    Precipitation   millimeters          Hundredths of mm
      !    Wind Speed      meters/second        Hundredths of m/s
      !    Cloud Cover     Percent              Percent
      !--------------------------------------------------------------------------
      SUBROUTINE ConvertToDatabaseUnits(DataVals)
      IMPLICIT NONE
      INTEGER (KIND=INTEGER2), INTENT(INOUT) :: DataVals(7)
      INTEGER  :: I
      
      DO I = 1, 7
         IF (DataVals(I) .LT. -990) THEN
            DataVals(I) = -9999                      ! Missing data.
         ELSE
            SELECT CASE(I)
               CASE (1,2,4,5)                        ! Air temperatures
                   DataVals(I) = DataVals(I) * 100   !    Hundredths of Deg C
               CASE (3)                              ! Precipitation
                   DataVals(I) = DataVals(I) * 100   !    Hundredths of mm
               CASE (6)                              ! Wind speed
                   DataVals(I) = DataVals(I) * 100   !    Hundredths of m/s
               CASE (7)                              ! Cloud Cover
                   DataVals(I) = DataVals(I) * 1     !    Percent (0-100)
            END SELECT
         ENDIF
      END DO

      END SUBROUTINE ConvertToDatabaseUnits

!------------------------------------------------------------------------------
      CHARACTER (LEN=4) FUNCTION ID_String (I1, I2)
!
!     This routine returns the concatenated binary image of the two 2-byte
!     integers as a 4-byte string.
!     An effort is made to enable subsequent ordering by latitude first and
!     then longitude by arranging the byte order so that the high order byte
!     of each integer is stored before the low order byte.  The byte order
!     in the resultant string is as follows:
!        INT1(hi) INT1(lo) INT2(hi) INT2(lo)
!
!     Rewritten again by TSH, 07mar2001, to use the TRANSFER function.
!     We found that it CHAR() of any negative number is, as far as we can
!     determine, not defined by the Fortran standard.  While debugging the
!     program the debugger exhibited strange behavior when processing
!     CHAR(-118) (I *think that was the number).  So I changed the code from
!     taking CHAR() of each individual byte (which will be a negative value
!     about 1/2 the time for the low-order byte) to using the TRANSFER function.
!     Note that the same thing could be accomplished with EQUIVALENCE, but
!     that feature has been declared obsolescent in the standard, so I wanted
!     to avoid it.
!  
!     Rewritten TECII, 21mar95, as a FUNCTION rather than a SUBROUTINE
!     which required the use of a COMMON block, after changes made to PROVSNAL
!     by TSH 18jan94.
!
!     TECII, 26jun92
!
      IMPLICIT NONE
      INTEGER (KIND=INTEGER2) :: I1, I2
      CHARACTER (LEN=2) :: S1, S2

      S1 = TRANSFER(I1, S1)      ! do a bit pattern assignment
      S2 = TRANSFER(I2, S2)      ! do a bit pattern assignment
      ID_String = S1 // S2
      RETURN

      END FUNCTION ID_String

!-----------------------------------------------------------------------------
      SUBROUTINE WriteBsnnmPD(NewPD)

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NewPd
      INTEGER             :: I, U1, PD
      CHARACTER (LEN=4)   :: OTL

      U1 = GetFreeUnitNumber(); IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(BsnFile), STATUS='OLD', RECL=12,           &
           ACCESS='DIRECT', FORM='FORMATTED', ERR=811)
      CALL FileWasOpened(U1); IF (ErrorLevel .NE. 0) GOTO 899
      READ (U1, 1011, REC=2, ERR=812) I, PD, OTL
      WRITE(U1, 1011, REC=2, ERR=813) I, NewPD, OTL, CHAR(10)
      CLOSE(U1)
      CALL FileWasClosed(U1); IF (ErrorLevel .NE. 0) GOTO 899
      RETURN

  811 ErrorMessage = 'ProvMetData: Error opening file '//TRIM(BsnFile)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  812 ErrorMessage = 'ProvMetData: Error reading file '//TRIM(BsnFile)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  813 ErrorMessage = 'ProvMetData: Error writing file '//TRIM(BsnFile)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : WriteBsnnmPD...'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)

 1001 FORMAT(3X, I3, 5X, A1)
 1011 FORMAT(I3, I3, A4, 1X, A1)

      END SUBROUTINE WriteBsnnmPD
!-----------------------------------------------------------------------------

END MODULE ProvMetDataSubs

!============================================================================
!============================================================================
!============================================================================
MODULE ProvMetDataMain
CONTAINS
      SUBROUTINE DO_ProvMetData(ErrStatus, HandleIn)
      USE ProvMetDataSubs
      IMPLICIT NONE

      INTEGER, INTENT(INOUT)  :: ErrStatus
      INTEGER, INTENT(IN)     :: HandleIn

      INTEGER (KIND=INTEGER2) :: ILat, ILon
      INTEGER ::  I, J, K, L, D, M, Y, BD, PD, LastRec, Seq, SeqNum, RNum
      INTEGER ::  Base_Day, Base_Month, Base_Year, Base, Start
      INTEGER ::  Prov_Day, Prov_Month, Prov_Year, PrvEndDate, PrvDays, PrvPct
      INTEGER ::  NumStations, Flag, CurRec, NxtRec, NxtBlk
      INTEGER ::  TotalRecs, StationsToProcess, U11
      INTEGER ::  IOS, IOS11, IOS22, LinesRemoved, Ndx
      LOGICAL ::  AllMissing, FileExist, OkToContinue
      REAL    ::  Lat, Lon, DV

      INTEGER (KIND=INTEGER2), DIMENSION(7) :: DatVals
      
      CHARACTER (LEN=1)  :: Lf
      CHARACTER (LEN=3)  :: Bsn, Dum3
      CHARACTER (LEN=4)  :: OTLCode
      CHARACTER (LEN=10) :: LkName
      CHARACTER (LEN=20) :: PRVName
      CHARACTER (LEN=40) :: StNumber
      CHARACTER (LEN=80) :: StnName, Line, File11, File22, File33
      CHARACTER (LEN=100):: EMsg

      INTEGER            :: FileVer
      CHARACTER (LEN=30) :: TFilename

      TYPE (MetCsvData)  :: StnData
      TYPE (StationRecordType), POINTER :: CurStation, SRT
      TYPE (StnRecList), POINTER        :: L1, L2

      WinHandle = HandleIn
      LogUnit = -1

      IF (UserWantsToCancel()) GOTO 999    ! test for user CANCEL

      DO I = 1, MaxOpenedFiles
         OpenedFileUnits(I) = -1
      END DO
      OpenedFiles = 0

      !
      !  Initialize the NullX character strings
      !
      DO I = 1, 58
        Null58(I:I) = CHAR(0)
      END DO
      Null46 = Null58(1:46)
      Null42 = Null58(1:42)
      Null40 = Null58(1:40)

      !
      Lf = CHAR(10)
      ErrStatus = 0
      ErrorLevel = 0       ! global variable in GLERLUTIL module

      CALL DateSequence(1, 1, 3000, EarliestNewData); IF (ErrorLevel .NE. 0) GOTO 899  ! arbitrary BIG number
      CALL DateSequence(1, 1,    1,   LatestNewData); IF (ErrorLevel .NE. 0) GOTO 899  ! arbitrary BIG number

      !
      !  Initialize stuff for the dynamic linked-list
      !
      ALLOCATE(L1, STAT=IOS)
      IF (IOS .NE. 0) THEN
         ErrorMessage = 'ProvMetData: Error allocating RAM for head of linked list!'
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         GOTO 898
      END IF
      L1%Used = 0
      NULLIFY(L1%Next)
      StnListHead => L1

      DO I = 1, Max_No_Days
         BlockRec(I)%SeqNum = -999
         BlockRec(I)%NumStations = 0
         BlockRec(I)%Processed = .FALSE.      ! default = block has NEW data
         NULLIFY(BlockRec(I)%Stations)
      END DO
      CALL DateSequence(1, 1, 1, EndOfPrvData); IF (ErrorLevel .NE. 0) GOTO 899    ! arbitrary small number

      !
      !  WARNING! The following date is the BASE DATE and corresponds to the
      !  first day of the provisional data which MUST BE the day after the last
      !  day of the climatologic data records available for the application.
      !  The user will not be allowed by this program to add or change any
      !  provisional data prior to this base date nor is it assumed that he/she
      !  will have any reason to do so.  DO NOT CHANGE THIS BASE DATE as it is
      !  used in companion programs and must agree.  When changed, the previous
      !  version of file: ProvMetData.xxx  must be amended to correspond to
      !  the new base date by deleting all earlier data and repacking the file.  The user
      !  should only perform this operation with program: REPACK by changing the
      !  base date and running that program; once done, earlier provisional data
      !  is lost and unrecoverable.  This operation need only be performed
      !  infrequently as climatologic data becomes available and the forecast
      !  data bases are rebuilt by GLERL (say, once every 2-3 years or so) or when
      !  the file: ProvMetData.xxx becomes too large to manage efficiently as a
      !  linked list.
      !
      !  Open EBSNNM/BSNNM and get basin name and base date.  The structure of
      !  record 2 is a little different, but the PD is in the same place, so for
      !  READ purposes we can use the same FORMAT statement.
      !
      File11 = 'BSNNM'
      U11 = GetFreeUnitNumber()
      OPEN(UNIT=U11, FILE=TRIM(File11), STATUS='OLD', RECL=12,      &
           ACCESS='DIRECT', FORM='FORMATTED')
      CALL FileWasOpened(U11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U11, 1000, REC=1, ERR=811) LkName
      READ(U11, 1001, REC=2, ERR=811) PD
      READ(U11, 1002, REC=3, ERR=811) Base_Day, Base_Month, Base_Year
      READ(U11, 1002, REC=4, ERR=811) Prov_Day, Prov_Month, Prov_Year
      CLOSE(U11)
      CALL FileWasClosed(U11); IF (ErrorLevel .NE. 0) GOTO 899
      IF (PD .NE. 1) THEN
        ErrorMessage = 'ProvMetData: Attempt to run ProvMetData in wrong sequence!'
        CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
        GOTO 898
      END IF
      CALL DateSequence(Base_Day, Base_Month, Base_Year, Base); IF (ErrorLevel .NE. 0) GOTO 899
      CALL DateSequence(Prov_Day, Prov_Month, Prov_Year, PrvEndDate); IF (ErrorLevel .NE. 0) GOTO 899
      PrvDays = PrvEndDate - Base
      Bsn = LkName(1:3)
      BlockRec(1)%SeqNum = Base   ! initialize entry 1 in BlockRec

      ErrorMessage = 'Adding provisional data for '//TRIM(LkName)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)

      !
      !  If the data base does not already exist, create a new empty file.
      !
      PRVName = 'ProvMetData.'//Bsn
      File11 = TRIM(PRVName)
      INQUIRE(FILE=TRIM(File11), EXIST=FileExist)
      IF (.NOT. FileExist) THEN
         CALL CreateInitialFile(File11, Base)
         IF (ErrorLevel .NE. 0) GOTO 899
      END IF
      
      !
      !  Open data base and read record 1 to verify validity of starting date.
      !
      U11 = GetFreeUnitNumber()
      OPEN(UNIT=U11, FILE=TRIM(File11), STATUS='OLD', ACCESS='DIRECT',      &
           RECL=62, FORM='UNFORMATTED', ERR=811)
      CALL FileWasOpened(U11); IF (ErrorLevel .NE. 0) GOTO 899
      READ(U11, REC=1, ERR=812) Start, NxtBlk, LastRec, Skip46, NxtRec
      IF (Start .NE. Base) THEN
         ErrorMessage = 'ProvMetData: '//TRIM(PRVName) // ' file starts on wrong date!'
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         CALL SequenceDate(D, M, Y, Base); IF (ErrorLevel .NE. 0) GOTO 899
         WRITE(ErrorMessage, 8000) 'Base Date ', D, M, Y
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         CALL SequenceDate(D, M, Y, Start); IF (ErrorLevel .NE. 0) GOTO 899
         WRITE(ErrorMessage, 8001) TRIM(PRVName), D, M, Y
         CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
         GOTO 898
      END IF
      CLOSE(U11)
      CALL FileWasClosed(U11); IF (ErrorLevel .NE. 0) GOTO 899

      IF (UserWantsToCancel()) GOTO 999    ! test for user CANCEL

      !
      !  Now read all of the existing data.
      !  To indicate to user that progress is being made (because this could
      !  be a time-consuming operation) assume that this will be 50% of the
      !  processing and give status reports accordingly.
      !
      CALL ReadExistingData(PrvName, PrvDays, 50, Base)

      !
      !  Reopen data base so that we can add to it.
      !
      File11 = TRIM(PRVName)
      U11 = GetFreeUnitNumber()
      OPEN(UNIT=U11, FILE=TRIM(File11), STATUS='OLD', ACCESS='DIRECT',      &
           RECL=62, FORM='UNFORMATTED', ERR=811)
      CALL FileWasOpened(U11); IF (ErrorLevel .NE. 0) GOTO 899
      
      !
      !  Open the list of stations.
      !
      File22 = 'PROVSTNS.LIS'
      OPEN(UNIT=22, FILE=TRIM(File22), STATUS='OLD', ERR=821)
      CALL FileWasOpened(22); IF (ErrorLevel .NE. 0) GOTO 899

      !
      !  Count the number of entries for display purposes.  Then rewind the file.
      !
      StationsToProcess = 0
      READ(22, 2200, IOSTAT=IOS22) StnName
      DO WHILE (IOS22 .EQ. 0)
        INQUIRE(FILE=TRIM(StnName), EXIST=FileExist)
        IF (FileExist) StationsToProcess = StationsToProcess + 1
        READ(22, 2200, IOSTAT=IOS22) StnName
      END DO
      REWIND(22)

      !
      !  Loop through list of stations, processing each one.
      !
      ErrorMessage = ' '; CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)     ! write blank line that will be overwritten
      NumStations = 0
      READ(22, 2200, IOSTAT=IOS22) StnName
      INQUIRE(FILE=TRIM(StnName), EXIST=FileExist)
      DO WHILE ((.NOT. FileExist) .AND. (IOS22 .EQ. 0))
         IF (IOS22 .GT. 0) GOTO 822
         READ(22, 2200, IOSTAT=IOS22) StnName
         INQUIRE(FILE=TRIM(StnName), EXIST=FileExist)
      END DO
      DO WHILE (IOS22 .EQ. 0)
         IF (UserWantsToCancel()) GOTO 999    ! test for user CANCEL
         EMsg = 'Processing station file: '//TRIM(StnName)
         CALL WriteErrorMessage(EMsg, '~^~', LogUnit, .TRUE.)

         !
         ! Update the "% finished" status display. I am calculating it as
         ! the 50% from reading old data into RAM plus the progress
         ! we've made on the new data.
         !
         NumStations = NumStations + 1
         D = 50 + ((NumStations*100) / StationsToProcess) / 2     ! % finished
         WRITE(EMsg, '(I3)') D
         CALL WriteErrorMessage(EMsg, '~%~', LogUnit, .TRUE.)
         
         !
         !  Read the data, then
         !    1) verify the station location is ok
         !    2) do any necessary data conversions so that the data from this point
         !       on will be in the standard GLERL setup (the required 7 data types 
         !       in the prescribed data units).
         !
         OkToContinue = .TRUE.
         CALL ReadMetCsvFile(StnName, StnData, LinesRemoved, ErrStatus)
         IF (ErrStatus .NE. 0) OkToContinue = .FALSE.

         IF (StnData%Lat  .LT. BigBoxBottom) OkToContinue = .FALSE.       ! S of box
         IF (StnData%Lat  .GT. BigBoxTop)    OkToContinue = .FALSE.       ! N of box
         IF (StnData%Long .LT. BigBoxLeft)   OkToContinue = .FALSE.       ! W of box
         IF (StnData%Long .GT. BigBoxRight)  OkToContinue = .FALSE.       ! E of box
         
         !
         !  Adjust the values in order to match the prescribed units of the
         !  ProvMetData.xxx file. Note that in this step I am only converting to
         !  the correct "basic" units (Celsius, m/s, mm, %). Before writing to the
         !  file, we will still need to adjust as needed to get into "hundredths".
         !
         CALL MakeDataGlerlStandard(StnData, ErrStatus)
         IF (ErrStatus .NE. 0) OkToContinue = .FALSE.
         
         !
         !  If station is usable, add all of the data to the database
         !  Note that only data on/after the "BASE" date can be used.
         !  Data prior to that date is discarded.
         !
         IF (OkToContinue) THEN
            ILat = NINT(StnData%Lat  * 100)
            ILon = NINT(StnData%Long * 100)
            DO Seq = StnData%SDate, StnData%EDate
               IF (Seq .GE. Base) THEN
                  Ndx = Seq - StnData%SDate + 1
                  !
                  !  All data values except cloud cover need to be multiplied by
                  !  100 in order to make them "hundredths of x".
                  !
                  DatVals(:) = -9999     ! "missing"
                  DO I = 1, 6
                     DV = StnData%DataValues(I,Ndx)
                     IF (.NOT. IsMissing(DV)) DatVals(I) = DV * 100
                  END DO
                  DV = StnData%DataValues(7,Ndx)
                  IF (.NOT. IsMissing(DV)) DatVals(7) = DV
                  IF (Seq .GT. LatestNewData) LatestNewData = Seq
                  CALL AddRecord(Seq, ILat,ILon, DatVals, StnData%ID, .TRUE.)
                  IF (ErrorLevel .NE. 0) GOTO 899
               END IF
            END DO
         END IF
         
         !
         !  Get the next station file name
         !
         READ(22, 2200, IOSTAT=IOS22) StnName
         INQUIRE(FILE=TRIM(StnName), EXIST=FileExist)
         DO WHILE ((.NOT. FileExist) .AND. (IOS22 .EQ. 0))
            IF (IOS22 .GT. 0) GOTO 822
            READ(22, 2200, IOSTAT=IOS22) StnName
            INQUIRE(FILE=TRIM(StnName), EXIST=FileExist)
         END DO
      END DO 

      CALL SequenceDate(D, M, Y, LatestNewData); IF (ErrorLevel .NE. 0) GOTO 899
      WRITE(ErrorMessage, 9127) NumStations, MonCode3(M), D, Y
      CALL WriteErrorMessage(ErrorMessage, '~^~', -1, .TRUE.)

      CLOSE(U11)
      CALL FileWasClosed(U11); IF (ErrorLevel .NE. 0) GOTO 899
      CLOSE(22)
      CALL FileWasClosed(22); IF (ErrorLevel .NE. 0) GOTO 899

      IF (UserWantsToCancel()) GOTO 999    ! test for user CANCEL

      !
      !  Determine the maximum number of stations on any single day
      !
      MaxEntriesOnOneDay = 0
      DO BD = 1, Max_No_Days
        L = 0
        CurStation => BlockRec(BD)%Stations
        DO WHILE (ASSOCIATED(CurStation))
           L = L + 1
           CurStation => CurStation%NxtSta
        END DO
        IF (L .GT. MaxEntriesOnOneDay) MaxEntriesOnOneDay = L
      END DO

      !
      !  Now that we've processed all of the station data, and it's been
      !  incorporated into the data structures, write it out into the
      !  binary database file.
      !
      CALL WriteBinaryFile(PRVName); IF (ErrorLevel .NE. 0) GOTO 899
      CALL WriteASCIIFile(PRVName); IF (ErrorLevel .NE. 0) GOTO 899     ! for debug purposes

      !
      !  Update the BSNNM file
      !
      IF (UserWantsToCancel()) GOTO 999    ! test for user CANCEL

      File11 = 'BSNNM'
      U11 = GetFreeUnitNumber()
      OPEN(UNIT=U11, FILE=TRIM(File11), STATUS='OLD', RECL=12,      &
           ACCESS='DIRECT', FORM='FORMATTED')
      CALL FileWasOpened(U11); IF (ErrorLevel .NE. 0) GOTO 899
      READ (U11, 1011, REC=2, ERR=812) I, J, OTLCode
      WRITE(U11, 1011, REC=2, ERR=813) I, 2, OTLCode, Lf
      CLOSE(U11)
      CALL FileWasClosed(U11); IF (ErrorLevel .NE. 0) GOTO 899

      ErrorLevel = 0
      ErrStatus = 0
      ErrorMessage = 'ProvMetData completed successfully for '//TRIM(LkName)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
      WRITE(ErrorMessage, 9100) MaxEntriesOnOneDay
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)

      GOTO 999

!
!  Error Handling
!
  811 ErrorMessage = 'ProvMetData: Error opening file '//TRIM(File11)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  812 ErrorMessage = 'ProvMetData: Error reading file '//TRIM(File11)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  813 ErrorMessage = 'ProvMetData: Error writing file '//TRIM(File11)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  821 ErrorMessage = 'ProvMetData: Error opening file '//TRIM(File22)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  822 ErrorMessage = 'ProvMetData: Error reading file '//TRIM(File22)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  823 ErrorMessage = 'ProvMetData: Error writing file '//TRIM(File22)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  831 ErrorMessage = 'ProvMetData: Error opening file '//TRIM(File33)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  832 ErrorMessage = 'ProvMetData: Error reading file '//TRIM(File33)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898
  833 ErrorMessage = 'ProvMetData: Error writing file '//TRIM(File33)
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.); GOTO 898

  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : ProvMetData_main...'
      CALL WriteErrorMessage(ErrorMessage, '', -1, .TRUE.)
      CALL WriteBsnnmPD(-1)

!
!    Return to calling procedure.  I used a GOTO 999 throughout the program
!    instead of a RETURN everywhere so that I can do any "cleanup" here.
!
 999 CONTINUE
     CALL CloseAllFiles()
     IF (ASSOCIATED(StnListHead)) CALL MyDealloc(StnListHead)

     CALL WriteErrorMessage('0', '~%~', -1, .TRUE.)
     RETURN


!
!  FORMATs
!
 1000 FORMAT(A10, 1X, A1)
 1001 FORMAT(3X, I3, 5X, A1)
 1002 FORMAT(2I3, I5, A1)
 1011 FORMAT(I3, I3, A4, 1X, A1)

 2200 FORMAT(A)

 3301 FORMAT(1X, A10, 2F10.3)
 3302 FORMAT(2I3, I5, 4I4, 2F10.3)
 3303 FORMAT(2I3, I5, 3I4, 4X, 2F10.3)
 3305 FORMAT(A)

 8000 FORMAT(A, ' is (D,M,Y): ', 2I3, I5)
 8001 FORMAT(A, ' starts on (D,M,Y): ', 2I3, I5)
 
 9100 FORMAT('Maximum numbers of stations reporting for a single day=', I5)
 9127 FORMAT('Added ', I0, ' stations - data ended on ', A3, 1X, I2, ', ', I4)

      END SUBROUTINE DO_ProvMetData

END MODULE ProvMetDataMain



